@@ -1,16 +1,21 @@
 #ifndef __CONTROLSOCKET_H__
 #define __CONTROLSOCKET_H__
 
+#include "socket.h"
+#include "logging_private.h"
+#include "backend.h"
+
 class COpData
 {
 public:
-	COpData(enum Command op_Id);
+	COpData(Command op_Id);
 	virtual ~COpData();
 
 	int opState;
-	const enum Command opId;
+	Command const opId;
 
 	bool waitForAsyncRequest;
+	bool holdsLock;
 
 	COpData *pNextOpData;
 };
@@ -19,7 +24,8 @@ class CConnectOpData : public COpData
 {
 public:
 	CConnectOpData()
-		: COpData(cmd_connect)
+		: COpData(Command::connect),
+		port(0)
 	{
 	}
 
@@ -30,14 +36,14 @@ public:
 class CFileTransferOpData : public COpData
 {
 public:
-	CFileTransferOpData();
+	CFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path);
 	virtual ~CFileTransferOpData();
 	// Transfer data
 	wxString localFile, remoteFile;
 	CServerPath remotePath;
-	bool download;
+	const bool download;
 
-	wxDateTime fileTime;
+	CDateTime fileTime;
 	wxFileOffset localFileSize;
 	wxFileOffset remoteFileSize;
 
@@ -55,7 +61,7 @@ class CMkdirOpData : public COpData
 {
 public:
 	CMkdirOpData()
-		: COpData(cmd_mkdir)
+		: COpData(Command::mkdir)
 	{
 	}
 
@@ -73,9 +79,10 @@ class CChangeDirOpData : public COpData
 {
 public:
 	CChangeDirOpData()
-		: COpData(cmd_cwd)
+		: COpData(Command::cwd)
 	{
-		triedMkd = false;
+		tryMkdOnFail = false;
+		link_discovery = false;
 	}
 
 	virtual ~CChangeDirOpData()
@@ -84,16 +91,19 @@ public:
 
 	CServerPath path;
 	wxString subDir;
-	bool triedMkd;
+	bool tryMkdOnFail;
 	CServerPath target;
+
+	bool link_discovery;
 };
 
-enum TransferEndReason
+enum class TransferEndReason
 {
 	none,
 	successful,
 	timeout,
-	transfer_failure,					// Error during the transfer, like lost connection lack of diskspace and so on
+	transfer_failure,					// Error during transfer, like lost connection. Retry automatically
+	transfer_failure_critical,			// Error during transfer like lack of diskspace. Needs user interaction
 	pre_transfer_command_failure,		// If a command fails prior to sending the transfer command
 	transfer_command_failure_immediate,	// Used if server does not send the 150 reply after the transfer command
 	transfer_command_failure,			// Used if the transfer command fails, but after receiving a 150 first
@@ -101,65 +111,59 @@ enum TransferEndReason
 	failed_resumetest
 };
 
-#include "logging_private.h"
-#include "backend.h"
-
 class CTransferStatus;
-class CControlSocket: public wxEvtHandler, public CLogging
+class CControlSocket: public CLogging, public CEventHandler
 {
 public:
-	CControlSocket(CFileZillaEnginePrivate *pEngine);
+	CControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CControlSocket();
 
 	virtual int Connect(const CServer &server) = 0;
-	virtual int ContinueConnect(const wxIPV4address *address) { return FZ_REPLY_ERROR; }
 	virtual int Disconnect();
 	virtual void Cancel();
-	virtual int List(CServerPath path = CServerPath(), wxString subDir = _T(""), bool refresh = false) { return FZ_REPLY_NOTSUPPORTED; }
+	virtual int List(CServerPath path = CServerPath(), wxString subDir = wxString(), int flags = 0);
 	virtual int FileTransfer(const wxString localFile, const CServerPath &remotePath,
 							 const wxString &remoteFile, bool download,
-							 const CFileTransferCommand::t_transferSettings& transferSettings) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual int RawCommand(const wxString& command = _T("")) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual int Delete(const CServerPath& path, const std::list<wxString>& files) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual int RemoveDir(const CServerPath& path = CServerPath(), const wxString& subDir = _T("")) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual int Mkdir(const CServerPath& path) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual int Rename(const CRenameCommand& command) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual int Chmod(const CChmodCommand& command) { return FZ_REPLY_NOTSUPPORTED; }
-	virtual bool Connected() const = 0;
+							 const CFileTransferCommand::t_transferSettings& transferSettings);
+	virtual int RawCommand(const wxString& command = wxString());
+	virtual int Delete(const CServerPath& path, const std::list<wxString>& files);
+	virtual int RemoveDir(const CServerPath& path = CServerPath(), const wxString& subDir = wxString());
+	virtual int Mkdir(const CServerPath& path);
+	virtual int Rename(const CRenameCommand& command);
+	virtual int Chmod(const CChmodCommand& command);
+	virtual bool Connected() = 0;
 
 	// If m_pCurrentOpData is zero, this function returns the current command
 	// from the engine.
-	enum Command GetCurrentCommandId() const;
+	Command GetCurrentCommandId() const;
 
-	virtual void TransferEnd() { }
+	virtual void TransferEnd() {}
 
+	void SendAsyncRequest(CAsyncRequestNotification* pNotification);
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification) = 0;
-
-	void InitTransferStatus(wxFileOffset totalSize, wxFileOffset startOffset, bool list);
-	void SetTransferStatusStartTime();
-	void UpdateTransferStatus(wxFileOffset transferredBytes);
-	void ResetTransferStatus();
-	bool GetTransferStatus(CTransferStatus &status, bool &changed);
+	bool SetFileExistsAction(CFileExistsNotification *pFileExistsNotification);
 
 	const CServer* GetCurrentServer() const;
 
 	// Conversion function which convert between local and server charset.
-	wxString ConvToLocal(const char* buffer);
-	wxChar* ConvToLocalBuffer(const char* buffer);
-	wxChar* ConvToLocalBuffer(const char* buffer, wxMBConv& conv);
+	wxString ConvToLocal(const char* buffer, size_t len);
+	wxChar* ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen);
+	wxChar* ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen);
 	wxCharBuffer ConvToServer(const wxString& str, bool force_utf8 = false);
 
+	void SetActive(CFileZillaEngine::_direction direction);
+
 	// ---
 	// The following two functions control the timeout behaviour:
 	// ---
-    
+
 	// Call this if data could be sent or retrieved
 	void SetAlive();
-	
+
 	// Set to true if waiting for data
 	void SetWait(bool waiting);
 
-	CFileZillaEnginePrivate* GetEngine() { return m_pEngine; }
+	CFileZillaEnginePrivate& GetEngine() { return engine_; }
 
 	// Only called from the engine, see there for description
 	void InvalidateCurrentWorkingDir(const CServerPath& path);
@@ -172,67 +176,75 @@ protected:
 
 	virtual int ResetOperation(int nErrorCode);
 
-	// Called by ResetOperation if there's a queued operation
-	virtual int SendNextCommand(int prevResult = FZ_REPLY_OK);
+	virtual int SendNextCommand();
 
-	wxString ConvertDomainName(wxString domain);
+	void LogTransferResultMessage(int nErrorCode, CFileTransferOpData *pData);
+
+	// Called by ResetOperation if there's a queued operation
+	virtual int ParseSubcommandResult(int prevResult);
+
+	wxString ConvertDomainName(wxString const& domain);
 
 	int CheckOverwriteFile();
 
+	void CreateLocalDir(const wxString &local_file);
+
 	bool ParsePwdReply(wxString reply, bool unquoted = false, const CServerPath& defaultPath = CServerPath());
 
 	COpData *m_pCurOpData;
 	int m_nOpState;
-	CFileZillaEnginePrivate *m_pEngine;
+	CFileZillaEnginePrivate & engine_;
 	CServer *m_pCurrentServer;
 
 	CServerPath m_CurrentPath;
-	
-	CTransferStatus *m_pTransferStatus;
-	int m_transferStatusSendState;
-	
+
 	wxCSConv *m_pCSConv;
 	bool m_useUTF8;
 
 	// Timeout data
-	wxTimer m_timer;
-	wxStopWatch m_stopWatch;
+	timer_id m_timer{};
+	CMonotonicClock m_lastActivity;
 
 	// -------------------------
 	// Begin cache locking stuff
 	// -------------------------
 
+	enum locking_reason
+	{
+		lock_unknown = -1,
+		lock_list,
+		lock_mkdir
+	};
+
 	// Tries to obtain lock. Returns true on success.
 	// On failure, caller has to pass control.
 	// SendNextCommand will be called once the lock gets available
 	// and engine could obtain it.
-	bool TryLockCache(const CServerPath& directory);
+	// Lock is recursive. Lock counter increases on suboperations.
+	bool TryLockCache(enum locking_reason reason, const CServerPath& directory);
+	bool IsLocked(enum locking_reason reason, const CServerPath& directory);
 
 	// Unlocks the cache. Can be called if not holding the lock
+	// Doesn't need reason as one engine can at most hold one lock
 	void UnlockCache();
 
 	// Called from the fzOBTAINLOCK event.
-	// Returns true iff engine is the first waiting engine
+	// Returns reason != unknown iff engine is the first waiting engine
 	// and obtains the lock.
 	// On failure, the engine was not waiting for a lock.
-	bool ObtainLockFromEvent();
+	enum locking_reason ObtainLockFromEvent();
 
-	bool HasLock();
+	bool IsWaitingForLock();
 
-#ifdef __VISUALC__
-	// Retarded compiler does not like my code
-	public:
-#endif
 	struct t_lockInfo
 	{
 		CControlSocket* pControlSocket;
 		CServerPath directory;
+		enum locking_reason reason;
 		bool waiting;
+		int lockcount;
 	};
 	static std::list<t_lockInfo> m_lockInfoList;
-#ifdef __VISUALC__
-	protected:
-#endif
 
 	const std::list<t_lockInfo>::iterator GetLockStatus();
 
@@ -242,45 +254,46 @@ protected:
 
 	bool m_invalidateCurrentPath;
 
-	DECLARE_EVENT_TABLE();
-	void OnTimer(wxTimerEvent& event);
-	void OnObtainLock(wxCommandEvent& event);
+	virtual void operator()(CEventBase const& ev);
+
+	void OnTimer(timer_id id);
+	void OnObtainLock();
 };
 
-class CRealControlSocket : public CControlSocket, public wxSocketClient
+class CProxySocket;
+class CRealControlSocket : public CControlSocket
 {
 public:
-	CRealControlSocket(CFileZillaEnginePrivate *pEngine);
+	CRealControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CRealControlSocket();
 
 	virtual int Connect(const CServer &server);
-	virtual int ContinueConnect(const wxIPV4address *address);
+	virtual int ContinueConnect();
 
-	virtual bool Connected() const { return IsConnected(); }
+	virtual bool Connected() { return m_pSocket->GetState() == CSocket::connected; }
 
 protected:
 	virtual int DoClose(int nErrorCode = FZ_REPLY_DISCONNECTED);
 	void ResetSocket();
 
-	// Easier functions to get the IP addresses
-	virtual wxString GetLocalIP() const;
-	virtual wxString GetPeerIP() const;
+	virtual void operator()(CEventBase const& ev);
+	void OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error);
+	void OnHostAddress(CSocketEventSource* source, wxString const& address);
 
-	DECLARE_EVENT_TABLE();
-	virtual void OnSocketEvent(wxSocketEvent &event);
 	virtual void OnConnect();
 	virtual void OnReceive();
-	virtual void OnSend();
-	virtual void OnClose();
-	
-	virtual bool Send(const char *buffer, int len);
+	void OnSend();
+	virtual void OnClose(int error);
+
+	bool Send(const char *buffer, int len);
+
+	CSocket* m_pSocket;
 
 	CBackend* m_pBackend;
+	CProxySocket* m_pProxyBackend;
 
 	char *m_pSendBuffer;
 	int m_nSendBufferLen;
-
-	bool m_onConnectCalled;
 };
 
 #endif
