@@ -4,7 +4,7 @@
 /* This class is responsible for parsing the directory listings returned by
  * the server.
  * Unfortunatly, RFC959 did not specify the format of directory listings, so
- * each server uses its own format. In addition to that, in most cases the 
+ * each server uses its own format. In addition to that, in most cases the
  * listings were not designed to be machine-parsable, they were meant to be
  * human readable by users of that particular server.
  * By far the most common format is the one returned by the Unix "ls -l"
@@ -17,8 +17,8 @@
  * Some servers send multiline listings where a single entry can span two
  * lines, this has to be detected as well, as far as possible.
  *
- * Some servers send MVS style listings which can consist of just the 
- * filename without any additional data. In order to prevent problems, this 
+ * Some servers send MVS style listings which can consist of just the
+ * filename without any additional data. In order to prevent problems, this
  * format is only parsed if the server is in fact recognizes as MVS server.
  *
  * Please see tests/dirparsertest.cpp for a list of supported formats and the
@@ -35,78 +35,117 @@
 class CLine;
 class CToken;
 class CControlSocket;
+
+namespace listingEncoding
+{
+	enum type
+	{
+		unknown,
+		normal,
+		ebcdic
+	};
+}
+
+
 class CDirectoryListingParser
 {
 public:
-	CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server);
+	CDirectoryListingParser(CControlSocket* pControlSocket, const CServer& server, listingEncoding::type encoding = listingEncoding::unknown, bool sftp_mode = false);
 	~CDirectoryListingParser();
 
 	CDirectoryListing Parse(const CServerPath &path);
 
-	void AddData(char *pData, int len);
-	void AddLine(const wxChar* pLine);
+	bool AddData(char *pData, int len);
+	bool AddLine(const wxChar* pLine);
 
 	void Reset();
 
 	void SetTimezoneOffset(const wxTimeSpan& span) { m_timezoneOffset = span; }
 
+	void SetServer(const CServer& server) { m_server = server; };
+
 protected:
-	CLine *GetLine(bool breakAtEnd = false);
+	CLine *GetLine(bool breakAtEnd, bool& error);
 
-	void ParseData(bool partial);
+	bool ParseData(bool partial);
 
-	bool ParseLine(CLine *pLine, const enum ServerType serverType, bool concatenated);
+	bool ParseLine(CLine &line, const enum ServerType serverType, bool concatenated);
+
+	bool ParseAsUnix(CLine &line, CDirentry &entry, bool expect_date);
+	bool ParseAsDos(CLine &line, CDirentry &entry);
+	bool ParseAsEplf(CLine &line, CDirentry &entry);
+	bool ParseAsVms(CLine &line, CDirentry &entry);
+	bool ParseAsIbm(CLine &line, CDirentry &entry);
+	bool ParseOther(CLine &line, CDirentry &entry);
+	bool ParseAsWfFtp(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_PDS(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_PDS2(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_Migrated(CLine &line, CDirentry &entry);
+	bool ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry);
+	int ParseAsMlsd(CLine &line, CDirentry &entry);
+	bool ParseAsOS9(CLine &line, CDirentry &entry);
 
-	bool ParseAsUnix(CLine *pLine, CDirentry &entry);
-	bool ParseAsDos(CLine *pLine, CDirentry &entry);
-	bool ParseAsEplf(CLine *pLine, CDirentry &entry);
-	bool ParseAsVms(CLine *pLine, CDirentry &entry);
-	bool ParseAsIbm(CLine *pLine, CDirentry &entry);
-	bool ParseOther(CLine *pLine, CDirentry &entry);
-	bool ParseAsWfFtp(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_PDS(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_PDS2(CLine *pLine, CDirentry &entry);
-	bool ParseAsIBM_MVS_Migrated(CLine *pLine, CDirentry &entry);
-	bool ParseAsMlsd(CLine *pLine, CDirentry &entry);
-	bool ParseAsOS9(CLine *pLine, CDirentry &entry);
-	
 	// Only call this if servertype set to ZVM since it conflicts
 	// with other formats.
-	bool ParseAsZVM(CLine *pLine, CDirentry &entry);
+	bool ParseAsZVM(CLine &line, CDirentry &entry);
+
+	// Only call this if servertype set to HPNONSTOP since it conflicts
+	// with other formats.
+	bool ParseAsHPNonstop(CLine &line, CDirentry &entry);
 
 	// Date / time parsers
-	bool ParseUnixDateTime(CLine *pLine, int &index, CDirentry &entry);
+	bool ParseUnixDateTime(CLine &line, int &index, CDirentry &entry);
 	bool ParseShortDate(CToken &token, CDirentry &entry, bool saneFieldOrder = false);
 	bool ParseTime(CToken &token, CDirentry &entry);
 
 	// Parse file sizes given like this: 123.4M
-	bool ParseComplexFileSize(CToken& token, wxLongLong& size);
+	bool ParseComplexFileSize(CToken& token, wxLongLong& size, int blocksize = -1);
+
+	bool GetMonthFromName(const wxString& name, int &month);
+
+	void DeduceEncoding();
+	void ConvertEncoding(char *pData, int len);
 
 	CControlSocket* m_pControlSocket;
 
 	static std::map<wxString, int> m_MonthNamesMap;
-	
+
 	struct t_list
 	{
+		t_list() = default;
+		t_list(char* s, int l)
+			: p(s), len(l)
+		{}
+
 		char *p;
 		int len;
 	};
+
 	int m_currentOffset;
 
 	std::list<t_list> m_DataList;
-	std::list<CDirentry> m_entryList;
+	std::deque<CRefcountObject<CDirentry>> m_entryList;
+	wxLongLong m_totalData;
 
 	CLine *m_prevLine;
 
-	const CServer& m_server;
+	CServer m_server;
 
 	bool m_fileListOnly;
 	std::list<wxString> m_fileList;
-	
+
 	bool m_maybeMultilineVms;
 
 	wxTimeSpan m_timezoneOffset;
+
+	listingEncoding::type m_listingEncoding;
+
+	bool sftp_mode_{};
+
+	// If not passing a default date/time to wxDateTime::ParseFormat, it internaly uses today as reference.
+	// Getting today is slow, so cache it.
+	wxDateTime const today_;
 };
 
 #endif
