@@ -1,34 +1,19 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "externalipresolver.h"
 #include "wx/regex.h"
 #include "socket.h"
-#include <errno.h>
+#include "misc.h"
 
-const wxEventType fzEVT_EXTERNALIPRESOLVE = wxNewEventType();
-
-fzExternalIPResolveEvent::fzExternalIPResolveEvent(int id)
-	: wxEvent(id, fzEVT_EXTERNALIPRESOLVE)
-{
+namespace {
+mutex s_sync;
+wxString ip;
+bool checked = false;
 }
 
-wxEvent* fzExternalIPResolveEvent::Clone() const
+CExternalIPResolver::CExternalIPResolver(CEventHandler & handler)
+	: CEventHandler(handler.event_loop_)
+	, m_handler(&handler)
 {
-	return new fzExternalIPResolveEvent(GetId());
-}
-
-wxString CExternalIPResolver::m_ip;
-bool CExternalIPResolver::m_checked = false;
-
-CExternalIPResolver::CExternalIPResolver(wxEvtHandler* handler, int id /*=wxID_ANY*/)
-	: m_handler(handler), m_id(id)
-{
-	m_pSocket = 0;
-	m_done = false;
-	m_pSendBuffer = 0;
-	m_sendBufferPos = 0;
-	m_pRecvBuffer = 0;
-	m_recvBufferPos = 0;
-
 	ResetHttpData(true);
 }
 
@@ -43,20 +28,22 @@ CExternalIPResolver::~CExternalIPResolver()
 	m_pSocket = 0;
 }
 
-void CExternalIPResolver::GetExternalIP(const wxString& address /*=_T("")*/, bool force /*=false*/)
+void CExternalIPResolver::GetExternalIP(const wxString& address, CSocket::address_family protocol, bool force /*=false*/)
 {
-	if (m_checked)
 	{
-		if (force)
-			m_checked = false;
-		else
-		{
-			m_done = true;
-			return;
+		scoped_lock l(s_sync);
+		if (checked) {
+			if (force)
+				checked = false;
+			else {
+				m_done = true;
+				return;
+			}
 		}
 	}
 
 	m_address = address;
+	m_protocol = protocol;
 
 	wxString host;
 	int pos;
@@ -70,8 +57,7 @@ void CExternalIPResolver::GetExternalIP(const wxString& address /*=_T("")*/, boo
 
 	wxString hostWithPort = host;
 
-	if ((pos = host.Find(':', true)) != -1)
-	{
+	if ((pos = host.Find(':', true)) != -1) {
 		wxString port = host.Mid(pos + 1);
 		if (!port.ToULong(&m_port) || m_port < 1 || m_port > 65535)
 			m_port = 80;
@@ -80,43 +66,46 @@ void CExternalIPResolver::GetExternalIP(const wxString& address /*=_T("")*/, boo
 	else
 		m_port = 80;
 
-	if (host == _T(""))
-	{
+	if (host.empty()) {
 		m_done = true;
 		return;
 	}
 
 	m_pSocket = new CSocket(this);
 
-	int res = m_pSocket->Connect(host, m_port);
-	if (res && res != EINPROGRESS)
-	{
+	int res = m_pSocket->Connect(host, m_port, protocol);
+	if (res && res != EINPROGRESS) {
 		Close(false);
 		return;
 	}
 
-	wxString buffer = wxString::Format(_T("GET %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n"), address.c_str(), hostWithPort.c_str(), wxString(PACKAGE_STRING, wxConvLocal).c_str());
+	wxString buffer = wxString::Format(_T("GET %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nConnection: close\r\n\r\n"), address, hostWithPort, wxString(PACKAGE_STRING, wxConvLocal));
 	m_pSendBuffer = new char[strlen(buffer.mb_str()) + 1];
 	strcpy(m_pSendBuffer, buffer.mb_str());
 }
 
-void CExternalIPResolver::OnSocketEvent(CSocketEvent& event)
+void CExternalIPResolver::operator()(CEventBase const& ev)
+{
+	Dispatch<CSocketEvent>(ev, this, &CExternalIPResolver::OnSocketEvent);
+}
+
+void CExternalIPResolver::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
 	if (!m_pSocket)
 		return;
 
-	switch (event.GetType())
+	switch (t)
 	{
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnReceive();
 		break;
-	case CSocketEvent::connection:
-		OnConnect(event.GetError());
+	case SocketEventType::connection:
+		OnConnect(error);
 		break;
-	case CSocketEvent::close:
+	case SocketEventType::close:
 		OnClose();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
 	default:
@@ -133,16 +122,15 @@ void CExternalIPResolver::OnConnect(int error)
 
 void CExternalIPResolver::OnClose()
 {
-	if (m_data != _T(""))
-		OnData(0, 0);
-	else
+	if (m_data.empty())
 		Close(false);
+	else
+		OnData(0, 0);
 }
 
 void CExternalIPResolver::OnReceive()
 {
-	if (!m_pRecvBuffer)
-	{
+	if (!m_pRecvBuffer) {
 		m_pRecvBuffer = new char[m_recvBufferLen];
 		m_recvBufferPos = 0;
 	}
@@ -150,26 +138,22 @@ void CExternalIPResolver::OnReceive()
 	if (m_pSendBuffer)
 		return;
 
-	while (m_pSocket)
-	{
+	while (m_pSocket) {
 		unsigned int len = m_recvBufferLen - m_recvBufferPos;
 		int error;
 		int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, len, error);
-		if (read == -1)
-		{
+		if (read == -1) {
 			if (error != EAGAIN)
 				Close(false);
 			return;
 		}
 
-		if (!read)
-		{
+		if (!read) {
 			Close(false);
 			return;
 		}
 
-		if (m_finished)
-		{
+		if (m_finished) {
 			// Just ignore all further data
 			m_recvBufferPos = 0;
 			return;
@@ -179,8 +163,7 @@ void CExternalIPResolver::OnReceive()
 
 		if (!m_gotHeader)
 			OnHeader();
-		else
-		{
+		else {
 			if (m_transferEncoding == chunked)
 				OnChunkedData();
 			else
@@ -191,26 +174,22 @@ void CExternalIPResolver::OnReceive()
 
 void CExternalIPResolver::OnSend()
 {
-	while (m_pSendBuffer)
-	{
+	while (m_pSendBuffer) {
 		unsigned int len = strlen(m_pSendBuffer + m_sendBufferPos);
 		int error;
 		int written = m_pSocket->Write(m_pSendBuffer + m_sendBufferPos, len, error);
-		if (written == -1)
-		{
+		if (written == -1) {
 			if (error != EAGAIN)
 				Close(false);
 			return;
 		}
 
-		if (!written)
-		{
+		if (!written) {
 			Close(false);
 			return;
 		}
 
-		if (written == (int)len)
-		{
+		if (written == (int)len) {
 			delete [] m_pSendBuffer;
 			m_pSendBuffer = 0;
 
@@ -237,14 +216,16 @@ void CExternalIPResolver::Close(bool successful)
 
 	m_done = true;
 
-	if (!successful)
-		m_ip = _T("");
-	m_checked = true;
-
-	if (m_handler)
 	{
-		fzExternalIPResolveEvent event;
-		wxPostEvent(m_handler, event);
+		scoped_lock l(s_sync);
+		if (!successful) {
+			ip.clear();
+		}
+		checked = true;
+	}
+
+	if (m_handler) {
+		m_handler->SendEvent<CExternalIPResolveEvent>();
 		m_handler = 0;
 	}
 }
@@ -255,26 +236,20 @@ void CExternalIPResolver::OnHeader()
 	// We do just the neccessary parsing and silently ignore most header fields
 	// Redirects are supported though if the server sends the Location field.
 
-	while (true)
-	{
+	for (;;) {
 		// Find line ending
 		unsigned int i = 0;
-		for (i = 0; (i + 1) < m_recvBufferPos; i++)
-		{
-			if (m_pRecvBuffer[i] == '\r')
-			{
-				if (m_pRecvBuffer[i + 1] != '\n')
-				{
+		for (i = 0; (i + 1) < m_recvBufferPos; ++i) {
+			if (m_pRecvBuffer[i] == '\r') {
+				if (m_pRecvBuffer[i + 1] != '\n') {
 					Close(false);
 					return;
 				}
 				break;
 			}
 		}
-		if ((i + 1) >= m_recvBufferPos)
-		{
-			if (m_recvBufferPos == m_recvBufferLen)
-			{
+		if ((i + 1) >= m_recvBufferPos) {
+			if (m_recvBufferPos == m_recvBufferLen) {
 				// We don't support header lines larger than 4096
 				Close(false);
 				return;
@@ -284,11 +259,9 @@ void CExternalIPResolver::OnHeader()
 
 		m_pRecvBuffer[i] = 0;
 
-		if (!m_responseCode)
-		{
+		if (!m_responseCode) {
 			m_responseString = wxString(m_pRecvBuffer, wxConvLocal);
-			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7))
-			{
+			if (m_recvBufferPos < 16 || memcmp(m_pRecvBuffer, "HTTP/1.", 7)) {
 				// Invalid HTTP Status-Line
 				Close(false);
 				return;
@@ -305,29 +278,24 @@ void CExternalIPResolver::OnHeader()
 
 			m_responseCode = (m_pRecvBuffer[9] - '0') * 100 + (m_pRecvBuffer[10] - '0') * 10 + m_pRecvBuffer[11] - '0';
 
-			if (m_responseCode >= 400)
-			{
+			if (m_responseCode >= 400) {
 				// Failed request
 				Close(false);
 				return;
 			}
 
-			if (m_responseCode == 305)
-			{
+			if (m_responseCode == 305) {
 				// Unsupported redirect
 				Close(false);
 				return;
 			}
 		}
-		else
-		{
-			if (!i)
-			{
+		else {
+			if (!i) {
 				// End of header, data from now on
 
 				// Redirect if neccessary
-				if (m_responseCode >= 300)
-				{
+				if (m_responseCode >= 300) {
 					delete m_pSocket;
 					m_pSocket = 0;
 
@@ -338,7 +306,7 @@ void CExternalIPResolver::OnHeader()
 
 					ResetHttpData(false);
 
-					GetExternalIP(location);
+					GetExternalIP(location, m_protocol);
 					return;
 				}
 
@@ -346,8 +314,7 @@ void CExternalIPResolver::OnHeader()
 
 				memmove(m_pRecvBuffer, m_pRecvBuffer + 2, m_recvBufferPos - 2);
 				m_recvBufferPos -= 2;
-				if (m_recvBufferPos)
-				{
+				if (m_recvBufferPos) {
 					if (m_transferEncoding == chunked)
 						OnChunkedData();
 					else
@@ -355,12 +322,10 @@ void CExternalIPResolver::OnHeader()
 				}
 				return;
 			}
-			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10))
-			{
+			if (m_recvBufferPos > 12 && !memcmp(m_pRecvBuffer, "Location: ", 10)) {
 				m_location = wxString(m_pRecvBuffer + 10, wxConvLocal);
 			}
-			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19))
-			{
+			else if (m_recvBufferPos > 21 && !memcmp(m_pRecvBuffer, "Transfer-Encoding: ", 19)) {
 				if (!strcmp(m_pRecvBuffer + 19, "chunked"))
 					m_transferEncoding = chunked;
 				else if (!strcmp(m_pRecvBuffer + 19, "identity"))
@@ -380,13 +345,15 @@ void CExternalIPResolver::OnHeader()
 
 void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 {
-	if (buffer)
-	{
+	if (buffer) {
 		unsigned int i;
-		for (i = 0; i < len; i++)
-		{
+		for (i = 0; i < len; ++i) {
 			if (buffer[i] == '\r' || buffer[i] == '\n')
 				break;
+			if (buffer[i] & 0x80) {
+				Close(false);
+				return;
+			}
 		}
 
 		if (i)
@@ -396,29 +363,51 @@ void CExternalIPResolver::OnData(char* buffer, unsigned int len)
 			return;
 	}
 
-	// Validate ip address
-	wxString digit = _T("0*[0-9]{1,3}");
-	const wxChar* dot = _T("\\.");
-	wxString exp = _T("(^|[^\\.[:digit:]])(") + digit + dot + digit + dot + digit + dot + digit + _T(")([^\\.[:digit:]]|$)");
-	wxRegEx regex;
-	regex.Compile(exp);
+	if (m_protocol == CSocket::ipv6) {
+		if (!m_data.empty() && m_data[0] == '[') {
+			if (m_data.Last() != ']') {
+				Close(false);
+				return;
+			}
+			m_data.RemoveLast();
+			m_data = m_data.Mid(1);
+		}
 
-	if (!regex.Matches(m_data))
-	{
-		Close(false);
-		return;
+		if (GetIPV6LongForm(m_data).empty()) {
+			Close(false);
+			return;
+		}
+
+		scoped_lock l(s_sync);
+		ip = m_data;
+	}
+	else {
+
+		// Validate ip address
+		wxString digit = _T("0*[0-9]{1,3}");
+		const wxChar* dot = _T("\\.");
+		wxString exp = _T("(^|[^\\.[:digit:]])(") + digit + dot + digit + dot + digit + dot + digit + _T(")([^\\.[:digit:]]|$)");
+		wxRegEx regex;
+		regex.Compile(exp);
+
+		if (!regex.Matches(m_data)) {
+			Close(false);
+			return;
+		}
+
+		scoped_lock l(s_sync);
+		ip = regex.GetMatch(m_data, 2);
 	}
 
-	m_ip = regex.GetMatch(m_data, 2);
 	Close(true);
 }
 
 void CExternalIPResolver::ResetHttpData(bool resetRedirectCount)
 {
 	m_gotHeader = false;
-	m_location = _T("");
+	m_location.clear();
 	m_responseCode = 0;
-	m_responseString = _T("");
+	m_responseString.clear();
 	if (resetRedirectCount)
 		m_redirectCount = 0;
 
@@ -436,10 +425,8 @@ void CExternalIPResolver::OnChunkedData()
 	char* p = m_pRecvBuffer;
 	unsigned int len = m_recvBufferPos;
 
-	while (true)
-	{
-		if (m_chunkData.size != 0)
-		{
+	for (;;) {
+		if (m_chunkData.size != 0) {
 			unsigned int dataLen = len;
 			if (m_chunkData.size < len)
 				dataLen = m_chunkData.size.GetLo();
@@ -447,7 +434,7 @@ void CExternalIPResolver::OnChunkedData()
 			if (!m_pRecvBuffer)
 				return;
 
-            m_chunkData.size -= dataLen;
+			m_chunkData.size -= dataLen;
 			p += dataLen;
 			len -= dataLen;
 
@@ -460,22 +447,17 @@ void CExternalIPResolver::OnChunkedData()
 
 		// Find line ending
 		unsigned int i = 0;
-		for (i = 0; (i + 1) < len; i++)
-		{
-			if (p[i] == '\r')
-			{
-				if (p[i + 1] != '\n')
-				{
+		for (i = 0; (i + 1) < len; ++i) {
+			if (p[i] == '\r') {
+				if (p[i + 1] != '\n') {
 					Close(false);
 					return;
 				}
 				break;
 			}
 		}
-		if ((i + 1) >= len)
-		{
-			if (len == m_recvBufferLen)
-			{
+		if ((i + 1) >= len) {
+			if (len == m_recvBufferLen) {
 				// We don't support lines larger than 4096
 				Close(false);
 				return;
@@ -485,20 +467,16 @@ void CExternalIPResolver::OnChunkedData()
 
 		p[i] = 0;
 
-		if (m_chunkData.terminateChunk)
-		{
-			if (i)
-			{
+		if (m_chunkData.terminateChunk) {
+			if (i) {
 				// Chunk has to end with CRLF
 				Close(false);
 				return;
 			}
 			m_chunkData.terminateChunk = false;
 		}
-		else if (m_chunkData.getTrailer)
-		{
-			if (!i)
-			{
+		else if (m_chunkData.getTrailer) {
+			if (!i) {
 				m_finished = true;
 				m_recvBufferPos = 0;
 				return;
@@ -506,31 +484,25 @@ void CExternalIPResolver::OnChunkedData()
 
 			// Ignore the trailer
 		}
-		else
-		{
+		else {
 			// Read chunk size
 			char* q = p;
-			while (*q)
-			{
-				if (*q >= '0' && *q <= '9')
-				{
+			while (*q) {
+				if (*q >= '0' && *q <= '9') {
 					m_chunkData.size *= 16;
 					m_chunkData.size += *q - '0';
 				}
-				else if (*q >= 'A' && *q <= 'F')
-				{
+				else if (*q >= 'A' && *q <= 'F') {
 					m_chunkData.size *= 10;
 					m_chunkData.size += *q - 'A' + 10;
 				}
-				else if (*q >= 'a' && *q <= 'f')
-				{
+				else if (*q >= 'a' && *q <= 'f') {
 					m_chunkData.size *= 10;
 					m_chunkData.size += *q - 'a' + 10;
 				}
 				else if (*q == ';' || *q == ' ')
 					break;
-				else
-				{
+				else {
 					// Invalid size
 					Close(false);
 					return;
@@ -548,9 +520,20 @@ void CExternalIPResolver::OnChunkedData()
 			break;
 	}
 
-	if (p != m_pRecvBuffer)
-	{
+	if (p != m_pRecvBuffer) {
 		memmove(m_pRecvBuffer, p, len);
 		m_recvBufferPos = len;
 	}
 }
+
+bool CExternalIPResolver::Successful() const
+{
+	scoped_lock l(s_sync);
+	return !ip.empty();
+}
+
+wxString CExternalIPResolver::GetIP() const
+{
+	scoped_lock l(s_sync);
+	return ip;
+}
