@@ -18,7 +18,7 @@ class CFtpControlSocket : public CRealControlSocket
 {
 	friend class CTransferSocket;
 public:
-	CFtpControlSocket(CFileZillaEnginePrivate *pEngine);
+	CFtpControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CFtpControlSocket();
 	virtual void TransferEnd();
 
@@ -83,7 +83,7 @@ protected:
 	virtual void OnConnect();
 	virtual void OnReceive();
 
-	virtual bool Send(wxString str, bool maskArgs = false, bool measureRTT = true);
+	bool SendCommand(wxString const& str, bool maskArgs = false, bool measureRTT = true);
 
 	// Parse the latest reply line from the server
 	void ParseLine(wxString line);
@@ -101,6 +101,7 @@ protected:
 	int LogonParseResponse();
 	int LogonSend();
 
+	wxString GetPassiveCommand(CRawTransferOpData& data);
 	bool ParsePasvResponse(CRawTransferOpData* pData);
 	bool ParseEpsvResponse(CRawTransferOpData* pData);
 
@@ -141,19 +142,18 @@ protected:
 
 	int m_lastTypeBinary;
 
-	int m_pendingTransferEndEvents;
-
 	// Used by keepalive code so that we're not using keep alive
 	// till the end of time. Stop after a couple of minutes.
 	wxDateTime m_lastCommandCompletionTime;
 
-	wxTimer m_idleTimer;
+	timer_id m_idleTimer{};
 
 	CLatencyMeasurement m_rtt;
 
-	DECLARE_EVENT_TABLE()
-	void OnExternalIPAddress(fzExternalIPResolveEvent& event);
-	void OnIdleTimer(wxTimerEvent& event);
+	virtual void operator()(CEventBase const& ev);
+
+	void OnExternalIPAddress();
+	void OnTimer(timer_id id);
 };
 
 class CIOThread;
@@ -164,14 +164,14 @@ public:
 	CFtpTransferOpData();
 	virtual ~CFtpTransferOpData() {}
 
-	enum TransferEndReason transferEndReason;
+	TransferEndReason transferEndReason;
 	bool tranferCommandSent;
 
 	wxLongLong resumeOffset;
 	bool binary;
 };
 
-class CFtpFileTransferOpData : public CFileTransferOpData, public CFtpTransferOpData
+class CFtpFileTransferOpData final : public CFileTransferOpData, public CFtpTransferOpData
 {
 public:
 	CFtpFileTransferOpData(bool is_download, const wxString& local_file, const wxString& remote_file, const CServerPath& remote_path);
@@ -181,7 +181,7 @@ public:
 	bool fileDidExist;
 };
 
-class CRawTransferOpData : public COpData
+class CRawTransferOpData final : public COpData
 {
 public:
 	CRawTransferOpData();
