@@ -1,6 +1,7 @@
 #include <filezilla.h>
 
 #include "local_filesys.h"
+#include <msgbox.h>
 
 #include <wx/dir.h>
 #include <wx/filename.h>
@@ -12,18 +13,12 @@ const wxChar CLocalFileSystem::path_separator = '\\';
 const wxChar CLocalFileSystem::path_separator = '/';
 #endif
 
-CLocalFileSystem::CLocalFileSystem()
-	: m_dirs_only()
-#ifdef __WXMSW__
-	, m_hFind(INVALID_HANDLE_VALUE)
-	, m_found()
-#else
-	, m_raw_path()
-	, m_file_part()
-	, m_buffer_length()
-	, m_dir()
-#endif
+namespace {
+template<typename T>
+int64_t make_int64_t(T hi, T lo)
 {
+	return (static_cast<int64_t>(hi) << 32) + static_cast<int64_t>(lo);
+}
 }
 
 CLocalFileSystem::~CLocalFileSystem()
@@ -31,20 +26,22 @@ CLocalFileSystem::~CLocalFileSystem()
 	EndFindFiles();
 }
 
-enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileType(const wxString& path)
+CLocalFileSystem::local_fileType CLocalFileSystem::GetFileType(const wxString& path)
 {
 #ifdef __WXMSW__
 	DWORD result = GetFileAttributes(path);
 	if (result == INVALID_FILE_ATTRIBUTES)
 		return unknown;
 
+	if (result & FILE_ATTRIBUTE_REPARSE_POINT)
+		return link;
+
 	if (result & FILE_ATTRIBUTE_DIRECTORY)
 		return dir;
 
 	return file;
 #else
-	if (path.Last() == '/' && path != _T("/"))
-	{
+	if (!path.empty() && path.Last() == '/' && path != _T("/")) {
 		wxString tmp = path;
 		tmp.RemoveLast();
 		return GetFileType(tmp);
@@ -84,28 +81,26 @@ bool CLocalFileSystem::RecursiveDelete(std::list<wxString> dirsToVisit, wxWindow
 	// SHFileOperation accepts a list of null-terminated strings. Go through all
 	// paths to get the required buffer length
 
-	int len = 1; // String list terminated by empty string
+	size_t len = 1; // String list terminated by empty string
 
-	for (std::list<wxString>::const_iterator const_iter = dirsToVisit.begin(); const_iter != dirsToVisit.end(); ++const_iter)
-		len += const_iter->Length() + 1;
+	for (auto const& dir : dirsToVisit) {
+		len += dir.size() + 1;
+	}
 
 	// Allocate memory
 	wxChar* pBuffer = new wxChar[len];
 	wxChar* p = pBuffer;
 
-	for (std::list<wxString>::iterator iter = dirsToVisit.begin(); iter != dirsToVisit.end(); ++iter)
-	{
-		wxString& path = *iter;
-		if (path.Last() == wxFileName::GetPathSeparator())
-			path.RemoveLast();
-		if (GetFileType(path) == unknown)
+	for (auto& dir : dirsToVisit) {
+		if (!dir.empty() && dir.Last() == wxFileName::GetPathSeparator())
+			dir.RemoveLast();
+		if (GetFileType(dir) == unknown)
 			continue;
 
-		_tcscpy(p, path);
-		p += path.Length() + 1;
+		_tcscpy(p, dir);
+		p += dir.size() + 1;
 	}
-	if (p != pBuffer)
-	{
+	if (p != pBuffer) {
 		*p = 0;
 
 		// Now we can delete the files in the buffer
@@ -115,8 +110,7 @@ bool CLocalFileSystem::RecursiveDelete(std::list<wxString> dirsToVisit, wxWindow
 		op.wFunc = FO_DELETE;
 		op.pFrom = pBuffer;
 
-		if (parent)
-		{
+		if (parent) {
 			// Move to trash if shift is not pressed, else delete
 			op.fFlags = wxGetKeyState(WXK_SHIFT) ? 0 : FOF_ALLOWUNDO;
 		}
@@ -129,17 +123,14 @@ bool CLocalFileSystem::RecursiveDelete(std::list<wxString> dirsToVisit, wxWindow
 
 	return true;
 #else
-	if (parent)
-	{
-		if (wxMessageBox(_("Really delete all selected files and/or directories from your computer?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, parent) != wxYES)
+	if (parent) {
+		if (wxMessageBoxEx(_("Really delete all selected files and/or directories from your computer?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, parent) != wxYES)
 			return true;
 	}
 
-	for (std::list<wxString>::iterator iter = dirsToVisit.begin(); iter != dirsToVisit.end(); ++iter)
-	{
-		wxString& path = *iter;
-		if (path.Last() == '/' && path != _T("/"))
-			path.RemoveLast();
+	for (auto& dir : dirsToVisit) {
+		if (!dir.empty() && dir.Last() == '/' && dir != _T("/"))
+			dir.RemoveLast();
 	}
 
 	bool encodingError = false;
@@ -147,34 +138,34 @@ bool CLocalFileSystem::RecursiveDelete(std::list<wxString> dirsToVisit, wxWindow
 	// Remember the directories to delete after recursing into them
 	std::list<wxString> dirsToDelete;
 
-	// Process all dirctories that have to be visited
-	while (!dirsToVisit.empty())
-	{
-		const wxString path = dirsToVisit.front();
-		dirsToVisit.pop_front();
-		dirsToDelete.push_front(path);
+	CLocalFileSystem fs;
 
-		if (GetFileType(path) != dir)
-		{
+	// Process all dirctories that have to be visited
+	while (!dirsToVisit.empty()) {
+		auto const iter = dirsToVisit.begin();
+		wxString const& path = *iter;
+
+		if (GetFileType(path) != dir) {
 			wxRemoveFile(path);
+			dirsToVisit.erase(iter);
 			continue;
 		}
 
-		wxDir dir;
-		if (!dir.Open(path))
+		dirsToDelete.splice(dirsToDelete.begin(), dirsToVisit, iter);
+
+		if (!fs.BeginFindFiles(path, false)) {
 			continue;
+		}
 
 		// Depending on underlying platform, wxDir does not handle
 		// changes to the directory contents very well.
-		// See bug [ 1946574 ]
+		// See http://trac.filezilla-project.org/ticket/3482
 		// To work around this, delete files after enumerating everything in current directory
 		std::list<wxString> filesToDelete;
 
 		wxString file;
-		for (bool found = dir.GetFirst(&file); found; found = dir.GetNext(&file))
-		{
-			if (file == _T(""))
-			{
+		while (fs.GetNextFile(file)) {
+			if (file.empty()) {
 				encodingError = true;
 				continue;
 			}
@@ -186,25 +177,27 @@ bool CLocalFileSystem::RecursiveDelete(std::list<wxString> dirsToVisit, wxWindow
 			else
 				filesToDelete.push_back(fullName);
 		}
+		fs.EndFindFiles();
 
 		// Delete all files and links in current directory enumerated before
-		for (std::list<wxString>::const_iterator iter = filesToDelete.begin(); iter != filesToDelete.end(); ++iter)
-			wxRemoveFile(*iter);
+		for (auto const& file : filesToDelete) {
+			wxRemoveFile(file);
+		}
 	}
 
 	// Delete the now empty directories
-	for (std::list<wxString>::const_iterator iter = dirsToDelete.begin(); iter != dirsToDelete.end(); ++iter)
-		wxRmdir(*iter);
+	for (auto const& dir : dirsToDelete) {
+		wxRmdir(dir);
+	}
 
 	return !encodingError;
 #endif
 }
 
-enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const wxString& path, bool &isLink, wxLongLong* size, wxDateTime* modificationTime, int *mode)
+CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const wxString& path, bool &isLink, int64_t* size, CDateTime* modificationTime, int *mode)
 {
 #ifdef __WXMSW__
-	if (path.Last() == wxFileName::GetPathSeparator() && path != wxFileName::GetPathSeparator())
-	{
+	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator() && path != wxFileName::GetPathSeparator()) {
 		wxString tmp = path;
 		tmp.RemoveLast();
 		return GetFileInfo(tmp, isLink, size, modificationTime, mode);
@@ -214,40 +207,78 @@ enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const wxStri
 
 	WIN32_FILE_ATTRIBUTE_DATA attributes;
 	BOOL result = GetFileAttributesEx(path, GetFileExInfoStandard, &attributes);
-	if (!result)
-	{
+	if (!result) {
 		if (size)
 			*size = -1;
 		if (mode)
 			*mode = 0;
 		if (modificationTime)
-			*modificationTime = wxDateTime();
+			*modificationTime = CDateTime();
 		return unknown;
 	}
 
-	if (modificationTime)
-	{
-		if (!ConvertFileTimeToWxDateTime(*modificationTime, attributes.ftLastWriteTime))
-			ConvertFileTimeToWxDateTime(*modificationTime, attributes.ftCreationTime);
+	bool is_dir = (attributes.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
+
+	if (attributes.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT) {
+		isLink = true;
+
+		HANDLE hFile = is_dir ? INVALID_HANDLE_VALUE : CreateFile(path, FILE_READ_ATTRIBUTES | FILE_READ_EA, FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+		if (hFile != INVALID_HANDLE_VALUE) {
+			BY_HANDLE_FILE_INFORMATION info{};
+			int ret = GetFileInformationByHandle(hFile, &info);
+			CloseHandle(hFile);
+			if (ret != 0 && !(info.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT)) {
+
+				if (modificationTime) {
+					if (!ConvertFileTimeToCDateTime(*modificationTime, info.ftLastWriteTime))
+						ConvertFileTimeToCDateTime(*modificationTime, info.ftCreationTime);
+				}
+
+				if (mode)
+					*mode = (int)info.dwFileAttributes;
+
+				if (info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
+					if (size)
+						*size = -1;
+					return dir;
+				}
+
+				if (size)
+					*size = make_int64_t(info.nFileSizeHigh, info.nFileSizeLow);
+
+				return file;
+			}
+		}
+
+		if (size)
+			*size = -1;
+		if (mode)
+			*mode = 0;
+		if (modificationTime)
+			*modificationTime = CDateTime();
+		return is_dir ? dir : unknown;
+	}
+
+	if (modificationTime) {
+		if (!ConvertFileTimeToCDateTime(*modificationTime, attributes.ftLastWriteTime))
+			ConvertFileTimeToCDateTime(*modificationTime, attributes.ftCreationTime);
 	}
 
 	if (mode)
 		*mode = (int)attributes.dwFileAttributes;
 
-	if (attributes.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
-	{
+	if (is_dir) {
 		if (size)
 			*size = -1;
 		return dir;
 	}
-
-	if (size)
-		*size = wxLongLong(attributes.nFileSizeHigh, attributes.nFileSizeLow);
-
-	return file;
+	else {
+		if (size)
+			*size = make_int64_t(attributes.nFileSizeHigh, attributes.nFileSizeLow);
+		return file;
+	}
 #else
-	if (path.Last() == '/' && path != _T("/"))
-	{
+	if (!path.empty() && path.Last() == '/' && path != _T("/")) {
 		wxString tmp = path;
 		tmp.RemoveLast();
 		return GetFileInfo(tmp, isLink, size, modificationTime, mode);
@@ -259,7 +290,7 @@ enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const wxStri
 }
 
 #ifndef __WXMSW__
-enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const char* path, bool &isLink, wxLongLong* size, wxDateTime* modificationTime, int *mode)
+CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const char* path, bool &isLink, int64_t* size, CDateTime* modificationTime, int *mode)
 {
 	struct stat buf;
 	int result = lstat(path, &buf);
@@ -271,7 +302,7 @@ enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const char*
 		if (mode)
 			*mode = -1;
 		if (modificationTime)
-			*modificationTime = wxDateTime();
+			*modificationTime = CDateTime();
 		return unknown;
 	}
 
@@ -287,7 +318,7 @@ enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const char*
 			if (mode)
 				*mode = -1;
 			if (modificationTime)
-				*modificationTime = wxDateTime();
+				*modificationTime = CDateTime();
 			return unknown;
 		}
 	}
@@ -296,7 +327,7 @@ enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const char*
 		isLink = false;
 
 	if (modificationTime)
-		modificationTime->Set(buf.st_mtime);
+		*modificationTime = CDateTime(wxDateTime(buf.st_mtime), CDateTime::seconds);
 
 	if (mode)
 		*mode = buf.st_mode & 0x777;
@@ -317,10 +348,10 @@ enum CLocalFileSystem::local_fileType CLocalFileSystem::GetFileInfo(const char*
 
 #ifdef __WXMSW__
 
-// This is the offset between FILETIME epoch and the Unix/wxDateTime Epoch. 
-static wxInt64 EPOCH_OFFSET_IN_MSEC = wxLL(11644473600000);
+// This is the offset between FILETIME epoch and the Unix/wxDateTime Epoch.
+static wxInt64 EPOCH_OFFSET_IN_MSEC = 11644473600000ll;
 
-bool CLocalFileSystem::ConvertFileTimeToWxDateTime(wxDateTime& time, const FILETIME &ft)
+bool CLocalFileSystem::ConvertFileTimeToCDateTime(CDateTime& time, const FILETIME &ft)
 {
 	if (!ft.dwHighDateTime && !ft.dwLowDateTime)
 		return false;
@@ -328,8 +359,8 @@ bool CLocalFileSystem::ConvertFileTimeToWxDateTime(wxDateTime& time, const FILET
 	// See http://trac.wxwidgets.org/changeset/74423 and http://trac.wxwidgets.org/ticket/13098
 	// Directly converting to time_t
 
-	wxLongLong t(ft.dwHighDateTime, ft.dwLowDateTime);
-	t /= 10000; // Convert hundreds of nanoseconds to milliseconds. 
+	int64_t t = make_int64_t(ft.dwHighDateTime, ft.dwLowDateTime);
+	t /= 10000; // Convert hundreds of nanoseconds to milliseconds.
 	t -= EPOCH_OFFSET_IN_MSEC;
 	if (t < 0) {
 		return false;
@@ -337,22 +368,22 @@ bool CLocalFileSystem::ConvertFileTimeToWxDateTime(wxDateTime& time, const FILET
 
 	// Interestingly wxDateTime has this constructor which
 	// even more interestingly isn't even marked explicit.
-	time = wxDateTime(t);
+	time = CDateTime(wxDateTime(wxLongLong(t)), CDateTime::milliseconds);
 	return time.IsValid();
 }
 
-bool CLocalFileSystem::ConvertWxDateTimeToFileTime(FILETIME &ft, const wxDateTime& time)
+bool CLocalFileSystem::ConvertCDateTimeToFileTime(FILETIME &ft, CDateTime const& time)
 {
 	if (!time.IsValid())
 		return false;
 
-	wxLongLong t = time.GetValue();
+	int64_t t = time.Degenerate().GetValue().GetValue();
 
 	t += EPOCH_OFFSET_IN_MSEC;
 	t *= 10000;
 
-	ft.dwHighDateTime = t.GetHi();
-	ft.dwLowDateTime = t.GetLo();
+	ft.dwHighDateTime = t >> 32;
+	ft.dwLowDateTime = t & 0xffffffffll;
 
 	return true;
 }
@@ -360,18 +391,25 @@ bool CLocalFileSystem::ConvertWxDateTimeToFileTime(FILETIME &ft, const wxDateTim
 
 bool CLocalFileSystem::BeginFindFiles(wxString path, bool dirs_only)
 {
+	if (path.empty()) {
+		return false;
+	}
+
 	EndFindFiles();
 
 	m_dirs_only = dirs_only;
 #ifdef __WXMSW__
-	if (path.Last() != '/' && path.Last() != '\\')
+	if (path.Last() != '/' && path.Last() != '\\') {
+		m_find_path = path + _T("\\");
 		path += _T("\\*");
-	else
+	}
+	else {
+		m_find_path = path;
 		path += '*';
+	}
 
 	m_hFind = FindFirstFileEx(path, FindExInfoStandard, &m_find_data, dirs_only ? FindExSearchLimitToDirectories : FindExSearchNameMatch, 0, 0);
-	if (m_hFind == INVALID_HANDLE_VALUE)
-	{
+	if (m_hFind == INVALID_HANDLE_VALUE) {
 		m_found = false;
 		return false;
 	}
@@ -431,11 +469,9 @@ bool CLocalFileSystem::GetNextFile(wxString& name)
 #ifdef __WXMSW__
 	if (!m_found)
 		return false;
-	do
-	{
+	do {
 		name = m_find_data.cFileName;
-		if (name == _T(""))
-		{
+		if (name.empty()) {
 			m_found = FindNextFile(m_hFind, &m_find_data) != 0;
 			return true;
 		}
@@ -455,16 +491,14 @@ bool CLocalFileSystem::GetNextFile(wxString& name)
 		return false;
 
 	struct dirent* entry;
-	while ((entry = readdir(m_dir)))
-	{
+	while ((entry = readdir(m_dir))) {
 		if (!entry->d_name[0] ||
 			!strcmp(entry->d_name, ".") ||
 			!strcmp(entry->d_name, ".."))
 			continue;
 
-		if (m_dirs_only)
-		{
-#ifdef _DIRENT_HAVE_D_TYPE
+		if (m_dirs_only) {
+#if HAVE_STRUCT_DIRENT_D_TYPE
 			if (entry->d_type == DT_LNK)
 			{
 				bool wasLink;
@@ -494,15 +528,14 @@ bool CLocalFileSystem::GetNextFile(wxString& name)
 #endif
 }
 
-bool CLocalFileSystem::GetNextFile(wxString& name, bool &isLink, bool &is_dir, wxLongLong* size, wxDateTime* modificationTime, int* mode)
+bool CLocalFileSystem::GetNextFile(wxString& name, bool &isLink, bool &is_dir, int64_t* size, CDateTime* modificationTime, int* mode)
 {
 #ifdef __WXMSW__
 	if (!m_found)
 		return false;
 	do
 	{
-		if (!m_find_data.cFileName[0])
-		{
+		if (!m_find_data.cFileName[0]) {
 			m_found = FindNextFile(m_hFind, &m_find_data) != 0;
 			return true;
 		}
@@ -513,27 +546,67 @@ bool CLocalFileSystem::GetNextFile(wxString& name, bool &isLink, bool &is_dir, w
 			continue;
 		name = m_find_data.cFileName;
 
-		isLink = false;
+		is_dir = (m_find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
 
-		if (modificationTime)
-			ConvertFileTimeToWxDateTime(*modificationTime, m_find_data.ftLastWriteTime);
+		isLink = (m_find_data.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT) != 0;
+		if (isLink) {
+			HANDLE hFile = is_dir ? INVALID_HANDLE_VALUE : CreateFile(m_find_path + name, FILE_READ_ATTRIBUTES | FILE_READ_EA, FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+			if (hFile != INVALID_HANDLE_VALUE) {
+				BY_HANDLE_FILE_INFORMATION info{};
+				int ret = GetFileInformationByHandle(hFile, &info);
+				CloseHandle(hFile);
+				if (ret != 0 && !(info.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT)) {
 
-		if (mode)
-			*mode = (int)m_find_data.dwFileAttributes;
+					if (modificationTime) {
+						if (!ConvertFileTimeToCDateTime(*modificationTime, info.ftLastWriteTime))
+							ConvertFileTimeToCDateTime(*modificationTime, info.ftCreationTime);
+					}
+
+					if (mode)
+						*mode = (int)info.dwFileAttributes;
+
+					is_dir = (info.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) != 0;
+					if (size) {
+						if (is_dir) {
+							*size = -1;
+						}
+						else {
+							*size = make_int64_t(info.nFileSizeHigh, info.nFileSizeLow);
+						}
+					}
+
+					m_found = FindNextFile(m_hFind, &m_find_data) != 0;
+					return true;
+				}
+			}
+
+			if (m_dirs_only && !is_dir) {
+				continue;
+			}
 
-		if (m_find_data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
-		{
-			is_dir = true;
 			if (size)
 				*size = -1;
+			if (mode)
+				*mode = 0;
+			if (modificationTime)
+				*modificationTime = CDateTime();
 		}
-		else
-		{
-			is_dir = false;
-			if (size)
-				*size = wxLongLong(m_find_data.nFileSizeHigh, m_find_data.nFileSizeLow);
-		}
+		else {
+			if (modificationTime)
+				ConvertFileTimeToCDateTime(*modificationTime, m_find_data.ftLastWriteTime);
 
+			if (mode)
+				*mode = (int)m_find_data.dwFileAttributes;
+
+			if (size) {
+				if (is_dir) {
+					*size = -1;
+				}
+				else {
+					*size = make_int64_t(m_find_data.nFileSizeHigh, m_find_data.nFileSizeLow);
+				}
+			}
+		}
 		m_found = FindNextFile(m_hFind, &m_find_data) != 0;
 		return true;
 	} while ((m_found = FindNextFile(m_hFind, &m_find_data) != 0));
@@ -551,14 +624,14 @@ bool CLocalFileSystem::GetNextFile(wxString& name, bool &isLink, bool &is_dir, w
 			!strcmp(entry->d_name, ".."))
 			continue;
 
-#ifdef _DIRENT_HAVE_D_TYPE
+#if HAVE_STRUCT_DIRENT_D_TYPE
 		if (m_dirs_only)
 		{
 			if (entry->d_type == DT_LNK)
 			{
 				AllocPathBuffer(entry->d_name);
 				strcpy(m_file_part, entry->d_name);
-				enum local_fileType type = GetFileInfo(m_raw_path, isLink, size, modificationTime, mode);
+				local_fileType type = GetFileInfo(m_raw_path, isLink, size, modificationTime, mode);
 				if (type != dir)
 					continue;
 
@@ -573,11 +646,11 @@ bool CLocalFileSystem::GetNextFile(wxString& name, bool &isLink, bool &is_dir, w
 
 		AllocPathBuffer(entry->d_name);
 		strcpy(m_file_part, entry->d_name);
-		enum local_fileType type = GetFileInfo(m_raw_path, isLink, size, modificationTime, mode);
+		local_fileType type = GetFileInfo(m_raw_path, isLink, size, modificationTime, mode);
 
 		if (type == unknown) // Happens for example in case of permission denied
 		{
-#ifdef _DIRENT_HAVE_D_TYPE
+#if HAVE_STRUCT_DIRENT_D_TYPE
 			type = entry->d_type == DT_DIR ? dir : file;
 #else
 			type = file;
@@ -586,7 +659,7 @@ bool CLocalFileSystem::GetNextFile(wxString& name, bool &isLink, bool &is_dir, w
 			if (size)
 				*size = -1;
 			if (modificationTime)
-				*modificationTime = wxDateTime();
+				*modificationTime = CDateTime();
 			if (mode)
 				*mode = 0;
 		}
@@ -622,25 +695,25 @@ void CLocalFileSystem::AllocPathBuffer(const char* file)
 }
 #endif
 
-wxDateTime CLocalFileSystem::GetModificationTime( const wxString& path)
+CDateTime CLocalFileSystem::GetModificationTime( const wxString& path)
 {
-	wxDateTime mtime;
+	CDateTime mtime;
 
 	bool tmp;
 	if (GetFileInfo(path, tmp, 0, &mtime, 0) == unknown)
-		mtime = wxDateTime();
+		mtime = CDateTime();
 
 	return mtime;
 }
 
-bool CLocalFileSystem::SetModificationTime(const wxString& path, const wxDateTime& t)
+bool CLocalFileSystem::SetModificationTime(const wxString& path, const CDateTime& t)
 {
 	if (!t.IsValid())
 		return false;
 
 #ifdef __WXMSW__
 	FILETIME ft;
-	if (!ConvertWxDateTimeToFileTime(ft, t))
+	if (!ConvertCDateTimeToFileTime(ft, t))
 		return false;
 
 	HANDLE h = CreateFile(path, GENERIC_WRITE, FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
@@ -652,6 +725,48 @@ bool CLocalFileSystem::SetModificationTime(const wxString& path, const wxDateTim
 	return ret;
 #else
 	wxFileName fn(path);
-	return fn.SetTimes( &t, &t, 0 );
+	wxDateTime d = t.Degenerate();
+	return fn.SetTimes( &d, &d, 0 );
 #endif
 }
+
+int64_t CLocalFileSystem::GetSize(wxString const& path, bool* isLink)
+{
+	int64_t ret = -1;
+	bool tmp{};
+	local_fileType t = GetFileInfo(path, isLink ? *isLink : tmp, &ret, 0, 0);
+	if( t != file ) {
+		ret = -1;
+	}
+
+	return ret;
+}
+
+wxString CLocalFileSystem::GetSymbolicLinkTarget(wxString const& path)
+{
+	wxString target;
+
+#ifdef __WXMSW__
+	HANDLE hFile = CreateFile(path, FILE_READ_ATTRIBUTES | FILE_READ_EA, FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+	if (hFile != INVALID_HANDLE_VALUE) {
+		DWORD const size = 1024;
+		wxChar out[size];
+		DWORD ret = GetFinalPathNameByHandle(hFile, out, size, 0);
+		if (ret > 0 && ret < size) {
+			target = out;
+		}
+		CloseHandle(hFile);
+	}
+#else
+	size_t const size = 1024;
+	char out[size];
+
+	const wxCharBuffer p = path.fn_str();
+	ssize_t res = readlink(static_cast<char const*>(p), out, size);
+	if( res > 0 && static_cast<size_t>(res) < size ) {
+		out[res] = 0;
+		target = wxString(out, *wxConvFileName);
+	}
+#endif
+	return target;
+}
