@@ -15,34 +15,37 @@ HANDLE CLogging::m_log_fd = INVALID_HANDLE_VALUE;
 #else
 int CLogging::m_log_fd = -1;
 #endif
-wxString CLogging::m_prefixes[MessageTypeCount];
+wxString CLogging::m_prefixes[static_cast<int>(MessageType::count)];
 unsigned int CLogging::m_pid;
 int CLogging::m_max_size;
 wxString CLogging::m_file;
 
 int CLogging::m_refcount = 0;
+mutex CLogging::mutex_(false);
 
-CLogging::CLogging(CFileZillaEnginePrivate *pEngine)
+thread_local int CLogging::debug_level_{0};
+thread_local int CLogging::raw_listing_{0};
+
+CLogging::CLogging(CFileZillaEnginePrivate & engine)
+	: engine_(engine)
 {
-	m_pEngine = pEngine;
+	scoped_lock l(mutex_);
 	m_refcount++;
 }
 
 CLogging::~CLogging()
 {
+	scoped_lock l(mutex_);
 	m_refcount--;
 
-	if (!m_refcount)
-	{
+	if (!m_refcount) {
 #ifdef __WXMSW__
-		if (m_log_fd != INVALID_HANDLE_VALUE)
-		{
+		if (m_log_fd != INVALID_HANDLE_VALUE) {
 			CloseHandle(m_log_fd);
 			m_log_fd = INVALID_HANDLE_VALUE;
 		}
 #else
-		if (m_log_fd != -1)
-		{
+		if (m_log_fd != -1) {
 			close(m_log_fd);
 			m_log_fd = -1;
 		}
@@ -51,139 +54,33 @@ CLogging::~CLogging()
 	}
 }
 
-void CLogging::LogMessage(MessageType nMessageType, const wxChar *msgFormat, ...) const
+bool CLogging::ShouldLog(MessageType nMessageType) const
 {
-	InitLogFile();
-
-	const int debugLevel = m_pEngine->GetOptions()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL);
-	switch (nMessageType)
-	{
-	case Debug_Warning:
-		if (!debugLevel)
-			return;
+	switch (nMessageType) {
+	case MessageType::Debug_Warning:
+		if (!debug_level_)
+			return false;
 		break;
-	case Debug_Info:
-		if (debugLevel < 2)
-			return;
+	case MessageType::Debug_Info:
+		if (debug_level_ < 2)
+			return false;
 		break;
-	case Debug_Verbose:
-		if (debugLevel < 3)
-			return;
+	case MessageType::Debug_Verbose:
+		if (debug_level_ < 3)
+			return false;
 		break;
-	case Debug_Debug:
-		if (debugLevel != 4)
-			return;
+	case MessageType::Debug_Debug:
+		if (debug_level_ != 4)
+			return false;
 		break;
-	case RawList:
-		if (!m_pEngine->GetOptions()->GetOptionVal(OPTION_LOGGING_RAWLISTING))
-			return;
+	case MessageType::RawList:
+		if (!raw_listing_)
+			return false;
 		break;
 	default:
 		break;
 	}
-
-	va_list ap;
-    
-    va_start(ap, msgFormat);
-	wxString text = wxString::FormatV(msgFormat, ap);
-	va_end(ap);
-
-	LogToFile(nMessageType, text);
-
-	CLogmsgNotification *notification = new CLogmsgNotification;
-	notification->msgType = nMessageType;
-	notification->msg = text;
-	m_pEngine->AddNotification(notification);
-}
-
-void CLogging::LogMessageRaw(MessageType nMessageType, const wxChar *msg) const
-{
-	InitLogFile();
-
-	const int debugLevel = m_pEngine->GetOptions()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL);
-	switch (nMessageType)
-	{
-	case Debug_Warning:
-		if (!debugLevel)
-			return;
-		break;
-	case Debug_Info:
-		if (debugLevel < 2)
-			return;
-		break;
-	case Debug_Verbose:
-		if (debugLevel < 3)
-			return;
-		break;
-	case Debug_Debug:
-		if (debugLevel != 4)
-			return;
-		break;
-	case RawList:
-		if (!m_pEngine->GetOptions()->GetOptionVal(OPTION_LOGGING_RAWLISTING))
-			return;
-		break;
-	default:
-		break;
-	}
-
-	LogToFile(nMessageType, msg);
-
-	CLogmsgNotification *notification = new CLogmsgNotification;
-	notification->msgType = nMessageType;
-	notification->msg = msg;
-	m_pEngine->AddNotification(notification);
-}
-
-void CLogging::LogMessage(wxString SourceFile, int nSourceLine, void *pInstance, MessageType nMessageType, const wxChar *msgFormat, ...) const
-{
-	InitLogFile();
-
-	const int debugLevel = m_pEngine->GetOptions()->GetOptionVal(OPTION_LOGGING_DEBUGLEVEL);
-	switch (nMessageType)
-	{
-	case Debug_Warning:
-		if (!debugLevel)
-			return;
-		break;
-	case Debug_Info:
-		if (debugLevel < 2)
-			return;
-		break;
-	case Debug_Verbose:
-		if (debugLevel < 3)
-			return;
-		break;
-	case Debug_Debug:
-		if (debugLevel != 4)
-			return;
-		break;
-	default:
-		break;
-	}
-
-	int pos = SourceFile.Find('\\', true);
-	if (pos != -1)
-		SourceFile = SourceFile.Mid(pos+1);
-
-	pos = SourceFile.Find('/', true);
-	if (pos != -1)
-		SourceFile = SourceFile.Mid(pos+1);
-
-	va_list ap;
-    
-	va_start(ap, msgFormat);
-	wxString text = wxString::FormatV(msgFormat, ap);
-	va_end(ap);
-
-	wxString msg = wxString::Format(_T("%s(%d): %s   caller=%p"), SourceFile.c_str(), nSourceLine, text.c_str(), this);
-
-	LogToFile(nMessageType, msg);
-
-	CLogmsgNotification *notification = new CLogmsgNotification;
-	notification->msgType = nMessageType;
-	notification->msg = msg;
-	m_pEngine->AddNotification(notification);
+	return true;
 }
 
 void CLogging::InitLogFile() const
@@ -193,35 +90,35 @@ void CLogging::InitLogFile() const
 
 	m_logfile_initialized = true;
 
-	m_file = m_pEngine->GetOptions()->GetOption(OPTION_LOGGING_FILE);
-	if (m_file == _T(""))
+	m_file = engine_.GetOptions().GetOption(OPTION_LOGGING_FILE);
+	if (m_file.empty())
 		return;
 
 #ifdef __WXMSW__
 	m_log_fd = CreateFile(m_file, FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 	if (m_log_fd == INVALID_HANDLE_VALUE)
 #else
-	m_log_fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT, 0644);
+	m_log_fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC, 0644);
 	if (m_log_fd == -1)
 #endif
 	{
-		LogMessage(Error, _("Could not open log file: %s"), wxSysErrorMsg());
+		LogMessage(MessageType::Error, _("Could not open log file: %s"), wxSysErrorMsg());
 		return;
 	}
 
-	m_prefixes[Status] = _("Status:");
-	m_prefixes[Error] = _("Error:");
-	m_prefixes[Command] = _("Command:");
-	m_prefixes[Response] = _("Response:");
-	m_prefixes[Debug_Warning] = _("Trace:");
-	m_prefixes[Debug_Info] = m_prefixes[Debug_Warning];
-	m_prefixes[Debug_Verbose] = m_prefixes[Debug_Warning];
-	m_prefixes[Debug_Debug] = m_prefixes[Debug_Warning];
-	m_prefixes[RawList] = _("Listing:");
+	m_prefixes[static_cast<int>(MessageType::Status)] = _("Status:");
+	m_prefixes[static_cast<int>(MessageType::Error)] = _("Error:");
+	m_prefixes[static_cast<int>(MessageType::Command)] = _("Command:");
+	m_prefixes[static_cast<int>(MessageType::Response)] = _("Response:");
+	m_prefixes[static_cast<int>(MessageType::Debug_Warning)] = _("Trace:");
+	m_prefixes[static_cast<int>(MessageType::Debug_Info)] = m_prefixes[static_cast<int>(MessageType::Debug_Warning)];
+	m_prefixes[static_cast<int>(MessageType::Debug_Verbose)] = m_prefixes[static_cast<int>(MessageType::Debug_Warning)];
+	m_prefixes[static_cast<int>(MessageType::Debug_Debug)] = m_prefixes[static_cast<int>(MessageType::Debug_Warning)];
+	m_prefixes[static_cast<int>(MessageType::RawList)] = _("Listing:");
 
 	m_pid = wxGetProcessId();
 
-	m_max_size = m_pEngine->GetOptions()->GetOptionVal(OPTION_LOGGING_FILE_SIZELIMIT);
+	m_max_size = engine_.GetOptions().GetOptionVal(OPTION_LOGGING_FILE_SIZELIMIT);
 	if (m_max_size < 0)
 		m_max_size = 0;
 	else if (m_max_size > 2000)
@@ -231,6 +128,9 @@ void CLogging::InitLogFile() const
 
 void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 {
+	scoped_lock l(mutex_);
+
+	InitLogFile();
 #ifdef __WXMSW__
 	if (m_log_fd == INVALID_HANDLE_VALUE)
 		return;
@@ -246,17 +146,14 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 #else
 		_T("\n"),
 #endif
-		now.Format(_T("%Y-%m-%d %H:%M:%S")).c_str(), m_pid, m_pEngine->GetEngineId(), m_prefixes[nMessageType].c_str(), msg.c_str()));
+		now.Format(_T("%Y-%m-%d %H:%M:%S")), m_pid, engine_.GetEngineId(), m_prefixes[static_cast<int>(nMessageType)], msg));
 
 	const wxWX2MBbuf utf8 = out.mb_str(wxConvUTF8);
-	if (utf8)
-	{
+	if (utf8) {
 #ifdef __WXMSW__
-		if (m_max_size)
-		{
+		if (m_max_size) {
 			LARGE_INTEGER size;
-			if (!GetFileSizeEx(m_log_fd, &size) || size.QuadPart > m_max_size)
-			{
+			if (!GetFileSizeEx(m_log_fd, &size) || size.QuadPart > m_max_size) {
 				CloseHandle(m_log_fd);
 
 				// m_log_fd might no longer be the original file.
@@ -264,8 +161,7 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 				HANDLE hMutex = ::CreateMutex(0, true, _T("FileZilla 3 Logrotate Mutex"));
 
 				HANDLE hFile = CreateFile(m_file, FILE_APPEND_DATA, FILE_SHARE_DELETE | FILE_SHARE_WRITE | FILE_SHARE_READ, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
-				if (hFile == INVALID_HANDLE_VALUE)
-				{
+				if (hFile == INVALID_HANDLE_VALUE) {
 					wxString error = wxSysErrorMsg();
 
 					// Oh dear..
@@ -273,15 +169,14 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 					CloseHandle(hMutex);
 
 					m_log_fd = INVALID_HANDLE_VALUE;
-					LogMessage(Error, _("Could not open log file: %s"), error.c_str());
+					LogMessage(MessageType::Error, _("Could not open log file: %s"), error);
 					return;
 				}
 
 				wxString error;
-				if (GetFileSizeEx(hFile, &size) && size.QuadPart > m_max_size)
-				{
+				if (GetFileSizeEx(hFile, &size) && size.QuadPart > m_max_size) {
 					CloseHandle(hFile);
-					
+
 					// MoveFileEx can fail if trying to access a deleted file for which another process still has
 					// a handle. Move it far away first.
 					// Todo: Handle the case in which logdir and tmpdir are on different volumes.
@@ -300,11 +195,13 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 				else
 					m_log_fd = hFile;
 
-				ReleaseMutex(hMutex);
-				CloseHandle(hMutex);
+				if (hMutex) {
+					ReleaseMutex(hMutex);
+					CloseHandle(hMutex);
+				}
 
-				if (error != _T(""))
-					LogMessage(Error, _("Could not open log file: %s"), error.c_str());
+				if (!error.empty())
+					LogMessage(MessageType::Error, _("Could not open log file: %s"), error);
 			}
 		}
 		DWORD len = (DWORD)strlen((const char*)utf8);
@@ -312,17 +209,15 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 		BOOL res = WriteFile(m_log_fd, (const char*)utf8, len, &written, 0);
 		if (!res || written != len)
 		{
-			LogMessage(Error, _("Could not write to log file: %s"), wxSysErrorMsg());
+			LogMessage(MessageType::Error, _("Could not write to log file: %s"), wxSysErrorMsg());
 			CloseHandle(m_log_fd);
 			m_log_fd = INVALID_HANDLE_VALUE;
 		}
 #else
-		if (m_max_size)
-		{
+		if (m_max_size) {
 			struct stat buf;
 			int rc = fstat(m_log_fd, &buf);
-			while (!rc && buf.st_size > 10000)//m_max_size)
-			{
+			while (!rc && buf.st_size > m_max_size) {
 				struct flock lock = {0};
 				lock.l_type = F_WRLCK;
 				lock.l_whence = SEEK_SET;
@@ -335,23 +230,21 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 				while ((rc = fcntl(m_log_fd, F_SETLKW, &lock)) == -1 && errno == EINTR);
 
 				// Ignore any other failures
-				int fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT, 0644);
-				if (fd == -1)
-				{
+				int fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC, 0644);
+				if (fd == -1) {
 					wxString error = wxSysErrorMsg();
 
 					close(m_log_fd);
 					m_log_fd = -1;
 
-					LogMessage(Error, error);
+					LogMessage(MessageType::Error, error);
 					return;
 				}
 				struct stat buf2;
 				rc = fstat(fd, &buf2);
 
 				// Different files
-				if (!rc && buf.st_ino != buf2.st_ino)
-				{
+				if (!rc && buf.st_ino != buf2.st_ino) {
 					close(m_log_fd); // Releases the lock
 					m_log_fd = fd;
 					buf = buf2;
@@ -366,10 +259,9 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 				close(fd);
 
 				// Get the new file
-				m_log_fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT, 0644);
-				if (m_log_fd == -1)
-				{
-					LogMessage(Error, wxSysErrorMsg());
+				m_log_fd = open(m_file.fn_str(), O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC, 0644);
+				if (m_log_fd == -1) {
+					LogMessage(MessageType::Error, wxSysErrorMsg());
 					return;
 				}
 
@@ -379,12 +271,17 @@ void CLogging::LogToFile(MessageType nMessageType, const wxString& msg) const
 		}
 		size_t len = strlen((const char*)utf8);
 		size_t written = write(m_log_fd, (const char*)utf8, len);
-		if (written != len)
-		{
+		if (written != len) {
 			close(m_log_fd);
 			m_log_fd = -1;
-			LogMessage(Error, _("Could not write to log file: %s"), wxSysErrorMsg());
+			LogMessage(MessageType::Error, _("Could not write to log file: %s"), wxSysErrorMsg());
 		}
 #endif
 	}
 }
+
+void CLogging::UpdateLogLevel(COptionsBase & options)
+{
+	debug_level_ = options.GetOptionVal(OPTION_LOGGING_DEBUGLEVEL);
+	raw_listing_ = options.GetOptionVal(OPTION_LOGGING_RAWLISTING);
+}
