@@ -1,8 +1,22 @@
 #include <filezilla.h>
 
-const wxEventType fzEVT_NOTIFICATION = wxNewEventType();
+wxDEFINE_EVENT(fzEVT_NOTIFICATION, wxFzEvent);
 
-wxFzEvent::wxFzEvent(int id /*=wxID_ANY*/) : wxEvent(id, fzEVT_NOTIFICATION)
+wxFzEvent::wxFzEvent()
+	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
+	, engine_()
+{
+}
+
+wxFzEvent::wxFzEvent(CFileZillaEngine const* engine)
+	: wxEvent(wxID_ANY, fzEVT_NOTIFICATION)
+	, engine_(engine)
+{
+}
+
+wxFzEvent::wxFzEvent(int id)
+	: wxEvent(id, fzEVT_NOTIFICATION)
+	, engine_()
 {
 }
 
@@ -11,78 +25,11 @@ wxEvent *wxFzEvent::Clone() const
 	return new wxFzEvent(*this);
 }
 
-CNotification::CNotification()
-{
-}
-
-CNotification::~CNotification()
-{
-}
-
-CLogmsgNotification::CLogmsgNotification()
-{
-}
-
-CLogmsgNotification::~CLogmsgNotification()
-{
-}
-
-enum NotificationId CLogmsgNotification::GetID() const
-{
-	return nId_logmsg;
-}
-
-COperationNotification::COperationNotification()
-{
-}
-
-COperationNotification::~COperationNotification()
-{
-}
-
-enum NotificationId COperationNotification::GetID() const
-{
-	return nId_operation;
-}
-
 CDirectoryListingNotification::CDirectoryListingNotification(const CServerPath& path, const bool modified /*=false*/, const bool failed /*=false*/)
 	: m_modified(modified), m_failed(failed), m_path(path)
 {
 }
 
-CDirectoryListingNotification::~CDirectoryListingNotification()
-{
-}
-
-enum NotificationId CDirectoryListingNotification::GetID() const
-{
-	return nId_listing;
-}
-
-CAsyncRequestNotification::CAsyncRequestNotification()
-{
-}
-
-CAsyncRequestNotification::~CAsyncRequestNotification()
-{
-}
-
-enum NotificationId CAsyncRequestNotification::GetID() const
-{
-	return nId_asyncrequest;
-}
-
-CFileExistsNotification::CFileExistsNotification()
-{
-	localSize = remoteSize = -1;
-	overwriteAction = unknown;
-	ascii = false;
-}
-
-CFileExistsNotification::~CFileExistsNotification()
-{
-}
-
 enum RequestId CFileExistsNotification::GetRequestID() const
 {
 	return reqId_fileexists;
@@ -90,11 +37,6 @@ enum RequestId CFileExistsNotification::GetRequestID() const
 
 CInteractiveLoginNotification::CInteractiveLoginNotification(const wxString& challenge)
 	: m_challenge(challenge)
-{
-	passwordSet = false;
-}
-
-CInteractiveLoginNotification::~CInteractiveLoginNotification()
 {
 }
 
@@ -108,41 +50,18 @@ CActiveNotification::CActiveNotification(int direction)
 {
 }
 
-CActiveNotification::~CActiveNotification()
+CTransferStatusNotification::CTransferStatusNotification(CTransferStatus const& status)
+	: status_(status)
 {
 }
 
-enum NotificationId CActiveNotification::GetID() const
+CTransferStatus const& CTransferStatusNotification::GetStatus() const
 {
-	return nId_active;
-}
-
-CTransferStatusNotification::CTransferStatusNotification(CTransferStatus *pStatus)
-{
-	m_pStatus = pStatus;
-}
-
-CTransferStatusNotification::~CTransferStatusNotification()
-{
-	delete m_pStatus;
-}
-
-enum NotificationId CTransferStatusNotification::GetID() const
-{
-	return nId_transferstatus;
-}
-
-const CTransferStatus* CTransferStatusNotification::GetStatus() const
-{
-	return m_pStatus;
+	return status_;
 }
 
 CHostKeyNotification::CHostKeyNotification(wxString host, int port, wxString fingerprint, bool changed /*=false*/)
-	: m_trust(false), m_alwaysTrust(false), m_host(host), m_port(port), m_fingerprint(fingerprint), m_changed(changed)
-{
-}
-
-CHostKeyNotification::~CHostKeyNotification()
+	: m_host(host), m_port(port), m_fingerprint(fingerprint), m_changed(changed)
 {
 }
 
@@ -184,57 +103,42 @@ char* CDataNotification::Detach(int& len)
 	return pData;
 }
 
-CCertificate::CCertificate()
-	: m_rawData()
-	, m_len()
-	, m_pkalgobits()
-{
-}
-
 CCertificate::CCertificate(
-		const unsigned char* rawData, unsigned int len,
-		wxDateTime activationTime, wxDateTime expirationTime,
-		const wxString& serial,
-		const wxString& pkalgoname, unsigned int bits,
-		const wxString& signalgoname,
-		const wxString& fingerprint_md5,
-		const wxString& fingerprint_sha1,
-		const wxString& subject,
-		const wxString& issuer)
+		unsigned char const* rawData, unsigned int len,
+		wxDateTime const& activationTime, wxDateTime const& expirationTime,
+		wxString const& serial,
+		wxString const& pkalgoname, unsigned int bits,
+		wxString const& signalgoname,
+		wxString const& fingerprint_sha256,
+		wxString const& fingerprint_sha1,
+		wxString const& issuer,
+		wxString const& subject,
+		std::vector<wxString> const& altSubjectNames)
+	: m_activationTime(activationTime)
+	, m_expirationTime(expirationTime)
+	, m_len(len)
+	, m_serial(serial)
+	, m_pkalgoname(pkalgoname)
+	, m_pkalgobits(bits)
+	, m_signalgoname(signalgoname)
+	, m_fingerprint_sha256(fingerprint_sha256)
+	, m_fingerprint_sha1(fingerprint_sha1)
+	, m_issuer(issuer)
+	, m_subject(subject)
+	, m_altSubjectNames(altSubjectNames)
 {
 	wxASSERT(len);
-	if (len)
-	{
+	if (len) {
 		m_rawData = new unsigned char[len];
 		memcpy(m_rawData, rawData, len);
 	}
-	else
-		m_rawData = 0;
-	m_len = len;
-
-	m_activationTime = activationTime;
-	m_expirationTime = expirationTime;
-
-	m_serial = serial;
-	m_pkalgoname = pkalgoname;
-	m_pkalgobits = bits;
-
-	m_signalgoname = signalgoname;
-
-	m_fingerprint_md5 = fingerprint_md5;
-	m_fingerprint_sha1 = fingerprint_sha1;
-
-	m_subject = subject;
-	m_issuer = issuer;
 }
 
 CCertificate::CCertificate(const CCertificate &op)
 {
-	if (op.m_rawData)
-	{
+	if (op.m_rawData) {
 		wxASSERT(op.m_len);
-		if (op.m_len)
-		{
+		if (op.m_len) {
 			m_rawData = new unsigned char[op.m_len];
 			memcpy(m_rawData, op.m_rawData, op.m_len);
 		}
@@ -254,11 +158,12 @@ CCertificate::CCertificate(const CCertificate &op)
 
 	m_signalgoname = op.m_signalgoname;
 
-	m_fingerprint_md5 = op.m_fingerprint_md5;
+	m_fingerprint_sha256 = op.m_fingerprint_sha256;
 	m_fingerprint_sha1 = op.m_fingerprint_sha1;
 
-	m_subject = op.m_subject;
 	m_issuer = op.m_issuer;
+	m_subject = op.m_subject;
+	m_altSubjectNames = op.m_altSubjectNames;
 }
 
 CCertificate::~CCertificate()
@@ -272,11 +177,9 @@ CCertificate& CCertificate::operator=(const CCertificate &op)
 		return *this;
 
 	delete [] m_rawData;
-	if (op.m_rawData)
-	{
+	if (op.m_rawData) {
 		wxASSERT(op.m_len);
-		if (op.m_len)
-		{
+		if (op.m_len) {
 			m_rawData = new unsigned char[op.m_len];
 			memcpy(m_rawData, op.m_rawData, op.m_len);
 		}
@@ -296,11 +199,12 @@ CCertificate& CCertificate::operator=(const CCertificate &op)
 
 	m_signalgoname = op.m_signalgoname;
 
-	m_fingerprint_md5 = op.m_fingerprint_md5;
+	m_fingerprint_sha256 = op.m_fingerprint_sha256;
 	m_fingerprint_sha1 = op.m_fingerprint_sha1;
 
-	m_subject = op.m_subject;
 	m_issuer = op.m_issuer;
+	m_subject = op.m_subject;
+	m_altSubjectNames = op.m_altSubjectNames;
 
 	return *this;
 }
@@ -321,10 +225,4 @@ CCertificateNotification::CCertificateNotification(const wxString& host, unsigne
 	m_sessionMac = sessionMac;
 
 	m_certificates = certificates;
-
-	m_trusted = false;
-}
-
-CCertificateNotification::~CCertificateNotification()
-{
 }
