@@ -1,6 +1,6 @@
 #include <filezilla.h>
+#include "engineprivate.h"
 #include "proxy.h"
-#include <errno.h>
 #include "ControlSocket.h"
 #include <wx/sckaddr.h>
 
@@ -17,25 +17,19 @@ enum handshake_state
 	socks4_handshake
 };
 
-CProxySocket::CProxySocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
-	: CBackend(pEvtHandler), m_pOwner(pOwner)
+CProxySocket::CProxySocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+	: CEventHandler(pOwner->event_loop_)
+	, CBackend(pEvtHandler)
+	, m_pSocket(pSocket)
+	, m_pOwner(pOwner)
 {
-	m_pSocket = pSocket;
 	m_pSocket->SetEventHandler(this);
-
-	m_proxyState = noconn;
-
-	m_pSendBuffer = 0;
-	m_pRecvBuffer = 0;
-
-	m_proxyType = unknown;
-
-	m_can_write = false;
-	m_can_read = false;
 }
 
 CProxySocket::~CProxySocket()
 {
+	RemoveHandler();
+
 	if (m_pSocket)
 		m_pSocket->SetEventHandler(0);
 	delete [] m_pSendBuffer;
@@ -47,7 +41,7 @@ static wxString base64encode(const wxString& str)
 	// Code shamelessly taken from wxWidgets and adopted to encode UTF-8 strings.
 	// wxWidget's http class encodes string from arbitrary encoding into base64,
 	// could as well encode /dev/random
-	static const char *base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	static char const*const base64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 	wxString buf;
 
@@ -74,9 +68,9 @@ static wxString base64encode(const wxString& str)
 	return buf;
 }
 
-int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass)
+int CProxySocket::Handshake(CProxySocket::ProxyType type, const wxString& host, unsigned int port, const wxString& user, const wxString& pass)
 {
-	if (type == CProxySocket::unknown || host == _T("") || port < 1 || port > 65535)
+	if (type == CProxySocket::unknown || host.empty() || port < 1 || port > 65535)
 		return EINVAL;
 
 	if (m_proxyState != noconn)
@@ -95,34 +89,25 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 
 	m_proxyState = handshake;
 
-	if (type == HTTP)
-	{
+	if (type == HTTP) {
 		m_handshakeState = http_wait;
 
-		wxWX2MBbuf challenge;
-		int challenge_len;
-		if (user != _T(""))
-		{
+		wxWX2MBbuf challenge{};
+		int challenge_len{};
+		if (!user.empty()) {
 			challenge = base64encode(user + _T(":") + pass).mb_str(wxConvUTF8);
 			challenge_len = strlen(challenge);
 		}
-		else
-		{
-			challenge = (size_t)0;
-			challenge_len = 0;
-		}
 
 		// Bit oversized, but be on the safe side
 		m_pSendBuffer = new char[70 + strlen(host_raw) * 2 + 2*5 + challenge_len + 23];
 
-		if (!challenge)
-		{
+		if (!challenge || !challenge_len) {
 			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nUser-Agent: FileZilla\r\n\r\n",
 				(const char*)host_raw, port,
 				(const char*)host_raw, port);
 		}
-		else
-		{
+		else {
 			m_sendBufferLen = sprintf(m_pSendBuffer, "CONNECT %s:%u HTTP/1.1\r\nHost: %s:%u\r\nProxy-Authorization: Basic %s\r\nUser-Agent: FileZilla\r\n\r\n",
 				(const char*)host_raw, port,
 				(const char*)host_raw, port,
@@ -133,25 +118,24 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 		m_recvBufferLen = 4096;
 		m_recvBufferPos = 0;
 	}
-	else if (type == SOCKS4)
-	{
+	else if (type == SOCKS4) {
 		wxString ip = m_host;
-		if (GetIPV6LongForm(m_host).empty()) {
-			m_pOwner->LogMessage(Error, _("IPv6 addresses are not supported with SOCKS4 proxy"));
+		if (!GetIPV6LongForm(m_host).empty()) {
+			m_pOwner->LogMessage(MessageType::Error, _("IPv6 addresses are not supported with SOCKS4 proxy"));
 			return EINVAL;
 		}
 
-		if (!IsIpAddress(m_host)) {			
+		if (!IsIpAddress(m_host)) {
 			wxIPV4address address;
 			if (!address.Hostname(host)) {
-				m_pOwner->LogMessage(Error, _("Cannot resolve hostname to IPv4 address for use with SOCKS4 proxy."));
+				m_pOwner->LogMessage(MessageType::Error, _("Cannot resolve hostname to IPv4 address for use with SOCKS4 proxy."));
 				return EINVAL;
 			}
 			ip = address.IPAddress();
 		}
 
-		m_pOwner->LogMessage(Status, _("SOCKS4 proxy will connect to: %s"), ip.c_str());		
-		
+		m_pOwner->LogMessage(MessageType::Status, _("SOCKS4 proxy will connect to: %s"), ip);
+
 		m_pSendBuffer = new char[9];
 		m_pSendBuffer[0] = 4; // Protocol version
 		m_pSendBuffer[1] = 1; // Stream mode
@@ -170,25 +154,22 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 			buf[i] += c - '0';
 		}
 		m_pSendBuffer[8] = 0;
-		m_sendBufferLen = 9;		
+		m_sendBufferLen = 9;
 		m_pRecvBuffer = new char[8];
 		m_recvBufferLen = 8;
 		m_recvBufferPos = 0;
 		m_handshakeState = socks4_handshake;
 	}
-	else
-	{
+	else {
 		m_pSendBuffer = new char[4];
 		m_pSendBuffer[0] = 5; // Protocol version
-		if (user != _T(""))
-		{
+		if (!user.empty()) {
 			m_pSendBuffer[1] = 2; // # auth methods supported
 			m_pSendBuffer[2] = 0; // Method: No auth
 			m_pSendBuffer[3] = 2; // Method: Username and password
 			m_sendBufferLen = 4;
 		}
-		else
-		{
+		else {
 			m_pSendBuffer[1] = 1; // # auth methods supported
 			m_pSendBuffer[2] = 0; // Method: No auth
 			m_sendBufferLen = 3;
@@ -204,50 +185,47 @@ int CProxySocket::Handshake(enum CProxySocket::ProxyType type, const wxString& h
 	return EINPROGRESS;
 }
 
-void CProxySocket::OnSocketEvent(CSocketEvent& event)
+void CProxySocket::operator()(CEventBase const& ev)
 {
-	switch (event.GetType())
-	{
-	case CSocketEvent::hostaddress:
-		{
-			const wxString& address = event.GetData();
-			m_pOwner->LogMessage(Status, _("Connecting to %s..."), address.c_str());
-		}
-	case CSocketEvent::connection_next:
-		if (event.GetError())
-			m_pOwner->LogMessage(Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(event.GetError()).c_str());
+	Dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+		&CProxySocket::OnSocketEvent,
+		&CProxySocket::OnHostAddress);
+}
+
+void CProxySocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
+{
+	switch (t) {
+	case SocketEventType::connection_next:
+		if (error)
+			m_pOwner->LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
 		break;
-	case CSocketEvent::connection:
-		if (event.GetError())
-		{
+	case SocketEventType::connection:
+		if (error) {
 			if (m_proxyState == handshake)
 				m_proxyState = noconn;
-			CSocketEvent *forwarded_event = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, event.GetError());
-			CSocketEventDispatcher::Get().SendEvent(forwarded_event);
+			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, error);
 		}
-		else
-		{
-			m_pOwner->LogMessage(Status, _("Connection with proxy established, performing handshake..."));
+		else {
+			m_pOwner->LogMessage(MessageType::Status, _("Connection with proxy established, performing handshake..."));
 		}
 		break;
-	case CSocketEvent::read:
+	case SocketEventType::read:
 		OnReceive();
 		break;
-	case CSocketEvent::write:
+	case SocketEventType::write:
 		OnSend();
 		break;
-	case CSocketEvent::close:
+	case SocketEventType::close:
 		OnReceive();
-		/*if (m_proxyState == handshake)
-			m_proxyState = noconn;
-		m_pEvtHandler->AddPendingEvent(event);*/
-		break;
-	default:
-		m_pOwner->LogMessage(Debug_Warning, _T("Unhandled socket event %d"), event.GetType());
 		break;
 	}
 }
 
+void CProxySocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+{
+	m_pOwner->LogMessage(MessageType::Status, _("Connecting to %s..."), address);
+}
+
 void CProxySocket::Detach()
 {
 	if (!m_pSocket)
@@ -267,59 +245,46 @@ void CProxySocket::OnReceive()
 	switch (m_handshakeState)
 	{
 	case http_wait:
-		for (;;)
-		{
+		for (;;) {
 			int error;
 			int do_read = m_recvBufferLen - m_recvBufferPos - 1;
 			char* end = 0;
-			for (int i = 0; i < 2; i++)
-			{
+			for (int i = 0; i < 2; ++i) {
 				int read;
 				if (!i)
 					read = m_pSocket->Peek(m_pRecvBuffer + m_recvBufferPos, do_read, error);
 				else
 					read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, do_read, error);
-				if (read == -1)
-				{
-					if (error != EAGAIN)
-					{
+				if (read == -1) {
+					if (error != EAGAIN) {
 						m_proxyState = noconn;
-						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
-						CSocketEventDispatcher::Get().SendEvent(evt);
+						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
 					}
 					else
 						m_can_read = false;
 					return;
 				}
-				if (!read)
-				{
+				if (!read) {
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
-				if (m_pSendBuffer)
-				{
+				if (m_pSendBuffer) {
 					m_proxyState = noconn;
-					m_pOwner->LogMessage(Debug_Warning, _T("Incoming data before requst fully sent"));
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming data before request fully sent"));
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 
-				if (!i)
-				{
+				if (!i) {
 					// Response ends with strstr
 					m_pRecvBuffer[m_recvBufferPos + read] = 0;
 					end = strstr(m_pRecvBuffer, "\r\n\r\n");
-					if (!end)
-					{
-						if (m_recvBufferPos + read + 1 == m_recvBufferLen)
-						{
+					if (!end) {
+						if (m_recvBufferPos + read + 1 == m_recvBufferLen) {
 							m_proxyState = noconn;
-							m_pOwner->LogMessage(Debug_Warning, _T("Incoming header too large"));
-							CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ENOMEM);
-							CSocketEventDispatcher::Get().SendEvent(evt);
+							m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Incoming header too large"));
+							m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ENOMEM);
 							return;
 						}
 						do_read = read;
@@ -327,14 +292,11 @@ void CProxySocket::OnReceive()
 					else
 						do_read = end - m_pRecvBuffer + 4 - m_recvBufferPos;
 				}
-				else
-				{
-					if (read != do_read)
-					{
+				else {
+					if (read != do_read) {
 						m_proxyState = noconn;
-						m_pOwner->LogMessage(Debug_Warning, _T("Could not read what got peeked"));
-						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-						CSocketEventDispatcher::Get().SendEvent(evt);
+						m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Could not read what got peeked"));
+						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 						return;
 					}
 					m_recvBufferPos += read;
@@ -344,48 +306,38 @@ void CProxySocket::OnReceive()
 			if (!end)
 				continue;
 
-			end = strchr(m_pRecvBuffer, '\r');
-			wxASSERT(end);
+			end = strchr(m_pRecvBuffer, '\r'); // Never fails as old value of end exists and starts with CR, we just look for an earlier case.
 			*end = 0;
 			wxString reply(m_pRecvBuffer, wxConvUTF8);
-			m_pOwner->LogMessage(Response, _("Proxy reply: %s"), reply.c_str());
+			m_pOwner->LogMessage(MessageType::Response, _("Proxy reply: %s"), reply);
 
-			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2"))
-			{
+			if (reply.Left(10) != _T("HTTP/1.1 2") && reply.Left(10) != _T("HTTP/1.0 2")) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNRESET);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNRESET);
 				return;
 			}
 
 			m_proxyState = conn;
-			CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
-			CSocketEventDispatcher::Get().SendEvent(evt);
+			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
 			return;
 		}
 	case socks4_handshake:
-		while (m_recvBufferLen && m_can_read && m_proxyState == handshake)
-		{
+		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int read_error;
 			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, read_error);
-			if (read == -1)
-			{
-				if (read_error != EAGAIN)
-				{
+			if (read == -1) {
+				if (read_error != EAGAIN) {
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, read_error);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, read_error);
 				}
 				else
 					m_can_read = false;
 				return;
 			}
-			
-			if (!read)
-			{
+
+			if (!read) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
@@ -395,7 +347,7 @@ void CProxySocket::OnReceive()
 				continue;
 
 			m_recvBufferPos = 0;
-			
+
 			if (m_pRecvBuffer[1] != 0x5A) {
 				wxString error;
 				switch (m_pRecvBuffer[1]) {
@@ -407,20 +359,18 @@ void CProxySocket::OnReceive()
 						break;
 					case 0x5D:
 						error = _("Request failed - client's identd could not confirm the user ID string");
-						break;				
+						break;
 					default:
 						error.Printf(_("Unassigned error code %d"), (int)(unsigned char) m_pRecvBuffer[1]);
 						break;
 				}
-				m_pOwner->LogMessage(Error, _("Proxy request failed: %s"), error.c_str());
+				m_pOwner->LogMessage(MessageType::Error, _("Proxy request failed: %s"), error);
 				m_proxyState = noconn;
-				CSocketEventDispatcher::Get()
-					.SendEvent(new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED));
+				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_proxyState = conn;
-			CSocketEventDispatcher::Get()
-				.SendEvent(new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0));
+			m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
 		}
 		return;
 	case socks5_method:
@@ -430,27 +380,21 @@ void CProxySocket::OnReceive()
 	case socks5_request_address:
 		if (m_pSendBuffer)
 			return;
-		while (m_recvBufferLen && m_can_read && m_proxyState == handshake)
-		{
+		while (m_recvBufferLen && m_can_read && m_proxyState == handshake) {
 			int error;
 			int read = m_pSocket->Read(m_pRecvBuffer + m_recvBufferPos, m_recvBufferLen, error);
-			if (read == -1)
-			{
-				if (error != EAGAIN)
-				{
+			if (read == -1) {
+				if (error != EAGAIN) {
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
 				}
 				else
 					m_can_read = false;
 				return;
 			}
-			if (!read)
-			{
+			if (!read) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 				return;
 			}
 			m_recvBufferPos += read;
@@ -462,25 +406,20 @@ void CProxySocket::OnReceive()
 			m_recvBufferPos = 0;
 
 			// All data got read, parse it
-			switch (m_handshakeState)
-			{
+			switch (m_handshakeState) {
 			default:
-				if (m_pRecvBuffer[0] != 5)
-				{
-					m_pOwner->LogMessage(Debug_Warning, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
+				if (m_pRecvBuffer[0] != 5) {
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown SOCKS protocol version: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				break;
 			case socks5_auth:
-				if (m_pRecvBuffer[0] != 1)
-				{
-					m_pOwner->LogMessage(Debug_Warning, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
+				if (m_pRecvBuffer[0] != 1) {
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Unknown protocol version of SOCKS Username/Password Authentication subnegotiation: %d"), (int)m_pRecvBuffer[0]);
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				break;
@@ -489,8 +428,7 @@ void CProxySocket::OnReceive()
 				// Nothing to do
 				break;
 			}
-			switch (m_handshakeState)
-			{
+			switch (m_handshakeState) {
 			case socks5_method:
 				{
 					const char method = m_pRecvBuffer[1];
@@ -503,10 +441,9 @@ void CProxySocket::OnReceive()
 						m_handshakeState = socks5_auth;
 						break;
 					default:
-						m_pOwner->LogMessage(Debug_Warning, _("No supported SOCKS5 auth method"));
+						m_pOwner->LogMessage(MessageType::Debug_Warning, _("No supported SOCKS5 auth method"));
 						m_proxyState = noconn;
-						CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-						CSocketEventDispatcher::Get().SendEvent(evt);
+						m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 						return;
 					}
 				}
@@ -514,59 +451,59 @@ void CProxySocket::OnReceive()
 			case socks5_auth:
 				if (m_pRecvBuffer[1] != 0)
 				{
-					m_pOwner->LogMessage(Debug_Warning, _("Proxy authentication failed"));
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy authentication failed"));
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request;
 				break;
 			case socks5_request:
-				if (m_pRecvBuffer[1])
-				{
-					wxString error;
+				if (m_pRecvBuffer[1]) {
+					wxString errorMsg;
 					switch (m_pRecvBuffer[1])
 					{
 					case 1:
-						error = _("General SOCKS server failure");
+						errorMsg = _("General SOCKS server failure");
 						break;
 					case 2:
-						error = _("Connection not allowed by ruleset");
+						errorMsg = _("Connection not allowed by ruleset");
 						break;
 					case 3:
-						error = _("Network unreachable");
+						errorMsg = _("Network unreachable");
 						break;
 					case 4:
-						error = _("Host unreachable");
+						errorMsg = _("Host unreachable");
 						break;
 					case 5:
-						error = _("Connection refused");
+						errorMsg = _("Connection refused");
 						break;
 					case 6:
-						error = _("TTL expired");
+						errorMsg = _("TTL expired");
 						break;
 					case 7:
-						error = _("Command not supported");
+						errorMsg = _("Command not supported");
 						break;
 					case 8:
-						error = _("Address type not supported");
+						errorMsg = _("Address type not supported");
 						break;
 					default:
-						error.Printf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
+						errorMsg.Printf(_("Unassigned error code %d"), (int)(unsigned char)m_pRecvBuffer[1]);
 						break;
 					}
 
-					m_pOwner->LogMessage(Debug_Warning, _("Proxy request failed: %s"), error.c_str());
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: %s"), errorMsg);
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_addrtype;
 				m_recvBufferLen = 3;
 				break;
 			case socks5_request_addrtype:
+				// We need to parse the returned address type to determine the length of the address that follows.
+				// Unfortunately the information in the type and address is useless, many proxies just return
+				// syntactically valid bogus values
 				switch (m_pRecvBuffer[1])
 				{
 				case 1:
@@ -579,10 +516,9 @@ void CProxySocket::OnReceive()
 					m_recvBufferLen = 17;
 					break;
 				default:
-					m_pOwner->LogMessage(Debug_Warning, _("Proxy request failed: Unknown address type in CONNECT reply"));
+					m_pOwner->LogMessage(MessageType::Debug_Warning, _("Proxy request failed: Unknown address type in CONNECT reply"));
 					m_proxyState = noconn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 					return;
 				}
 				m_handshakeState = socks5_request_address;
@@ -591,8 +527,7 @@ void CProxySocket::OnReceive()
 				{
 					// We're done
 					m_proxyState = conn;
-					CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::connection, 0);
-					CSocketEventDispatcher::Get().SendEvent(evt);
+					m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::connection, 0);
 					return;
 				}
 			default:
@@ -630,12 +565,10 @@ void CProxySocket::OnReceive()
 					m_pSendBuffer[1] = 1; // CONNECT
 					m_pSendBuffer[2] = 0; // Reserved
 
-					if (IsIpAddress(m_host))
-					{
+					if (IsIpAddress(m_host)) {
 						// Quite ugly
 						wxString ipv6 = GetIPV6LongForm(m_host);
-						if (!ipv6.empty())
-						{
+						if (!ipv6.empty()) {
 							ipv6.Replace(_T(":"), _T(""));
 							addrlen = 16;
 							for (int i = 0; i < 16; i++)
@@ -643,8 +576,7 @@ void CProxySocket::OnReceive()
 
 							m_pSendBuffer[3] = 4; // IPv6
 						}
-						else
-						{
+						else {
 							unsigned char *buf = (unsigned char*)m_pSendBuffer + 4;
 							int i = 0;
 							memset(buf, 0, 4);
@@ -665,8 +597,7 @@ void CProxySocket::OnReceive()
 							m_pSendBuffer[3] = 1; // IPv4
 						}
 					}
-					else
-					{
+					else {
 						m_pSendBuffer[3] = 3; // Domain name
 						m_pSendBuffer[4] = hostlen;
 						memcpy(m_pSendBuffer + 5, (const char*)host, hostlen);
@@ -695,9 +626,8 @@ void CProxySocket::OnReceive()
 		break;
 	default:
 		m_proxyState = noconn;
-		m_pOwner->LogMessage(Debug_Warning, _T("Unhandled handshake state %d"), m_handshakeState);
-		CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, ECONNABORTED);
-		CSocketEventDispatcher::Get().SendEvent(evt);
+		m_pOwner->LogMessage(MessageType::Debug_Warning, _T("Unhandled handshake state %d"), m_handshakeState);
+		m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, ECONNABORTED);
 		return;
 	}
 }
@@ -708,17 +638,13 @@ void CProxySocket::OnSend()
 	if (m_proxyState != handshake || !m_pSendBuffer)
 		return;
 
-	for (;;)
-	{
+	for (;;) {
 		int error;
 		int written = m_pSocket->Write(m_pSendBuffer, m_sendBufferLen, error);
-		if (written == -1)
-		{
-			if (error != EAGAIN)
-			{
+		if (written == -1) {
+			if (error != EAGAIN) {
 				m_proxyState = noconn;
-				CSocketEvent *evt = new CSocketEvent(m_pEvtHandler, this, CSocketEvent::close, error);
-				CSocketEventDispatcher::Get().SendEvent(evt);
+				m_pEvtHandler->SendEvent<CSocketEvent>(this, SocketEventType::close, error);
 			}
 			else
 				m_can_write = false;
@@ -726,8 +652,7 @@ void CProxySocket::OnSend()
 			return;
 		}
 
-		if (written == m_sendBufferLen)
-		{
+		if (written == m_sendBufferLen) {
 			delete [] m_pSendBuffer;
 			m_pSendBuffer = 0;
 
@@ -740,19 +665,19 @@ void CProxySocket::OnSend()
 	}
 }
 
-int CProxySocket::Read(void *buffer, unsigned int size, int& error)
+int CProxySocket::Read(void *, unsigned int, int& error)
 {
 	error = EAGAIN;
 	return -1;
 }
 
-int CProxySocket::Peek(void *buffer, unsigned int size, int& error)
+int CProxySocket::Peek(void *, unsigned int, int& error)
 {
 	error = EAGAIN;
 	return -1;
 }
 
-int CProxySocket::Write(const void *buffer, unsigned int size, int& error)
+int CProxySocket::Write(const void *, unsigned int, int& error)
 {
 	error = EAGAIN;
 	return -1;
