@@ -1,6 +1,5 @@
 #include <filezilla.h>
 #include "serverpath.h"
-#include "string_coalescer.h"
 
 #define FTP_MVS_DOUBLE_QUOTE (wxChar)0xDC
 
@@ -30,16 +29,6 @@ static const CServerTypeTraits traits[SERVERTYPE_MAX] = {
 	{ _T("/"),   true,     0,    0,    false, 0, 0,   true,  true  }  // Cygwin is like Unix but has optional prefix of form "//server"
 };
 
-CServerPathData::CServerPathData()
-{
-}
-
-CServerPathData::CServerPathData(const CServerPathData& v)
-	: m_segments(v.m_segments)
-	, m_prefix(v.m_prefix)
-{
-}
-
 bool CServerPathData::operator==(const CServerPathData& cmp) const
 {
 	if (m_prefix != cmp.m_prefix)
@@ -52,44 +41,35 @@ bool CServerPathData::operator==(const CServerPathData& cmp) const
 }
 
 CServerPath::CServerPath()
-	: m_bEmpty(true)
-	, m_type(DEFAULT)
+	: m_type(DEFAULT)
 {
 }
 
 CServerPath::CServerPath(const CServerPath &path, wxString subdir)
-	: m_bEmpty(path.m_bEmpty)
-	, m_type(path.m_type)
+	: m_type(path.m_type)
 	, m_data(path.m_data)
 {
-	if (subdir == _T(""))
+	if (subdir.empty())
 		return;
 
 	if (!ChangePath(subdir))
-		Clear();
+		clear();
 }
 
 CServerPath::CServerPath(const CServerPath &path)
-	: m_bEmpty(path.m_bEmpty)
-	, m_type(path.m_type)
+	: m_type(path.m_type)
 	, m_data(path.m_data)
 {
 }
 
-CServerPath::CServerPath(wxString path, ServerType type /*=DEFAULT*/)
+CServerPath::CServerPath(wxString const& path, ServerType type /*=DEFAULT*/)
 	: m_type(type)
 {
 	SetPath(path);
 }
 
-CServerPath::~CServerPath()
+void CServerPath::clear()
 {
-}
-
-void CServerPath::Clear()
-{
-	m_bEmpty = true;
-	m_type = DEFAULT;
 	m_data.clear();
 }
 
@@ -103,14 +83,12 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	wxString path = newPath;
 	wxString file;
 
-	if (path == _T(""))
+	if (path.empty())
 		return false;
 
-	if (m_type == DEFAULT)
-	{
+	if (m_type == DEFAULT) {
 		int pos1 = path.Find(_T(":["));
-		if (pos1 != -1)
-		{
+		if (pos1 != -1) {
 			int pos2 = path.Find(']', true);
 			if (pos2 != -1 && static_cast<size_t>(pos2) == (path.Length() - 1) && !isFile)
 				m_type = VMS;
@@ -137,7 +115,6 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 	}
 
 	m_data.clear();
-	m_bEmpty = true;
 
 	if (!ChangePath(path, isFile))
 		return false;
@@ -149,32 +126,29 @@ bool CServerPath::SetPath(wxString &newPath, bool isFile)
 
 wxString CServerPath::GetPath() const
 {
-	if (m_bEmpty)
-		return _T("");
+	if (empty())
+		return wxString();
 
 	wxString path;
 
-	if (!traits[m_type].prefixmode)
-		path = m_data->m_prefix;
+	if (!traits[m_type].prefixmode && m_data->m_prefix)
+		path = *m_data->m_prefix;
 
 	if (traits[m_type].left_enclosure != 0)
 		path += traits[m_type].left_enclosure;
-	if (m_data->m_segments.empty() && (!traits[m_type].has_root || m_data->m_prefix.empty() || traits[m_type].separator_after_prefix))
+	if (m_data->m_segments.empty() && (!traits[m_type].has_root || !m_data->m_prefix || traits[m_type].separator_after_prefix))
 		path += traits[m_type].separators[0];
 
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-	{
+	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter) {
 		const wxString& segment = *iter;
 		if (iter != m_data->m_segments.begin())
 			path += traits[m_type].separators[0];
-		else if (traits[m_type].has_root)
-		{
-			if (m_data->m_prefix.empty() || traits[m_type].separator_after_prefix)
+		else if (traits[m_type].has_root) {
+			if (!m_data->m_prefix || traits[m_type].separator_after_prefix)
 				path += traits[m_type].separators[0];
 		}
 
-		if (traits[m_type].separatorEscape)
-		{
+		if (traits[m_type].separatorEscape) {
 			wxString tmp = segment;
 			EscapeSeparators(m_type, tmp);
 			path += tmp;
@@ -183,8 +157,8 @@ wxString CServerPath::GetPath() const
 			path += segment;
 	}
 
-	if (traits[m_type].prefixmode)
-		path += m_data->m_prefix;
+	if (traits[m_type].prefixmode && m_data->m_prefix)
+		path += *m_data->m_prefix;
 
 	if (traits[m_type].right_enclosure != 0)
 		path += traits[m_type].right_enclosure;
@@ -199,7 +173,7 @@ wxString CServerPath::GetPath() const
 
 bool CServerPath::HasParent() const
 {
-	if (m_bEmpty)
+	if (empty())
 		return false;
 
 	if (!traits[m_type].has_root)
@@ -210,7 +184,7 @@ bool CServerPath::HasParent() const
 
 CServerPath CServerPath::GetParent() const
 {
-	if (!HasParent())
+	if (empty() || !HasParent())
 		return CServerPath();
 
 	CServerPath parent(*this);
@@ -219,20 +193,20 @@ CServerPath CServerPath::GetParent() const
 	parent_data.m_segments.pop_back();
 
 	if (m_type == MVS)
-		parent_data.m_prefix = _T(".");
+		parent_data.m_prefix = CSparseOptional<wxString>(_T("."));
 
 	return parent;
 }
 
 wxString CServerPath::GetLastSegment() const
 {
-	if (!HasParent())
-		return _T("");
+	if (empty() || !HasParent())
+		return wxString();
 
 	if (!m_data->m_segments.empty())
 		return m_data->m_segments.back();
 	else
-		return _T("");
+		return wxString();
 }
 
 // libc sprintf can be so slow at times...
@@ -259,53 +233,61 @@ wxChar* fast_sprint_number(wxChar* s, size_t n)
 
 wxString CServerPath::GetSafePath() const
 {
-	if (m_bEmpty)
-		return _T("");
+	if (empty())
+		return wxString();
 
 	#define INTLENGTH 20 // 2^64 - 1
 
 	int len = 5 // Type and 2x' ' and terminating 0
 		+ INTLENGTH; // Max length of prefix
 
-	len += m_data->m_prefix.size();
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-		len += iter->Length() + 2 + INTLENGTH;
+	len += m_data->m_prefix ? m_data->m_prefix->size() : 0;
+	for( auto const& segment : m_data->m_segments ) {
+		len += segment.size() + 2 + INTLENGTH;
+	}
 
 	wxString safepath;
-	wxChar* start = safepath.GetWriteBuf(len);
-	wxChar* t = start;
-
-	t = fast_sprint_number(t, m_type);
-	*(t++) = ' ';
-	t = fast_sprint_number(t, m_data->m_prefix.size());
-
-	if (!m_data->m_prefix.empty())
 	{
-		*(t++) = ' ';
-		tstrcpy(t, m_data->m_prefix);
-		t += m_data->m_prefix.size();
-	}
+		wxStringBuffer start(safepath, len);
+		wxChar* t = start;
 
-	for (tConstSegmentIter iter = m_data->m_segments.begin(); iter != m_data->m_segments.end(); ++iter)
-	{
+		t = fast_sprint_number(t, m_type);
 		*(t++) = ' ';
-		t = fast_sprint_number(t, iter->size());
-		*(t++) = ' ';
-		tstrcpy(t, *iter);
-		t += iter->size();
-	}
-	*t = 0;
+		t = fast_sprint_number(t, m_data->m_prefix ? m_data->m_prefix->size() : 0);
 
-	safepath.UngetWriteBuf( t - start );
+		if (m_data->m_prefix) {
+			*(t++) = ' ';
+			tstrcpy(t, m_data->m_prefix->c_str());
+			t += m_data->m_prefix->size();
+		}
+
+		for( auto const& segment : m_data->m_segments ) {
+			*(t++) = ' ';
+			t = fast_sprint_number(t, segment.size());
+			*(t++) = ' ';
+			tstrcpy(t, segment.c_str());
+			t += segment.size();
+		}
+		*t = 0;
+	}
 	safepath.Shrink();
 
 	return safepath;
 }
 
-bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
+bool CServerPath::SetSafePath(const wxString& path)
+{
+	bool const ret = DoSetSafePath(path);
+	if (!ret) {
+		clear();
+	}
+
+	return ret;
+}
+
+bool CServerPath::DoSetSafePath(const wxString& path)
 {
 	CServerPathData& data = m_data.Get();
-	m_bEmpty = true;
 	data.m_prefix.clear();
 	data.m_segments.clear();
 
@@ -314,9 +296,10 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 	// most CPU cycles used during loading of transfer queues
 	// from file
 	const int len = (int)path.Len();
-	wxChar* begin = new wxChar[len + 1];
-	CScopedArray<wxChar> tmp(begin);
-	memcpy(begin, (const wxChar*)path, (len + 1) * sizeof(wxChar));
+	wxCharTypeBuffer<wxChar> buf(len + 1);
+	wxChar* begin = buf.data();
+
+	memcpy(begin, (const wxChar*)path.c_str(), (len + 1) * sizeof(wxChar));
 	wxChar* p = begin;
 
 	int type = 0;
@@ -349,14 +332,11 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 	}
 	while (*p && *p != ' ');
 
-	if (!*p)
-	{
+	if (!*p) {
 		if (prefix_len != 0)
 			return false;
-		else
-		{
+		else {
 			// Is root directory, like / on unix like systems.
-			m_bEmpty = false;
 			return true;
 		}
 	}
@@ -368,9 +348,9 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 	if (prefix_len)
 	{
 		*(p + prefix_len) = 0;
-		data.m_prefix = p;
-		if (coalesce)
-			::Coalesce(data.m_prefix);
+		if( *p ) {
+			data.m_prefix = CSparseOptional<wxString>(p);
+		}
 
 		p += prefix_len + 1;
 	}
@@ -399,21 +379,17 @@ bool CServerPath::SetSafePath(const wxString& path, bool coalesce)
 			return false;
 		*(p + segment_len) = 0;
 		wxString s(p);
-		if (coalesce)
-			::Coalesce(s);
 		data.m_segments.push_back(s);
 
 		p += segment_len + 1;
 	}
 
-	m_bEmpty = false;
-
 	return true;
 }
 
-bool CServerPath::SetType(enum ServerType type)
+bool CServerPath::SetType(ServerType type)
 {
-	if (!m_bEmpty && m_type != DEFAULT)
+	if (!empty() && m_type != DEFAULT)
 		return false;
 
 	m_type = type;
@@ -421,14 +397,14 @@ bool CServerPath::SetType(enum ServerType type)
 	return true;
 }
 
-enum ServerType CServerPath::GetType() const
+ServerType CServerPath::GetType() const
 {
 	return m_type;
 }
 
 bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 {
-	if (m_bEmpty || path.m_bEmpty)
+	if (empty() || path.empty())
 		return false;
 
 	if (m_type != path.m_type)
@@ -437,27 +413,33 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 	if (!HasParent())
 		return false;
 
-	if (traits[m_type].prefixmode != 1)
-	{
-		if (cmpNoCase && m_data->m_prefix.CmpNoCase(path.m_data->m_prefix))
-			return false;
+	if (traits[m_type].prefixmode != 1) {
+		if (cmpNoCase ) {
+			if( m_data->m_prefix && !path.m_data->m_prefix ) {
+				return false;
+			}
+			else if( !m_data->m_prefix && path.m_data->m_prefix ) {
+				return false;
+			}
+			else if( m_data->m_prefix && path.m_data->m_prefix && m_data->m_prefix->CmpNoCase(*path.m_data->m_prefix) ) {
+				return false;
+			}
+		}
 		if (!cmpNoCase && m_data->m_prefix != path.m_data->m_prefix)
 			return false;
 	}
 
 	// On MVS, dirs like 'FOO.BAR' without trailing dot cannot have
 	// subdirectories
-	if (traits[m_type].prefixmode == 1 && path.m_data->m_prefix == _T(""))
+	if (traits[m_type].prefixmode == 1 && !path.m_data->m_prefix)
 		return false;
 
 	tConstSegmentIter iter1 = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
-	while (iter1 != m_data->m_segments.end())
-	{
+	while (iter1 != m_data->m_segments.end()) {
 		if (iter2 == path.m_data->m_segments.end())
 			return true;
-		if (cmpNoCase)
-		{
+		if (cmpNoCase) {
 			if (iter1->CmpNoCase(*iter2))
 				return false;
 		}
@@ -473,9 +455,6 @@ bool CServerPath::IsSubdirOf(const CServerPath &path, bool cmpNoCase) const
 
 bool CServerPath::IsParentOf(const CServerPath &path, bool cmpNoCase) const
 {
-	if (!this)
-		return false;
-
 	return path.IsSubdirOf(*this, cmpNoCase);
 }
 
@@ -486,18 +465,28 @@ bool CServerPath::ChangePath(wxString subdir)
 }
 
 bool CServerPath::ChangePath(wxString &subdir, bool isFile)
+{
+	bool ret = DoChangePath(subdir, isFile);
+	if (!ret) {
+		clear();
+	}
+
+	return ret;
+}
+
+bool CServerPath::DoChangePath(wxString &subdir, bool isFile)
 {
 	wxString dir = subdir;
 	wxString file;
 
-	if (dir == _T(""))
-	{
-		if (IsEmpty() || isFile)
+	if (dir.empty()) {
+		if (empty() || isFile)
 			return false;
 		else
 			return true;
 	}
 
+	bool const was_empty = empty();
 	CServerPathData& data = m_data.Get();
 
 	switch (m_type)
@@ -513,7 +502,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 				if (isFile)
 				{
-					if (IsEmpty())
+					if (was_empty)
 						return false;
 
 					file = dir;
@@ -535,7 +524,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				dir = dir.Left(pos2);
 
 				if (pos1)
-					data.m_prefix = dir.Left(pos1);
+					data.m_prefix = CSparseOptional<wxString>(dir.Left(pos1));
 				dir = dir.Mid(pos1 + 1);
 
 				data.m_segments.clear();
@@ -543,7 +532,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 			if (!Segmentize(dir, data.m_segments))
 				return false;
-			if (data.m_segments.empty() && m_bEmpty)
+			if (data.m_segments.empty() && was_empty)
 				return false;
 		}
 		break;
@@ -574,7 +563,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 			if (!Segmentize(dir, data.m_segments))
 				return false;
-			if (data.m_segments.empty() && m_bEmpty)
+			if (data.m_segments.empty() && was_empty)
 				return false;
 		}
 		break;
@@ -587,8 +576,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				c = dir.c_str()[++i];
 			dir.Remove(0, i);
 
-			while (dir != _T(""))
-			{
+			while (!dir.empty()) {
 				c = dir.Last();
 				if (c != FTP_MVS_DOUBLE_QUOTE)
 					break;
@@ -596,11 +584,10 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 					dir.RemoveLast();
 			}
 		}
-		if (dir == _T(""))
+		if (dir.empty())
 			return false;
 
-		if (dir.c_str()[0] == traits[m_type].left_enclosure)
-		{
+		if (dir.c_str()[0] == traits[m_type].left_enclosure) {
 			if (dir.Last() != traits[m_type].right_enclosure)
 				return false;
 
@@ -611,11 +598,10 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		}
 		else if (dir.Last() == traits[m_type].right_enclosure)
 			return false;
-		else if (m_bEmpty)
+		else if (was_empty)
 			return false;
 
-		if (dir.Last() == ')')
-		{
+		if (!dir.empty() && dir.Last() == ')') {
 			// Partitioned dataset member
 			if (!isFile)
 				return false;
@@ -627,27 +613,25 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 			file = dir.Mid(pos + 1);
 			dir = dir.Left(pos);
 
-			if (!m_bEmpty && data.m_prefix == _T("") && !dir.empty())
+			if (!was_empty && !data.m_prefix && !dir.empty())
 				return false;
 
 			data.m_prefix.clear();
 		}
-		else
-		{
-			if (!m_bEmpty && data.m_prefix == _T(""))
+		else {
+			if (!was_empty && !data.m_prefix)
 			{
 				if (dir.Find('.') != -1 || !isFile)
 					return false;
 			}
 
-			if (isFile)
-			{
+			if (isFile) {
 				if (!ExtractFile(dir, file))
 					return false;
-				data.m_prefix = _T(".");
+				data.m_prefix = CSparseOptional<wxString>(_T("."));
 			}
-			else if (dir.Last() == '.')
-				data.m_prefix = _T(".");
+			else if (!dir.empty() && dir.Last() == '.')
+				data.m_prefix = CSparseOptional<wxString>(_T("."));
 			else
 				data.m_prefix.clear();
 		}
@@ -664,7 +648,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 
 		if (!Segmentize(dir, data.m_segments))
 			return false;
-		if (data.m_segments.empty() && m_bEmpty)
+		if (data.m_segments.empty() && was_empty)
 			return false;
 
 		break;
@@ -672,7 +656,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		{
 			if (dir[0] != ':')
 			{
-				if (IsEmpty())
+				if (was_empty)
 					return false;
 			}
 			else
@@ -680,7 +664,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				int colon2;
 				if ((colon2 = dir.Mid(1).Find(':')) < 1)
 					return false;
-				data.m_prefix = dir.Left(colon2 + 2);
+				data.m_prefix = CSparseOptional<wxString>(dir.Left(colon2 + 2));
 				dir = dir.Mid(colon2 + 2);
 
 				data.m_segments.clear();
@@ -700,11 +684,11 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 				data.m_segments.clear();
 				data.m_prefix.clear();
 			}
-			else if (m_bEmpty)
+			else if (was_empty)
 				return false;
 			if (dir.Left(2) == _T("//"))
 			{
-				data.m_prefix = traits[m_type].separators[0];
+				data.m_prefix = CSparseOptional<wxString>(traits[m_type].separators[0]);
 				dir = dir.Mid(1);
 			}
 
@@ -719,7 +703,7 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		{
 			if (wxString(traits[m_type].separators).Contains(dir[0]))
 				data.m_segments.clear();
-			else if (m_bEmpty)
+			else if (was_empty)
 				return false;
 
 			if (isFile && !ExtractFile(dir, file))
@@ -744,13 +728,12 @@ bool CServerPath::ChangePath(wxString &subdir, bool isFile)
 		subdir = file;
 	}
 
-	m_bEmpty = false;
 	return true;
 }
 
 bool CServerPath::operator==(const CServerPath &op) const
 {
-	if (m_bEmpty != op.m_bEmpty)
+	if (empty() != op.empty())
 		return false;
 	else if (m_type != op.m_type)
 		return false;
@@ -762,27 +745,26 @@ bool CServerPath::operator==(const CServerPath &op) const
 
 bool CServerPath::operator!=(const CServerPath &op) const
 {
-	if (!this)
-		return false;
-
 	return !(*this == op);
 }
 
 bool CServerPath::operator<(const CServerPath &op) const
 {
-	if (m_bEmpty)
-	{
-		if (!op.m_bEmpty)
+	if (empty()) {
+		if (!op.empty())
 			return false;
 	}
-	else if (op.m_bEmpty)
+	else if (op.empty())
 		return true;
 
-	const int cmp = m_data->m_prefix.Cmp(op.m_data->m_prefix);
-	if (cmp < 0)
-		return true;
-	if (cmp > 0)
-		return false;
+	if( m_data->m_prefix || op.m_data->m_prefix ) {
+		if( m_data->m_prefix < op.m_data->m_prefix ) {
+			return true;
+		}
+		else if( op.m_data->m_prefix < m_data->m_prefix ) {
+			return false;
+		}
+	}
 
 	if (m_type > op.m_type)
 		return false;
@@ -790,8 +772,7 @@ bool CServerPath::operator<(const CServerPath &op) const
 		return true;
 
 	tConstSegmentIter iter1, iter2;
-	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2)
-	{
+	for (iter1 = m_data->m_segments.begin(), iter2 = op.m_data->m_segments.begin(); iter1 != m_data->m_segments.end(); ++iter1, ++iter2) {
 		if (iter2 == op.m_data->m_segments.end())
 			return false;
 
@@ -807,16 +788,13 @@ bool CServerPath::operator<(const CServerPath &op) const
 
 wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=false*/) const
 {
-	if (m_bEmpty)
+	if (empty())
 		return filename;
 
-	if (filename == _T(""))
-		return _T("");
+	if (filename.empty())
+		return wxString();
 
-	if (m_bEmpty)
-		return _T("");
-
-	if (omitPath && (!traits[m_type].prefixmode || m_data->m_prefix == _T(".")))
+	if (omitPath && (!traits[m_type].prefixmode || (m_data->m_prefix && *m_data->m_prefix == _T("."))))
 		return filename;
 
 	wxString result = GetPath();
@@ -826,19 +804,19 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 	switch (m_type)
 	{
 		case VXWORKS:
-			if (!wxString(traits[m_type].separators).Contains(result.Last()) && !m_data->m_segments.empty())
+			if (!result.empty() && !wxString(traits[m_type].separators).Contains(result.Last()) && !m_data->m_segments.empty())
 				result += traits[m_type].separators[0];
 			break;
 		case VMS:
 		case MVS:
 			break;
 		default:
-			if (!wxString(traits[m_type].separators).Contains(result.Last()))
+			if (!result.empty() && !wxString(traits[m_type].separators).Contains(result.Last()))
 				result += traits[m_type].separators[0];
 			break;
 	}
 
-	if (traits[m_type].prefixmode == 1 && m_data->m_prefix.empty())
+	if (traits[m_type].prefixmode == 1 && !m_data->m_prefix)
 		result += _T("(") + filename + _T(")");
 	else
 		result += filename;
@@ -851,7 +829,7 @@ wxString CServerPath::FormatFilename(const wxString &filename, bool omitPath /*=
 
 int CServerPath::CmpNoCase(const CServerPath &op) const
 {
-	if (m_bEmpty != op.m_bEmpty)
+	if (empty() != op.empty())
 		return 1;
 	else if (m_data->m_prefix != op.m_data->m_prefix)
 		return 1;
@@ -865,8 +843,7 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = op.m_data->m_segments.begin();
-	while (iter != m_data->m_segments.end())
-	{
+	while (iter != m_data->m_segments.end()) {
 		int res = iter++->CmpNoCase(*iter2++);
 		if (res)
 			return res;
@@ -877,7 +854,7 @@ int CServerPath::CmpNoCase(const CServerPath &op) const
 
 bool CServerPath::AddSegment(const wxString& segment)
 {
-	if (IsEmpty())
+	if (empty())
 		return false;
 
 	// TODO: Check for invalid characters
@@ -891,7 +868,7 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	if (*this == path)
 		return *this;
 
-	if (m_bEmpty || path.m_bEmpty)
+	if (empty() || path.empty())
 		return CServerPath();
 
 	if (m_type != path.m_type ||
@@ -916,18 +893,16 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 	}
 
 	CServerPath parent;
-	parent.m_bEmpty = false;
 	parent.m_type = m_type;
 
 	CServerPathData& parentData = parent.m_data.Get();
 
 	tConstSegmentIter last = m_data->m_segments.end();
 	tConstSegmentIter last2 = path.m_data->m_segments.end();
-	if (traits[m_type].prefixmode == 1)
-	{
-		if (m_data->m_prefix.empty())
+	if (traits[m_type].prefixmode == 1) {
+		if (!m_data->m_prefix)
 			--last;
-		if (path.m_data->m_prefix.empty())
+		if (!path.m_data->m_prefix)
 			--last2;
 		parentData.m_prefix = GetParent().m_data->m_prefix;
 	}
@@ -936,10 +911,8 @@ CServerPath CServerPath::GetCommonParent(const CServerPath& path) const
 
 	tConstSegmentIter iter = m_data->m_segments.begin();
 	tConstSegmentIter iter2 = path.m_data->m_segments.begin();
-	while (iter != last && iter2 != last2)
-	{
-		if (*iter != *iter2)
-		{
+	while (iter != last && iter2 != last2) {
+		if (*iter != *iter2) {
 			if (!traits[m_type].has_root && parentData.m_segments.empty())
 				return CServerPath();
 			else
@@ -969,36 +942,29 @@ wxString CServerPath::FormatSubdir(const wxString &subdir) const
 bool CServerPath::Segmentize(wxString str, tSegmentList& segments)
 {
 	bool append = false;
-	while (!str.empty())
-	{
+	while (!str.empty()) {
 		wxString segment;
 		int pos = str.find_first_of(traits[m_type].separators);
-		if (pos == -1)
-		{
+		if (pos == -1) {
 			segment = str,
 			str.clear();
 		}
-		else if (!pos)
-		{
+		else if (!pos) {
 			str = str.Mid(1);
 			continue;
 		}
-		else
-		{
+		else {
 			segment = str.Left(pos);
 			str = str.Mid(pos + 1);
 		}
 
-		if (traits[m_type].has_dots)
-		{
+		if (traits[m_type].has_dots) {
 			if (segment == _T("."))
 				continue;
-			else if (segment == _T(".."))
-			{
+			else if (segment == _T("..")) {
 				if (segments.empty())
 					return false;
-				else
-				{
+				else {
 					segments.pop_back();
 					continue;
 				}
@@ -1006,8 +972,7 @@ bool CServerPath::Segmentize(wxString str, tSegmentList& segments)
 		}
 
 		bool append_next = false;
-		if (traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape)
-		{
+		if (!segment.empty() && traits[m_type].separatorEscape && segment.Last() == traits[m_type].separatorEscape) {
 			append_next = true;
 			segment.RemoveLast();
 			segment += traits[m_type].separators[0];
@@ -1047,17 +1012,8 @@ bool CServerPath::ExtractFile(wxString& dir, wxString& file)
 
 void CServerPath::EscapeSeparators(ServerType type, wxString& subdir)
 {
-	if (traits[type].separatorEscape)
-	{
+	if (traits[type].separatorEscape) {
 		for (const wxChar* p = traits[type].separators; *p; ++p)
 			subdir.Replace((wxString)*p, (wxString)traits[type].separatorEscape + traits[type].separators[0]);
 	}
 }
-
-void CServerPath::Coalesce()
-{
-	CServerPathData& data = m_data.Get();
-	::Coalesce(data.m_prefix);
-	for (tSegmentIter iter = data.m_segments.begin(); iter != data.m_segments.end(); ++iter)
-		::Coalesce(*iter);
-}
