@@ -4,33 +4,33 @@
 #include "ControlSocket.h"
 #include <wx/process.h>
 
-typedef enum
-{
-	sftpUnknown = -1,
-	sftpReply = 0,
-	sftpDone,
-	sftpError,
-	sftpVerbose,
-	sftpStatus,
-	sftpRecv,
-	sftpSend,
-	sftpClose,
-	sftpRequest,
-	sftpListentry,
-	sftpRead,
-	sftpWrite,
-	sftpRequestPreamble,
-	sftpRequestInstruction,
-	sftpUsedQuotaRecv,
-	sftpUsedQuotaSend,
-	sftpKexAlgorithm,
-	sftpKexHash,
-	sftpCipherClientToServer,
-	sftpCipherServerToClient,
-	sftpMacClientToServer,
-	sftpMacServerToClient,
-	sftpHostkey
-} sftpEventTypes;
+enum class sftpEvent {
+	Unknown = -1,
+	Reply = 0,
+	Done,
+	Error,
+	Verbose,
+	Status,
+	Recv,
+	Send,
+	Close,
+	Request,
+	Listentry,
+	Transfer,
+	RequestPreamble,
+	RequestInstruction,
+	UsedQuotaRecv,
+	UsedQuotaSend,
+	KexAlgorithm,
+	KexHash,
+	CipherClientToServer,
+	CipherServerToClient,
+	MacClientToServer,
+	MacServerToClient,
+	Hostkey,
+
+	max = Hostkey
+};
 
 enum sftpRequestTypes
 {
@@ -40,11 +40,13 @@ enum sftpRequestTypes
 	sftpReqUnknown
 };
 
+class CProcess;
 class CSftpInputThread;
+
 class CSftpControlSocket : public CControlSocket, public CRateLimiterObject
 {
 public:
-	CSftpControlSocket(CFileZillaEnginePrivate* pEngine);
+	CSftpControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CSftpControlSocket();
 
 	virtual int Connect(const CServer &server);
@@ -61,8 +63,6 @@ public:
 
 	virtual bool SetAsyncRequestReply(CAsyncRequestNotification *pNotification);
 
-	void SetActive(bool recv);
-
 protected:
 	// Replaces filename"with"quotes with
 	// "filename""with""quotes"
@@ -113,28 +113,21 @@ protected:
 	int RenameSubcommandResult(int prevResult);
 	int RenameSend();
 
-	bool Send(wxString cmd, const wxString& show = _T(""));
+	bool SendCommand(wxString const& cmd, const wxString& show = wxString());
 	bool AddToStream(const wxString& cmd, bool force_utf8 = false);
 
-	virtual void OnRateAvailable(enum CRateLimiter::rate_direction direction);
-	void OnQuotaRequest(enum CRateLimiter::rate_direction direction);
+	virtual void OnRateAvailable(CRateLimiter::rate_direction direction);
+	void OnQuotaRequest(CRateLimiter::rate_direction direction);
 
 	// see src/putty/wildcard.c
 	wxString WildcardEscape(const wxString& file);
 
-	wxProcess* m_pProcess;
-	CSftpInputThread* m_pInputThread;
-	int m_pid;
+	CProcess* m_pProcess{};
+	CSftpInputThread* m_pInputThread{};
 
-	DECLARE_EVENT_TABLE()
-	void OnTerminate(wxProcessEvent& event);
-	void OnSftpEvent(wxCommandEvent& event);
-
-	// Set to true in destructor of CSftpControlSocket
-	// If sftp process dies (or gets killed), OnTerminate will be called.
-	// To avoid a race condition, abort OnTerminate if m_inDestructor is set
-	bool m_inDestructor;
-	bool m_termindatedInDestructor;
+	virtual void operator()(CEventBase const& ev);
+	void OnSftpEvent();
+	void OnTerminate();
 
 	wxString m_requestPreamble;
 	wxString m_requestInstruction;
