@@ -5,24 +5,23 @@
 
 // Command IDs
 // -----------
-enum Command
+enum class Command
 {
-	cmd_none = 0,
-	cmd_connect,
-	cmd_disconnect,
-	cmd_cancel,
-	cmd_list,
-	cmd_transfer,
-	cmd_delete,
-	cmd_removedir,
-	cmd_mkdir,
-	cmd_rename,
-	cmd_chmod,
-	cmd_raw,
+	none = 0,
+	connect,
+	disconnect,
+	list,
+	transfer,
+	del,
+	removedir,
+	mkdir,
+	rename,
+	chmod,
+	raw,
 
 	// Only used internally
-	cmd_cwd,
-	cmd_rawtransfer
+	cwd,
+	rawtransfer
 };
 
 // Reply codes
@@ -47,16 +46,6 @@ enum Command
 #define FZ_REPLY_WRITEFAILED	(0x2000 | FZ_REPLY_ERROR) // Happens if local file could not be written during transfer
 #define FZ_REPLY_LINKNOTDIR		(0x4000)
 
-// Small macro to simplify command class declaration
-// Basically all this macro does, is to declare the class and add the required
-// standard functions to it.
-#define DECLARE_COMMAND(name, id) \
-	class name : public CCommand \
-	{ \
-	public: \
-		virtual enum Command GetId() const { return id; } \
-		virtual CCommand* Clone() const { return new name(*this); }
-
 // --------------- //
 // Actual commands //
 // --------------- //
@@ -64,33 +53,60 @@ enum Command
 class CCommand
 {
 public:
-	CCommand() {}
-	virtual ~CCommand() {}
-	virtual enum Command GetId() const = 0;
+	CCommand() = default;
+	virtual ~CCommand() {}; // TODO: One GCC >= 4.8 is in Debian Stable (Jessie by then), make default and add testcase to configure.
+
+	virtual Command GetId() const = 0;
 	virtual CCommand *Clone() const = 0;
+
+	virtual bool valid() const { return true; }
+
+protected:
+	CCommand(CCommand const&) = default;
+	CCommand& operator=(CCommand const&) = default;
 };
 
-DECLARE_COMMAND(CConnectCommand, cmd_connect)
-	CConnectCommand(const CServer &server, bool retry_conncting = true);
+template<typename Derived, Command id>
+class CCommandHelper : public CCommand
+{
+public:
+	virtual Command GetId() const final { return id; }
 
-	const CServer GetServer() const;
+	virtual CCommand* Clone() const final {
+		return new Derived(static_cast<Derived const&>(*this));
+	}
+
+protected:
+	CCommandHelper<Derived, id>() = default;
+	CCommandHelper<Derived, id>(CCommandHelper<Derived, id> const&) = default;
+	CCommandHelper<Derived, id>& operator=(CCommandHelper<Derived, id> const&) = default;
+};
+
+template<Command id>
+class CBasicCommand final : public CCommandHelper<CBasicCommand<id>, id>
+{
+};
+
+class CConnectCommand final : public CCommandHelper<CConnectCommand, Command::connect>
+{
+public:
+	explicit CConnectCommand(CServer const& server, bool retry_conncting = true);
+
+	CServer const& GetServer() const;
 	bool RetryConnecting() const { return m_retry_connecting; }
 protected:
-	CServer m_Server;
-	bool m_retry_connecting;
+	CServer const m_Server;
+	bool const m_retry_connecting;
 };
 
-DECLARE_COMMAND(CDisconnectCommand, cmd_disconnect)
-};
-
-DECLARE_COMMAND(CCancelCommand, cmd_cancel)
-};
+typedef CBasicCommand<Command::disconnect> CDisconnectCommand;
 
 #define LIST_FLAG_REFRESH 1
 #define LIST_FLAG_AVOID 2
 #define LIST_FLAG_FALLBACK_CURRENT 4
 #define LIST_FLAG_LINK 8
-DECLARE_COMMAND(CListCommand, cmd_list)
+class CListCommand final : public CCommandHelper<CListCommand, Command::list>
+{
 	// Without a given directory, the current directory will be listed.
 	// Directories can either be given as absolute path or as
 	// pair of an absolute path and the very last path segments.
@@ -108,23 +124,27 @@ DECLARE_COMMAND(CListCommand, cmd_list)
 	// LIST_FLAG_LINK is used for symlink discovery. There's unfortunately
 	// no sane way to distinguish between symlinks to files and symlinks to
 	// directories.
-	CListCommand(int flags = 0);
-	CListCommand(CServerPath path, wxString subDir = _T(""), int flags = 0);
+public:
+	explicit CListCommand(int flags = 0);
+	explicit CListCommand(CServerPath path, wxString subDir = wxString(), int flags = 0);
 
 	CServerPath GetPath() const;
 	wxString GetSubDir() const;
 
 	int GetFlags() const { return m_flags; }
 
+	bool valid() const;
+
 protected:
-	CServerPath m_path;
-	wxString m_subDir;
-	int m_flags;
+	CServerPath const m_path;
+	wxString const m_subDir;
+	int const m_flags;
 };
 
-DECLARE_COMMAND(CFileTransferCommand, cmd_transfer)
-
-	class t_transferSettings
+class CFileTransferCommand final : public CCommandHelper<CFileTransferCommand, Command::transfer>
+{
+public:
+	class t_transferSettings final
 	{
 	public:
 		t_transferSettings()
@@ -138,7 +158,7 @@ DECLARE_COMMAND(CFileTransferCommand, cmd_transfer)
 	// For downloads, localFile can be left empty if supported by protocol.
 	// Check for nId_data notification.
 	// FIXME: localFile empty iff protocol is HTTP.
-	CFileTransferCommand(const wxString &localFile, const CServerPath& remotePath, const wxString &remoteFile, bool download, const t_transferSettings& m_transferSettings);
+	CFileTransferCommand(wxString const& localFile, CServerPath const& remotePath, wxString const& remoteFile, bool download, t_transferSettings const& m_transferSettings);
 
 	wxString GetLocalFile() const;
 	CServerPath GetRemotePath() const;
@@ -147,75 +167,93 @@ DECLARE_COMMAND(CFileTransferCommand, cmd_transfer)
 	const t_transferSettings& GetTransferSettings() const { return m_transferSettings; }
 
 protected:
-	wxString m_localFile;
-	CServerPath m_remotePath;
-	wxString m_remoteFile;
-	bool m_download;
-	t_transferSettings m_transferSettings;
+	wxString const m_localFile;
+	CServerPath const m_remotePath;
+	wxString const m_remoteFile;
+	bool const m_download;
+	t_transferSettings const m_transferSettings;
 };
 
-DECLARE_COMMAND(CRawCommand, cmd_raw)
-	CRawCommand(const wxString &command);
+class CRawCommand final : public CCommandHelper<CRawCommand, Command::raw>
+{
+public:
+	explicit CRawCommand(const wxString &command);
 
 	wxString GetCommand() const;
 
+	bool valid() const { return !m_command.empty(); }
+
 protected:
 	wxString m_command;
 };
 
-DECLARE_COMMAND(CDeleteCommand, cmd_delete)
-	CDeleteCommand(const CServerPath& path, const std::list<wxString>& files);
+class CDeleteCommand final : public CCommandHelper<CDeleteCommand, Command::del>
+{
+public:
+	CDeleteCommand(CServerPath const& path, std::list<wxString> const& files);
 
 	CServerPath GetPath() const { return m_path; }
 	const std::list<wxString>& GetFiles() const { return m_files; }
 
+	bool valid() const { return !GetPath().empty() && !GetFiles().empty(); }
 protected:
-
-	const CServerPath m_path;
-	const std::list<wxString> m_files;
+	CServerPath const m_path;
+	std::list<wxString> const m_files;
 };
 
-DECLARE_COMMAND(CRemoveDirCommand, cmd_removedir)
+class CRemoveDirCommand final : public CCommandHelper<CRemoveDirCommand, Command::removedir>
+{
+public:
 	// Directories can either be given as absolute path or as
 	// pair of an absolute path and the very last path segments.
-	CRemoveDirCommand(const CServerPath& path, const wxString& subdDir);
+	CRemoveDirCommand(CServerPath const& path, wxString const& subdDir);
 
 	CServerPath GetPath() const { return m_path; }
 	wxString GetSubDir() const { return m_subDir; }
 
-protected:
+	bool valid() const;
 
-	CServerPath m_path;
-	wxString m_subDir;
+protected:
+	CServerPath const m_path;
+	wxString const m_subDir;
 };
 
-DECLARE_COMMAND(CMkdirCommand, cmd_mkdir)
-	CMkdirCommand(const CServerPath& path);
+class CMkdirCommand final : public CCommandHelper<CMkdirCommand, Command::mkdir>
+{
+public:
+	explicit CMkdirCommand(CServerPath const& path);
 
 	CServerPath GetPath() const { return m_path; }
 
-protected:
+	bool valid() const;
 
-	CServerPath m_path;
+protected:
+	CServerPath const m_path;
 };
 
-DECLARE_COMMAND(CRenameCommand, cmd_rename)
-	CRenameCommand(const CServerPath& fromPath, const wxString& fromFile,
-				   const CServerPath& toPath, const wxString& toFile);
+class CRenameCommand final : public CCommandHelper<CRenameCommand, Command::rename>
+{
+public:
+	CRenameCommand(CServerPath const& fromPath, wxString const& fromFile,
+				   CServerPath const& toPath, wxString const& toFile);
 
 	CServerPath GetFromPath() const { return m_fromPath; }
 	CServerPath GetToPath() const { return m_toPath; }
 	wxString GetFromFile() const { return m_fromFile; }
 	wxString GetToFile() const { return m_toFile; }
 
+	bool valid() const;
+
 protected:
-	CServerPath m_fromPath;
-	CServerPath m_toPath;
-	wxString m_fromFile;
-	wxString m_toFile;
+	CServerPath const m_fromPath;
+	CServerPath const m_toPath;
+	wxString const m_fromFile;
+	wxString const m_toFile;
 };
 
-DECLARE_COMMAND(CChmodCommand, cmd_chmod)
+class CChmodCommand final : public CCommandHelper<CChmodCommand, Command::chmod>
+{
+public:
 	// The permission string should be given in a format understandable by the server.
 	// Most likely it's the defaut octal representation used by the unix chmod command,
 	// i.e. chmod 755 foo.bar
@@ -225,10 +263,12 @@ DECLARE_COMMAND(CChmodCommand, cmd_chmod)
 	wxString GetFile() const { return m_file; }
 	wxString GetPermission() const { return m_permission; }
 
+	bool valid() const;
+
 protected:
-	CServerPath m_path;
-	wxString m_file;
-	wxString m_permission;
+	CServerPath const m_path;
+	wxString const m_file;
+	wxString const m_permission;
 };
 
 #endif
