@@ -3,6 +3,8 @@
 
 #ifdef HAVE_CONFIG_H
   #include <config.h>
+#else
+#define HAVE_MAP_EMPLACE 1
 #endif
 
 #ifndef PACKAGE_STRING
@@ -15,12 +17,19 @@
 #include "setup.h"
 
 #ifdef __WXMSW__
+#ifndef NOMINMAX
+#define NOMINMAX
+#endif
+#ifndef STRICT
+#define STRICT 1
+#endif
 #include <windows.h>
 #endif
 
 #include <wx/datetime.h>
 #include <wx/event.h>
 #include <wx/string.h>
+#include <wx/translation.h>
 
 #include <list>
 #include <vector>
@@ -37,4 +46,6 @@
 
 #include "misc.h"
 
+#define TRANSLATE_T(str) _T(str)
+
 #endif //__LIBFILEZILLA_H__
