@@ -9,16 +9,16 @@
 
 enum engineOptions
 {
-	OPTION_USEPASV,			// Use passive mode unless overridden by 
+	OPTION_USEPASV,			// Use passive mode unless overridden by
 							// server settings
 	OPTION_LIMITPORTS,
 	OPTION_LIMITPORTS_LOW,
 	OPTION_LIMITPORTS_HIGH,
 	OPTION_EXTERNALIPMODE,		/* External IP Address mode for use in active mode
 								   Values: 0: ask operating system
-								           1: use provided IP
+										   1: use provided IP
 										   2: use provided resolver
-							    */
+								*/
 	OPTION_EXTERNALIP,
 	OPTION_EXTERNALIPRESOLVER,
 	OPTION_LASTRESOLVEDIP,
@@ -36,10 +36,13 @@ enum engineOptions
 	OPTION_RECONNECTCOUNT,
 	OPTION_RECONNECTDELAY,
 
+	OPTION_SPEEDLIMIT_ENABLE,
 	OPTION_SPEEDLIMIT_INBOUND,
 	OPTION_SPEEDLIMIT_OUTBOUND,
 	OPTION_SPEEDLIMIT_BURSTTOLERANCE,
 
+	OPTION_PREALLOCATE_SPACE,
+
 	OPTION_VIEW_HIDDEN_FILES,
 
 	OPTION_PRESERVE_TIMESTAMPS,
@@ -65,6 +68,11 @@ enum engineOptions
 
 	OPTION_LOGGING_FILE,
 	OPTION_LOGGING_FILE_SIZELIMIT,
+	OPTION_LOGGING_SHOW_DETAILED_LOGS,
+
+	OPTION_SIZE_FORMAT,
+	OPTION_SIZE_USETHOUSANDSEP,
+	OPTION_SIZE_DECIMALPLACES,
 
 	OPTIONS_ENGINE_NUM
 };
@@ -77,7 +85,7 @@ public:
 	virtual wxString GetOption(unsigned int nID) = 0;
 
 	virtual bool SetOption(unsigned int nID, int value) = 0;
-	virtual bool SetOption(unsigned int nID, wxString value) = 0;
+	virtual bool SetOption(unsigned int nID, wxString const& value) = 0;
 };
 
 #endif
