@@ -1,37 +1,37 @@
 #ifndef __SERVERPATH_H__
 #define __SERVERPATH_H__
 
+#include "optional.h"
 #include "refcount.h"
 
-class CServerPathData
+#include <deque>
+
+class CServerPathData final
 {
 public:
-	CServerPathData();
-	CServerPathData(const CServerPathData& v);
-	std::list<wxString> m_segments;
-	wxString m_prefix;
+	std::deque<wxString> m_segments;
+	CSparseOptional<wxString> m_prefix;
 
 	bool operator==(const CServerPathData& cmp) const;
 };
 
-class CServerPath
+class CServerPath final
 {
 public:
 	CServerPath();
-	CServerPath(wxString path, ServerType type = DEFAULT);
+	explicit CServerPath(wxString const& path, ServerType type = DEFAULT);
 	CServerPath(const CServerPath &path, wxString subdir); // Ignores parent on absolute subdir
 	CServerPath(const CServerPath &path);
-	virtual ~CServerPath();
 
-	bool IsEmpty() const { return m_bEmpty; };
-	void Clear();
+	bool empty() const { return !m_data; }
+	void clear();
 
 	bool SetPath(wxString newPath);
 	bool SetPath(wxString &newPath, bool isFile);
 	bool SetSafePath(const wxString& path);
 
-	// If ChangePath returns false, the object will be left an
-	// undefined state
+	// If ChangePath returns false, the object will be left
+	// empty.
 	bool ChangePath(wxString subdir);
 	bool ChangePath(wxString &subdir, bool isFile);
 
@@ -44,8 +44,8 @@ public:
 
 	CServerPath GetCommonParent(const CServerPath& path) const;
 
-	bool SetType(enum ServerType type);
-	enum ServerType GetType() const;
+	bool SetType(ServerType type);
+	ServerType GetType() const;
 
 	bool IsSubdirOf(const CServerPath &path, bool cmpNoCase) const;
 	bool IsParentOf(const CServerPath &path, bool cmpNoCase) const;
@@ -66,10 +66,12 @@ public:
 	bool AddSegment(const wxString& segment);
 
 protected:
-	bool m_bEmpty;
+	bool DoSetSafePath(const wxString& path);
+	bool DoChangePath(wxString &subdir, bool isFile);
+
 	ServerType m_type;
 
-	typedef std::list<wxString> tSegmentList;
+	typedef std::deque<wxString> tSegmentList;
 	typedef tSegmentList::iterator tSegmentIter;
 	typedef tSegmentList::const_iterator tConstSegmentIter;
 
@@ -78,7 +80,7 @@ protected:
 
 	static void EscapeSeparators(ServerType type, wxString& subdir);
 
-	CRefcountObject<CServerPathData> m_data;
+	CRefcountObject_Uninitialized<CServerPathData> m_data;
 };
 
 #endif
