@@ -10,11 +10,13 @@
 #ifdef __WXMSW__
 #include <wx/msw/registry.h> // Needed by CheckForWin2003FirewallBug
 #endif
-#include "xmlfunctions.h"
 #include <wx/tokenzr.h>
 #include "cmdline.h"
 #include "welcome_dialog.h"
+#include <msgbox.h>
+#include "local_filesys.h"
 
+#include <wx/xrc/xh_animatctrl.h>
 #include <wx/xrc/xh_bmpbt.h>
 #include <wx/xrc/xh_bttn.h>
 #include <wx/xrc/xh_chckb.h>
@@ -35,11 +37,10 @@
 #include <wx/xrc/xh_stbox.h>
 #include <wx/xrc/xh_stlin.h>
 #include <wx/xrc/xh_sttxt.h>
-#include <wx/xrc/xh_text.h>
+#include "xh_text_ex.h"
 #include <wx/xrc/xh_tree.h>
 #include <wx/xrc/xh_hyperlink.h>
 #include "xh_toolb_ex.h"
-#include "xh_menu_ex.h"
 #ifdef __WXMSW__
 #include <wx/socket.h>
 #include <wx/dynlib.h>
@@ -52,37 +53,27 @@
 #include <wx/stdpaths.h>
 #endif
 
-#ifdef __WXGTK__
 #include "locale_initializer.h"
-#endif
 
 #ifdef ENABLE_BINRELOC
 	#define BR_PTHREADS 0
 	#include "prefix.h"
 #endif
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
 #ifndef __WXGTK__
-IMPLEMENT_APP(CFileZillaApp);
+IMPLEMENT_APP(CFileZillaApp)
 #else
-IMPLEMENT_APP_NO_MAIN(CFileZillaApp);
+IMPLEMENT_APP_NO_MAIN(CFileZillaApp)
 #endif //__WXGTK__
 
 CFileZillaApp::CFileZillaApp()
 {
-	m_pWrapEngine = 0;
-	m_pLocale = 0;
-	m_pCommandLine = 0;
+	m_profilingActive = true;
+	AddStartupProfileRecord(_T("CFileZillaApp::CFileZillaApp()"));
 }
 
 CFileZillaApp::~CFileZillaApp()
 {
-	delete m_pLocale;
-	delete m_pWrapEngine;
-	delete m_pCommandLine;
 	COptions::Destroy();
 }
 
@@ -90,26 +81,23 @@ CFileZillaApp::~CFileZillaApp()
 bool IsServiceRunning(const wxString& serviceName)
 {
 	SC_HANDLE hScm = OpenSCManager(0, 0, GENERIC_READ);
-	if (!hScm)
-	{
-		//wxMessageBox(_T("OpenSCManager failed"));
+	if (!hScm) {
+		//wxMessageBoxEx(_T("OpenSCManager failed"));
 		return false;
 	}
 
 	SC_HANDLE hService = OpenService(hScm, serviceName, GENERIC_READ);
-	if (!hService)
-	{
+	if (!hService) {
 		CloseServiceHandle(hScm);
-		//wxMessageBox(_T("OpenService failed"));
+		//wxMessageBoxEx(_T("OpenService failed"));
 		return false;
 	}
 
 	SERVICE_STATUS status;
-	if (!ControlService(hService, SERVICE_CONTROL_INTERROGATE, &status))
-	{
+	if (!ControlService(hService, SERVICE_CONTROL_INTERROGATE, &status)) {
 		CloseServiceHandle(hService);
 		CloseServiceHandle(hScm);
-		//wxMessageBox(_T("ControlService failed"));
+		//wxMessageBoxEx(_T("ControlService failed"));
 		return false;
 	}
 
@@ -138,7 +126,7 @@ bool CheckForWin2003FirewallBug()
 	if (!key.Exists() || !key.Open(wxRegKey::Read))
 		return false;
 
-	long value;
+	long value = 0;
 	if (!key.HasValue(_T("EnableFirewall")) || !key.QueryValue(_T("EnableFirewall"), &value))
 		return false;
 
@@ -148,7 +136,7 @@ bool CheckForWin2003FirewallBug()
 	return true;
 }
 
-extern "C" 
+extern "C"
 {
 	typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
 }
@@ -173,8 +161,66 @@ static void SetAppId()
 
 #endif //__WXMSW__
 
+void CFileZillaApp::InitLocale()
+{
+	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
+	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
+	if (!language.empty())
+	{
+#ifdef __WXGTK__
+		if (CInitializer::error) {
+			wxString error;
+
+			wxLocale *loc = wxGetLocale();
+			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
+			if (!loc || !currentInfo) {
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language."),
+						language);
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language."),
+						pInfo->Description, language);
+			}
+			else {
+				wxString currentName = currentInfo->CanonicalName;
+
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
+						language, loc->GetLocale(),
+						currentName);
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
+						pInfo->Description, language, loc->GetLocale(),
+						currentName);
+			}
+
+			error += _T("\n");
+			error += _("Please make sure the requested locale is installed on your system.");
+			wxMessageBoxEx(error, _("Failed to change language"), wxICON_EXCLAMATION);
+
+			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
+		}
+#else
+		if (!pInfo || !SetLocale(pInfo->Language)) {
+			for( language = GetFallbackLocale(language); !language.empty(); language = GetFallbackLocale(language) ) {
+				const wxLanguageInfo* fallbackInfo = wxLocale::FindLanguageInfo(language);
+				if( fallbackInfo && SetLocale(fallbackInfo->Language )) {
+					return;
+				}
+			}
+			if (pInfo && !pInfo->Description.empty())
+				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description, language), _("Failed to change language"), wxICON_EXCLAMATION);
+			else
+				wxMessageBoxEx(wxString::Format(_("Failed to set language to %s, using default system language"), language), _("Failed to change language"), wxICON_EXCLAMATION);
+		}
+#endif
+	}
+}
+
 bool CFileZillaApp::OnInit()
 {
+	AddStartupProfileRecord(_T("CFileZillaApp::OnInit()"));
+
 	srand( (unsigned)time( NULL ) );
 
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
@@ -196,13 +242,12 @@ bool CFileZillaApp::OnInit()
 	wxSystemOptions::SetOption(_T("no-maskblt"), 0);
 	wxSystemOptions::SetOption(_T("msw.window.no-clip-children"), 0);
 	wxSystemOptions::SetOption(_T("msw.font.no-proof-quality"), 0);
-#endif
-
-#ifdef __WXMSW__
 	wxSystemOptions::SetOption(_T("msw.remap"), 0);
+	wxSystemOptions::SetOption(_T("msw.staticbox.optimized-paint"), 0);
 #endif
 #ifdef __WXMAC__
 	wxSystemOptions::SetOption(_T("mac.listctrl.always_use_generic"), 1);
+	wxSystemOptions::SetOption(_T("mac.textcontrol-use-spell-checker"), 0);
 #endif
 
 	int cmdline_result = ProcessCommandLine();
@@ -211,9 +256,10 @@ bool CFileZillaApp::OnInit()
 
 	LoadLocales();
 
-	if (cmdline_result < 0)
-	{
-		m_pCommandLine->DisplayUsage();
+	if (cmdline_result < 0) {
+		if (m_pCommandLine) {
+			m_pCommandLine->DisplayUsage();
+		}
 		return false;
 	}
 
@@ -221,71 +267,29 @@ bool CFileZillaApp::OnInit()
 
 	COptions::Init();
 
-	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
-	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
-	if (language != _T(""))
-	{
-#ifdef __WXGTK__
-		if (CInitializer::error)
-		{
-			wxString error;
-
-			wxLocale *loc = wxGetLocale();
-			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
-			if (!loc || !currentInfo)
-			{
-				if (!pInfo)
-					error.Printf(_("Failed to set language to %s, using default system language."),
-						language.c_str());
-				else
-					error.Printf(_("Failed to set language to %s (%s), using default system language."),
-						pInfo->Description.c_str(), language.c_str());
-			}
-			else
-			{
-				wxString currentName = currentInfo->CanonicalName;
-
-				if (!pInfo)
-					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
-						language.c_str(), loc->GetLocale(),
-						currentName.c_str());
-				else
-					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
-						pInfo->Description.c_str(), language.c_str(), loc->GetLocale(),
-						currentName.c_str());
-			}
-
-			error += _T("\n");
-			error += _("Please make sure the requested locale is installed on your system.");
-			wxMessageBox(error, _("Failed to change language"), wxICON_EXCLAMATION);
-
-			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
-		}
-#else
-		if (!pInfo || !SetLocale(pInfo->Language))
-		{
-			if (pInfo && pInfo->Description)
-				wxMessageBox(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description.c_str(), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
-			else
-				wxMessageBox(wxString::Format(_("Failed to set language to %s, using default system language"), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
-		}
-#endif
-	}
+	InitLocale();
 
 #ifndef _DEBUG
 	const wxString& buildType = CBuildInfo::GetBuildType();
-	if (buildType == _T("nightly"))
-        wxMessageBox(_T("You are using a nightly development version of FileZilla 3, do not expect anything to work.\r\nPlease use the official releases instead.\r\n\r\n\
+	if (buildType == _T("nightly")) {
+		wxMessageBoxEx(_T("You are using a nightly development version of FileZilla 3, do not expect anything to work.\r\nPlease use the official releases instead.\r\n\r\n\
 Unless explicitly instructed otherwise,\r\n\
 DO NOT post bugreports,\r\n\
 DO NOT use it in production environments,\r\n\
 DO NOT distribute the binaries,\r\n\
 DO NOT complain about it\r\n\
 USE AT OWN RISK"), _T("Important Information"));
+	}
+	else {
+		wxString v;
+		if (!wxGetEnv(_T("FZDEBUG"), &v) || v != _T("1")) {
+			COptions::Get()->SetOption(OPTION_LOGGING_DEBUGLEVEL, 0);
+			COptions::Get()->SetOption(OPTION_LOGGING_RAWLISTING, 0);
+		}
+	}
 #endif
 
-	if (!LoadResourceFiles())
-	{
+	if (!LoadResourceFiles()) {
 		COptions::Destroy();
 		return false;
 	}
@@ -293,8 +297,7 @@ USE AT OWN RISK"), _T("Important Information"));
 	CheckExistsFzsftp();
 
 #ifdef __WXMSW__
-	if (CheckForWin2003FirewallBug())
-	{
+	if (CheckForWin2003FirewallBug()) {
 		const wxString& error = _("Warning!\n\nA bug in Windows causes problems with FileZilla\n\n\
 The bug occurs if you have\n\
 - Windows Server 2003 or XP 64\n\
@@ -304,7 +307,7 @@ See http://support.microsoft.com/kb/931130 for background information.\n\n\
 Unless you either disable Windows Firewall or the Application Layer Gateway service,\n\
 FileZilla will timeout on big transfers.\
 ");
-		wxMessageBox(error, _("Operating system problem detected"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(error, _("Operating system problem detected"), wxICON_EXCLAMATION);
 	}
 #endif
 
@@ -318,7 +321,7 @@ FileZilla will timeout on big transfers.\
 #endif
 
 	// Load the text wrapping engine
-	m_pWrapEngine = new CWrapEngine();
+	m_pWrapEngine = make_unique<CWrapEngine>();
 	m_pWrapEngine->LoadCache();
 
 	CMainFrame *frame = new CMainFrame();
@@ -331,6 +334,8 @@ FileZilla will timeout on big transfers.\
 	frame->ProcessCommandLine();
 	frame->PostInitialize();
 
+	ShowStartupProfile();
+
 	return true;
 }
 
@@ -350,12 +355,12 @@ int CFileZillaApp::OnExit()
 bool CFileZillaApp::FileExists(const wxString& file) const
 {
 	int pos = file.Find('*');
-	if (pos == -1)
+	if (pos < 0)
 		return wxFileExists(file);
 
 	wxASSERT(pos > 0);
 	wxASSERT(file[pos - 1] == '/');
-	wxASSERT(file[pos + 1] == '/');
+	wxASSERT(file.size() > static_cast<size_t>(pos + 1) && file[pos + 1] == '/');
 
 	wxLogNull nullLog;
 	wxDir dir(file.Left(pos));
@@ -364,8 +369,7 @@ bool CFileZillaApp::FileExists(const wxString& file) const
 
 	wxString subDir;
 	bool found = dir.GetFirst(&subDir, _T(""), wxDIR_DIRS);
-	while (found)
-	{
+	while (found) {
 		if (FileExists(file.Left(pos) + subDir + file.Mid(pos + 1)))
 			return true;
 
@@ -375,7 +379,7 @@ bool CFileZillaApp::FileExists(const wxString& file) const
 	return false;
 }
 
-wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
+CLocalPath CFileZillaApp::GetDataDir(wxString fileToFind) const
 {
 	/*
 	 * Finding the resources in all cases is a difficult task,
@@ -389,11 +393,11 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	 */
 
 #ifdef __WXMAC__
-	wxString path = wxStandardPaths::Get().GetDataDir();
-	if (FileExists(path + fileToFind))
+	CLocalPath path(wxStandardPaths::Get().GetDataDir());
+	if (FileExists(path.GetPath() + fileToFind))
 		return path;
 
-	return _T("");
+	return CLocalPath();
 #else
 
 	wxPathList pathList;
@@ -408,29 +412,26 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 #ifdef ENABLE_BINRELOC
 	const char* path = SELFPATH;
-	if (path && *path)
-	{
+	if (path && *path) {
 		wxString datadir(SELFPATH , *wxConvCurrent);
 		wxFileName fn(datadir);
 		datadir = fn.GetPath();
-		if (datadir != _T(""))
+		if (!datadir.empty())
 			pathList.Add(datadir);
 
 	}
 	path = DATADIR;
-	if (path && *path)
-	{
+	if (path && *path) {
 		wxString datadir(DATADIR, *wxConvCurrent);
-		if (datadir != _T(""))
+		if (!datadir.empty())
 			pathList.Add(datadir);
 	}
 #elif defined __WXMSW__
 	wxChar path[1024];
 	int res = GetModuleFileName(0, path, 1000);
-	if (res > 0 && res < 1000)
-	{
+	if (res > 0 && res < 1000) {
 		wxFileName fn(path);
-		pathList.Add(fn.GetPath());
+		pathList.Add(fn.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
 	}
 #endif //ENABLE_BINRELOC and __WXMSW__ blocks
 
@@ -445,54 +446,48 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 	// For each path, check for the resources
 	wxPathList::const_iterator node;
-	for (node = pathList.begin(); node != pathList.end(); node++)
-	{
-		wxString cur = *node;
+	for (node = pathList.begin(); node != pathList.end(); ++node) {
+		wxString cur = CLocalPath(*node).GetPath();
 		if (FileExists(cur + fileToFind))
-			return cur;
-		if (FileExists(cur + _T("/share/filezilla") + fileToFind))
-			return cur + _T("/share/filezilla");
-		if (FileExists(cur + _T("/filezilla") + fileToFind))
-			return cur + _T("/filezilla");
+			return CLocalPath(cur);
+		if (FileExists(cur + _T("share/filezilla/") + fileToFind))
+			return CLocalPath(cur + _T("/share/filezilla"));
+		if (FileExists(cur + _T("filezilla/") + fileToFind))
+			return CLocalPath(cur + _T("filezilla"));
 	}
 
-	for (node = pathList.begin(); node != pathList.end(); node++)
-	{
-		wxString cur = *node;
-		if (FileExists(cur + _T("/..") + fileToFind))
-			return cur + _T("/..");
-		if (FileExists(cur + _T("/../share/filezilla") + fileToFind))
-			return cur + _T("/../share/filezilla");
+	for (node = pathList.begin(); node != pathList.end(); ++node) {
+		wxString cur = CLocalPath(*node).GetPath();
+		if (FileExists(cur + _T("../") + fileToFind))
+			return CLocalPath(cur + _T("/.."));
+		if (FileExists(cur + _T("../share/filezilla/") + fileToFind))
+			return CLocalPath(cur + _T("../share/filezilla"));
 	}
 
-	for (node = pathList.begin(); node != pathList.end(); node++)
-	{
-		wxString cur = *node;
-		if (FileExists(cur + _T("/../../") + fileToFind))
-			return cur + _T("/../..");
+	for (node = pathList.begin(); node != pathList.end(); ++node) {
+		wxString cur = CLocalPath(*node).GetPath();
+		if (FileExists(cur + _T("../../") + fileToFind))
+			return CLocalPath(cur + _T("../.."));
 	}
 
-	return _T("");
+	return CLocalPath();
 #endif //__WXMAC__
 }
 
 bool CFileZillaApp::LoadResourceFiles()
 {
-	m_resourceDir = GetDataDir(_T("/resources/menus.xrc"));
+	AddStartupProfileRecord(_T("CFileZillaApp::LoadResourceFiles"));
+	m_resourceDir = GetDataDir(_T("resources/theme.xml"));
 
 	wxImage::AddHandler(new wxPNGHandler());
 
-	if (m_resourceDir == _T(""))
-	{
+	if (m_resourceDir.empty()) {
 		wxString msg = _("Could not find the resource files for FileZilla, closing FileZilla.\nYou can set the data directory of FileZilla using the '--datadir <custompath>' commandline option or by setting the FZ_DATADIR environment variable.");
-		wxMessageBox(msg, _("FileZilla Error"), wxOK | wxICON_ERROR);
+		wxMessageBoxEx(msg, _("FileZilla Error"), wxOK | wxICON_ERROR);
 		return false;
 	}
 
-	if (m_resourceDir[m_resourceDir.Length() - 1] != wxFileName::GetPathSeparator())
-		m_resourceDir += wxFileName::GetPathSeparator();
-
-	m_resourceDir += _T("resources/");
+	m_resourceDir.AddSegment(_T("resources"));
 
 	wxXmlResource *pResource = wxXmlResource::Get();
 
@@ -501,7 +496,7 @@ bool CFileZillaApp::LoadResourceFiles()
 #endif
 
 	pResource->AddHandler(new wxMenuXmlHandler);
-	pResource->AddHandler(new wxMenuBarXmlHandlerEx);
+	pResource->AddHandler(new wxMenuBarXmlHandler);
 	pResource->AddHandler(new wxDialogXmlHandler);
 	pResource->AddHandler(new wxPanelXmlHandler);
 	pResource->AddHandler(new wxSizerXmlHandler);
@@ -519,75 +514,79 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->AddHandler(new wxSpinCtrlXmlHandler);
 	pResource->AddHandler(new wxRadioButtonXmlHandler);
 	pResource->AddHandler(new wxNotebookXmlHandler);
-	pResource->AddHandler(new wxTextCtrlXmlHandler);
+	pResource->AddHandler(new wxTextCtrlXmlHandlerEx);
 	pResource->AddHandler(new wxListBoxXmlHandler);
 	pResource->AddHandler(new wxToolBarXmlHandlerEx);
 	pResource->AddHandler(new wxStaticLineXmlHandler);
 	pResource->AddHandler(new wxScrolledWindowXmlHandler);
 	pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
+	pResource->AddHandler(new wxAnimationCtrlXmlHandler);
+	pResource->AddHandler(new wxStdDialogButtonSizerXmlHandler);
 
-	wxString resourceDir = m_resourceDir;
-#if wxUSE_FILESYSTEM
-	resourceDir.Replace(_T("%"), _T("%25"));
-	resourceDir.Replace(_T(":"), _T("%3A"));
-	resourceDir.Replace(_T("#"), _T("%23"));
-#endif
-	pResource->Load(resourceDir + _T("*.xrc"));
+	if (CLocalFileSystem::GetFileType(m_resourceDir.GetPath() + _T("xrc/resources.xrc")) == CLocalFileSystem::file) {
+		pResource->LoadFile(m_resourceDir.GetPath() + _T("xrc/resources.xrc"));
+	}
+	else {
+		CLocalFileSystem fs;
+		wxString dir = m_resourceDir.GetPath() + _T("xrc/");
+		bool found = fs.BeginFindFiles(dir, false);
+		while (found) {
+			wxString name;
+			found = fs.GetNextFile(name);
+			if (name.Right(4) != _T(".xrc")) {
+				continue;
+			}
+			pResource->LoadFile(dir + name);
+		}
+	}
 
 	return true;
 }
 
 bool CFileZillaApp::InitDefaultsDir()
 {
+	AddStartupProfileRecord(_T("InitDefaultsDir"));
 #ifdef __WXGTK__
-	wxFileName fn = wxFileName(wxGetHomeDir(), _T("fzdefaults.xml"));
-	fn.AppendDir(_T(".filezilla"));
-	if (fn.FileExists())
-		m_defaultsDir = fn.GetPath();
-	else if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml")))
-		m_defaultsDir = _T("/etc/filezilla");
-	else
-#endif
-	m_defaultsDir = GetDataDir(_T("/fzdefaults.xml"));
+	m_defaultsDir = COptions::GetUnadjustedSettingsDir();
+	if( m_defaultsDir.empty() || !wxFileName::FileExists(m_defaultsDir.GetPath() + _T("fzdefaults.xml"))) {
+		if (wxFileName::FileExists(_T("/etc/filezilla/fzdefaults.xml"))) {
+			m_defaultsDir.SetPath(_T("/etc/filezilla"));
+		}
+		else {
+			m_defaultsDir.clear();
+		}
+	}
 
-	return m_defaultsDir != _T("");
+#endif
+	if( m_defaultsDir.empty() ) {
+		m_defaultsDir = GetDataDir(_T("fzdefaults.xml"));
+	}
+
+	return !m_defaultsDir.empty();
 }
 
 bool CFileZillaApp::LoadLocales()
 {
+	AddStartupProfileRecord(_T("CFileZillaApp::LoadLocales"));
 #ifndef __WXMAC__
-	m_localesDir = GetDataDir(_T("/../locale/*/filezilla.mo"));
-	if (m_localesDir == _T(""))
-		m_localesDir = GetDataDir(_T("/../locale/*/LC_MESSAGES/filezilla.mo"));
-	if (m_localesDir != _T(""))
-	{
-		if (m_localesDir[m_localesDir.Length() - 1] != wxFileName::GetPathSeparator())
-			m_localesDir += wxFileName::GetPathSeparator();
-
-		m_localesDir += _T("../locale");
+	m_localesDir = GetDataDir(_T("../locale/*/filezilla.mo"));
+	if (m_localesDir.empty())
+		m_localesDir = GetDataDir(_T("../locale/*/LC_MESSAGES/filezilla.mo"));
+	if (!m_localesDir.empty()) {
+		m_localesDir.ChangePath( _T("../locale") );
 	}
-	else
-	{
-		m_localesDir = GetDataDir(_T("/locales/*/filezilla.mo"));
-		if (m_localesDir != _T(""))
-		{
-			if (m_localesDir[m_localesDir.Length() - 1] != wxFileName::GetPathSeparator())
-				m_localesDir += wxFileName::GetPathSeparator();
-
-			m_localesDir += _T("locales");
+	else {
+		m_localesDir = GetDataDir(_T("locales/*/filezilla.mo"));
+		if (!m_localesDir.empty()) {
+			m_localesDir.AddSegment(_T("locales"));
 		}
 	}
 #else
-	m_localesDir = wxStandardPaths::Get().GetDataDir() + _T("/locales");
+	m_localesDir.SetPath(wxStandardPaths::Get().GetDataDir() + _T("/locales"));
 #endif
 
-	if (m_localesDir != _T(""))
-	{
-		wxFileName fn(m_localesDir, _T(""));
-		fn.Normalize();
-		m_localesDir = fn.GetFullPath();
-
-		wxLocale::AddCatalogLookupPathPrefix(m_localesDir);
+	if (!m_localesDir.empty()) {
+		wxLocale::AddCatalogLookupPathPrefix(m_localesDir.GetPath());
 	}
 
 	SetLocale(wxLANGUAGE_DEFAULT);
@@ -598,30 +597,25 @@ bool CFileZillaApp::LoadLocales()
 bool CFileZillaApp::SetLocale(int language)
 {
 	// First check if we can load the new locale
-	wxLocale* pLocale = new wxLocale();
+	auto pLocale = make_unique<wxLocale>();
 	wxLogNull log;
 	pLocale->Init(language);
-	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla")))
-	{
-		delete pLocale;
+	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
 		return false;
 	}
 
 	// Now unload old locale
 	// We unload new locale as well, else the internal locale chain in wxWidgets get's broken.
-	delete pLocale;
-	delete m_pLocale;
-	m_pLocale = 0;
+	pLocale.reset();
+	m_pLocale.reset();
 
 	// Finally load new one
-	pLocale = new wxLocale();
+	pLocale = make_unique<wxLocale>();
 	pLocale->Init(language);
-	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla")))
-	{
-		delete pLocale;
+	if (!pLocale->IsOk() || !pLocale->AddCatalog(_T("filezilla"))) {
 		return false;
 	}
-	m_pLocale = pLocale;
+	m_pLocale = std::move(pLocale);
 
 	return true;
 }
@@ -637,38 +631,15 @@ int CFileZillaApp::GetCurrentLanguage() const
 wxString CFileZillaApp::GetCurrentLanguageCode() const
 {
 	if (!m_pLocale)
-		return _T("");
+		return wxString();
 
 	return m_pLocale->GetCanonicalName();
 }
 
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
-
 void CFileZillaApp::OnFatalException()
 {
-	//GenerateReport(wxDebugReport::Context_Exception);
 }
-
-void CFileZillaApp::GenerateReport(wxDebugReport::Context ctx)
-{
-	/*
-	wxDebugReport report;
-
-	// add all standard files: currently this means just a minidump and an
-	// XML file with system info and stack trace
-	report.AddAll(ctx);
-
-	// calling Show() is not mandatory, but is more polite
-	if ( wxDebugReportPreviewStd().Show(report) )
-	{
-		if ( report.Process() )
-		{
-			// report successfully uploaded
-		}
-	}*/
-	//else: user cancelled the report
-}
-
 #endif
 
 void CFileZillaApp::DisplayEncodingWarning()
@@ -679,16 +650,17 @@ void CFileZillaApp::DisplayEncodingWarning()
 
 	displayedEncodingWarning = true;
 
-	wxMessageBox(_("A local filename could not be decoded.\nPlease make sure the LC_CTYPE (or LC_ALL) environment variable is set correctly.\nUnless you fix this problem, files might be missing in the file listings.\nNo further warning will be displayed this session."), _("Character encoding issue"), wxICON_EXCLAMATION);
+	wxMessageBoxEx(_("A local filename could not be decoded.\nPlease make sure the LC_CTYPE (or LC_ALL) environment variable is set correctly.\nUnless you fix this problem, files might be missing in the file listings.\nNo further warning will be displayed this session."), _("Character encoding issue"), wxICON_EXCLAMATION);
 }
 
 CWrapEngine* CFileZillaApp::GetWrapEngine()
 {
-	return m_pWrapEngine;
+	return m_pWrapEngine.get();
 }
 
 void CFileZillaApp::CheckExistsFzsftp()
 {
+	AddStartupProfileRecord(_T("CFileZillaApp::CheckExistsFzstp"));
 	// Get the correct path to the fzsftp executable
 
 #ifdef __WXMAC__
@@ -699,7 +671,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 	executable += _T("/fzsftp");
 	if (!wxFileName::FileExists(executable))
 	{
-		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable.c_str()),
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable),
 			_("File not found"), wxICON_ERROR);
 		executable.clear();
 	}
@@ -715,14 +687,12 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 	// First check the FZ_FZSFTP environment variable
 	wxString executable;
-	if (wxGetEnv(_T("FZ_FZSFTP"), &executable))
-	{
+	if (wxGetEnv(_T("FZ_FZSFTP"), &executable)) {
 		if (wxFileName::FileExists(executable))
 			found = true;
 	}
 
-	if (!found)
-	{
+	if (!found) {
 		wxPathList pathList;
 
 		// Add current working directory
@@ -733,13 +703,11 @@ void CFileZillaApp::CheckExistsFzsftp()
 		// Add executable path
 		wxChar modulePath[1000];
 		DWORD len = GetModuleFileName(0, modulePath, 999);
-		if (len)
-		{
+		if (len) {
 			modulePath[len] = 0;
 			wxString path(modulePath);
 			int pos = path.Find('\\', true);
-			if (pos != -1)
-			{
+			if (pos != -1) {
 				path = path.Left(pos);
 				pathList.Add(path);
 			}
@@ -752,66 +720,56 @@ void CFileZillaApp::CheckExistsFzsftp()
 		pathList.Add(cwd + _T("/putty"));
 
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (executable != _T(""))
+		if (!executable.empty())
 			found = true;
 	}
 
-	if (!found)
-	{
 #ifdef __UNIX__
+	if (!found) {
 		const wxString prefix = ((const wxStandardPaths&)wxStandardPaths::Get()).GetInstallPrefix();
-		if (prefix != _T("/usr/local"))
-		{
+		if (prefix != _T("/usr/local")) {
 			// /usr/local is the fallback value. /usr/local/bin is most likely in the PATH
 			// environment variable already so we don't have to check it. Furthermore, other
 			// directories might be listed before it (For example a developer's own
 			// application prefix)
 			wxFileName fn(prefix + _T("/bin/"), program);
 			fn.Normalize();
-			if (fn.FileExists())
-			{
+			if (fn.FileExists()) {
 				executable = fn.GetFullPath();
 				found = true;
 			}
 		}
-#endif
 	}
+#endif
 
-	if (!found)
-	{
+	if (!found) {
 		// Check PATH
 		wxPathList pathList;
 		pathList.AddEnvList(_T("PATH"));
 		executable = pathList.FindAbsoluteValidPath(program);
-		if (executable != _T(""))
+		if (!executable.empty())
 			found = true;
 	}
 
-	// Quote path if it contains spaces
-	if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
-		executable = _T("\"") + executable + _T("\"");
+	if (!found) {
+		// Quote path if it contains spaces
+		if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
+			executable = _T("\"") + executable + _T("\"");
 
-	if (!found)
-	{
-		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program.c_str(), program.c_str(), program.c_str()),
-			_("File not found"), wxICON_ERROR);
+		wxMessageBoxEx(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program, program, program),
+			_("File not found"), wxICON_ERROR | wxOK);
 		executable.clear();
 	}
 #endif
 
-	// Quote path if it contains spaces
-	if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
-		executable = _T("\"") + executable + _T("\"");
-
 	COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
 }
 
 #ifdef __WXMSW__
-extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM lParam)
+extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM)
 {
 	HWND child = FindWindowEx(hwnd, 0, 0, _T("FileZilla process identificator 3919DB0A-082D-4560-8E2F-381A35969FB4"));
-	if (child)
-	{
+	if (child) {
 		::PostMessage(hwnd, WM_ENDSESSION, (WPARAM)TRUE, (LPARAM)ENDSESSION_LOGOFF);
 	}
 
@@ -821,22 +779,20 @@ extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM lParam)
 
 int CFileZillaApp::ProcessCommandLine()
 {
-	m_pCommandLine = new CCommandLine(argc, argv);
+	AddStartupProfileRecord(_T("CFileZillaApp::ProcessCommandLine"));
+	m_pCommandLine = make_unique<CCommandLine>(argc, argv);
 	int res = m_pCommandLine->Parse() ? 1 : -1;
 
-	if (res > 0)
-	{
-		if (m_pCommandLine->HasSwitch(CCommandLine::close))
-		{
+	if (res > 0) {
+		if (m_pCommandLine->HasSwitch(CCommandLine::close)) {
 #ifdef __WXMSW__
 			EnumWindows((WNDENUMPROC)EnumWindowCallback, 0);
 #endif
 			return 0;
 		}
 
-		if (m_pCommandLine->HasSwitch(CCommandLine::version))
-		{
-			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion().c_str());
+		if (m_pCommandLine->HasSwitch(CCommandLine::version)) {
+			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion());
 			if (!CBuildInfo::GetBuildType().empty())
 				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
 			out += _T(", compiled on ") + CBuildInfo::GetBuildDateString();
@@ -848,3 +804,37 @@ int CFileZillaApp::ProcessCommandLine()
 
 	return res;
 }
+
+void CFileZillaApp::AddStartupProfileRecord(const wxString& msg)
+{
+	if (!m_profilingActive)
+		return;
+
+	m_startupProfile.push_back(std::make_pair(wxDateTime::UNow(), msg));
+}
+
+void CFileZillaApp::ShowStartupProfile()
+{
+	m_profilingActive = false;
+
+	std::list<std::pair<wxDateTime, wxString> > profile;
+	profile.swap(m_startupProfile);
+
+	if (m_pCommandLine && !m_pCommandLine->HasSwitch(CCommandLine::debug_startup))
+		return;
+
+	wxString msg = _T("Profile:\n");
+	for (auto const& p : profile) {
+		msg += p.first.Format(_T("%Y-%m-%d %H:%M:%S %l"));
+		msg += _T(" ");
+		msg += p.second;
+		msg += _T("\n");
+	}
+
+	wxMessageBoxEx(msg);
+}
+
+wxString CFileZillaApp::GetSettingsFile(wxString const& name) const
+{
+	return COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR) + name + _T(".xml");
+}
