@@ -10,21 +10,21 @@ class CLocalListViewDropTarget;
 class CVolumeDescriptionEnumeratorThread;
 #endif
 
-class CLocalFileData : public CGenericFileData
+class CLocalFileData final : public CGenericFileData
 {
 public:
 	wxString name;
 #ifdef __WXMSW__
-	wxString label;
+	CSparseOptional<wxString> label;
 #endif
-	bool dir;
-	wxLongLong size;
-	bool hasTime;
-	wxDateTime lastModified;
+	CDateTime time;
+	int64_t size;
 	int attributes;
+	bool dir;
+	bool is_dir() const { return dir; }
 };
 
-class CLocalListView : public CFileListCtrl<CLocalFileData>, CSystemImageList, CStateEventHandler
+class CLocalListView : public CFileListCtrl<CLocalFileData>, CStateEventHandler
 {
 	friend class CLocalListViewDropTarget;
 	friend class CLocalListViewSortType;
@@ -34,8 +34,8 @@ public:
 	virtual ~CLocalListView();
 
 protected:
-	void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
-	bool DisplayDir(wxString dirname);
+	void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void*);
+	bool DisplayDir(CLocalPath const& dirname);
 	void ApplyCurrentFilter();
 
 	// Declared const due to design error in wxWidgets.
@@ -51,7 +51,7 @@ protected:
 	// Select a list of items based in their names.
 	// Sort order may not change between call to RememberSelectedItems and
 	// ReselectItems
-	void ReselectItems(const std::list<wxString>& selectedNames, wxString focused);
+	void ReselectItems(const std::list<wxString>& selectedNames, wxString focused, bool ensureVisible = false);
 
 #ifdef __WXMSW__
 	void DisplayDrives();
@@ -59,11 +59,9 @@ protected:
 #endif
 
 public:
-	void InitDateFormat();
-
 	virtual bool CanStartComparison(wxString* pError);
 	virtual void StartComparison();
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, wxDateTime& date, bool &hasTime);
+	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, CDateTime& date);
 	virtual void FinishComparison();
 
 	virtual bool ItemIsDir(int index) const;
@@ -79,24 +77,27 @@ protected:
 
 	void RefreshFile(const wxString& file);
 
-	wxString m_dir;
+	virtual void OnNavigationEvent(bool forward);
+
+	virtual bool OnBeginRename(const wxListEvent& event);
+	virtual bool OnAcceptRename(const wxListEvent& event);
+
+	CLocalPath m_dir;
 
-	wxDropTarget* m_pDropTarget;
 	int m_dropTarget;
 
-	wxString m_dateFormat;
+	wxString MenuMkdir();
 
 	// Event handlers
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent& event);
 	void OnContextMenu(wxContextMenuEvent& event);
 	void OnMenuUpload(wxCommandEvent& event);
 	void OnMenuMkdir(wxCommandEvent& event);
+	void OnMenuMkdirChgDir(wxCommandEvent&);
 	void OnMenuDelete(wxCommandEvent& event);
 	void OnMenuRename(wxCommandEvent& event);
 	void OnKeyDown(wxKeyEvent& event);
-	void OnBeginLabelEdit(wxListEvent& event);
-	void OnEndLabelEdit(wxListEvent& event);
 	void OnBeginDrag(wxListEvent& event);
 	void OnMenuOpen(wxCommandEvent& event);
 	void OnMenuEdit(wxCommandEvent& event);
@@ -105,6 +106,7 @@ protected:
 	void OnVolumesEnumerated(wxCommandEvent& event);
 	CVolumeDescriptionEnumeratorThread* m_pVolumeEnumeratorThread;
 #endif
+	void OnMenuRefresh(wxCommandEvent& event);
 };
 
 #endif
