@@ -3,15 +3,14 @@
 #include "queue.h"
 #include "filezillaapp.h"
 #include "filter.h"
+#include "file_utils.h"
 #include <wx/dnd.h>
 #include "dndobjects.h"
 #include "inputdialog.h"
 #include "local_filesys.h"
 #include "dragdropmanager.h"
-
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
+#include "drop_target_ex.h"
+#include "Options.h"
 
 #ifdef __WXMSW__
 #include <wx/msw/registry.h>
@@ -27,11 +26,12 @@ public:
 	wxString m_known_subdir;
 };
 
-class CLocalTreeViewDropTarget : public wxDropTarget
+class CLocalTreeViewDropTarget : public CScrollableDropTarget<wxTreeCtrlEx>
 {
 public:
 	CLocalTreeViewDropTarget(CLocalTreeView* pLocalTreeView)
-		: m_pLocalTreeView(pLocalTreeView), m_pFileDataObject(new wxFileDataObject()),
+		: CScrollableDropTarget(pLocalTreeView)
+		, m_pLocalTreeView(pLocalTreeView), m_pFileDataObject(new wxFileDataObject()),
 		m_pRemoteDataObject(new CRemoteDataObject())
 	{
 		m_pDataObject = new wxDataObjectComposite;
@@ -56,7 +56,7 @@ public:
 
 #ifdef __WXMSW__
 		if (dir == _T("/"))
-			return _T("");
+			return wxString();
 #endif
 
 		return dir;
@@ -79,6 +79,9 @@ public:
 			def == wxDragNone ||
 			def == wxDragCancel)
 			return def;
+		if( def == wxDragLink ) {
+			def = wxDragCopy;
+		}
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
 		if (!hit)
@@ -101,13 +104,13 @@ public:
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
 			{
-				wxMessageBox(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			if (!m_pLocalTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pLocalTreeView->m_pState->GetServer()))
 			{
-				wxMessageBox(_("Drag&drop between different servers has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
@@ -120,6 +123,10 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
+		if (!CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y)) {
+			return false;
+		}
+
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
@@ -127,27 +134,25 @@ public:
 			return false;
 
 		const wxString dir = GetDirFromItem(hit);
-		if (dir == _T("") || !CLocalPath(dir).IsWriteable())
+		if (dir.empty() || !CLocalPath(dir).IsWriteable())
 			return false;
 
 		return true;
 	}
 
-	wxString DisplayDropHighlight(wxPoint point)
+	wxTreeItemId DisplayDropHighlight(wxPoint point)
 	{
 		wxTreeItemId hit = GetHit(point);
-		if (!hit)
-		{
+		if (!hit) {
 			ClearDropHighlight();
-			return _T("");
+			return hit;
 		}
 
 		wxString dir = GetDirFromItem(hit);
 
-		if (dir == _T(""))
-		{
+		if (dir.empty()) {
 			ClearDropHighlight();
-			return _T("");
+			return wxTreeItemId();
 		}
 
 		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
@@ -158,11 +163,13 @@ public:
 		m_pLocalTreeView->m_dropHighlight = hit;
 
 
-		return dir;
+		return hit;
 	}
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnDragOver(x, y, def);
+
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -171,8 +178,8 @@ public:
 			return def;
 		}
 
-		const wxString& dir = DisplayDropHighlight(wxPoint(x, y));
-		if (dir == _T(""))
+		wxTreeItemId hit = DisplayDropHighlight(wxPoint(x, y));
+		if (!hit.IsOk())
 			return wxDragNone;
 
 		if (def == wxDragLink)
@@ -183,11 +190,13 @@ public:
 
 	virtual void OnLeave()
 	{
+		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -207,9 +216,7 @@ EVT_TREE_SEL_CHANGING(wxID_ANY, CLocalTreeView::OnSelectionChanging)
 #endif
 EVT_TREE_SEL_CHANGED(wxID_ANY, CLocalTreeView::OnSelectionChanged)
 EVT_TREE_BEGIN_DRAG(wxID_ANY, CLocalTreeView::OnBeginDrag)
-#ifndef __WXMSW__
-EVT_KEY_DOWN(CLocalTreeView::OnKeyDown)
-#else
+#ifdef __WXMSW__
 EVT_COMMAND(-1, fzEVT_VOLUMESENUMERATED, CLocalTreeView::OnVolumesEnumerated)
 EVT_COMMAND(-1, fzEVT_VOLUMEENUMERATED, CLocalTreeView::OnVolumesEnumerated)
 #endif //__WXMSW__
@@ -219,6 +226,7 @@ EVT_MENU(XRCID("ID_ADDTOQUEUE"), CLocalTreeView::OnMenuUpload)
 EVT_MENU(XRCID("ID_DELETE"), CLocalTreeView::OnMenuDelete)
 EVT_MENU(XRCID("ID_RENAME"), CLocalTreeView::OnMenuRename)
 EVT_MENU(XRCID("ID_MKDIR"), CLocalTreeView::OnMenuMkdir)
+EVT_MENU(XRCID("ID_MKDIR_CHGDIR"), CLocalTreeView::OnMenuMkdirChgDir)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CLocalTreeView::OnEndLabelEdit)
 EVT_CHAR(CLocalTreeView::OnChar)
@@ -231,6 +239,7 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 	CStateEventHandler(pState),
 	m_pQueueView(pQueueView)
 {
+	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::CLocalTreeView"));
 #ifdef __WXMAC__
 	SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
 #endif
@@ -240,16 +249,19 @@ CLocalTreeView::CLocalTreeView(wxWindow* parent, wxWindowID id, CState *pState,
 
 	SetImageList(GetSystemImageList());
 
+	UpdateSortMode();
+	RegisterOption(OPTION_FILELIST_NAMESORT);
+
 #ifdef __WXMSW__
 	m_pVolumeEnumeratorThread = 0;
 
 	CreateRoot();
 #else
 	wxTreeItemId root = AddRoot(_T("/"));
-	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Normal);
-	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_Selected);
-	SetItemImage(root, GetIconIndex(dir), wxTreeItemIcon_Expanded);
-	SetItemImage(root, GetIconIndex(opened_dir), wxTreeItemIcon_SelectedExpanded);
+	SetItemImage(root, GetIconIndex(iconType::dir), wxTreeItemIcon_Normal);
+	SetItemImage(root, GetIconIndex(iconType::opened_dir), wxTreeItemIcon_Selected);
+	SetItemImage(root, GetIconIndex(iconType::dir), wxTreeItemIcon_Expanded);
+	SetItemImage(root, GetIconIndex(iconType::opened_dir), wxTreeItemIcon_SelectedExpanded);
 
 	SetDir(_T("/"));
 #endif
@@ -268,7 +280,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 {
 	if (m_currentDir == localDir)
 	{
-		Refresh();
+		RefreshListing();
 		return;
 	}
 
@@ -297,7 +309,7 @@ void CLocalTreeView::SetDir(wxString localDir)
 		return;
 	}
 
-	if (subDirs == _T(""))
+	if (subDirs.empty())
 	{
 		SafeSelectItem(parent);
 		return;
@@ -341,7 +353,7 @@ wxTreeItemId CLocalTreeView::GetNearestParent(wxString& localDir)
 		wxTreeItemId root = GetRootItem();
 #endif
 
-	while (localDir != _T(""))
+	while (!localDir.empty())
 	{
 		wxString subDir;
 		int pos = localDir.Find(separator);
@@ -386,66 +398,28 @@ wxTreeItemId CLocalTreeView::GetSubdir(wxTreeItemId parent, const wxString& subD
 
 bool CLocalTreeView::DisplayDrives(wxTreeItemId parent)
 {
-	long drivesToHide = 0;
-	// Adhere to the NODRIVES group policy
-	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
-	if (key.Exists())
-	{
-		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
-			drivesToHide = 0;
-	}
+	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives"));
 
-	int len = GetLogicalDriveStrings(0, 0);
-	if (!len)
-		return false;
-
-	wxChar* drives = new wxChar[len + 1];
-
-	if (!GetLogicalDriveStrings(len, drives))
-	{
-		delete [] drives;
-		return false;
-	}
+	std::list<wxString> drives = CVolumeDescriptionEnumeratorThread::GetDrives();
 
 	m_pVolumeEnumeratorThread = new CVolumeDescriptionEnumeratorThread(this);
-	if (m_pVolumeEnumeratorThread->Failed())
-	{
+	if (m_pVolumeEnumeratorThread->Failed()) {
 		delete m_pVolumeEnumeratorThread;
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	const wxChar* pDrive = drives;
-	while (*pDrive)
-	{
-		// Check if drive should be hidden by default
-		if (pDrive[0] != 0 && pDrive[1] == ':')
-		{
-			int bit = 0;
-			char letter = pDrive[0];
-			if (letter >= 'A' && letter <= 'Z')
-				bit = 1 << (letter - 'A');
-			if (letter >= 'a' && letter <= 'z')
-				bit = 1 << (letter - 'a');
-
-			if (drivesToHide & bit)
-			{
-				pDrive += wxStrlen(pDrive) + 1;
-				continue;
-			}
-		}
-
-		wxString drive = pDrive;
+	for( std::list<wxString>::const_iterator it = drives.begin(); it != drives.end(); ++it ) {
+		wxString drive = *it;
 		if (drive.Right(1) == _T("\\"))
 			drive = drive.RemoveLast();
 
-		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(dir, pDrive));
+		wxGetApp().AddStartupProfileRecord(wxString::Format(_T("CLocalTreeView::DisplayDrives adding drive %s"), drive.c_str()));
+		wxTreeItemId item = AppendItem(parent, drive, GetIconIndex(iconType::dir, drive));
 		AppendItem(item, _T(""));
 		SortChildren(parent);
-
-		pDrive += wxStrlen(pDrive) + 1;
 	}
 
-	delete [] drives;
+	wxGetApp().AddStartupProfileRecord(_T("CLocalTreeView::DisplayDrives adding drives done"));
 
 	return true;
 }
@@ -460,18 +434,18 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		wxLogNull log;
 		if (!local_filesystem.BeginFindFiles(dirname, true))
 		{
-			if (knownSubdir != _T(""))
+			if (!knownSubdir.empty())
 			{
 				wxTreeItemId item = GetSubdir(parent, knownSubdir);
 				if (item != wxTreeItemId())
 					return;
 
 				const wxString fullName = dirname + knownSubdir;
-				item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+				item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 						-1
 #else
-						GetIconIndex(opened_dir, fullName)
+						GetIconIndex(iconType::opened_dir, fullName)
 #endif
 					);
 				CheckSubdirStatus(item, fullName);
@@ -500,11 +474,11 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 	int attributes;
 	bool is_dir;
 	const wxLongLong size(-1);
-	wxDateTime date;
+	CDateTime date;
 	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
 	{
 		wxASSERT(is_dir);
-		if (file == _T(""))
+		if (file.empty())
 		{
 			wxGetApp().DisplayEncodingWarning();
 			continue;
@@ -517,31 +491,31 @@ void CLocalTreeView::DisplayDir(wxTreeItemId parent, const wxString& dirname, co
 		if (file != knownSubdir)
 #endif
 		{
-			if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date.IsValid() ? &date : 0))
+			if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
 				continue;
 		}
 		else
 			matchedKnown = true;
 
-		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, file, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
-				GetIconIndex(opened_dir, fullName)
+				GetIconIndex(iconType::opened_dir, fullName)
 #endif
 			);
 
 		CheckSubdirStatus(item, fullName);
 	}
 
-	if (!matchedKnown && knownSubdir != _T(""))
+	if (!matchedKnown && !knownSubdir.empty())
 	{
 		const wxString fullName = dirname + knownSubdir;
-		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(::dir, fullName),
+		wxTreeItemId item = AppendItem(parent, knownSubdir, GetIconIndex(iconType::dir, fullName),
 #ifdef __WXMSW__
 				-1
 #else
-				GetIconIndex(opened_dir, fullName)
+				GetIconIndex(iconType::opened_dir, fullName)
 #endif
 			);
 
@@ -556,40 +530,40 @@ wxString CLocalTreeView::HasSubdir(const wxString& dirname)
 	wxLogNull nullLog;
 
 	CFilterManager filter;
-	
+
 	CLocalFileSystem local_filesystem;
 	if (!local_filesystem.BeginFindFiles(dirname, true))
-		return _T("");
+		return wxString();
 
 	wxString file;
 	bool wasLink;
 	int attributes;
 	bool is_dir;
 	const wxLongLong size(-1);
-	wxDateTime date;
+	CDateTime date;
 	while (local_filesystem.GetNextFile(file, wasLink, is_dir, 0, &date, &attributes))
 	{
 		wxASSERT(is_dir);
-		if (file == _T(""))
+		if (file.empty())
 		{
 			wxGetApp().DisplayEncodingWarning();
 			continue;
 		}
 
-		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date.IsValid() ? &date : 0))
+		if (filter.FilenameFiltered(file, dirname, true, size, true, attributes, date))
 			continue;
 
 		return file;
 	}
 
-	return _T("");
+	return wxString();
 }
 
 wxTreeItemId CLocalTreeView::MakeSubdirs(wxTreeItemId parent, wxString dirname, wxString subDir)
 {
 	const wxString& separator = wxFileName::GetPathSeparator();
 
-	while (subDir != _T(""))
+	while (!subDir.empty())
 	{
 		int pos = subDir.Find(separator);
 		wxString segment;
@@ -625,7 +599,7 @@ void CLocalTreeView::OnItemExpanding(wxTreeEvent& event)
 
 	wxTreeItemIdValue value;
 	wxTreeItemId child = GetFirstChild(item, value);
-	if (child && GetItemText(child) == _T(""))
+	if (child && GetItemText(child).empty())
 		DisplayDir(item, GetDirFromItem(item));
 }
 
@@ -636,39 +610,33 @@ wxString CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 	while (item)
 	{
 #ifdef __WXMSW__
-		if (item == m_desktop)
-		{
+		if (item == m_desktop) {
 			wxChar path[MAX_PATH + 1];
-			if (SHGetFolderPath(0, CSIDL_DESKTOPDIRECTORY, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
-			{
-				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
-				{
-					wxMessageBox(_("Failed to get desktop path"));
+			if (SHGetFolderPath(0, CSIDL_DESKTOPDIRECTORY, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
+				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
+					wxMessageBoxEx(_("Failed to get desktop path"));
 					return _T("/");
 				}
 			}
 			dir = path;
-			if (dir.Last() != separator)
+			if (dir.empty() || dir.Last() != separator)
 				dir += separator;
 			return dir;
 		}
-		else if (item == m_documents)
-		{
+		else if (item == m_documents) {
 			wxChar path[MAX_PATH + 1];
-			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK)
-			{
-				wxMessageBox(_("Failed to get 'My Documents' path"));
+			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
+				wxMessageBoxEx(_("Failed to get 'My Documents' path"));
 				return _T("/");
 			}
 			dir = path;
-			if (dir.Last() != separator)
+			if (dir.empty() || dir.Last() != separator)
 				dir += separator;
 			return dir;
 		}
 		else if (item == m_drives)
 			return _T("/");
-		else if (GetItemParent(item) == m_drives)
-		{
+		else if (GetItemParent(item) == m_drives) {
 			wxString text = GetItemText(item);
 			int pos = text.Find(_T(" "));
 			if (pos == -1)
@@ -695,16 +663,24 @@ struct t_dir
 	wxTreeItemId item;
 };
 
-static bool sortfunc(const wxString& a, const wxString& b)
+void CLocalTreeView::UpdateSortMode()
 {
-#ifdef __WXMSW__
-	return b.CmpNoCase(a) > 0;
-#else
-	return b.Cmp(a) > 0;
-#endif
+	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
+	{
+	case 0:
+	default:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
 }
 
-void CLocalTreeView::Refresh()
+void CLocalTreeView::RefreshListing()
 {
 	wxLogNull nullLog;
 
@@ -748,6 +724,7 @@ void CLocalTreeView::Refresh()
 		t_dir dir = dirsToCheck.front();
 		dirsToCheck.pop_front();
 
+		// Step 1: Check if directory exists
 		CLocalFileSystem local_filesystem;
 		if (!local_filesystem.BeginFindFiles(dir.dir, true))
 		{
@@ -767,43 +744,52 @@ void CLocalTreeView::Refresh()
 			continue;
 		}
 
-		std::list<wxString> dirs;
+		// Step 2: Enumerate subdirectories on disk and sort them
+		wxArrayString dirs;
 
-		
 		wxString file;
 		const wxLongLong size(-1);
 		bool was_link;
 		bool is_dir;
 		int attributes;
-		wxDateTime date;
+		CDateTime date;
 		while (local_filesystem.GetNextFile(file, was_link, is_dir, 0, &date, &attributes))
 		{
-			if (file == _T(""))
+			if (file.empty())
 			{
 				wxGetApp().DisplayEncodingWarning();
 				continue;
 			}
 
-			if (filter.FilenameFiltered(file, dir.dir, true, size, true, attributes, date.IsValid() ? &date : 0))
+			if (filter.FilenameFiltered(file, dir.dir, true, size, true, attributes, date))
 				continue;
 
 			dirs.push_back(file);
 		}
-		dirs.sort(sortfunc);		
+		dirs.Sort(CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode));
 
 		bool inserted = false;
 
+		// Step 3: Merge list of subdirectories with subtree.
 		wxTreeItemId child = GetLastChild(dir.item);
-		std::list<wxString>::reverse_iterator iter = dirs.rbegin();
-		while (child && iter != dirs.rend())
+		wxArrayString::reverse_iterator iter = dirs.rbegin();
+		while (child || iter != dirs.rend())
 		{
+			int cmp;
+			if (child && iter != dirs.rend())
 #ifdef __WXMSW__
-			int cmp = GetItemText(child).CmpNoCase(*iter);
+				cmp = GetItemText(child).CmpNoCase(*iter);
 #else
-			int cmp = GetItemText(child).Cmp(*iter);
+				cmp = GetItemText(child).Cmp(*iter);
 #endif
+			else if (child)
+				cmp = 1;
+			else
+				cmp = -1;
+
 			if (!cmp)
 			{
+				// Found item with same name. Mark it for further processing
 				if (!IsExpanded(child))
 				{
 					wxString path = dir.dir + *iter + separator;
@@ -823,10 +809,13 @@ void CLocalTreeView::Refresh()
 					dirsToCheck.push_front(subdir);
 				}
 				child = GetPrevSibling(child);
-				iter++;
+				++iter;
 			}
 			else if (cmp > 0)
 			{
+				// Subdirectory currently in tree no longer exists.
+				// Delete child from tree, unless current selection
+				// is in the subtree.
 				wxTreeItemId sel = GetSelection();
 				while (sel && sel != child)
 					sel = GetItemParent(sel);
@@ -837,45 +826,21 @@ void CLocalTreeView::Refresh()
 			}
 			else if (cmp < 0)
 			{
+				// New subdirectory, add treeitem
 				wxString fullname = dir.dir + *iter + separator;
-				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
+				wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(iconType::dir, fullname),
 #ifdef __WXMSW__
 						-1
 #else
-						GetIconIndex(opened_dir, fullname)
+						GetIconIndex(iconType::opened_dir, fullname)
 #endif
 					);
 
 				CheckSubdirStatus(newItem, fullname);
-				iter++;
+				++iter;
 				inserted = true;
 			}
 		}
-		while (child)
-		{
-			wxTreeItemId sel = GetSelection();
-			while (sel && sel != child)
-				sel = GetItemParent(sel);
-			wxTreeItemId prev = GetPrevSibling(child);
-			if (!sel)
-				Delete(child);
-			child = prev;
-		}
-		while (iter != dirs.rend())
-		{
-			wxString fullname = dir.dir + *iter + separator;
-			wxTreeItemId newItem = AppendItem(dir.item, *iter, GetIconIndex(::dir, fullname),
-#ifdef __WXMSW__
-					-1
-#else
-					GetIconIndex(opened_dir, fullname)
-#endif
-				);
-
-			CheckSubdirStatus(newItem, fullname);
-			iter++;
-			inserted = true;
-		}
 		if (inserted)
 			SortChildren(dir.item);
 	}
@@ -902,8 +867,8 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	wxString error;
 	if (!m_pState->SetLocalDir(dir, &error))
 	{
-		if (error != _T(""))
-			wxMessageBox(error, _("Failed to change directory"), wxICON_INFORMATION);
+		if (!error.empty())
+			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
 		else
 			wxBell();
 		m_setSelection = true;
@@ -912,25 +877,13 @@ void CLocalTreeView::OnSelectionChanged(wxTreeEvent& event)
 	}
 }
 
-int CLocalTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2)
-{
-	wxString label1 = GetItemText(item1);
-	wxString label2 = GetItemText(item2);
-#ifdef __WXMSW__
-	return label1.CmpNoCase(label2);
-#else
-	return label1.Cmp(label2);
-#endif
-}
-
-void CLocalTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CLocalTreeView::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_LOCAL_DIR)
 		SetDir(m_pState->GetLocalDir().GetPath());
-	else
-	{
+	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
-		Refresh();
+		RefreshListing();
 	}
 }
 
@@ -950,14 +903,14 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 
 #ifdef __WXMSW__
-	if (dir.Last() == '\\')
+	if (!dir.empty() && dir.Last() == '\\')
 		dir.RemoveLast();
 #endif
-	if (dir.Last() == '/')
+	if (!dir.empty() && dir.Last() == '/')
 		dir.RemoveLast();
 
 #ifdef __WXMSW__
-	if (dir.Last() == ':')
+	if (!dir.empty() && dir.Last() == ':')
 		return;
 #endif
 
@@ -984,41 +937,23 @@ void CLocalTreeView::OnBeginDrag(wxTreeEvent& event)
 	}
 }
 
-#ifndef __WXMSW__
-void CLocalTreeView::OnKeyDown(wxKeyEvent& event)
-{
-	if (event.GetKeyCode() != WXK_TAB)
-	{
-		event.Skip();
-		return;
-	}
-
-	wxNavigationKeyEvent navEvent;
-	navEvent.SetEventObject(this);
-	navEvent.SetDirection(!event.ShiftDown());
-	navEvent.SetFromTab(true);
-	navEvent.ResumePropagation(1);
-	ProcessEvent(navEvent);
-}
-#endif
-
 #ifdef __WXMSW__
 
 wxString CLocalTreeView::GetSpecialFolder(int folder, int &iconIndex, int &openIconIndex)
 {
 	LPITEMIDLIST list;
 	if (SHGetSpecialFolderLocation((HWND)GetHandle(), folder, &list) != S_OK)
-		return _T("");
+		return wxString();
 
 	SHFILEINFO shFinfo;
 	if (!SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON))
-		return _T("");
+		return wxString();
 
 	DestroyIcon(shFinfo.hIcon);
 	iconIndex = shFinfo.iIcon;
 
 	if (!SHGetFileInfo((LPCTSTR)list, 0, &shFinfo, sizeof(shFinfo), SHGFI_PIDL | SHGFI_ICON | SHGFI_SMALLICON | SHGFI_OPENICON | SHGFI_DISPLAYNAME))
-		return _T("");
+		return wxString();
 
 	DestroyIcon(shFinfo.hIcon);
 	openIconIndex = shFinfo.iIcon;
@@ -1026,7 +961,7 @@ wxString CLocalTreeView::GetSpecialFolder(int folder, int &iconIndex, int &openI
 	wxString name = shFinfo.szDisplayName;
 
 	LPMALLOC pMalloc;
-    SHGetMalloc(&pMalloc);
+	SHGetMalloc(&pMalloc);
 
 	if (pMalloc)
 	{
@@ -1043,7 +978,7 @@ bool CLocalTreeView::CreateRoot()
 {
 	int iconIndex, openIconIndex;
 	wxString name = GetSpecialFolder(CSIDL_DESKTOP, iconIndex, openIconIndex);
-	if (name == _T(""))
+	if (name.empty())
 	{
 		name = _("Desktop");
 		iconIndex = openIconIndex = -1;
@@ -1052,7 +987,7 @@ bool CLocalTreeView::CreateRoot()
 	m_desktop = AddRoot(name, iconIndex, openIconIndex);
 
 	name = GetSpecialFolder(CSIDL_PERSONAL, iconIndex, openIconIndex);
-	if (name == _T(""))
+	if (name.empty())
 	{
 		name = _("My Documents");
 		iconIndex = openIconIndex = -1;
@@ -1062,7 +997,7 @@ bool CLocalTreeView::CreateRoot()
 
 
 	name = GetSpecialFolder(CSIDL_DRIVES, iconIndex, openIconIndex);
-	if (name == _T(""))
+	if (name.empty())
 	{
 		name = _("My Computer");
 		iconIndex = openIconIndex = -1;
@@ -1091,7 +1026,7 @@ void CLocalTreeView::OnVolumesEnumerated(wxCommandEvent& event)
 		m_pVolumeEnumeratorThread = 0;
 	}
 
-	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); iter++)
+	for (std::list<CVolumeDescriptionEnumeratorThread::t_VolumeInfo>::const_iterator iter = volumeInfo.begin(); iter != volumeInfo.end(); ++iter)
 	{
 		wxString drive = iter->volume;
 
@@ -1127,7 +1062,7 @@ void CLocalTreeView::OnContextMenu(wxTreeEvent& event)
 	const bool hasParent = path.HasParent();
 	const bool writeable = path.IsWriteable();
 
-	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().IsEmpty();
+	const bool remoteConnected = m_pState->IsRemoteConnected() && !m_pState->GetRemotePath().empty();
 
 	pMenu->Enable(XRCID("ID_UPLOAD"), hasParent && remoteConnected);
 	pMenu->Enable(XRCID("ID_ADDTOQUEUE"), hasParent && remoteConnected);
@@ -1154,7 +1089,7 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 
 	const CServer server = *m_pState->GetServer();
 	CServerPath remotePath = m_pState->GetRemotePath();
-	if (remotePath.IsEmpty())
+	if (remotePath.empty())
 		return;
 
 	if (!remotePath.ChangePath(GetItemText(m_contextMenuItem)))
@@ -1163,10 +1098,41 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 	m_pQueueView->QueueFolder(event.GetId() == XRCID("ID_ADDTOQUEUE"), false, path, remotePath, server);
 }
 
-void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
+// Create a new Directory
+void CLocalTreeView::OnMenuMkdir(wxCommandEvent&)
+{
+	wxString newdir = MenuMkdir();
+	if (!newdir.empty()) {
+		RefreshListing();
+		m_pState->RefreshLocal();
+	}
+}
+
+// Create a new Directory and enter the new Directory
+void CLocalTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
+{
+	wxString newdir = MenuMkdir();
+	if (newdir.empty()) {
+		return;
+	}
+
+	// OnMenuEnter
+	wxString error;
+	if (!m_pState->SetLocalDir(newdir, &error))
+	{
+		if (!error.empty())
+			wxMessageBoxEx(error, _("Failed to change directory"), wxICON_INFORMATION);
+		else
+			wxBell();
+	}
+}
+
+// Helper-Function to create a new Directory
+// Returns the name of the new directory
+wxString CLocalTreeView::MenuMkdir()
 {
 	if (!m_contextMenuItem.IsOk())
-		return;
+		return wxString();
 
 	wxString path = GetDirFromItem(m_contextMenuItem);
 	if (path.Last() != wxFileName::GetPathSeparator())
@@ -1175,25 +1141,25 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 	if (!CLocalPath(path).IsWriteable())
 	{
 		wxBell();
-		return;
+		return wxString();
 	}
 
 	CInputDialog dlg;
 	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
-		return;
+		return wxString();
 
 	wxString newName = _("New directory");
 	dlg.SetValue(path + newName);
 	dlg.SelectText(path.Len(), path.Len() + newName.Len());
 
 	if (dlg.ShowModal() != wxID_OK)
-		return;
+		return wxString();
 
 	wxFileName fn(dlg.GetValue(), _T(""));
 	if (!fn.Normalize(wxPATH_NORM_ALL, path))
 	{
 		wxBell();
-		return;
+		return wxString();
 	}
 
 	bool res;
@@ -1202,29 +1168,28 @@ void CLocalTreeView::OnMenuMkdir(wxCommandEvent& event)
 		res = fn.Mkdir(fn.GetPath(), 0777, wxPATH_MKDIR_FULL);
 	}
 
-	if (!res)
+	if (!res) {
 		wxBell();
+		return wxString();
+	}
 
-	Refresh();
-	m_pState->RefreshLocal();
+	return fn.GetPath();
 }
 
-void CLocalTreeView::OnMenuRename(wxCommandEvent& event)
+void CLocalTreeView::OnMenuRename(wxCommandEvent&)
 {
 	if (!m_contextMenuItem.IsOk())
 		return;
 
 #ifdef __WXMSW__
-	if (m_contextMenuItem == m_desktop || m_contextMenuItem == m_documents)
-	{
+	if (m_contextMenuItem == m_desktop || m_contextMenuItem == m_documents) {
 		wxBell();
 		return;
 	}
 #endif
 
 	CLocalPath path(GetDirFromItem(m_contextMenuItem));
-	if (!path.HasParent() || !path.IsWriteable())
-	{
+	if (!path.HasParent() || !path.IsWriteable()) {
 		wxBell();
 		return;
 	}
@@ -1232,7 +1197,7 @@ void CLocalTreeView::OnMenuRename(wxCommandEvent& event)
 	EditLabel(m_contextMenuItem);
 }
 
-void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
+void CLocalTreeView::OnMenuDelete(wxCommandEvent&)
 {
 	if (!m_contextMenuItem.IsOk())
 		return;
@@ -1250,16 +1215,15 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 	while (item && item != m_contextMenuItem)
 		item = GetItemParent(item);
 
-	if (!item)
-	{
+	if (!item) {
 		if (GetItemParent(m_contextMenuItem) == GetSelection())
 			m_pState->RefreshLocal();
 		else
-			Refresh();
+			RefreshListing();
 		return;
 	}
 
-	if (path.Last() == wxFileName::GetPathSeparator())
+	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
 		path.RemoveLast();
 	int pos = path.Find(wxFileName::GetPathSeparator(), true);
 	if (pos < 1)
@@ -1268,7 +1232,7 @@ void CLocalTreeView::OnMenuDelete(wxCommandEvent& event)
 		path = path.Left(pos);
 
 	m_pState->SetLocalDir(path);
-	Refresh();
+	RefreshListing();
 }
 
 void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
@@ -1294,13 +1258,9 @@ void CLocalTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 }
 
-// Defined in LocalListView.cpp
-extern bool Rename(wxWindow* pWnd, wxString dir, wxString from, wxString to);
-
 void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 {
-	if (event.IsEditCancelled())
-	{
+	if (event.IsEditCancelled()) {
 		event.Veto();
 		return;
 	}
@@ -1308,8 +1268,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	wxTreeItemId item = event.GetItem();
 
 #ifdef __WXMSW__
-	if (item == m_desktop || item == m_documents)
-	{
+	if (item == m_desktop || item == m_documents) {
 		wxBell();
 		event.Veto();
 		return;
@@ -1319,14 +1278,13 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	wxString path = GetDirFromItem(item);
 
 	CLocalPath local_path(path);
-	if (!local_path.HasParent() || !local_path.IsWriteable())
-	{
+	if (!local_path.HasParent() || !local_path.IsWriteable()) {
 		wxBell();
 		event.Veto();
 		return;
 	}
 
-	if (path.Last() == wxFileName::GetPathSeparator())
+	if (!path.empty() && path.Last() == wxFileName::GetPathSeparator())
 		path.RemoveLast();
 
 	int pos = path.Find(wxFileName::GetPathSeparator(), true);
@@ -1336,8 +1294,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 
 	const wxString& oldName = GetItemText(item);
 	const wxString& newName = event.GetLabel();
-	if (newName == _T(""))
-	{
+	if (newName.empty()) {
 		wxBell();
 		event.Veto();
 		return;
@@ -1348,8 +1305,7 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	if (oldName == newName)
 		return;
 
-	if (!Rename(this, parent, oldName, newName))
-	{
+	if (!RenameFile(this, parent, oldName, newName)) {
 		event.Veto();
 		return;
 	}
@@ -1361,14 +1317,12 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	SetItemText(item, newName);
 
 	wxTreeItemId currentSel = GetSelection();
-	if (currentSel == wxTreeItemId())
-	{
-		Refresh();
+	if (currentSel == wxTreeItemId()) {
+		RefreshListing();
 		return;
 	}
 
-	if (item == currentSel)
-	{
+	if (item == currentSel) {
 		m_pState->SetLocalDir(parent + newName);
 		return;
 	}
@@ -1376,14 +1330,12 @@ void CLocalTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	wxString sub;
 
 	wxTreeItemId tmp = currentSel;
-	while (tmp != GetRootItem() && tmp != item)
-	{
+	while (tmp != GetRootItem() && tmp != item) {
 		sub = wxFileName::GetPathSeparator() + GetItemText(tmp) + sub;
 		tmp = GetItemParent(tmp);
 	}
 
-	if (tmp == GetRootItem())
-	{
+	if (tmp == GetRootItem()) {
 		// Rename unrelated to current selection
 		return;
 	}
@@ -1420,7 +1372,7 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 		CFilterManager filter;
 
 		const int attributes = S_IRUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
-		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, 0))
+		if (!filter.FilenameFiltered(_T("localhost"), path, true, size, true, attributes, CDateTime()))
 		{
 			if (!child)
 				AppendItem(item, _T(""));
@@ -1429,34 +1381,30 @@ bool CLocalTreeView::CheckSubdirStatus(wxTreeItemId& item, const wxString& path)
 	}
 #endif
 
-	if (child)
-	{
-		if (GetItemText(child) != _T(""))
+	if (child) {
+		if (!GetItemText(child).empty())
 			return false;
 
 		CTreeItemData* pData = (CTreeItemData*)GetItemData(child);
-		if (pData)
-		{
+		if (pData) {
 			bool wasLink;
 			int attributes;
 			enum CLocalFileSystem::local_fileType type;
-			wxDateTime date;
-			if (path.Last() == CLocalFileSystem::path_separator)
+			CDateTime date;
+			if (!path.empty() && path.Last() == CLocalFileSystem::path_separator)
 				type = CLocalFileSystem::GetFileInfo(path + pData->m_known_subdir, wasLink, 0, &date, &attributes);
 			else
 				type = CLocalFileSystem::GetFileInfo(path + CLocalFileSystem::path_separator + pData->m_known_subdir, wasLink, 0, &date, &attributes);
-			if (type == CLocalFileSystem::dir)
-			{
+			if (type == CLocalFileSystem::dir) {
 				CFilterManager filter;
-				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date.IsValid() ? &date : 0))
+				if (!filter.FilenameFiltered(pData->m_known_subdir, path, true, size, true, attributes, date))
 					return true;
 			}
 		}
 	}
 
 	wxString sub = HasSubdir(path);
-	if (sub != _T(""))
-	{
+	if (!sub.empty()) {
 		wxTreeItemId subItem = AppendItem(item, _T(""));
 		SetItemData(subItem, new CTreeItemData(sub));
 	}
@@ -1472,8 +1420,7 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 	if (!m_drives)
 		return;
 
-	if (wParam == DBT_DEVICEARRIVAL || wParam == DBT_DEVICEREMOVECOMPLETE)
-	{
+	if (wParam == DBT_DEVICEARRIVAL || wParam == DBT_DEVICEREMOVECOMPLETE) {
 		DEV_BROADCAST_HDR* pHdr = (DEV_BROADCAST_HDR*)lParam;
 		if (pHdr->dbch_devicetype != DBT_DEVTYP_VOLUME)
 			return;
@@ -1483,18 +1430,14 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 
 		wxChar drive = 'A';
 		int mask = 1;
-		while (drive <= 'Z')
-		{
-			if (pVolume->dbcv_unitmask & mask)
-			{
+		while (drive <= 'Z') {
+			if (pVolume->dbcv_unitmask & mask) {
 				if (wParam == DBT_DEVICEARRIVAL)
 					AddDrive(drive);
-				else
-				{
+				else {
 					RemoveDrive(drive);
 
-					if (pVolume->dbcv_flags & DBTF_MEDIA)
-					{
+					if (pVolume->dbcv_flags & DBTF_MEDIA) {
 						// E.g. disk removed from CD-ROM drive, need to keep the drive letter
 						AddDrive(drive);
 					}
@@ -1509,37 +1452,24 @@ void CLocalTreeView::OnDevicechange(WPARAM wParam, LPARAM lParam)
 	}
 }
 
-wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
+wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 {
-	// Adhere to the NODRIVES group policy
-	long drivesToHide = 0;
-	wxRegKey key(_T("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Explorer"));
-	if (key.Exists())
-	{
-		if (!key.HasValue(_T("NoDrives")) || !key.QueryValue(_T("NoDrives"), &drivesToHide))
-			drivesToHide = 0;
-	}
+	wxString drive = letter;
+	drive += _T(":");
 
-	int bit = 0;
-	if (drive >= 'A' && drive <= 'Z')
-		bit = 1 << (drive - 'A');
-	else if (drive >= 'a' && drive <= 'z')
-		bit = 1 << (drive - 'a');
-	if (drivesToHide & bit)
+	long drivesToHide = CVolumeDescriptionEnumeratorThread::GetDrivesToHide();
+	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive, drivesToHide) ) {
 		return wxTreeItemId();
-
-	wxString driveName = drive;
-	driveName += _T(":");
+	}
 
 	// Get the label of the drive
 	wxChar volumeName[501];
 	int oldErrorMode = SetErrorMode(SEM_FAILCRITICALERRORS);
-	BOOL res = GetVolumeInformation(driveName + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
+	BOOL res = GetVolumeInformation(drive + _T("\\"), volumeName, 500, 0, 0, 0, 0, 0);
 	SetErrorMode(oldErrorMode);
 
-	wxString itemLabel = driveName;
-	if (res && volumeName[0])
-	{
+	wxString itemLabel = drive;
+	if (res && volumeName[0]) {
 		itemLabel += _T(" (");
 		itemLabel += volumeName;
 		itemLabel += _T(")");
@@ -1549,7 +1479,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
 	wxTreeItemId driveItem = GetFirstChild(m_drives, value);
 	while (driveItem)
 	{
-		if (!GetItemText(driveItem).Left(2).CmpNoCase(driveName))
+		if (!GetItemText(driveItem).Left(2).CmpNoCase(drive))
 			break;
 
 		driveItem = GetNextSibling(driveItem);
@@ -1557,7 +1487,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
 	if (driveItem)
 	{
 		SetItemText(driveItem, itemLabel);
-		int icon = GetIconIndex(dir, driveName + _T("\\"));
+		int icon = GetIconIndex(iconType::dir, drive + _T("\\"));
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Normal);
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Selected);
 		SetItemImage(driveItem, icon, wxTreeItemIcon_Expanded);
@@ -1566,7 +1496,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar drive)
 		return driveItem;
 	}
 
-	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(dir, driveName + _T("\\")));
+	wxTreeItemId item = AppendItem(m_drives, itemLabel, GetIconIndex(iconType::dir, drive + _T("\\")));
 	AppendItem(item, _T(""));
 	SortChildren(m_drives);
 
@@ -1602,7 +1532,7 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 
 	if (GetItemImage(item, wxTreeItemIcon_Selected) == -1)
 	{
-		int icon = GetIconIndex(opened_dir, GetDirFromItem(item));
+		int icon = GetIconIndex(iconType::opened_dir, GetDirFromItem(item));
 		SetItemImage(item, icon, wxTreeItemIcon_Selected);
 		SetItemImage(item, icon, wxTreeItemIcon_SelectedExpanded);
 	}
@@ -1610,18 +1540,22 @@ void CLocalTreeView::OnSelectionChanging(wxTreeEvent& event)
 
 #endif
 
-void CLocalTreeView::OnMenuOpen(wxCommandEvent& event)
+void CLocalTreeView::OnMenuOpen(wxCommandEvent&)
 {
 	if (!m_contextMenuItem.IsOk())
 		return;
 
 	wxString path = GetDirFromItem(m_contextMenuItem);
-	if (path == _T(""))
+	if (path.empty())
 		return;
 
-	wxString url = CState::GetAsURL(path);
-	if (url == _T(""))
-		return;
+	OpenInFileManager(path);
+}
 
-	wxLaunchDefaultBrowser(url);
+void CLocalTreeView::OnOptionsChanged(changed_options_t const& options)
+{
+	if (options.test(OPTION_FILELIST_NAMESORT)) {
+		UpdateSortMode();
+		RefreshListing();
+	}
 }
