@@ -5,27 +5,36 @@
 #include <wx/taskbar.h>
 #endif
 
-class CStatusView;
-class CQueueView;
-class CState;
-class CAsyncRequestQueue;
-class CLed;
-class CThemeProvider;
-class CQuickconnectBar;
-#if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-class CUpdateWizard;
-#endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-class CSiteManagerItemData_Site;
-class CQueue;
-class CWindowStateManager;
-class CStatusBar;
-class CMainFrameStateEventHandler;
-class CSplitterWindowEx;
-class CContextControl;
-class CToolBar;
-class CMenuBar;
+#include "statusbar.h"
 
-class CMainFrame : public wxFrame
+#include "engine_context.h"
+
+class CAsyncRequestQueue;
+class CContextControl;
+class CLed;
+class CMainFrameStateEventHandler;
+class CMenuBar;
+class CQueue;
+class CQueueView;
+class CQuickconnectBar;
+class CSiteManagerItemData_Site;
+class CSplitterWindowEx;
+class CStatusView;
+class CState;
+class CThemeProvider;
+class CToolBar;
+class CWindowStateManager;
+
+
+#if FZ_MANUALUPDATECHECK
+#include "updater.h"
+#endif
+class CWindowStateManager;
+
+class CMainFrame : public wxNavigationEnabled<wxFrame>
+#if FZ_MANUALUPDATECHECK
+	, protected CUpdateHandler
+#endif
 {
 	friend class CMainFrameStateEventHandler;
 public:
@@ -34,7 +43,6 @@ public:
 
 	void UpdateActivityLed(int direction);
 
-	void AddToRequestQueue(CFileZillaEngine* pEngine, CAsyncRequestNotification* pNotification);
 	CStatusView* GetStatusView() { return m_pStatusView; }
 	CQueueView* GetQueue() { return m_pQueueView; }
 	CQuickconnectBar* GetQuickconnectBar() { return m_pQuickconnectBar; }
@@ -50,7 +58,7 @@ public:
 
 	void ConnectNavigationHandler(wxEvtHandler* handler);
 
-	CStatusBar* GetStatusBar() { return m_pStatusBar; }
+	wxStatusBar* GetStatusBar() const { return m_pStatusBar; }
 
 	void ProcessCommandLine();
 
@@ -60,41 +68,46 @@ public:
 
 	CContextControl* GetContextControl() { return m_pContextControl; }
 
-	bool ConnectToSite(CSiteManagerItemData_Site* const pData, bool newTab = false);
+	bool ConnectToSite(CSiteManagerItemData_Site & data, bool newTab = false);
 
+	CFileZillaEngineContext& GetEngineContext() { return m_engineContext; }
 protected:
+	void FixTabOrder();
+
 	bool CloseDialogsAndQuit(wxCloseEvent &event);
 	bool CreateMenus();
 	bool CreateQuickconnectBar();
-	bool CreateToolBar();
+	bool CreateMainToolBar();
 	void OpenSiteManager(const CServer* pServer = 0);
 
 	void FocusNextEnabled(std::list<wxWindow*>& windowOrder, std::list<wxWindow*>::iterator iter, bool skipFirst, bool forward);
 
 	void SetBookmarksFromPath(const wxString& path);
 
-	CStatusBar* m_pStatusBar;
-	CMenuBar* m_pMenuBar;
-	CToolBar* m_pToolBar;
-	CQuickconnectBar* m_pQuickconnectBar;
+	CStatusBar* m_pStatusBar{};
+	CMenuBar* m_pMenuBar{};
+	CToolBar* m_pToolBar{};
+	CQuickconnectBar* m_pQuickconnectBar{};
 
-	CSplitterWindowEx* m_pTopSplitter; // If log position is 0, splits message log from rest of panes
-	CSplitterWindowEx* m_pBottomSplitter; // Top contains view splitter, bottom queue (or queuelog splitter if in position 1)
-	CSplitterWindowEx* m_pQueueLogSplitter;
+	CSplitterWindowEx* m_pTopSplitter{}; // If log position is 0, splits message log from rest of panes
+	CSplitterWindowEx* m_pBottomSplitter{}; // Top contains view splitter, bottom queue (or queuelog splitter if in position 1)
+	CSplitterWindowEx* m_pQueueLogSplitter{};
 
-	CContextControl* m_pContextControl;
+	CFileZillaEngineContext m_engineContext;
+	CContextControl* m_pContextControl{};
 
-	CStatusView* m_pStatusView;
-	CQueueView* m_pQueueView;
+	CStatusView* m_pStatusView{};
+	CQueueView* m_pQueueView{};
 	CLed* m_pActivityLed[2];
-	wxTimer m_transferStatusTimer;
-	CThemeProvider* m_pThemeProvider;
-#if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	CUpdateWizard* m_pUpdateWizard;
-#endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
+	CThemeProvider* m_pThemeProvider{};
+#if FZ_MANUALUPDATECHECK
+	CUpdater* m_pUpdater{};
+	virtual void UpdaterStateChanged( UpdaterState s, build const& v );
+	void TriggerUpdateDialog();
+	wxTimer update_dialog_timer_;
+#endif
 
-	void ShowLocalTree();
-	void ShowRemoteTree();
+	void ShowDirectoryTree(bool local, bool show);
 
 	void ShowDropdownMenu(wxMenu* pMenu, wxToolBar* pToolBar, wxCommandEvent& event);
 
@@ -108,23 +121,22 @@ protected:
 	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent& event);
 	void OnMenuHandler(wxCommandEvent& event);
-	void OnEngineEvent(wxEvent& event);
-	void OnUpdateLedTooltip(wxCommandEvent& event);
-	void OnDisconnect(wxCommandEvent& event);
-	void OnCancel(wxCommandEvent& event);
+	void OnEngineEvent(wxFzEvent& event);
+	void OnUpdateLedTooltip(wxCommandEvent&);
+	void OnDisconnect(wxCommandEvent&);
+	void OnCancel(wxCommandEvent&);
 	void OnClose(wxCloseEvent& event);
-	void OnReconnect(wxCommandEvent& event);
-	void OnRefresh(wxCommandEvent& event);
+	void OnReconnect(wxCommandEvent&);
+	void OnRefresh(wxCommandEvent&);
 	void OnTimer(wxTimerEvent& event);
-	void OnSiteManager(wxCommandEvent& event);
+	void OnSiteManager(wxCommandEvent&);
 	void OnProcessQueue(wxCommandEvent& event);
-	void OnMenuEditSettings(wxCommandEvent& event);
+	void OnMenuEditSettings(wxCommandEvent&);
 	void OnToggleToolBar(wxCommandEvent& event);
-	void OnToggleLogView(wxCommandEvent& event);
-	void OnToggleLocalTreeView(wxCommandEvent& event);
-	void OnToggleRemoteTreeView(wxCommandEvent& event);
+	void OnToggleLogView(wxCommandEvent&);
+	void OnToggleDirectoryTreeView(wxCommandEvent& event);
 	void OnToggleQueueView(wxCommandEvent& event);
-	void OnMenuHelpAbout(wxCommandEvent& event);
+	void OnMenuHelpAbout(wxCommandEvent&);
 	void OnFilter(wxCommandEvent& event);
 	void OnFilterRightclicked(wxCommandEvent& event);
 #if FZ_MANUALUPDATECHECK
@@ -132,7 +144,6 @@ protected:
 #endif //FZ_MANUALUPDATECHECK
 	void OnSitemanagerDropdown(wxCommandEvent& event);
 	void OnNavigationKeyEvent(wxNavigationKeyEvent& event);
-	void OnGetFocus(wxFocusEvent& event);
 	void OnChar(wxKeyEvent& event);
 	void OnActivate(wxActivateEvent& event);
 	void OnToolbarComparison(wxCommandEvent& event);
@@ -140,9 +151,11 @@ protected:
 	void OnDropdownComparisonMode(wxCommandEvent& event);
 	void OnDropdownComparisonHide(wxCommandEvent& event);
 	void OnSyncBrowse(wxCommandEvent& event);
-#ifndef __WXMAC__
+#ifdef __WXMAC__
+	void OnChildFocused(wxChildFocusEvent& event);
+#else
 	void OnIconize(wxIconizeEvent& event);
-	void OnTaskBarClick(wxTaskBarIconEvent& event);
+	void OnTaskBarClick(wxTaskBarIconEvent&);
 #endif
 #ifdef __WXGTK__
 	void OnTaskBarClick_Delayed(wxCommandEvent& event);
@@ -151,20 +164,20 @@ protected:
 	void OnMenuNewTab(wxCommandEvent& event);
 	void OnMenuCloseTab(wxCommandEvent& event);
 
-	bool m_bInitDone;
-	bool m_bQuit;
-	wxEventType m_closeEvent;
+	bool m_bInitDone{};
+	bool m_bQuit{};
+	wxEventType m_closeEvent{};
 	wxTimer m_closeEventTimer;
 
-	CAsyncRequestQueue* m_pAsyncRequestQueue;
-	CMainFrameStateEventHandler* m_pStateEventHandler;
+	CAsyncRequestQueue* m_pAsyncRequestQueue{};
+	CMainFrameStateEventHandler* m_pStateEventHandler{};
 
-	CWindowStateManager* m_pWindowStateManager;
+	CWindowStateManager* m_pWindowStateManager{};
 
-	CQueue* m_pQueuePane;
+	CQueue* m_pQueuePane{};
 
 #ifndef __WXMAC__
-	wxTaskBarIcon* m_taskBarIcon;
+	wxTaskBarIcon* m_taskBarIcon{};
 #endif
 #ifdef __WXGTK__
 	// There is a bug in KDE, causing the window to toggle iconized state
@@ -172,10 +185,14 @@ protected:
 	// Set m_taskbar_is_uniconizing in OnTaskBarClick and unset the
 	// next time the pending event processing runs and calls OnTaskBarClick_Delayed.
 	// While set, ignore iconize events.
-	bool m_taskbar_is_uniconizing;
+	bool m_taskbar_is_uniconizing{};
 #endif
 
-	int m_comparisonToggleAcceleratorId;
+	int m_comparisonToggleAcceleratorId{};
+
+#ifdef __WXMAC__
+	int m_lastFocusedChild{-1};
+#endif
 };
 
 #endif
