@@ -4,9 +4,10 @@
 #include "filezillaapp.h"
 #include <wx/tokenzr.h>
 #include "ipcmutex.h"
-#include "option_change_event_handler.h"
+#include <option_change_event_handler.h>
 #include "sizeformatting.h"
 
+#include <algorithm>
 #include <string>
 
 #ifdef __WXMSW__
@@ -18,10 +19,6 @@
 	#endif
 #endif
 
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
-
 COptions* COptions::m_theOptions = 0;
 
 enum Type
@@ -46,6 +43,18 @@ struct t_Option
 	const Flags flags; // internal items won't get written to settings file nor loaded from there
 };
 
+#ifdef __WXMSW__
+//case insensitive
+#define DEFAULT_FILENAME_SORT   _T("0")
+#else
+//case sensitive
+#define DEFAULT_FILENAME_SORT   _T("1")
+#endif
+
+// In C++14 we should be able to use this instead:
+//   static_assert(OPTIONS_NUM <= changed_options_t().size());
+static_assert(OPTIONS_NUM <= changed_options_size, "OPTIONS_NUM too big for changed_options_t");
+
 static const t_Option options[OPTIONS_NUM] =
 {
 	// Note: A few options are versioned due to a changed
@@ -73,6 +82,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Speedlimit inbound", number, _T("100"), normal },
 	{ "Speedlimit outbound", number, _T("20"), normal },
 	{ "Speedlimit burst tolerance", number, _T("0"), normal },
+	{ "Preallocate space", number, _T("0"), normal },
 	{ "View hidden files", number, _T("0"), normal },
 	{ "Preserve timestamps", number, _T("0"), normal },
 	{ "Socket recv buffer size (v2)", number, _T("4194304"), normal }, // Make it large enough by default
@@ -92,7 +102,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Proxy password", string, _T(""), normal },
 	{ "Logging file", string, _T(""), normal },
 	{ "Logging filesize limit", number, _T("10"), normal },
-	{ "Trusted root certificate", string, _T(""), internal },
+	{ "Logging show detailed logs", number, _T("0"), internal },
 	{ "Size format", number, _T("0"), normal },
 	{ "Size thousands separator", number, _T("1"), normal },
 	{ "Size decimal places", number, _T("1"), normal },
@@ -100,7 +110,7 @@ static const t_Option options[OPTIONS_NUM] =
 	// Interface settings
 	{ "Number of Transfers", number, _T("2"), normal },
 	{ "Ascii Binary mode", number, _T("0"), normal },
-	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
+	{ "Auto Ascii files", string, _T("am|asp|bat|c|cfm|cgi|conf|cpp|css|dhtml|diz|h|hpp|htm|html|in|inc|java|js|jsp|lua|m4|mak|md5|nfo|nsi|pas|patch|php|phtml|pl|po|py|qmail|sh|sha1|sha256|sha512|shtml|sql|svg|tcl|tpl|txt|vbs|xhtml|xml|xrc"), normal },
 	{ "Auto Ascii no extension", number, _T("1"), normal },
 	{ "Auto Ascii dotfiles", number, _T("1"), normal },
 	{ "Theme", string, _T("opencrystal/"), normal },
@@ -113,7 +123,6 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Last automatic update check", string, _T(""), normal },
 	{ "Update Check New Version", string, _T(""), normal },
 	{ "Update Check Check Beta", number, _T("0"), normal },
-	{ "Update Check Download Dir", string, _T(""), normal },
 	{ "Show debug menu", number, _T("0"), normal },
 	{ "File exists action download", number, _T("0"), normal },
 	{ "File exists action upload", number, _T("0"), normal },
@@ -126,6 +135,7 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "File Pane Swap", number, _T("0"), normal },
 	{ "Last local directory", string, _T(""), normal },
 	{ "Filelist directory sort", number, _T("0"), normal },
+	{ "Filelist name sort", number, DEFAULT_FILENAME_SORT, normal },
 	{ "Queue successful autoclear", number, _T("0"), normal },
 	{ "Queue column widths", string, _T(""), normal },
 	{ "Local filelist colwidths", string, _T(""), normal },
@@ -171,10 +181,14 @@ static const t_Option options[OPTIONS_NUM] =
 	{ "Filteredit window size", string, _T(""), normal },
 	{ "Enable invalid char filter", number, _T("1"), normal },
 	{ "Invalid char replace", string, _T("_"), normal },
-	{ "Already connected choice", number, _T("0"), normal },	
+	{ "Already connected choice", number, _T("0"), normal },
 	{ "Edit status dialog size", string, _T(""), normal },
 	{ "Display current speed", number, _T("0"), normal },
 	{ "Toolbar hidden", number, _T("0"), normal },
+	{ "Strip VMS revisions", number, _T("0"), normal },
+	{ "Show Site Manager on startup", number, _T("0"), normal },
+	{ "Prompt password change", number, _T("0"), normal },
+	{ "Persistent Choices", number, _T("0"), normal },
 
 	// Default/internal options
 	{ "Config Location", string, _T(""), default_only },
@@ -186,31 +200,46 @@ BEGIN_EVENT_TABLE(COptions, wxEvtHandler)
 EVT_TIMER(wxID_ANY, COptions::OnTimer)
 END_EVENT_TABLE()
 
+t_OptionsCache& t_OptionsCache::operator=(wxString const& v)
+{
+	strValue = v;
+	long l;
+	if (v.ToLong(&l)) {
+		numValue = l;
+	}
+	else {
+		numValue = 0;
+	}
+	return *this;
+}
+
+t_OptionsCache& t_OptionsCache::operator=(int v)
+{
+	numValue = v;
+	strValue.Printf("%d", v);
+	return *this;
+}
+
 COptions::COptions()
 {
 	m_theOptions = this;
 	m_pXmlFile = 0;
 	m_pLastServer = 0;
 
-	m_acquired = false;
-
 	SetDefaultValues();
 
 	m_save_timer.SetOwner(this);
 
-	std::map<std::string, int> nameOptionMap;
-	GetNameOptionMap(nameOptionMap);
-
+	auto const nameOptionMap = GetNameOptionMap();
 	LoadGlobalDefaultOptions(nameOptionMap);
 
-	InitSettingsDir();
+	CLocalPath const dir = InitSettingsDir();
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
-	m_pXmlFile = new CXmlFile(_T("filezilla"));
-	if (!m_pXmlFile->Load())
-	{
+	m_pXmlFile = new CXmlFile(dir.GetPath() + _T("filezilla.xml"));
+	if (!m_pXmlFile->Load()) {
 		wxString msg = m_pXmlFile->GetError() + _T("\n\n") + _("For this session the default settings will be used. Any changes to the settings will not be saved.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 		delete m_pXmlFile;
 		m_pXmlFile = 0;
 	}
@@ -220,13 +249,14 @@ COptions::COptions()
 	LoadOptions(nameOptionMap);
 }
 
-void COptions::GetNameOptionMap(std::map<std::string, int>& nameOptionMap) const
+std::map<std::string, unsigned int> COptions::GetNameOptionMap() const
 {
-	for (int i = 0; i < OPTIONS_NUM; ++i)
-	{
+	std::map<std::string, unsigned int> ret;
+	for (unsigned int i = 0; i < OPTIONS_NUM; ++i) {
 		if (options[i].flags != internal)
-			nameOptionMap.insert(std::make_pair(std::string(options[i].name), i));
+			ret.insert(std::make_pair(std::string(options[i].name), i));
 	}
+	return ret;
 }
 
 COptions::~COptions()
@@ -242,20 +272,16 @@ int COptions::GetOptionVal(unsigned int nID)
 	if (nID >= OPTIONS_NUM)
 		return 0;
 
-	if (options[nID].type != number)
-		return 0;
-
+	scoped_lock l(m_sync_);
 	return m_optionsCache[nID].numValue;
 }
 
 wxString COptions::GetOption(unsigned int nID)
 {
 	if (nID >= OPTIONS_NUM)
-		return _T("");
-
-	if (options[nID].type != string)
-		return wxString::Format(_T("%d"), GetOptionVal(nID));
+		return wxString();
 
+	scoped_lock l(m_sync_);
 	return m_optionsCache[nID].strValue;
 }
 
@@ -267,36 +293,16 @@ bool COptions::SetOption(unsigned int nID, int value)
 	if (options[nID].type != number)
 		return false;
 
-	value = Validate(nID, value);
-
-	if (m_optionsCache[nID].numValue == value)
-	{
-		// Nothing to do
-		return true;
-	}
-
-	m_optionsCache[nID].numValue = value;
-
-	if (m_pXmlFile && options[nID].flags == normal)
-	{
-		SetXmlValue(nID, wxString::Format(_T("%d"), value));
-
-		if (!m_save_timer.IsRunning())
-			m_save_timer.Start(15000, true);
-	}
-
-	COptionChangeEventHandler::DoNotify(nID);
-
+	ContinueSetOption(nID, value);
 	return true;
 }
 
-bool COptions::SetOption(unsigned int nID, wxString value)
+bool COptions::SetOption(unsigned int nID, wxString const& value)
 {
 	if (nID >= OPTIONS_NUM)
 		return false;
 
-	if (options[nID].type != string)
-	{
+	if (options[nID].type != string) {
 		long tmp;
 		if (!value.ToLong(&tmp))
 			return false;
@@ -304,26 +310,47 @@ bool COptions::SetOption(unsigned int nID, wxString value)
 		return SetOption(nID, tmp);
 	}
 
-	Validate(nID, value);
+	ContinueSetOption(nID, value);
+	return true;
+}
+
+template<typename T>
+void COptions::ContinueSetOption(unsigned int nID, T const& value)
+{
+	T validated = Validate(nID, value);
 
-	if (m_optionsCache[nID].strValue == value)
 	{
-		// Nothing to do
-		return true;
+		scoped_lock l(m_sync_);
+		if (m_optionsCache[nID] == validated) {
+			// Nothing to do
+			return;
+		}
+		m_optionsCache[nID] = validated;
 	}
-	m_optionsCache[nID].strValue = value;
 
-	if (m_pXmlFile && options[nID].flags == normal)
-	{
-		SetXmlValue(nID, value);
+	// Fixme: Setting options from other threads
+	if (!wxIsMainThread())
+		return;
+
+	if (options[nID].flags == normal || options[nID].flags == default_priority) {
+		SetXmlValue(nID, validated);
 
 		if (!m_save_timer.IsRunning())
 			m_save_timer.Start(15000, true);
 	}
 
-	COptionChangeEventHandler::DoNotify(nID);
+	if (changedOptions_.none()) {
+		CallAfter(&COptions::NotifyChangedOptions);
+	}
+	changedOptions_.set(nID);
+}
 
-	return true;
+void COptions::NotifyChangedOptions()
+{
+	// Reset prior to notifying to correctly handle the case of an option being set while notifying
+	auto changedOptions = changedOptions_;
+	changedOptions_.reset();
+	COptionChangeEventHandler::DoNotify(changedOptions);
 }
 
 bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
@@ -331,95 +358,76 @@ bool COptions::OptionFromFzDefaultsXml(unsigned int nID)
 	if (nID >= OPTIONS_NUM)
 		return false;
 
+	scoped_lock l(m_sync_);
 	return m_optionsCache[nID].from_default;
 }
 
-void COptions::CreateSettingsXmlElement()
+TiXmlElement* COptions::CreateSettingsXmlElement()
 {
 	if (!m_pXmlFile)
-		return;
+		return 0;
 
-	TiXmlElement *element = m_pXmlFile->GetElement();
-	if (element->FirstChildElement("Settings"))
-		return;
-
-	TiXmlElement *settings = new TiXmlElement("Settings");
-	element->LinkEndChild(settings);
-
-	for (int i = 0; i < OPTIONS_NUM; i++)
-	{
-		if (options[i].type == string)
-			SetXmlValue(i, m_optionsCache[i].strValue);
-		else
-		{
-			wxString s(wxString::Format(_T("%d"), m_optionsCache[i].numValue));
-			SetXmlValue(i, s);
-		}
-		
+	TiXmlElement* element = m_pXmlFile->GetElement();
+	if (!element) {
+		return 0;
 	}
 
-	m_pXmlFile->Save();
+	TiXmlElement* settings = element->FirstChildElement("Settings");
+	if (settings) {
+		return settings;
+	}
+
+	settings = new TiXmlElement("Settings");
+	element->LinkEndChild(settings);
+
+	for (int i = 0; i < OPTIONS_NUM; ++i) {
+		if (options[i].type == string) {
+			SetXmlValue(i, GetOption(i));
+		}
+		else {
+			SetXmlValue(i, GetOptionVal(i));
+		}
+	}
+
+	return settings;
 }
 
-void COptions::SetXmlValue(unsigned int nID, wxString value)
+void COptions::SetXmlValue(unsigned int nID, int value)
+{
+	SetXmlValue(nID, wxString::Format(_T("%d"), value));
+}
+
+void COptions::SetXmlValue(unsigned int nID, wxString const& value)
 {
 	if (!m_pXmlFile)
 		return;
 
 	// No checks are made about the validity of the value, that's done in SetOption
 
-	char *utf8 = ConvUTF8(value);
+	wxScopedCharBuffer utf8 = value.utf8_str();
 	if (!utf8)
 		return;
 
-	TiXmlElement *settings = m_pXmlFile->GetElement()->FirstChildElement("Settings");
-	if (!settings)
-	{
-		TiXmlNode *node = m_pXmlFile->GetElement()->LinkEndChild(new TiXmlElement("Settings"));
-		if (!node)
-		{
-			delete [] utf8;
-			return;
-		}
-		settings = node->ToElement();
-		if (!settings)
-		{
-			delete [] utf8;
-			return;
-		}
-	}
-	else
-	{
-		TiXmlNode *node = 0;
-		while ((node = settings->IterateChildren("Setting", node)))
-		{
-			TiXmlElement *setting = node->ToElement();
-			if (!setting)
-				continue;
-
+	TiXmlElement *settings = CreateSettingsXmlElement();
+	if (settings) {
+		TiXmlElement *setting = 0;
+		for (setting = settings->FirstChildElement("Setting"); setting; setting = setting->NextSiblingElement("Setting")) {
 			const char *attribute = setting->Attribute("name");
 			if (!attribute)
 				continue;
-			if (strcmp(attribute, options[nID].name))
-				continue;
-
-			//setting->RemoveAttribute("type");
-			setting->Clear();
-			//setting->SetAttribute("type", (options[nID].type == string) ? "string" : "number");
-			setting->LinkEndChild(new TiXmlText(utf8));
-
-			delete [] utf8;
-			return;
+			if (!strcmp(attribute, options[nID].name))
+				break;
 		}
+		if (setting) {
+			setting->Clear();
+		}
+		else {
+			setting = new TiXmlElement("Setting");
+			setting->SetAttribute("name", options[nID].name);
+			settings->LinkEndChild(setting);
+		}
+		setting->LinkEndChild(new TiXmlText(utf8));
 	}
-	wxASSERT(options[nID].name[0]);
-	TiXmlElement *setting = new TiXmlElement("Setting");
-	setting->SetAttribute("name", options[nID].name);
-	//setting->SetAttribute("type", (options[nID].type == string) ? "string" : "number");
-	setting->LinkEndChild(new TiXmlText(utf8));
-	settings->LinkEndChild(setting);
-
-	delete [] utf8;
 }
 
 int COptions::Validate(unsigned int nID, int value)
@@ -427,7 +435,7 @@ int COptions::Validate(unsigned int nID, int value)
 	switch (nID)
 	{
 	case OPTION_UPDATECHECK_INTERVAL:
-		if (value < 7 || value > 9999)
+		if (value < 1 || value > 7)
 			value = 7;
 		break;
 	case OPTION_LOGGING_DEBUGLEVEL:
@@ -456,6 +464,7 @@ int COptions::Validate(unsigned int nID, int value)
 			value = 0;
 		break;
 	case OPTION_FILELIST_DIRSORT:
+	case OPTION_FILELIST_NAMESORT:
 		if (value < 0 || value > 2)
 			value = 0;
 		break;
@@ -496,12 +505,11 @@ int COptions::Validate(unsigned int nID, int value)
 	return value;
 }
 
-wxString COptions::Validate(unsigned int nID, wxString value)
+wxString COptions::Validate(unsigned int nID, wxString const& value)
 {
-	if (nID == OPTION_INVALID_CHAR_REPLACE)
-	{
+	if (nID == OPTION_INVALID_CHAR_REPLACE) {
 		if (value.Len() > 1)
-			value = _T("_");
+			return _T("_");
 	}
 	return value;
 }
@@ -511,39 +519,30 @@ void COptions::SetServer(wxString path, const CServer& server)
 	if (!m_pXmlFile)
 		return;
 
-	if (path == _T(""))
+	if (path.empty())
 		return;
 
 	TiXmlElement *element = m_pXmlFile->GetElement();
 
-	while (path != _T(""))
-	{
+	while (!path.empty()) {
 		wxString sub;
 		int pos = path.Find('/');
-		if (pos != -1)
-		{
+		if (pos != -1) {
 			sub = path.Left(pos);
 			path = path.Mid(pos + 1);
 		}
-		else
-		{
+		else {
 			sub = path;
 			path = _T("");
 		}
-		char *utf8 = ConvUTF8(sub);
+		wxScopedCharBuffer utf8 = sub.utf8_str();
 		if (!utf8)
 			return;
 		TiXmlElement *newElement = element->FirstChildElement(utf8);
-		delete [] utf8;
 		if (newElement)
 			element = newElement;
-		else
-		{
-			char *utf8 = ConvUTF8(sub);
-			if (!utf8)
-				return;
+		else {
 			TiXmlNode *node = element->LinkEndChild(new TiXmlElement(utf8));
-			delete [] utf8;
 			if (!node || !node->ToElement())
 				return;
 			element = node->ToElement();
@@ -554,39 +553,35 @@ void COptions::SetServer(wxString path, const CServer& server)
 
 	if (GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 		return;
-	
+
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
-	m_pXmlFile->Save();
+	m_pXmlFile->Save(true);
 }
 
 bool COptions::GetServer(wxString path, CServer& server)
 {
-	if (path == _T(""))
+	if (path.empty())
 		return false;
 
 	if (!m_pXmlFile)
 		return false;
 	TiXmlElement *element = m_pXmlFile->GetElement();
 
-	while (path != _T(""))
-	{
+	while (!path.empty()) {
 		wxString sub;
 		int pos = path.Find('/');
-		if (pos != -1)
-		{
+		if (pos != -1) {
 			sub = path.Left(pos);
 			path = path.Mid(pos + 1);
 		}
-		else
-		{
+		else {
 			sub = path;
 			path = _T("");
 		}
-		char *utf8 = ConvUTF8(sub);
+		wxScopedCharBuffer utf8 = sub.utf8_str();
 		if (!utf8)
 			return false;
 		element = element->FirstChildElement(utf8);
-		delete [] utf8;
 		if (!element)
 			return false;
 	}
@@ -607,15 +602,13 @@ void COptions::SetLastServer(const CServer& server)
 
 bool COptions::GetLastServer(CServer& server)
 {
-	if (!m_pLastServer)
-	{
+	if (!m_pLastServer) {
 		bool res = GetServer(_T("Settings/LastServer"), server);
 		if (res)
 			m_pLastServer = new CServer(server);
 		return res;
 	}
-	else
-	{
+	else {
 		server = *m_pLastServer;
 		if (server == CServer())
 			return false;
@@ -646,35 +639,22 @@ COptions* COptions::Get()
 
 void COptions::Import(TiXmlElement* pElement)
 {
-	std::map<std::string, int> nameOptionMap;
-	GetNameOptionMap(nameOptionMap);
-	LoadOptions(nameOptionMap, pElement);
+	LoadOptions(GetNameOptionMap(), pElement);
 	if (!m_save_timer.IsRunning())
 		m_save_timer.Start(15000, true);
 }
 
-void COptions::LoadOptions(const std::map<std::string, int>& nameOptionMap, TiXmlElement* settings /*=0*/)
+void COptions::LoadOptions(std::map<std::string, unsigned int> const& nameOptionMap, TiXmlElement* settings)
 {
-	if (!settings)
-	{
-		if (!m_pXmlFile)
+	if (!settings) {
+		settings = CreateSettingsXmlElement();
+		if (!settings) {
 			return;
-
-		settings = m_pXmlFile->GetElement()->FirstChildElement("Settings");
-		if (!settings)
-		{
-			TiXmlNode *node = m_pXmlFile->GetElement()->LinkEndChild(new TiXmlElement("Settings"));
-			if (!node)
-				return;
-			settings = node->ToElement();
-			if (!settings)
-				return;
 		}
 	}
 
 	TiXmlNode *node = 0;
-	while ((node = settings->IterateChildren("Setting", node)))
-	{
+	while ((node = settings->IterateChildren("Setting", node))) {
 		TiXmlElement *setting = node->ToElement();
 		if (!setting)
 			continue;
@@ -682,15 +662,14 @@ void COptions::LoadOptions(const std::map<std::string, int>& nameOptionMap, TiXm
 	}
 }
 
-void COptions::LoadOptionFromElement(TiXmlElement* pOption, const std::map<std::string, int>& nameOptionMap, bool allowDefault)
+void COptions::LoadOptionFromElement(TiXmlElement* pOption, std::map<std::string, unsigned int> const& nameOptionMap, bool allowDefault)
 {
 	const char* name = pOption->Attribute("name");
 	if (!name)
 		return;
 
-	std::map<std::string, int>::const_iterator iter = nameOptionMap.find(name);
-	if (iter != nameOptionMap.end())
-	{
+	auto const iter = nameOptionMap.find(name);
+	if (iter != nameOptionMap.end()) {
 		if (!allowDefault && options[iter->second].flags == default_only)
 			return;
 		wxString value;
@@ -703,40 +682,40 @@ void COptions::LoadOptionFromElement(TiXmlElement* pOption, const std::map<std::
 			value = ConvLocal(text->Value());
 		}
 
-		if (options[iter->second].flags == default_priority)
-		{
-			if (allowDefault)
+		if (options[iter->second].flags == default_priority) {
+			if (allowDefault) {
+				scoped_lock l(m_sync_);
 				m_optionsCache[iter->second].from_default = true;
-			else
-			{
+			}
+			else {
+				scoped_lock l(m_sync_);
 				if (m_optionsCache[iter->second].from_default)
 					return;
 			}
 		}
 
-		if (options[iter->second].type == number)
-		{
+		if (options[iter->second].type == number) {
 			long numValue = 0;
 			value.ToLong(&numValue);
 			numValue = Validate(iter->second, numValue);
-			m_optionsCache[iter->second].numValue = numValue;
+			scoped_lock l(m_sync_);
+			m_optionsCache[iter->second] = numValue;
 		}
-		else
-		{
+		else {
 			value = Validate(iter->second, value);
-			m_optionsCache[iter->second].strValue = value;
+			scoped_lock l(m_sync_);
+			m_optionsCache[iter->second] = value;
 		}
 	}
 }
 
-void COptions::LoadGlobalDefaultOptions(const std::map<std::string, int>& nameOptionMap)
+void COptions::LoadGlobalDefaultOptions(std::map<std::string, unsigned int> const& nameOptionMap)
 {
-	const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir == _T(""))
+	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+	if (defaultsDir.empty())
 		return;
-	
-	wxFileName name(defaultsDir, _T("fzdefaults.xml"));
-	CXmlFile file(name);
+
+	CXmlFile file(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	if (!file.Load())
 		return;
 
@@ -748,13 +727,12 @@ void COptions::LoadGlobalDefaultOptions(const std::map<std::string, int>& nameOp
 	if (!pElement)
 		return;
 
-	for (TiXmlElement* pSetting = pElement->FirstChildElement("Setting"); pSetting; pSetting = pSetting->NextSiblingElement("Setting"))
-	{
+	for (TiXmlElement* pSetting = pElement->FirstChildElement("Setting"); pSetting; pSetting = pSetting->NextSiblingElement("Setting")) {
 		LoadOptionFromElement(pSetting, nameOptionMap, true);
 	}
 }
 
-void COptions::OnTimer(wxTimerEvent& event)
+void COptions::OnTimer(wxTimerEvent&)
 {
 	Save();
 }
@@ -768,7 +746,7 @@ void COptions::Save()
 		return;
 
 	CInterProcessMutex mutex(MUTEX_OPTIONS);
-	m_pXmlFile->Save();
+	m_pXmlFile->Save(true);
 }
 
 void COptions::SaveIfNeeded()
@@ -780,27 +758,92 @@ void COptions::SaveIfNeeded()
 	Save();
 }
 
-void COptions::InitSettingsDir()
+namespace {
+#ifndef __WXMSW__
+wxString TryDirectory( wxString path, wxString const& suffix, bool check_exists )
+{
+	if( !path.empty() && path[0] == '/' ) {
+		if( path[path.size() - 1] != '/' ) {
+			path += '/';
+		}
+
+		path += suffix;
+
+		if( check_exists ) {
+			if( !wxFileName::DirExists(path) ) {
+				path.clear();
+			}
+		}
+	}
+	else {
+		path.clear();
+	}
+	return path;
+}
+#endif
+
+wxString GetEnv(wxString const& env)
+{
+	wxString ret;
+	if( !wxGetEnv(env, &ret) ) {
+		ret.clear();
+	}
+	return ret;
+}
+}
+
+CLocalPath COptions::GetUnadjustedSettingsDir()
 {
 	wxFileName fn;
+#ifdef __WXMSW__
+	wxChar buffer[MAX_PATH * 2 + 1];
+
+	if (SUCCEEDED(SHGetFolderPath(0, CSIDL_APPDATA, 0, SHGFP_TYPE_CURRENT, buffer))) {
+		fn = wxFileName(buffer, _T(""));
+		fn.AppendDir(_T("FileZilla"));
+	}
+	else {
+		// Fall back to directory where the executable is
+		if (GetModuleFileName(0, buffer, MAX_PATH * 2))
+			fn = buffer;
+	}
+#else
+	wxString cfg = TryDirectory(GetEnv(_T("XDG_CONFIG_HOME")), _T("filezilla/"), true);
+	if( cfg.empty() ) {
+		cfg = TryDirectory(wxGetHomeDir(), _T(".config/filezilla/"), true);
+	}
+	if( cfg.empty() ) {
+		cfg = TryDirectory(wxGetHomeDir(), _T(".filezilla/"), true);
+	}
+	if( cfg.empty() ) {
+		cfg = TryDirectory(GetEnv(_T("XDG_CONFIG_HOME")), _T("filezilla/"), false);
+	}
+	if( cfg.empty() ) {
+		cfg = TryDirectory(wxGetHomeDir(), _T(".config/filezilla/"), false);
+	}
+	if( cfg.empty() ) {
+		cfg = TryDirectory(wxGetHomeDir(), _T(".filezilla/"), false);
+	}
+	fn = wxFileName(cfg, _T(""));
+#endif
+	return CLocalPath(fn.GetPath());
+}
+
+CLocalPath COptions::InitSettingsDir()
+{
+	CLocalPath p;
 
 	wxString dir(GetOption(OPTION_DEFAULT_SETTINGSDIR));
-	if (!dir.empty())
-	{
+	if (!dir.empty()) {
 		wxStringTokenizer tokenizer(dir, _T("/\\"), wxTOKEN_RET_EMPTY_ALL);
 		dir = _T("");
-		while (tokenizer.HasMoreTokens())
-		{
+		while (tokenizer.HasMoreTokens()) {
 			wxString token = tokenizer.GetNextToken();
-			if (token[0] == '$')
-			{
-				if (token[1] == '$')
+			if (!token.empty() && token[0] == '$') {
+				if (token.size() > 1 && token[1] == '$')
 					token = token.Mid(1);
-				else
-				{
-					wxString value;
-					if (wxGetEnv(token.Mid(1), &value))
-						token = value;
+				else {
+					token = GetEnv(token.Mid(1));
 				}
 			}
 			dir += token;
@@ -809,47 +852,26 @@ void COptions::InitSettingsDir()
 				dir += delimiter;
 		}
 
-		fn = wxFileName(dir, _T(""));
-		fn.Normalize(wxPATH_NORM_ALL, wxGetApp().GetDefaultsDir());
+		p.SetPath(wxGetApp().GetDefaultsDir().GetPath());
+		p.ChangePath(dir);
+	}
+	else {
+		p = GetUnadjustedSettingsDir();
 	}
-	else
-	{
-#ifdef __WXMSW__
-		wxChar buffer[MAX_PATH * 2 + 1];
 
-		if (SUCCEEDED(SHGetFolderPath(0, CSIDL_APPDATA, 0, SHGFP_TYPE_CURRENT, buffer)))
-		{
-			fn = wxFileName(buffer, _T(""));
-			fn.AppendDir(_T("FileZilla"));
-		}
-		else
-		{
-			// Fall back to directory where the executable is
-			if (GetModuleFileName(0, buffer, MAX_PATH * 2))
-				fn = buffer;
-		}
-#else
-		fn = wxFileName(wxGetHomeDir(), _T(""));
-		fn.AppendDir(_T(".filezilla"));
-#endif
-	}
-	if (!fn.DirExists())
-		wxMkdir(fn.GetPath(), 0700);
-	SetOption(OPTION_DEFAULT_SETTINGSDIR, fn.GetPath());
+	if (!p.empty() && !p.Exists())
+		wxFileName::Mkdir( p.GetPath(), 0700, wxPATH_MKDIR_FULL );
+
+	SetOption(OPTION_DEFAULT_SETTINGSDIR, p.GetPath());
+
+	return p;
 }
 
 void COptions::SetDefaultValues()
 {
-	for (int i = 0; i < OPTIONS_NUM; ++i)
-	{
-		if (options[i].type == string)
-			m_optionsCache[i].strValue = options[i].defaultValue;
-		else
-		{
-			long numValue = 0;
-			options[i].defaultValue.ToLong(&numValue);
-			m_optionsCache[i].numValue = numValue;
-		}
+	scoped_lock l(m_sync_);
+	for (int i = 0; i < OPTIONS_NUM; ++i) {
+		m_optionsCache[i] = options[i].defaultValue;
 		m_optionsCache[i].from_default = false;
 	}
 }
