@@ -10,27 +10,20 @@ class CChmodDialog;
 class CInfoText;
 class CRemoteListViewDropTarget;
 
-class CRemoteListView : public CFileListCtrl<CGenericFileData>, CSystemImageList, CStateEventHandler
+class CRemoteListView : public CFileListCtrl<CGenericFileData>, CStateEventHandler
 {
 	friend class CRemoteListViewDropTarget;
-	friend class CRemoteListViewSortType;
-
 public:
 	CRemoteListView(wxWindow* pParent, CState* pState, CQueueView* pQueue);
 	virtual ~CRemoteListView();
 
-	bool DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, const CLocalPath& path, bool queueOnly);
-
-	void InitDateFormat();
-
 	virtual bool CanStartComparison(wxString* pError);
 	virtual void StartComparison();
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, wxDateTime& date, bool& hasTime);
+	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, CDateTime& date);
 	virtual void FinishComparison();
 	virtual void OnExitComparisonMode();
 
 	void LinkIsNotDir(const CServerPath& path, const wxString& link);
-
 protected:
 	virtual wxString GetItemText(int item, unsigned int column);
 
@@ -40,7 +33,7 @@ protected:
 	// Select a list of items based in their names.
 	// Sort order may not change between call to RememberSelectedItems and
 	// ReselectItems
-	void ReselectItems(std::list<wxString>& selectedNames, wxString focused);
+	void ReselectItems(std::list<wxString>& selectedNames, wxString focused, bool ensureVisible = false);
 
 
 	// Declared const due to design error in wxWidgets.
@@ -53,24 +46,25 @@ protected:
 	virtual bool ItemIsDir(int index) const;
 	virtual wxLongLong ItemGetSize(int index) const;
 
-protected:
 	bool IsItemValid(unsigned int item) const;
 	int GetItemIndex(unsigned int item) const;
 
 	virtual CSortComparisonObject GetSortComparisonObject();
 
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
+	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 	void ApplyCurrentFilter();
-	void SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing, bool modified = false);
-	bool UpdateDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing);
-	void UpdateDirectoryListing_Removed(const CSharedPointer<const CDirectoryListing> &pDirectoryListing);
-	void UpdateDirectoryListing_Added(const CSharedPointer<const CDirectoryListing> &pDirectoryListing);
+	void SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
+	bool UpdateDirectoryListing(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
+	void UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
+	void UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryListing> const& pDirectoryListing);
 
 #ifdef __WXDEBUG__
 	void ValidateIndexMapping();
 #endif
 
-	CSharedPointer <const CDirectoryListing> m_pDirectoryListing;
+	virtual void OnNavigationEvent(bool forward);
+
+	std::shared_ptr<CDirectoryListing> m_pDirectoryListing;
 
 	// Caller is responsible to check selection is valid!
 	void TransferSelectedFiles(const CLocalPath& local_parent, bool queueOnly);
@@ -82,11 +76,14 @@ protected:
 	void RepositionInfoText();
 	void SetInfoText();
 
-	wxDropTarget* m_pDropTarget;
-	int m_dropTarget;
+	virtual bool OnBeginRename(const wxListEvent& event);
+	virtual bool OnAcceptRename(const wxListEvent& event);
 
-	wxString m_dateFormat;
-	wxString m_timeFormat;
+#ifdef __WXMSW__
+	virtual int GetOverlayIndex(int item);
+#endif
+
+	int m_dropTarget;
 
 	// Used to track state for resolving symlinks
 	// While being resolved, global state might have changed
@@ -96,25 +93,29 @@ protected:
 		CServer server;
 		CServerPath remote_path;
 		wxString link;
-		wxString local_path;
-	} *m_pLinkResolveState;
+		CLocalPath local_path;
+	};
+	std::unique_ptr<t_linkResolveState> m_pLinkResolveState;
+
+	CServerPath MenuMkdir();
 
 	DECLARE_EVENT_TABLE()
 	void OnItemActivated(wxListEvent &event);
 	void OnContextMenu(wxContextMenuEvent& event);
 	void OnMenuDownload(wxCommandEvent& event);
-	void OnMenuMkdir(wxCommandEvent& event);
-	void OnMenuDelete(wxCommandEvent& event);
-	void OnMenuRename(wxCommandEvent& event);
+	void OnMenuMkdir(wxCommandEvent&);
+	void OnMenuMkdirChgDir(wxCommandEvent&);
+	void OnMenuDelete(wxCommandEvent&);
+	void OnMenuRename(wxCommandEvent&);
 	void OnKeyDown(wxKeyEvent& event);
-	void OnBeginLabelEdit(wxListEvent& event);
-	void OnEndLabelEdit(wxListEvent& event);
 	void OnMenuChmod(wxCommandEvent& event);
 	void OnSize(wxSizeEvent& event);
-	void OnBeginDrag(wxListEvent& event);
-	void OnMenuEdit(wxCommandEvent& event);
-	void OnMenuEnter(wxCommandEvent& event);
-	void OnMenuGeturl(wxCommandEvent& event);
+	void OnBeginDrag(wxListEvent&);
+	void OnMenuEdit(wxCommandEvent&);
+	void OnMenuEnter(wxCommandEvent&);
+	void OnMenuGeturl(wxCommandEvent&);
+	void OnMenuRefresh(wxCommandEvent&);
+	void OnMenuNewfile(wxCommandEvent& event);
 };
 
 #endif
