@@ -7,14 +7,12 @@
 #include "recursive_operation.h"
 #include "inputdialog.h"
 #include "dragdropmanager.h"
+#include "drop_target_ex.h"
 #include <wx/clipbrd.h>
 #include "queue.h"
 #include "QueueView.h"
 #include "themeprovider.h"
-
-#ifdef _DEBUG
-#define new DEBUG_NEW
-#endif
+#include "Options.h"
 
 class CItemData : public wxTreeItemData
 {
@@ -23,12 +21,14 @@ public:
 	CServerPath m_path;
 };
 
-class CRemoteTreeViewDropTarget : public wxDropTarget
+class CRemoteTreeViewDropTarget : public CScrollableDropTarget<wxTreeCtrlEx>
 {
 public:
 	CRemoteTreeViewDropTarget(CRemoteTreeView* pRemoteTreeView)
-		: m_pRemoteTreeView(pRemoteTreeView), m_pFileDataObject(new wxFileDataObject()),
-		m_pRemoteDataObject(new CRemoteDataObject())
+		: CScrollableDropTarget<wxTreeCtrlEx>(pRemoteTreeView)
+		, m_pRemoteTreeView(pRemoteTreeView)
+		, m_pFileDataObject(new wxFileDataObject())
+		, m_pRemoteDataObject(new CRemoteDataObject())
 	{
 		m_pDataObject = new wxDataObjectComposite;
 		m_pDataObject->Add(m_pRemoteDataObject, true);
@@ -39,8 +39,7 @@ public:
 	void ClearDropHighlight()
 	{
 		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId())
-		{
+		if (dropHighlight != wxTreeItemId()) {
 			m_pRemoteTreeView->SetItemDropHighlight(dropHighlight, false);
 			m_pRemoteTreeView->m_dropHighlight = wxTreeItemId();
 		}
@@ -69,7 +68,7 @@ public:
 			return wxDragNone;
 
 		CServerPath path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.IsEmpty())
+		if (path.empty())
 			return wxDragNone;
 
 		if (!GetData())
@@ -85,40 +84,42 @@ public:
 		{
 			if (m_pRemoteDataObject->GetProcessId() != (int)wxGetProcessId())
 			{
-				wxMessageBox(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different instances of FileZilla has not been implemented yet."));
 				return wxDragNone;
 			}
 
-			if (!m_pRemoteTreeView->m_pState->GetServer() || m_pRemoteDataObject->GetServer() != *m_pRemoteTreeView->m_pState->GetServer())
+			if (!m_pRemoteTreeView->m_pState->GetServer() || !m_pRemoteDataObject->GetServer().EqualsNoPass(*m_pRemoteTreeView->m_pState->GetServer()))
 			{
-				wxMessageBox(_("Drag&drop between different servers has not been implemented yet."));
+				wxMessageBoxEx(_("Drag&drop between different servers has not been implemented yet."));
 				return wxDragNone;
 			}
 
 			// Make sure path path is valid
 			if (path == m_pRemoteDataObject->GetServerPath())
 			{
-				wxMessageBox(_("Source and path of the drop operation are identical"));
+				wxMessageBoxEx(_("Source and path of the drop operation are identical"));
 				return wxDragNone;
 			}
 
 			const std::list<CRemoteDataObject::t_fileInfo>& files = m_pRemoteDataObject->GetFiles();
-			for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+			for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 			{
 				const CRemoteDataObject::t_fileInfo& info = *iter;
 				if (info.dir)
 				{
 					CServerPath dir = m_pRemoteDataObject->GetServerPath();
 					dir.AddSegment(info.name);
-					if (dir == path || dir.IsParentOf(path, false))
+					if (dir == path)
+						return wxDragNone;
+					else if (dir.IsParentOf(path, false))
 					{
-						wxMessageBox(_("A directory cannot be dragged into one of its subdirectories."));
+						wxMessageBoxEx(_("A directory cannot be dragged into one of its subdirectories."));
 						return wxDragNone;
 					}
 				}
 			}
 
-			for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+			for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 			{
 				const CRemoteDataObject::t_fileInfo& info = *iter;
 				m_pRemoteTreeView->m_pState->m_pCommandQueue->ProcessCommand(
@@ -134,6 +135,9 @@ public:
 
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
+		if (!CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y)) {
+			return false;
+		}
 		ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
@@ -141,27 +145,25 @@ public:
 			return false;
 
 		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
-		if (path.IsEmpty())
+		if (path.empty())
 			return false;
 
 		return true;
 	}
 
-	CServerPath DisplayDropHighlight(wxPoint point)
+	wxTreeItemId DisplayDropHighlight(wxPoint point)
 	{
 		wxTreeItemId hit = GetHit(point);
-		if (!hit)
-		{
+		if (!hit) {
 			ClearDropHighlight();
-			return CServerPath();
+			return wxTreeItemId();
 		}
 
 		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
 
-		if (path.IsEmpty())
-		{
+		if (path.empty()) {
 			ClearDropHighlight();
-			return CServerPath();
+			return wxTreeItemId();
 		}
 
 		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
@@ -171,11 +173,13 @@ public:
 		m_pRemoteTreeView->SetItemDropHighlight(hit, true);
 		m_pRemoteTreeView->m_dropHighlight = hit;
 
-		return path;
+		return hit;
 	}
 
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnDragOver(x, y, def);
+
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -184,8 +188,8 @@ public:
 			return def;
 		}
 
-		const CServerPath& path = DisplayDropHighlight(wxPoint(x, y));
-		if (path.IsEmpty())
+		wxTreeItemId hit = DisplayDropHighlight(wxPoint(x, y));
+		if (!hit.IsOk())
 			return wxDragNone;
 
 		if (def == wxDragLink)
@@ -196,11 +200,13 @@ public:
 
 	virtual void OnLeave()
 	{
+		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
 		ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
 	{
+		def = CScrollableDropTarget<wxTreeCtrlEx>::OnEnter(x, y, def);
 		return OnDragOver(x, y, def);
 	}
 
@@ -218,9 +224,6 @@ EVT_TREE_ITEM_EXPANDING(wxID_ANY, CRemoteTreeView::OnItemExpanding)
 EVT_TREE_SEL_CHANGED(wxID_ANY, CRemoteTreeView::OnSelectionChanged)
 EVT_TREE_ITEM_ACTIVATED(wxID_ANY, CRemoteTreeView::OnItemActivated)
 EVT_TREE_BEGIN_DRAG(wxID_ANY, CRemoteTreeView::OnBeginDrag)
-#ifndef __WXMSW__
-EVT_KEY_DOWN(CRemoteTreeView::OnKeyDown)
-#endif //__WXMSW__
 EVT_TREE_ITEM_MENU(wxID_ANY, CRemoteTreeView::OnContextMenu)
 EVT_MENU(XRCID("ID_CHMOD"), CRemoteTreeView::OnMenuChmod)
 EVT_MENU(XRCID("ID_DOWNLOAD"), CRemoteTreeView::OnMenuDownload)
@@ -230,6 +233,7 @@ EVT_MENU(XRCID("ID_RENAME"), CRemoteTreeView::OnMenuRename)
 EVT_TREE_BEGIN_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnBeginLabelEdit)
 EVT_TREE_END_LABEL_EDIT(wxID_ANY, CRemoteTreeView::OnEndLabelEdit)
 EVT_MENU(XRCID("ID_MKDIR"), CRemoteTreeView::OnMkdir)
+EVT_MENU(XRCID("ID_MKDIR_CHGDIR"), CRemoteTreeView::OnMenuMkdirChgDir)
 EVT_CHAR(CRemoteTreeView::OnChar)
 EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
@@ -247,13 +251,16 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
 
+	CreateImageList();
+
+	UpdateSortMode();
+	RegisterOption(OPTION_FILELIST_NAMESORT);
+
 	m_busy = false;
 	m_pQueue = pQueue;
 	AddRoot(_T(""));
 	m_ExpandAfterList = wxTreeItemId();
 
-	CreateImageList();
-
 	SetDropTarget(new CRemoteTreeViewDropTarget(this));
 
 	Enable(false);
@@ -261,45 +268,46 @@ CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState
 
 CRemoteTreeView::~CRemoteTreeView()
 {
+	SetImageList(0);
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
 		SetDirectoryListing(pState->GetRemoteDir(), false);
 	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
 		SetDirectoryListing(pState->GetRemoteDir(), true);
 	else if (notification == STATECHANGE_APPLYFILTER)
-		ApplyFilters();
+		ApplyFilters(false);
 }
 
-void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pListing, bool modified)
+void CRemoteTreeView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> const& pListing, bool modified)
 {
 	m_busy = true;
 
-	if (!pListing)
-	{
+	if (!pListing) {
 		m_ExpandAfterList = wxTreeItemId();
 		DeleteAllItems();
 		AddRoot(_T(""));
 		m_busy = false;
-		if (FindFocus() == this)
-		{
-			wxNavigationKeyEvent evt;
-			evt.SetFromTab(true);
-			evt.SetEventObject(this);
-			evt.SetDirection(true);
-			AddPendingEvent(evt);
+		if (FindFocus() == this) {
+			wxNavigationKeyEvent *evt = new wxNavigationKeyEvent();
+			evt->SetFromTab(true);
+			evt->SetEventObject(this);
+			evt->SetDirection(true);
+			QueueEvent(evt);
 		}
 		Enable(false);
 		m_contextMenuItem = wxTreeItemId();
 		return;
 	}
 	Enable(true);
+#ifdef __WXGTK__
+	GetParent()->m_dirtyTabOrder = true;
+#endif
 
-	if (pListing->m_hasUnsureEntries && !(pListing->m_hasUnsureEntries & ~(CDirectoryListing::unsure_unknown | CDirectoryListing::unsure_file_mask)))
-	{
+	if (pListing->get_unsure_flags() && !(pListing->get_unsure_flags() & ~(CDirectoryListing::unsure_unknown | CDirectoryListing::unsure_file_mask))) {
 		// Just files changed, does not affect directory tree
 		m_busy = false;
 		return;
@@ -347,7 +355,7 @@ void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 	SetItemImages(parent, false);
 
 #ifndef __WXMSW__
-	m_freezeCount--;
+	Thaw();
 #endif
 	if (!modified)
 		SafeSelectItem(parent);
@@ -367,20 +375,20 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 		pieces.push_front(path.GetLastSegment());
 		path = path.GetParent();
 	}
-	wxASSERT(path.GetPath() != _T(""));
+	wxASSERT(!path.GetPath().empty());
 	pieces.push_front(path.GetPath());
 
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemId parent = root;
 
-	for (std::list<wxString>::const_iterator iter = pieces.begin(); iter != pieces.end(); iter++)
+	for (std::list<wxString>::const_iterator iter = pieces.begin(); iter != pieces.end(); ++iter)
 	{
 		if (iter != pieces.begin())
 			path.AddSegment(*iter);
 
 		wxTreeItemIdValue cookie;
 		wxTreeItemId child = GetFirstChild(parent, cookie);
-		if (child && GetItemText(child) == _T(""))
+		if (child && GetItemText(child).empty())
 		{
 			Delete(child);
 			child = wxTreeItemId();
@@ -410,7 +418,7 @@ wxTreeItemId CRemoteTreeView::MakeParent(CServerPath path, bool select)
 			SortChildren(parent);
 
 			std::list<wxString>::const_iterator nextIter = iter;
-			nextIter++;
+			++nextIter;
 			if (nextIter != pieces.end())
 				DisplayItem(child, listing);
 		}
@@ -454,12 +462,12 @@ wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay /*=_T(""
 	GetSystemImageList()->Draw(index, dc, 0, 0, wxIMAGELIST_DRAW_TRANSPARENT);
 
 	// Load overlay
-	if (overlay != _T(""))
+	if (!overlay.empty())
 	{
 		wxImage unknownIcon = wxArtProvider::GetBitmap(overlay, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)).ConvertToImage();
 
 		// Convert mask into alpha channel
-		if (!unknownIcon.HasAlpha())
+		if (unknownIcon.IsOk() && !unknownIcon.HasAlpha())
 		{
 			wxASSERT(unknownIcon.HasMask());
 			unknownIcon.InitAlpha();
@@ -469,7 +477,7 @@ wxBitmap CRemoteTreeView::CreateIcon(int index, const wxString& overlay /*=_T(""
 		dc.DrawBitmap(unknownIcon, 0, 0, true);
 	}
 
-    dc.SelectObject(wxNullBitmap);
+	dc.SelectObject(wxNullBitmap);
 	return bmp;
 }
 
@@ -479,12 +487,12 @@ void CRemoteTreeView::CreateImageList()
 	m_pImageList = new wxImageList(s.x, s.y, true, 4);
 
 	// Normal directory
-	int index = GetIconIndex(dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
+	int index = GetIconIndex(iconType::dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
 	m_pImageList->Add(CreateIcon(index));
 	m_pImageList->Add(CreateIcon(index, _T("ART_UNKNOWN")));
 
 	// Opened directory
-	index = GetIconIndex(opened_dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
+	index = GetIconIndex(iconType::opened_dir, _T("{78013B9C-3532-4fe1-A418-5CD1955127CC}"), false);
 	m_pImageList->Add(CreateIcon(index));
 	m_pImageList->Add(CreateIcon(index, _T("ART_UNKNOWN")));
 
@@ -493,7 +501,7 @@ void CRemoteTreeView::CreateImageList()
 
 bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilterManager& filter)
 {
-	if (!listing.m_hasDirs)
+	if (!listing.has_dirs())
 		return false;
 
 	if (!filter.HasActiveFilters())
@@ -505,7 +513,7 @@ bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilter
 		if (!listing[i].is_dir())
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
 			continue;
 
 		return true;
@@ -526,7 +534,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 		if (!listing[i].is_dir())
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
 			continue;
 
 		const wxString& name = listing[i].name;
@@ -552,24 +560,13 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 	SortChildren(parent);
 }
 
-static bool sortfunc(const wxString& a, const wxString& b)
-{
-	int cmp = a.CmpNoCase(b);
-
-	if (!cmp)
-		cmp = a.Cmp(b);
-
-	return cmp < 0;
-}
-
-
 void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing& listing, bool will_select_parent)
 {
 	SetItemImages(parent, false);
 
 	wxTreeItemIdValue cookie;
 	wxTreeItemId child = GetFirstChild(parent, cookie);
-	if (!child || GetItemText(child) == _T(""))
+	if (!child || GetItemText(child).empty())
 	{
 		DisplayItem(parent, listing);
 		return;
@@ -579,29 +576,26 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 
 	const wxString path = listing.path.GetPath();
 
-	std::list<wxString> dirs;
+	wxArrayString dirs;
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
 	{
 		if (!listing[i].is_dir())
 			continue;
 
-		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
+		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].time))
 			dirs.push_back(listing[i].name);
 	}
 
-	dirs.sort(sortfunc);
+	auto const& sortFunc = CFileListCtrlSortBase::GetCmpFunction(m_nameSortMode);
+	dirs.Sort(sortFunc);
 
 	bool inserted = false;
 	child = GetLastChild(parent);
-	std::list<wxString>::reverse_iterator iter = dirs.rbegin();
-	while (child && iter != dirs.rend())
-	{
-		int cmp = GetItemText(child).CmpNoCase(*iter);
-		if (!cmp)
-			cmp = GetItemText(child).Cmp(*iter);
+	wxArrayString::reverse_iterator iter = dirs.rbegin();
+	while (child && iter != dirs.rend()) {
+		int cmp = sortFunc(GetItemText(child), *iter);
 
-		if (!cmp)
-		{
+		if (!cmp) {
 			CServerPath path = listing.path;
 			path.AddSegment(*iter);
 
@@ -616,10 +610,9 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 				SetItemImages(child, true);
 
 			child = GetPrevSibling(child);
-			iter++;
+			++iter;
 		}
-		else if (cmp > 0)
-		{
+		else if (cmp > 0) {
 			// Child no longer exists
 			wxTreeItemId sel = GetSelection();
 			while (sel && sel != child)
@@ -629,34 +622,30 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 				Delete(child);
 			child = prev;
 		}
-		else if (cmp < 0)
-		{
+		else if (cmp < 0) {
 			// New directory
 			CServerPath path = listing.path;
 			path.AddSegment(*iter);
 
 			CDirectoryListing subListing;
-			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
-			{
+			if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK) {
 				wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
 				SetItemImages(child, false);
 
 				if (HasSubdirs(subListing, filter))
 					AppendItem(child, _T(""), -1, -1);
 			}
-			else
-			{
+			else {
 				wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
 				if (child)
 					SetItemImages(child, true);
 			}
 
-			iter++;
+			++iter;
 			inserted = true;
 		}
 	}
-	while (child)
-	{
+	while (child) {
 		// Child no longer exists
 		wxTreeItemId sel = GetSelection();
 		while (sel && sel != child)
@@ -666,27 +655,24 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 			Delete(child);
 		child = prev;
 	}
-	while (iter != dirs.rend())
-	{
+	while (iter != dirs.rend()) {
 		CServerPath path = listing.path;
 		path.AddSegment(*iter);
 
 		CDirectoryListing subListing;
-		if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK)
-		{
+		if (m_pState->m_pEngine->CacheLookup(path, subListing) == FZ_REPLY_OK) {
 			wxTreeItemId child = AppendItem(parent, *iter, 0, 2, 0);
 			SetItemImages(child, false);
 
 			if (HasSubdirs(subListing, filter))
 				AppendItem(child, _T(""), -1, -1);
 		}
-		else
-		{
+		else {
 			wxTreeItemId child = AppendItem(parent, *iter, 1, 3, 0);
 			SetItemImages(child, true);
 		}
 
-		iter++;
+		++iter;
 		inserted = true;
 	}
 
@@ -694,17 +680,6 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 		SortChildren(parent);
 }
 
-int CRemoteTreeView::OnCompareItems(const wxTreeItemId& item1, const wxTreeItemId& item2)
-{
-	wxString label1 = GetItemText(item1);
-	wxString label2 = GetItemText(item2);
-
-	int cmp = label1.CmpNoCase(label2);
-	if (cmp)
-		return cmp;
-	return label1.Cmp(label2);
-}
-
 void CRemoteTreeView::OnItemExpanding(wxTreeEvent& event)
 {
 	if (m_busy)
@@ -749,7 +724,7 @@ void CRemoteTreeView::SetItemImages(wxTreeItemId item, bool unknown)
 void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 {
 	if (event.GetItem() != m_ExpandAfterList)
-        m_ExpandAfterList = wxTreeItemId();
+		m_ExpandAfterList = wxTreeItemId();
 	if (m_busy)
 		return;
 
@@ -764,8 +739,8 @@ void CRemoteTreeView::OnSelectionChanged(wxTreeEvent& event)
 		return;
 
 	const CServerPath path = GetPathFromItem(item);
-	wxASSERT(!path.IsEmpty());
-	if (path.IsEmpty())
+	wxASSERT(!path.empty());
+	if (path.empty())
 		return;
 
 	m_pState->ChangeRemoteDir(path);
@@ -788,7 +763,7 @@ CServerPath CRemoteTreeView::GetPathFromItem(const wxTreeItemId& item) const
 		if (pData)
 		{
 			CServerPath path = pData->m_path;
-			for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); iter++)
+			for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
 			{
 				if (!path.AddSegment(*iter))
 					return CServerPath();
@@ -817,12 +792,12 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(item);
-	if (path.IsEmpty() || !path.HasParent())
+	if (path.empty() || !path.HasParent())
 		return;
 
 	const CServerPath& parent = path.GetParent();
 	const wxString& lastSegment = path.GetLastSegment();
-	if (lastSegment == _T(""))
+	if (lastSegment.empty())
 		return;
 
 	wxDataObjectComposite object;
@@ -844,7 +819,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 	{
 		const wxString& file = ext->GetDragDirectory();
 
-		wxASSERT(file != _T(""));
+		wxASSERT(!file.empty());
 
 		wxFileDataObject *pFileDataObject = new wxFileDataObject;
 		pFileDataObject->AddFile(file);
@@ -874,8 +849,6 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 		return;
 	}
 
-	const wxDataFormat fmt = object.GetReceivedFormat();
-
 #if FZ3_USESHELLEXT
 	if (ext)
 	{
@@ -895,7 +868,7 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 			{
 				delete ext;
 				ext = 0;
-				wxMessageBox(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
+				wxMessageBoxEx(_("Could not determine the target of the Drag&Drop operation.\nEither the shell extension is not installed properly or you didn't drop the files into an Explorer window."));
 				return;
 			}
 
@@ -911,24 +884,6 @@ void CRemoteTreeView::OnBeginDrag(wxTreeEvent& event)
 #endif
 }
 
-#ifndef __WXMSW__
-void CRemoteTreeView::OnKeyDown(wxKeyEvent& event)
-{
-	if (event.GetKeyCode() != WXK_TAB)
-	{
-		event.Skip();
-		return;
-	}
-
-	wxNavigationKeyEvent navEvent;
-	navEvent.SetEventObject(this);
-	navEvent.SetDirection(!event.ShiftDown());
-	navEvent.SetFromTab(true);
-	navEvent.ResumePropagation(1);
-	ProcessEvent(navEvent);
-}
-#endif
-
 void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 {
 	m_contextMenuItem = event.GetItem();
@@ -937,14 +892,14 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 		return;
 
 	const CServerPath& path = m_contextMenuItem ? GetPathFromItem(m_contextMenuItem) : CServerPath();
-	if (!m_pState->IsRemoteIdle() || path.IsEmpty())
+	if (!m_pState->IsRemoteIdle() || path.empty())
 	{
 		pMenu->Enable(XRCID("ID_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_ADDTOQUEUE"), false);
 		pMenu->Enable(XRCID("ID_MKDIR"), false);
+		pMenu->Enable(XRCID("ID_MKDIR_CHGDIR"), false);
 		pMenu->Enable(XRCID("ID_DELETE"), false);
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
-		pMenu->Enable(XRCID("ID_MKDIR"), false);
 		pMenu->Enable(XRCID("ID_RENAME"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
 	}
@@ -961,7 +916,7 @@ void CRemoteTreeView::OnContextMenu(wxTreeEvent& event)
 	delete pMenu;
 }
 
-void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -970,7 +925,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
 	const bool hasParent = path.HasParent();
@@ -997,7 +952,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 				if (listing[i].name != name)
 					continue;
 
-				pChmodDlg->ConvertPermissions(listing[i].permissions, permissions);
+				pChmodDlg->ConvertPermissions(*listing[i].permissions, permissions);
 			}
 		}
 	}
@@ -1089,7 +1044,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
 	const wxString& name = GetItemText(m_contextMenuItem);
@@ -1109,7 +1064,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 	pRecursiveOperation->StartRecursiveOperation(addOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, path, filter.GetActiveFilters(false), true, currentPath);
 }
 
-void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -1118,16 +1073,16 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
-	if (wxMessageBox(_("Really delete all selected files and/or directories?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
+	if (wxMessageBoxEx(_("Really delete all selected files and/or directories from the server?"), _("Confirmation needed"), wxICON_QUESTION | wxYES_NO, this) != wxYES)
 		return;
 
 	const bool hasParent = path.HasParent();
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-	
+
 	CServerPath startDir;
 	if (hasParent)
 	{
@@ -1145,14 +1100,14 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent& event)
 	const wxTreeItemId selected = GetSelection();
 	if (selected)
 		currentPath = GetPathFromItem(selected);
-	if (!currentPath.IsEmpty() && (path == currentPath || path.IsParentOf(currentPath, false)))
+	if (!currentPath.empty() && (path == currentPath || path.IsParentOf(currentPath, false)))
 		currentPath = startDir;
 
 	CFilterManager filter;
 	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, startDir, filter.GetActiveFilters(false), !hasParent, currentPath);
 }
 
-void CRemoteTreeView::OnMenuRename(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuRename(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -1161,7 +1116,7 @@ void CRemoteTreeView::OnMenuRename(wxCommandEvent& event)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 		return;
 
 	if (!path.HasParent())
@@ -1179,7 +1134,7 @@ void CRemoteTreeView::OnBeginLabelEdit(wxTreeEvent& event)
 	}
 
 	const CServerPath& path = GetPathFromItem(event.GetItem());
-	if (path.IsEmpty())
+	if (path.empty())
 	{
 		event.Veto();
 		return;
@@ -1231,7 +1186,7 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 	const wxTreeItemId selected = GetSelection();
 	if (selected)
 		currentPath = GetPathFromItem(selected);
-	if (currentPath.IsEmpty())
+	if (currentPath.empty())
 		return;
 
 	if (currentPath == old_path || currentPath.IsSubdirOf(old_path, false))
@@ -1251,7 +1206,7 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		}
 		currentPath = parent;
 		currentPath.AddSegment(newName);
-		for (std::list<wxString>::const_iterator iter = subdirs.begin(); iter != subdirs.end(); iter++)
+		for (std::list<wxString>::const_iterator iter = subdirs.begin(); iter != subdirs.end(); ++iter)
 			currentPath.AddSegment(*iter);
 		m_pState->ChangeRemoteDir(currentPath);
 	}
@@ -1259,21 +1214,54 @@ void CRemoteTreeView::OnEndLabelEdit(wxTreeEvent& event)
 		m_pState->ChangeRemoteDir(currentPath);
 }
 
-void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
+
+// Create a new Directory
+void CRemoteTreeView::OnMkdir(wxCommandEvent&)
+{
+	CServerPath newpath = MenuMkdir();
+
+	CServerPath listed;
+	if (newpath.HasParent())
+	{
+		listed = newpath.GetParent();
+		m_pState->ChangeRemoteDir(listed);
+	}
+
+	CServerPath currentPath;
+	const wxTreeItemId selected = GetSelection();
+	if (selected)
+		currentPath = GetPathFromItem(selected);
+	if (!currentPath.empty() && currentPath != listed)
+		m_pState->ChangeRemoteDir(currentPath);
+
+}
+
+// Create a new Directory and enter the new Directory
+void CRemoteTreeView::OnMenuMkdirChgDir(wxCommandEvent&)
+{
+	CServerPath newpath = MenuMkdir();
+	if (!newpath.empty()) {
+		m_pState->ChangeRemoteDir(newpath);
+	}
+}
+
+// Help-Function to create a new Directory
+// Returns the name of the new directory
+CServerPath CRemoteTreeView::MenuMkdir()
 {
 	if (!m_pState->IsRemoteIdle())
-		return;
+		return CServerPath();
 
 	if (!m_contextMenuItem)
-		return;
+		return CServerPath();
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
-		return;
+	if (path.empty())
+		return CServerPath();
 
 	CInputDialog dlg;
 	if (!dlg.Create(this, _("Create directory"), _("Please enter the name of the directory which should be created:")))
-		return;
+		return CServerPath();
 
 	CServerPath newPath = path;
 
@@ -1293,36 +1281,25 @@ void CRemoteTreeView::OnMkdir(wxCommandEvent& event)
 	}
 
 	if (dlg.ShowModal() != wxID_OK)
-		return;
+		return CServerPath();
 
 	newPath = path;
 	if (!newPath.ChangePath(dlg.GetValue()))
 	{
 		wxBell();
-		return;
+		return CServerPath();
 	}
 
 	m_pState->m_pCommandQueue->ProcessCommand(new CMkdirCommand(newPath));
-	CServerPath listed;
-	if (newPath.HasParent())
-	{
-		listed = newPath.GetParent();
-		m_pState->ChangeRemoteDir(listed);
-	}
 
-	CServerPath currentPath;
-	const wxTreeItemId selected = GetSelection();
-	if (selected)
-		currentPath = GetPathFromItem(selected);
-	if (!currentPath.IsEmpty() && currentPath != listed)
-		m_pState->ChangeRemoteDir(currentPath);
+	return newPath;
 }
 
 bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 {
 	const CServerPath path = GetPathFromItem(item);
-	wxASSERT(!path.IsEmpty());
-	if (path.IsEmpty())
+	wxASSERT(!path.empty());
+	if (path.empty())
 		return false;
 
 	CDirectoryListing listing;
@@ -1333,7 +1310,7 @@ bool CRemoteTreeView::ListExpand(wxTreeItemId item)
 		SetItemImages(item, true);
 
 		wxTreeItemId child = GetLastChild(item);
-		if (!child || GetItemText(child) == _T(""))
+		if (!child || GetItemText(child).empty())
 			return false;
 	}
 
@@ -1359,16 +1336,15 @@ struct _parents
 	CServerPath path;
 };
 
-void CRemoteTreeView::ApplyFilters()
+void CRemoteTreeView::ApplyFilters(bool resort)
 {
 	std::list<struct _parents> parents;
 
 	const wxTreeItemId root = GetRootItem();
 	wxTreeItemIdValue cookie;
-	for (wxTreeItemId child = GetFirstChild(root, cookie); child; child = GetNextSibling(child))
-	{
+	for (wxTreeItemId child = GetFirstChild(root, cookie); child; child = GetNextSibling(child)) {
 		CServerPath path = GetPathFromItem(child);
-		if (path.IsEmpty())
+		if (path.empty())
 			continue;
 
 		struct _parents dir;
@@ -1376,36 +1352,35 @@ void CRemoteTreeView::ApplyFilters()
 		dir.path = path;
 		parents.push_back(dir);
 	}
-	
+
 	CFilterManager filter;
-	while (!parents.empty())
-	{
+	while (!parents.empty()) {
 		struct _parents parent = parents.back();
 		parents.pop_back();
 
+		if (resort) {
+			SortChildren(parent.item);
+		}
+
 		CDirectoryListing listing;
 		if (m_pState->m_pEngine->CacheLookup(parent.path, listing) == FZ_REPLY_OK)
 			RefreshItem(parent.item, listing, false);
-		else if (filter.HasActiveFilters())
-		{
-			for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child))
-			{
+		else if (filter.HasActiveFilters()) {
+			for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child)) {
 				CServerPath path = GetPathFromItem(child);
-				if (path.IsEmpty())
+				if (path.empty())
 					continue;
 
-				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, 0))
-				{
+				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, CDateTime())) {
 					wxTreeItemId sel = GetSelection();
 					while (sel && sel != child)
 						sel = GetItemParent(sel);
-					if (!sel)
-					{
+					if (!sel) {
 						Delete(child);
 						continue;
 					}
 				}
-				
+
 				struct _parents dir;
 				dir.item = child;
 				dir.path = path;
@@ -1415,10 +1390,9 @@ void CRemoteTreeView::ApplyFilters()
 			// The stuff below has already been done above in this one case
 			continue;
 		}
-		for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child))
-		{
+		for (wxTreeItemId child = GetFirstChild(parent.item, cookie); child; child = GetNextSibling(child)) {
 			CServerPath path = GetPathFromItem(child);
-			if (path.IsEmpty())
+			if (path.empty())
 				continue;
 
 			struct _parents dir;
@@ -1429,13 +1403,13 @@ void CRemoteTreeView::ApplyFilters()
 	}
 }
 
-void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
+void CRemoteTreeView::OnMenuGeturl(wxCommandEvent&)
 {
 	if (!m_contextMenuItem)
 		return;
 
 	const CServerPath& path = GetPathFromItem(m_contextMenuItem);
-	if (path.IsEmpty())
+	if (path.empty())
 	{
 		wxBell();
 		return;
@@ -1450,7 +1424,7 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 
 	if (!wxTheClipboard->Open())
 	{
-		wxMessageBox(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("Could not open clipboard"), _("Could not copy URLs"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -1465,3 +1439,28 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
 }
+
+void CRemoteTreeView::UpdateSortMode()
+{
+	switch (COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT))
+	{
+	case 0:
+	default:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		m_nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
+}
+
+void CRemoteTreeView::OnOptionsChanged(changed_options_t const& options)
+{
+	if (options.test(OPTION_FILELIST_NAMESORT)) {
+		UpdateSortMode();
+		ApplyFilters(true);
+	}
+}
