@@ -5,15 +5,17 @@
 #include "richedit.h"
 #endif
 
+#include <wx/timer.h>
+
 class CFastTextCtrl;
-class CStatusView : public wxWindow
+class CStatusView : public wxNavigationEnabled<wxWindow>
 {
 public:
 	CStatusView(wxWindow* parent, wxWindowID id);
 	virtual ~CStatusView();
 
-	void AddToLog(CLogmsgNotification *pNotification);
-	void AddToLog(enum MessageType messagetype, const wxString& message, const wxDateTime& time);
+	void AddToLog(CLogmsgNotification const& pNotification);
+	void AddToLog(MessageType messagetype, const wxString& message, const wxDateTime& time);
 
 	void InitDefAttr();
 
@@ -23,18 +25,19 @@ public:
 
 protected:
 
-	int m_nLineCount;
+	int m_nLineCount{};
 	wxString m_Content;
-	CFastTextCtrl *m_pTextCtrl;
-
-	void OnSize(wxSizeEvent &event);
+	CFastTextCtrl *m_pTextCtrl{};
 
 	DECLARE_EVENT_TABLE()
-	void OnContextMenu(wxContextMenuEvent& event);
-	void OnClear(wxCommandEvent& event);
-	void OnCopy(wxCommandEvent& event);
+	void OnSize(wxSizeEvent &);
+	void OnContextMenu(wxContextMenuEvent&);
+	void OnClear(wxCommandEvent& );
+	void OnCopy(wxCommandEvent& );
+	void OnTimer(wxTimerEvent&);
 
 	std::list<int> m_lineLengths;
+	std::list<int> m_unusedLineLengths;
 
 	struct t_attributeCache
 	{
@@ -44,11 +47,11 @@ protected:
 #ifdef __WXMSW__
 		CHARFORMAT2 cf;
 #endif
-	} m_attributeCache[MessageTypeCount];
+	} m_attributeCache[static_cast<int>(MessageType::count)];
 
-	bool m_rtl;
+	bool m_rtl{};
 
-	bool m_shown;
+	bool m_shown{};
 
 	// Don't update actual log window if not shown,
 	// do it later when showing the window.
@@ -60,13 +63,9 @@ protected:
 	};
 	std::list<t_line> m_hiddenLines;
 
-	bool m_showTimestamps;
+	bool m_showTimestamps{};
 	wxDateTime m_lastTime;
 	wxString m_lastTimeString;
-
-#ifdef __WXMAC__
-	int m_insertionPoint;
-#endif
 };
 
 #endif
