@@ -1,5 +1,6 @@
 #include <filezilla.h>
 #include "bookmarks_dialog.h"
+#include "filezillaapp.h"
 #include "sitemanager.h"
 #include "ipcmutex.h"
 #include "themeprovider.h"
@@ -30,29 +31,29 @@ CNewBookmarkDialog::CNewBookmarkDialog(wxWindow* parent, wxString& site_path, co
 {
 }
 
-int CNewBookmarkDialog::ShowModal(const wxString &local_path, const CServerPath &remote_path)
+int CNewBookmarkDialog::Run(const wxString &local_path, const CServerPath &remote_path)
 {
 	if (!Load(m_parent, _T("ID_NEWBOOKMARK")))
 		return wxID_CANCEL;
 
 	XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl)->ChangeValue(local_path);
-	if (!remote_path.IsEmpty())
+	if (!remote_path.empty())
 		XRCCTRL(*this, "ID_REMOTEPATH", wxTextCtrl)->ChangeValue(remote_path.GetPath());
 
 	if (!m_server)
 		XRCCTRL(*this, "ID_TYPE_SITE", wxRadioButton)->Enable(false);
 
-	return wxDialogEx::ShowModal();
+	return ShowModal();
 }
 
-void CNewBookmarkDialog::OnOK(wxCommandEvent& event)
+void CNewBookmarkDialog::OnOK(wxCommandEvent&)
 {
 	const bool global = XRCCTRL(*this, "ID_TYPE_GLOBAL", wxRadioButton)->GetValue();
 
 	const wxString name = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
 	if (name.empty())
 	{
-		wxMessageBox(_("You need to enter a name for the bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("You need to enter a name for the bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
@@ -66,37 +67,37 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent& event)
 			remote_path.SetType(m_server->GetType());
 		if (!remote_path.SetPath(remote_path_raw))
 		{
-			wxMessageBox(_("Could not parse remote path."), _("New bookmark"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("Could not parse remote path."), _("New bookmark"), wxICON_EXCLAMATION);
 			return;
 		}
 	}
 
 	if (local_path.empty() && remote_path_raw.empty())
 	{
-		wxMessageBox(_("You need to enter at least one path, empty bookmarks are not supported."), _("New bookmark"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("You need to enter at least one path, empty bookmarks are not supported."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
 	bool sync = XRCCTRL(*this, "ID_SYNC", wxCheckBox)->GetValue();
 	if (sync && (local_path.empty() || remote_path_raw.empty()))
 	{
-		wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
-	if (!global)
+	if (!global && m_server)
 	{
 		std::list<wxString> bookmarks;
 
 		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks))
 		{
-			if (wxMessageBox(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
+			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
 				return;
 
 			m_site_path = CSiteManager::AddServer(*m_server);
-			if (m_site_path == _T(""))
+			if (m_site_path.empty())
 			{
-				wxMessageBox(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
 				EndModal(wxID_CANCEL);
 				return;
 			}
@@ -105,7 +106,7 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent& event)
 		{
 			if (*iter == name)
 			{
-				wxMessageBox(_("A bookmark with the entered name already exists. Please enter an unused name."), _("New bookmark"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("A bookmark with the entered name already exists. Please enter an unused name."), _("New bookmark"), wxICON_EXCLAMATION, this);
 				return;
 			}
 		}
@@ -123,7 +124,7 @@ void CNewBookmarkDialog::OnOK(wxCommandEvent& event)
 	}
 }
 
-void CNewBookmarkDialog::OnBrowse(wxCommandEvent& event)
+void CNewBookmarkDialog::OnBrowse(wxCommandEvent&)
 {
 	wxTextCtrl *pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
 
@@ -164,18 +165,15 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("bookmarks"));
-
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-	{
+	for (TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
 		wxString name;
 		wxString local_dir;
 		wxString remote_dir_raw;
@@ -192,11 +190,11 @@ void CBookmarksDialog::LoadGlobalBookmarks()
 			if (!remote_dir.SetSafePath(remote_dir_raw))
 				continue;
 		}
-		if (local_dir.empty() && remote_dir.IsEmpty())
+		if (local_dir.empty() && remote_dir.empty())
 			continue;
 
 		bool sync;
-		if (local_dir.empty() || remote_dir.IsEmpty())
+		if (local_dir.empty() || remote_dir.empty())
 			sync = false;
 		else
 			sync = GetTextElementBool(pBookmark, "SyncBrowsing");
@@ -225,18 +223,17 @@ void CBookmarksDialog::LoadSiteSpecificBookmarks()
 		path.Replace(_T("/"), _T("\\/"));
 		path = m_site_path + _T("/") + path;
 
-		CSiteManagerItemData_Site* data = CSiteManager::GetSiteByPath(path);
-
-		CBookmarkItemData* new_data = new CBookmarkItemData(data->m_localDir, data->m_remoteDir, data->m_sync);
-		m_pTree->AppendItem(m_bookmarks_site, *iter, 1, 1, new_data);
-
-		delete data;
+		std::unique_ptr<CSiteManagerItemData_Site> data = CSiteManager::GetSiteByPath(path);
+		if (data) {
+			CBookmarkItemData* new_data = new CBookmarkItemData(data->m_localDir, data->m_remoteDir, data->m_sync);
+			m_pTree->AppendItem(m_bookmarks_site, *iter, 1, 1, new_data);
+		}
 	}
 
 	m_pTree->SortChildren(m_bookmarks_site);
 }
 
-int CBookmarksDialog::ShowModal(const wxString &local_path, const CServerPath &remote_path)
+int CBookmarksDialog::Run(const wxString &local_path, const CServerPath &remote_path)
 {
 	if (!Load(m_parent, _T("ID_BOOKMARKS")))
 		return wxID_CANCEL;
@@ -283,29 +280,28 @@ int CBookmarksDialog::ShowModal(const wxString &local_path, const CServerPath &r
 
 	m_pTree->SelectItem(m_bookmarks_global);
 
-	return wxDialogEx::ShowModal();
+	return ShowModal();
 }
 
 void CBookmarksDialog::SaveGlobalBookmarks()
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("bookmarks"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n\n") + _("The global bookmarks could not be saved.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return;
 	}
 
-	TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark");
-	while (pBookmark)
 	{
-		pDocument->RemoveChild(pBookmark);
-		pBookmark = pDocument->FirstChildElement("Bookmark");
+		TiXmlElement *pBookmark = pDocument->FirstChildElement("Bookmark");
+		while (pBookmark) {
+			pDocument->RemoveChild(pBookmark);
+			pBookmark = pDocument->FirstChildElement("Bookmark");
+		}
 	}
 
 	wxTreeItemIdValue cookie;
@@ -318,17 +314,15 @@ void CBookmarksDialog::SaveGlobalBookmarks()
 		AddTextElement(pBookmark, "Name", m_pTree->GetItemText(child));
 		if (!data->m_local_dir.empty())
 			AddTextElement(pBookmark, "LocalDir", data->m_local_dir);
-		if (!data->m_remote_dir.IsEmpty())
+		if (!data->m_remote_dir.empty())
 			AddTextElement(pBookmark, "RemoteDir", data->m_remote_dir.GetSafePath());
 		if (data->m_sync)
 			AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
 	}
 
-	wxString error;
-	if (!file.Save(&error))
-	{
-		wxString msg = wxString::Format(_("Could not write \"%s\", the global bookmarks could no be saved: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+	if (!file.Save(false)) {
+		wxString msg = wxString::Format(_("Could not write \"%s\", the global bookmarks could no be saved: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 }
 
@@ -351,7 +345,7 @@ void CBookmarksDialog::SaveSiteSpecificBookmarks()
 	}
 }
 
-void CBookmarksDialog::OnOK(wxCommandEvent& event)
+void CBookmarksDialog::OnOK(wxCommandEvent&)
 {
 	if (!Verify())
 		return;
@@ -363,7 +357,7 @@ void CBookmarksDialog::OnOK(wxCommandEvent& event)
 	EndModal(wxID_OK);
 }
 
-void CBookmarksDialog::OnBrowse(wxCommandEvent& event)
+void CBookmarksDialog::OnBrowse(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item)
@@ -393,7 +387,7 @@ void CBookmarksDialog::OnSelChanging(wxTreeEvent& event)
 	UpdateBookmark();
 }
 
-void CBookmarksDialog::OnSelChanged(wxTreeEvent& event)
+void CBookmarksDialog::OnSelChanged(wxTreeEvent&)
 {
 	DisplayBookmark();
 }
@@ -415,7 +409,7 @@ bool CBookmarksDialog::Verify()
 		server = 0;
 
 	const wxString remotePathRaw = XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue();
-	if (remotePathRaw != _T(""))
+	if (!remotePathRaw.empty())
 	{
 		CServerPath remotePath;
 		if (server)
@@ -427,13 +421,13 @@ bool CBookmarksDialog::Verify()
 			{
 				wxString msg;
 				if (server->GetType() != DEFAULT)
-					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), server->GetNameFromServerType(server->GetType()).c_str());
+					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the current site's servertype (%s)."), server->GetNameFromServerType(server->GetType()));
 				else
 					msg = _("Remote path cannot be parsed. Make sure it is a valid absolute path.");
-				wxMessageBox(msg);
+				wxMessageBoxEx(msg);
 			}
 			else
-				wxMessageBox(_("Remote path cannot be parsed. Make sure it is a valid absolute path."));
+				wxMessageBoxEx(_("Remote path cannot be parsed. Make sure it is a valid absolute path."));
 			return false;
 		}
 	}
@@ -443,14 +437,14 @@ bool CBookmarksDialog::Verify()
 	if (remotePathRaw.empty() && localPath.empty())
 	{
 		XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->SetFocus();
-		wxMessageBox(_("You need to enter at least one path, empty bookmarks are not supported."));
+		wxMessageBoxEx(_("You need to enter at least one path, empty bookmarks are not supported."));
 		return false;
 	}
 
 	bool sync = XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->GetValue();
 	if (sync && (localPath.empty() || remotePathRaw.empty()))
 	{
-		wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("New bookmark"), wxICON_EXCLAMATION, this);
 		return false;
 	}
 
@@ -474,7 +468,7 @@ void CBookmarksDialog::UpdateBookmark()
 		server = 0;
 
 	const wxString remotePathRaw = XRCCTRL(*this, "ID_BOOKMARK_REMOTEDIR", wxTextCtrl)->GetValue();
-	if (remotePathRaw != _T(""))
+	if (!remotePathRaw.empty())
 	{
 		if (server)
 			data->m_remote_dir.SetType(server->GetType());
@@ -523,7 +517,7 @@ void CBookmarksDialog::DisplayBookmark()
 	XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->SetValue(data->m_sync);
 }
 
-void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
+void CBookmarksDialog::OnNewBookmark(wxCommandEvent&)
 {
 	if (!Verify())
 		return;
@@ -542,13 +536,13 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
 
 		if (m_site_path.empty() || !CSiteManager::GetBookmarks(m_site_path, bookmarks))
 		{
-			if (wxMessageBox(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
+			if (wxMessageBoxEx(_("Site-specific bookmarks require the server to be stored in the Site Manager.\nAdd current connection to the site manager?"), _("New bookmark"), wxYES_NO | wxICON_QUESTION, this) != wxYES)
 				return;
 
 			m_site_path = CSiteManager::AddServer(*m_server);
-			if (m_site_path == _T(""))
+			if (m_site_path.empty())
 			{
-				wxMessageBox(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("Could not add connection to Site Manager"), _("New bookmark"), wxICON_EXCLAMATION, this);
 				return;
 			}
 		}
@@ -586,7 +580,7 @@ void CBookmarksDialog::OnNewBookmark(wxCommandEvent& event)
 	m_pTree->EditLabel(child);
 }
 
-void CBookmarksDialog::OnRename(wxCommandEvent& event)
+void CBookmarksDialog::OnRename(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
@@ -595,7 +589,7 @@ void CBookmarksDialog::OnRename(wxCommandEvent& event)
 	m_pTree->EditLabel(item);
 }
 
-void CBookmarksDialog::OnDelete(wxCommandEvent& event)
+void CBookmarksDialog::OnDelete(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
@@ -607,7 +601,7 @@ void CBookmarksDialog::OnDelete(wxCommandEvent& event)
 	m_pTree->SelectItem(parent);
 }
 
-void CBookmarksDialog::OnCopy(wxCommandEvent& event)
+void CBookmarksDialog::OnCopy(wxCommandEvent&)
 {
 	wxTreeItemId item = m_pTree->GetSelection();
 	if (!item.IsOk())
@@ -616,7 +610,7 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 	if (!Verify())
 		return;
 
-	CBookmarkItemData* data = reinterpret_cast<CBookmarkItemData *>(m_pTree->GetItemData(item));
+	CBookmarkItemData* data = static_cast<CBookmarkItemData *>(m_pTree->GetItemData(item));
 	if (!data)
 		return;
 
@@ -624,21 +618,18 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 
 	wxTreeItemId parent = m_pTree->GetItemParent(item);
 
-	const wxString name = m_pTree->GetItemText(item);
-	wxString newName = wxString::Format(_("Copy of %s"), name.c_str());
+	const wxString oldName = m_pTree->GetItemText(item);
+	wxString newName = wxString::Format(_("Copy of %s"), oldName);
 	int index = 2;
-	for (;;)
-	{
+	for (;;) {
 		wxTreeItemId child;
 		wxTreeItemIdValue cookie;
 		child = m_pTree->GetFirstChild(parent, cookie);
 		bool found = false;
-		while (child.IsOk())
-		{
+		while (child.IsOk()) {
 			wxString name = m_pTree->GetItemText(child);
 			int cmp = name.CmpNoCase(newName);
-			if (!cmp)
-			{
+			if (!cmp) {
 				found = true;
 				break;
 			}
@@ -648,7 +639,7 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 		if (!found)
 			break;
 
-		newName = wxString::Format(_("Copy (%d) of %s"), index++, name.c_str());
+		newName = wxString::Format(_("Copy (%d) of %s"), index++, oldName);
 	}
 
 	CBookmarkItemData* newData = new CBookmarkItemData(*data);
@@ -662,17 +653,14 @@ void CBookmarksDialog::OnCopy(wxCommandEvent& event)
 void CBookmarksDialog::OnBeginLabelEdit(wxTreeEvent& event)
 {
 	wxTreeItemId item = event.GetItem();
-	if (item != m_pTree->GetSelection())
-	{
-		if (!Verify())
-		{
+	if (item != m_pTree->GetSelection()) {
+		if (!Verify()) {
 			event.Veto();
 			return;
 		}
 	}
 
-	if (!item || item == m_bookmarks_global || item == m_bookmarks_site)
-	{
+	if (!item || item == m_bookmarks_global || item == m_bookmarks_site) {
 		event.Veto();
 		return;
 	}
@@ -710,7 +698,7 @@ void CBookmarksDialog::OnEndLabelEdit(wxTreeEvent& event)
 			continue;
 		if (!name.CmpNoCase(m_pTree->GetItemText(child)))
 		{
-			wxMessageBox(_("Name already exists"), _("Cannot rename entry"), wxICON_EXCLAMATION, this);
+			wxMessageBoxEx(_("Name already exists"), _("Cannot rename entry"), wxICON_EXCLAMATION, this);
 			event.Veto();
 			return;
 		}
@@ -723,12 +711,10 @@ bool CBookmarksDialog::GetBookmarks(std::list<wxString> &bookmarks)
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("bookmarks"));
-
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -751,7 +737,7 @@ bool CBookmarksDialog::GetBookmarks(std::list<wxString> &bookmarks)
 			if (!remote_dir.SetSafePath(remote_dir_raw))
 				continue;
 		}
-		if (local_dir.empty() && remote_dir.IsEmpty())
+		if (local_dir.empty() && remote_dir.empty())
 			continue;
 
 		bookmarks.push_back(name);
@@ -764,12 +750,10 @@ bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CS
 {
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("bookmarks"));
-
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -805,35 +789,31 @@ bool CBookmarksDialog::GetBookmark(const wxString &name, wxString &local_dir, CS
 
 bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_dir, const CServerPath &remote_dir, bool sync)
 {
-	if (local_dir.empty() && remote_dir.IsEmpty())
+	if (local_dir.empty() && remote_dir.empty())
 		return false;
-	if ((local_dir.empty() || remote_dir.IsEmpty()) && sync)
+	if ((local_dir.empty() || remote_dir.empty()) && sync)
 		return false;
 
 	CInterProcessMutex mutex(MUTEX_GLOBALBOOKMARKS);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("bookmarks"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("bookmarks")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n\n") + _("The bookmark could not be added.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
 
 	TiXmlElement *pInsertBefore = 0;
 	TiXmlElement *pBookmark;
-	for (pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-	{
+	for (pBookmark = pDocument->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
 		wxString remote_dir_raw;
 
 		wxString old_name = GetTextElement(pBookmark, "Name");
 
-		if (!name.CmpNoCase(old_name))
-		{
-			wxMessageBox(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
+		if (!name.CmpNoCase(old_name)) {
+			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
 		if (name < old_name && !pInsertBefore)
@@ -847,16 +827,14 @@ bool CBookmarksDialog::AddBookmark(const wxString &name, const wxString &local_d
 	AddTextElement(pBookmark, "Name", name);
 	if (!local_dir.empty())
 		AddTextElement(pBookmark, "LocalDir", local_dir);
-	if (!remote_dir.IsEmpty())
+	if (!remote_dir.empty())
 		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
 	if (sync)
 		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
 
-	wxString error;
-	if (!file.Save(&error))
-	{
-		wxString msg = wxString::Format(_("Could not write \"%s\", the bookmark could not be added: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+	if (!file.Save(false)) {
+		wxString msg = wxString::Format(_("Could not write \"%s\", the bookmark could not be added: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 		return false;
 	}
 
