@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "buildinfo.h"
 
 wxString CBuildInfo::GetVersion()
@@ -14,9 +14,9 @@ wxString CBuildInfo::GetBuildDateString()
 	while (date.Replace(_T("  "), _T(" ")));
 
 	const wxChar months[][4] = { _T("Jan"), _T("Feb"), _T("Mar"),
-								_T("Apr"), _T("May"), _T("Jun"), 
+								_T("Apr"), _T("May"), _T("Jun"),
 								_T("Jul"), _T("Aug"), _T("Sep"),
-								_T("Oct"), _T("Nov"), _T("Dev") };
+								_T("Oct"), _T("Nov"), _T("Dec") };
 
 	int pos = date.Find(_T(" "));
 	if (pos == -1)
@@ -45,7 +45,7 @@ wxString CBuildInfo::GetBuildDateString()
 	if (!tmp.Mid(pos + 1).ToLong(&year))
 		return date;
 
-	return wxString::Format(_T("%04d-%02d-%02d"), year, i + 1, day);
+	return wxString::Format(_T("%04d-%02d-%02d"), (int)year, i + 1, (int)day);
 }
 
 wxString CBuildInfo::GetBuildTimeString()
@@ -75,7 +75,7 @@ wxString CBuildInfo::GetCompiler()
 wxString CBuildInfo::GetCompilerFlags()
 {
 #ifndef USED_CXXFLAGS
-	return _T("");
+	return wxString();
 #else
 	wxString flags(USED_CXXFLAGS, wxConvLocal);
 	return flags;
@@ -89,10 +89,10 @@ wxString CBuildInfo::GetBuildType()
 	if (buildtype == _T("official") || buildtype == _T("nightly"))
 		return buildtype;
 #endif
-	return _T("");
+	return wxString();
 }
 
-wxULongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
+wxLongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
 {
 	// Crude conversion from version string into number for easy comparison
 	// Supported version formats:
@@ -116,8 +116,10 @@ wxULongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
 	// And these can be compared easily
 
 	wxASSERT(*version >= '0' && *version <= '9');
+	if (*version < '0' || *version > '9')
+		return -1;
 
-	wxULongLong v = 0;
+	wxLongLong v = 0;
 	int segment = 0;
 
 	int shifts = 0;
@@ -155,9 +157,30 @@ wxULongLong CBuildInfo::ConvertToVersionNumber(const wxChar* version)
 wxString CBuildInfo::GetHostname()
 {
 #ifndef USED_HOST
-	return _T("");
+	return wxString();
 #else
 	wxString flags(USED_HOST, wxConvLocal);
 	return flags;
 #endif
 }
+
+wxString CBuildInfo::GetBuildSystem()
+{
+#ifndef USED_BUILD
+	return wxString();
+#else
+	wxString flags(USED_BUILD, wxConvLocal);
+	return flags;
+#endif
+}
+
+bool CBuildInfo::IsUnstable()
+{
+	if (GetVersion().Find(_T("beta")) != -1)
+		return true;
+
+	if (GetVersion().Find(_T("rc")) != -1)
+		return true;
+
+	return false;
+}
