@@ -18,7 +18,8 @@ public:
 	void ProcessNextCommand();
 	bool Idle() const;
 	bool Cancel();
-	void Finish(COperationNotification *pNotification);
+	bool Quit();
+	void Finish(std::unique_ptr<COperationNotification> && pNotification);
 
 	void RequestExclusiveEngine(bool requestExclusive);
 
@@ -27,6 +28,8 @@ public:
 	bool EngineLocked() const { return m_exclusiveEngineLock; }
 
 protected:
+	void ProcessReply(int nReplyCode, Command commandId);
+
 	void GrantExclusiveEngineRequest();
 
 	CFileZillaEngine *m_pEngine;
@@ -38,9 +41,11 @@ protected:
 	static int m_requestIdCounter;
 
 	// Used to make this class reentrance-safe
-	bool m_inside_commandqueue;
+	int m_inside_commandqueue{};
 
-	std::list<CCommand *> m_CommandList;
+	std::list<std::unique_ptr<CCommand>> m_CommandList;
+
+	bool m_quit{};
 };
 
 #endif //__COMMANDQUEUE_H__
