@@ -2,6 +2,7 @@
 #include "commandqueue.h"
 #include "context_control.h"
 #include "filelist_statusbar.h"
+#include "filezillaapp.h"
 #include "LocalListView.h"
 #include "LocalTreeView.h"
 #include "Mainfrm.h"
@@ -15,8 +16,10 @@
 #include "view.h"
 #include "viewheader.h"
 
-DECLARE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED, -1);
-DEFINE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED);
+#include <wx/wupdlock.h>
+
+DECLARE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED, -1)
+DEFINE_EVENT_TYPE(fzEVT_TAB_CLOSING_DEFERRED)
 
 BEGIN_EVENT_TABLE(CContextControl, wxSplitterWindow)
 EVT_MENU(XRCID("ID_TABCONTEXT_REFRESH"), CContextControl::OnTabRefresh)
@@ -49,16 +52,20 @@ void CContextControl::Create(wxWindow *parent)
 
 void CContextControl::CreateTab()
 {
-	Freeze();
+	wxGetApp().AddStartupProfileRecord(_T("CContextControl::CreateTab"));
+#ifdef __WXMSW__
+	// Some reparenting is being done when creating tabs. Reparenting of frozen windows isn't working
+	// on OS X.
+	wxWindowUpdateLocker lock(this);
+#endif
 
 	CState* pState = 0;
-	
+
 	// See if we can reuse an existing context
-	for (size_t i = 0; i < m_context_controls.size(); i++)
-	{
+	for (size_t i = 0; i < m_context_controls.size(); i++) {
 		if (m_context_controls[i].tab_index != -1)
 			continue;
-		
+
 		if (m_context_controls[i].pState->IsRemoteConnected() ||
 			!m_context_controls[i].pState->IsRemoteIdle())
 			continue;
@@ -69,12 +76,10 @@ void CContextControl::CreateTab()
 			m_current_context_controls--;
 		break;
 	}
-	if (!pState)
-	{
+	if (!pState) {
 		pState = CContextManager::Get()->CreateState(m_pMainFrame);
-		if (!pState->CreateEngine())
-		{
-			wxMessageBox(_("Failed to initialize FTP engine"));
+		if (!pState->CreateEngine()) {
+			wxMessageBoxEx(_("Failed to initialize FTP engine"));
 		}
 	}
 
@@ -99,12 +104,11 @@ void CContextControl::CreateTab()
 
 	if (m_tabs)
 		m_tabs->SetSelection(m_tabs->GetPageCount() - 1);
-
-	Thaw();
 }
 
 void CContextControl::CreateContextControls(CState* pState)
 {
+	wxGetApp().AddStartupProfileRecord(_T("CContextControl::CreateContextControls"));
 	wxWindow* parent = this;
 
 #ifdef __WXGTK__
@@ -114,10 +118,8 @@ void CContextControl::CreateContextControls(CState* pState)
 	const wxPoint initial_position(wxDefaultPosition);
 #endif
 
-	if (!m_context_controls.empty())
-	{
-		if (!m_tabs )
-		{
+	if (!m_context_controls.empty()) {
+		if (!m_tabs) {
 			m_tabs = new wxAuiNotebookEx();
 
 			wxSize splitter_size = m_context_controls[m_current_context_controls].pViewSplitter->GetSize();
@@ -136,6 +138,11 @@ void CContextControl::CreateContextControls(CState* pState)
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_BG_DCLICK, wxAuiNotebookEventHandler(CContextControl::OnTabBgDoubleclick), 0, this);
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_TAB_MIDDLE_UP, wxAuiNotebookEventHandler(CContextControl::OnTabClosing), 0, this);
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP, wxAuiNotebookEventHandler(CContextControl::OnTabRightclick), 0, this);
+
+#ifdef __WXMAC__
+			// We need to select the first page as the default selection is -1. Not doing so prevents selecting other pages later on.
+			m_tabs->SetSelection(0);
+#endif
 		}
 
 		m_pMainFrame->RememberSplitterPositions();
@@ -191,24 +198,21 @@ void CContextControl::CreateContextControls(CState* pState)
 	const int layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
 	const int swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
 
-	if (layout == 1)
-	{
+	if (layout == 1) {
 		if (swap)
 			context_controls.pViewSplitter->SplitHorizontally(context_controls.pRemoteSplitter, context_controls.pLocalSplitter);
 		else
 			context_controls.pViewSplitter->SplitHorizontally(context_controls.pLocalSplitter, context_controls.pRemoteSplitter);
 	}
-	else
-	{
+	else {
 		if (swap)
 			context_controls.pViewSplitter->SplitVertically(context_controls.pRemoteSplitter, context_controls.pLocalSplitter);
 		else
 			context_controls.pViewSplitter->SplitVertically(context_controls.pLocalSplitter, context_controls.pRemoteSplitter);
 	}
 
-	context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalSplitter, pState);
-	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL))
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL)) {
+		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalTreeViewPanel, pState);
 		context_controls.pLocalTreeViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		if (layout == 3 && swap)
 			context_controls.pLocalSplitter->SplitVertically(context_controls.pLocalListViewPanel, context_controls.pLocalTreeViewPanel);
@@ -217,16 +221,15 @@ void CContextControl::CreateContextControls(CState* pState)
 		else
 			context_controls.pLocalSplitter->SplitHorizontally(context_controls.pLocalTreeViewPanel, context_controls.pLocalListViewPanel);
 	}
-	else
-	{
+	else {
 		context_controls.pLocalTreeViewPanel->Hide();
+		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalListViewPanel, pState);
 		context_controls.pLocalListViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		context_controls.pLocalSplitter->Initialize(context_controls.pLocalListViewPanel);
 	}
 
-	context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteSplitter, pState);
-	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE))
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE)) {
+		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteTreeViewPanel, pState);
 		context_controls.pRemoteTreeViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		if (layout == 3 && !swap)
 			context_controls.pRemoteSplitter->SplitVertically(context_controls.pRemoteListViewPanel, context_controls.pRemoteTreeViewPanel);
@@ -235,15 +238,14 @@ void CContextControl::CreateContextControls(CState* pState)
 		else
 			context_controls.pRemoteSplitter->SplitHorizontally(context_controls.pRemoteTreeViewPanel, context_controls.pRemoteListViewPanel);
 	}
-	else
-	{
+	else {
 		context_controls.pRemoteTreeViewPanel->Hide();
+		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteListViewPanel, pState);
 		context_controls.pRemoteListViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		context_controls.pRemoteSplitter->Initialize(context_controls.pRemoteListViewPanel);
 	}
 
-	if (layout == 3)
-	{
+	if (layout == 3) {
 		if (!swap)
 			context_controls.pRemoteSplitter->SetSashGravity(1.0);
 		else
@@ -259,8 +261,7 @@ void CContextControl::CreateContextControls(CState* pState)
 
 	pState->GetComparisonManager()->SetListings(context_controls.pLocalListView, context_controls.pRemoteListView);
 
-	if (m_tabs)
-	{
+	if (m_tabs) {
 		context_controls.tab_index = m_tabs->GetPageCount();
 		m_tabs->AddPage(context_controls.pViewSplitter, pState->GetTitle());
 
@@ -271,11 +272,10 @@ void CContextControl::CreateContextControls(CState* pState)
 
 		context_controls.site_bookmarks = m_context_controls[m_current_context_controls].site_bookmarks;
 	}
-	else
-	{
+	else {
 		context_controls.tab_index = 0;
-		context_controls.site_bookmarks = new CContextControl::_context_controls::_site_bookmarks;
-		
+		context_controls.site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
+
 		context_controls.site_bookmarks->path = COptions::Get()->GetOption(OPTION_LAST_CONNECTED_SITE);
 		CSiteManager::GetBookmarks(context_controls.site_bookmarks->path,
 			context_controls.site_bookmarks->bookmarks);
@@ -286,7 +286,7 @@ void CContextControl::CreateContextControls(CState* pState)
 	m_context_controls.push_back(context_controls);
 }
 
-void CContextControl::OnTabRefresh(wxCommandEvent& event)
+void CContextControl::OnTabRefresh(wxCommandEvent&)
 {
 	if (m_right_clicked_tab == -1)
 		return;
@@ -342,16 +342,18 @@ bool CContextControl::CloseTab(int tab)
 
 	if (!pState->m_pCommandQueue->Idle())
 	{
-		if (wxMessageBox(_("Cannot close tab while busy.\nCancel current operation and close tab?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
+		if (wxMessageBoxEx(_("Cannot close tab while busy.\nCancel current operation and close tab?"), _T("FileZilla"), wxYES_NO | wxICON_QUESTION) != wxYES)
 			return false;
-
-		Freeze();
-
-		pState->m_pCommandQueue->Cancel();
-		pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 	}
-	else
-		Freeze();
+
+#ifndef __WXMAC__
+	// Some reparenting is being done when closing tabs. Reparenting of frozen windows isn't working
+	// on OS X.
+	wxWindowUpdateLocker lock(this);
+#endif
+
+	pState->m_pCommandQueue->Cancel();
+	pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 
 	pState->GetComparisonManager()->SetListings(0, 0);
 
@@ -381,7 +383,7 @@ bool CContextControl::CloseTab(int tab)
 		m_tabs = 0;
 
 		m_context_controls[i].tab_index = -1;
-		m_context_controls[i].site_bookmarks.clear();
+		m_context_controls[i].site_bookmarks.reset();
 
 		CContextManager::Get()->SetCurrentContext(m_context_controls[j].pState);
 
@@ -409,18 +411,16 @@ bool CContextControl::CloseTab(int tab)
 				m_context_controls[j].tab_index--;
 		}
 		m_context_controls[i].tab_index = -1;
-		m_context_controls[i].site_bookmarks.clear();
+		m_context_controls[i].site_bookmarks.reset();
 		m_tabs->DeletePage(tab);
 	}
 
 	pState->Disconnect();
 
-	Thaw();
-
 	return true;
 }
 
-void CContextControl::OnTabBgDoubleclick(wxAuiNotebookEvent& event)
+void CContextControl::OnTabBgDoubleclick(wxAuiNotebookEvent&)
 {
 	CreateTab();
 }
@@ -454,14 +454,12 @@ void CContextControl::OnTabContextClose(wxCommandEvent& event)
 
 	// Need to defer event, wxAUI would write to free'd memory
 	// if we'd actually delete tab and potenially the notebook with it
-	wxCommandEvent evt(fzEVT_TAB_CLOSING_DEFERRED, m_right_clicked_tab);
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent(fzEVT_TAB_CLOSING_DEFERRED, m_right_clicked_tab));
 }
 
 void CContextControl::OnTabContextCloseOthers(wxCommandEvent& event)
 {
-	wxCommandEvent evt(fzEVT_TAB_CLOSING_DEFERRED, -m_right_clicked_tab - 1);
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent (fzEVT_TAB_CLOSING_DEFERRED, -m_right_clicked_tab - 1));
 }
 
 void CContextControl::OnTabClosing_Deferred(wxCommandEvent& event)
@@ -482,7 +480,7 @@ void CContextControl::OnTabClosing_Deferred(wxCommandEvent& event)
 }
 
 
-void CContextControl::OnTabChanged(wxAuiNotebookEvent& event)
+void CContextControl::OnTabChanged(wxAuiNotebookEvent&)
 {
 	int i = m_tabs->GetSelection();
 	if (i < 0 || i >= (int)m_context_controls.size())
@@ -502,8 +500,7 @@ void CContextControl::OnTabClosing(wxAuiNotebookEvent& event)
 {
 	// Need to defer event, wxAUI would write to free'd memory
 	// if we'd actually delete tab and potenially the notebook with it
-	wxCommandEvent evt(fzEVT_TAB_CLOSING_DEFERRED, event.GetSelection());
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent(fzEVT_TAB_CLOSING_DEFERRED, event.GetSelection()));
 
 	event.Veto();
 }
@@ -558,7 +555,7 @@ void CContextControl::AdvanceTab(bool forward)
 	m_tabs->AdvanceTab(forward);
 }
 
-void CContextControl::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CContextControl::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString&, const void*)
 {
 	if (notification == STATECHANGE_CHANGEDCONTEXT)
 	{
@@ -588,7 +585,7 @@ void CContextControl::OnStateChange(CState* pState, enum t_statechange_notificat
 	}
 }
 
-void CContextControl::OnTabContextNew(wxCommandEvent& event)
+void CContextControl::OnTabContextNew(wxCommandEvent&)
 {
 	CreateTab();
 }
