@@ -1,15 +1,14 @@
 #include <filezilla.h>
-#include "edithandler.h"
+#include "conditionaldialog.h"
 #include "dialogex.h"
+#include "edithandler.h"
 #include "filezillaapp.h"
-#include "queue.h"
-#include "Options.h"
-#include "window_state_manager.h"
+#include "file_utils.h"
 #include "local_filesys.h"
-
-// Defined in optionspage_edit.cpp
-bool UnquoteCommand(wxString& command, wxString& arguments, bool is_dde = false);
-bool ProgramExists(const wxString& editor);
+#include "Options.h"
+#include "queue.h"
+#include "window_state_manager.h"
+#include "xrc_helper.h"
 
 class CChangedFileDialog : public wxDialogEx
 {
@@ -72,7 +71,7 @@ CEditHandler* CEditHandler::Get()
 	return m_pEditHandler;
 }
 
-void CEditHandler::RemoveTemporaryFiles(const wxString& temp)
+void CEditHandler::RemoveTemporaryFiles(wxString const& temp)
 {
 	wxDir dir(temp);
 	if (!dir.IsOpened())
@@ -82,51 +81,66 @@ void CEditHandler::RemoveTemporaryFiles(const wxString& temp)
 	if (!dir.GetFirst(&file, _T("fz3temp-*"), wxDIR_DIRS))
 		return;
 
-	const wxChar& sep = wxFileName::GetPathSeparator();
-	do
-	{
-		const wxString lockfile = temp + file + sep + _("fz3temp-lockfile");
-		if (wxFileName::FileExists(lockfile))
-		{
-#ifndef __WXMSW__
-			int fd = open(lockfile.mb_str(), O_RDWR, 0);
-			if (fd >= 0)
-			{
-				// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
-				struct flock f = {0};
-				f.l_type = F_WRLCK;
-				f.l_whence = SEEK_SET;
-				f.l_start = 0;
-				f.l_len = 1;
-				f.l_pid = getpid();
-				if (fcntl(fd, F_SETLK, &f))
-				{
-					// In use by other process
-					continue;
-				}
-				close(fd);
-			}
-#endif
-			wxRemoveFile(lockfile);
-			if (wxFileName::FileExists(lockfile))
-				continue;
+	wxChar const& sep = wxFileName::GetPathSeparator();
+	do {
+		if (!m_localDir.empty() && temp + file + sep == m_localDir) {
+			// Don't delete own working directory
+			continue;
 		}
 
-		{
-			wxString file2;
-			wxDir dir2(temp + file);
-			bool res;
-			for ((res = dir2.GetFirst(&file2, _T(""), wxDIR_FILES)); res; res = dir2.GetNext(&file2));
-				wxRemoveFile(temp + file + sep + file2);
-		}
-
-		wxRmdir(temp + file + sep);
+		RemoveTemporaryFilesInSpecificDir(temp + file + sep);
 	} while (dir.GetNext(&file));
 }
 
+void CEditHandler::RemoveTemporaryFilesInSpecificDir(wxString const& temp)
+{
+	const wxString lockfile = temp + _("fz3temp-lockfile");
+	if (wxFileName::FileExists(lockfile)) {
+#ifndef __WXMSW__
+		int fd = open(lockfile.mb_str(), O_RDWR | O_CLOEXEC, 0);
+		if (fd >= 0)
+		{
+			// Try to lock 1 byte region in the lockfile. m_type specifies the byte to lock.
+			struct flock f = { 0 };
+			f.l_type = F_WRLCK;
+			f.l_whence = SEEK_SET;
+			f.l_start = 0;
+			f.l_len = 1;
+			f.l_pid = getpid();
+			if (fcntl(fd, F_SETLK, &f)) {
+				// In use by other process
+				close(fd);
+				return;
+			}
+			close(fd);
+		}
+#endif
+		{
+			wxLogNull log;
+			wxRemoveFile(lockfile);
+		}
+		if (wxFileName::FileExists(lockfile))
+			return;
+	}
+
+	wxLogNull log;
+
+	{
+		wxString file;
+		wxDir dir(temp);
+		bool res;
+		for ((res = dir.GetFirst(&file, _T(""), wxDIR_FILES)); res; res = dir.GetNext(&file)) {
+			wxRemoveFile(temp + file);
+		}
+	}
+
+	wxRmdir(temp);
+
+}
+
 wxString CEditHandler::GetLocalDirectory()
 {
-	if (m_localDir != _T(""))
+	if (!m_localDir.empty())
 		return m_localDir;
 
 	wxFileName tmpdir(wxFileName::GetTempDir(), _T(""));
@@ -134,32 +148,35 @@ wxString CEditHandler::GetLocalDirectory()
 	// which will cause problems when calculating maximum allowed file
 	// length
 	wxString dir = tmpdir.GetLongPath();
-	if (dir == _T("") || !wxFileName::DirExists(dir))
-		return _T("");
+	if (dir.empty() || !wxFileName::DirExists(dir))
+		return wxString();
 
 	if (dir.Last() != wxFileName::GetPathSeparator())
 		dir += wxFileName::GetPathSeparator();
 
-	RemoveTemporaryFiles(dir);
-
 	// On POSIX, the permissions of the created directory (700) ensure
 	// that this is a safe operation.
 	// On Windows, the user's profile directory and associated temp dir
 	// already has the correct permissions which get inherited.
 	int i = 1;
-	do
-	{
+	do {
 		wxString newDir = dir + wxString::Format(_T("fz3temp-%d"), i++);
 		if (wxFileName::FileExists(newDir) || wxFileName::DirExists(newDir))
 			continue;
 
 		if (!wxMkdir(newDir, 0700))
-			return _T("");
+			return wxString();
 
 		m_localDir = newDir + wxFileName::GetPathSeparator();
 		break;
 	} while (true);
 
+	// Defer deleting stale directories until after having created our own
+	// working directory.
+	// This avoids some strange errors where freshly deleted directories
+	// cannot be instantly recreated.
+	RemoveTemporaryFiles(dir);
+
 #ifdef __WXMSW__
 	m_lockfile_handle = ::CreateFile(m_localDir + _T("fz3temp-lockfile"), GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_TEMPORARY, 0);
 	if (m_lockfile_handle == INVALID_HANDLE_VALUE)
@@ -169,7 +186,7 @@ wxString CEditHandler::GetLocalDirectory()
 	}
 #else
 	wxString file = m_localDir + _T("fz3temp-lockfile");
-	m_lockfile_descriptor = open(file.mb_str(), O_CREAT | O_RDWR, 0600);
+	m_lockfile_descriptor = open(file.mb_str(), O_CREAT | O_RDWR | O_CLOEXEC, 0600);
 	if (m_lockfile_descriptor >= 0)
 	{
 		// Lock 1 byte region in the lockfile.
@@ -193,8 +210,7 @@ void CEditHandler::Release()
 	if (m_busyTimer.IsRunning())
 		m_busyTimer.Stop();
 
-	if (m_localDir != _T(""))
-	{
+	if (!m_localDir.empty()) {
 #ifdef __WXMSW__
 		if (m_lockfile_handle != INVALID_HANDLE_VALUE)
 			CloseHandle(m_lockfile_handle);
@@ -205,10 +221,10 @@ void CEditHandler::Release()
 			close(m_lockfile_descriptor);
 #endif
 
+		wxLogNull log;
 		wxRemoveFile(m_localDir + _T("empty_file_yq744zm"));
-
 		RemoveAll(true);
-		wxRmdir(m_localDir);
+		RemoveTemporaryFilesInSpecificDir(m_localDir);
 	}
 
 	m_pEditHandler = 0;
@@ -236,37 +252,30 @@ enum CEditHandler::fileState CEditHandler::GetFileState(const wxString& fileName
 int CEditHandler::GetFileCount(enum CEditHandler::fileType type, enum CEditHandler::fileState state, const CServer* pServer /*=0*/) const
 {
 	int count = 0;
-	if (state == unknown)
-	{
+	if (state == unknown) {
 		wxASSERT(!pServer);
 		if (type != remote)
 			count += m_fileDataList[local].size();
 		if (type != local)
 			count += m_fileDataList[remote].size();
 	}
-	else
-	{
-		if (type != remote)
-		{
-			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-			{
-				if (iter->state != state)
+	else {
+		auto f = [state, pServer](decltype(m_fileDataList[0]) & items) {
+			int count = 0;
+			for (auto const& data : items) {
+				if (data.state != state)
 					continue;
 
-				if (!pServer || iter->server == *pServer)
-					count++;
+				if (!pServer || data.server == *pServer)
+					++count;
 			}
+			return count;
+		};
+		if (type != remote) {
+			count += f(m_fileDataList[local]);
 		}
-		if (type != local)
-		{
-			for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-			{
-				if (iter->state != state)
-					continue;
-
-				if (!pServer || iter->server == *pServer)
-					count++;
-			}
+		if (type != local) {
+			count += f(m_fileDataList[remote]);
 		}
 	}
 
@@ -277,27 +286,24 @@ bool CEditHandler::AddFile(enum CEditHandler::fileType type, wxString& fileName,
 {
 	wxASSERT(type != none);
 
-	enum fileState state;
+	fileState state;
 	if (type == local)
 		state = GetFileState(fileName);
 	else
 		state = GetFileState(fileName, remotePath, server);
-	if (state != unknown)
-	{
+	if (state != unknown) {
 		wxFAIL_MSG(_T("File state not unknown"));
 		return false;
 	}
 
 	t_fileData data;
-	if (type == remote)
-	{
+	if (type == remote) {
 		data.state = download;
 		data.name = fileName;
 		data.file = GetTemporaryFile(fileName);
 		fileName = data.file;
 	}
-	else
-	{
+	else {
 		data.file = fileName;
 		data.name = wxFileName(fileName).GetFullName();
 		data.state = edit;
@@ -357,18 +363,14 @@ bool CEditHandler::RemoveAll(bool force)
 {
 	std::list<t_fileData> keep;
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
-		if (!force && (iter->state == download || iter->state == upload || iter->state == upload_and_remove))
-		{
+	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
+		if (!force && (iter->state == download || iter->state == upload || iter->state == upload_and_remove)) {
 			keep.push_back(*iter);
 			continue;
 		}
 
-		if (wxFileName::FileExists(iter->file))
-		{
-			if (!wxRemoveFile(iter->file))
-			{
+		if (wxFileName::FileExists(iter->file)) {
+			if (!wxRemoveFile(iter->file)) {
 				iter->state = removing;
 				keep.push_back(*iter);
 				continue;
@@ -378,13 +380,11 @@ bool CEditHandler::RemoveAll(bool force)
 	m_fileDataList[remote].swap(keep);
 	keep.clear();
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-	{
+	for (auto iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
 		if (force)
 			continue;
 
-		if (iter->state == upload || iter->state == upload_and_remove)
-		{
+		if (iter->state == upload || iter->state == upload_and_remove) {
 			keep.push_back(*iter);
 			continue;
 		}
@@ -403,24 +403,19 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 
 	std::list<t_fileData> keep;
 
-	for (std::list<t_fileData>::iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
-		if (iter->state != state)
-		{
+	for (auto iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
+		if (iter->state != state) {
 			keep.push_back(*iter);
 			continue;
 		}
 
-		if (pServer && iter->server != *pServer)
-		{
+		if (pServer && iter->server != *pServer) {
 			keep.push_back(*iter);
 			continue;
 		}
 
-		if (wxFileName::FileExists(iter->file))
-		{
-			if (!wxRemoveFile(iter->file))
-			{
+		if (wxFileName::FileExists(iter->file)) {
+			if (!wxRemoveFile(iter->file)) {
 				iter->state = removing;
 				keep.push_back(*iter);
 				continue;
@@ -435,8 +430,7 @@ bool CEditHandler::RemoveAll(enum fileState state, const CServer* pServer /*=0*/
 std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName)
 {
 	std::list<t_fileData>::iterator iter;
-	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-	{
+	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
 		if (iter->file == fileName)
 			break;
 	}
@@ -447,8 +441,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName) const
 {
 	std::list<t_fileData>::const_iterator iter;
-	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter)
-	{
+	for (iter = m_fileDataList[local].begin(); iter != m_fileDataList[local].end(); ++iter) {
 		if (iter->file == fileName)
 			break;
 	}
@@ -459,8 +452,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server)
 {
 	std::list<t_fileData>::iterator iter;
-	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
+	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
 		if (iter->name != fileName)
 			continue;
 
@@ -479,8 +471,7 @@ std::list<CEditHandler::t_fileData>::iterator CEditHandler::GetFile(const wxStri
 std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const wxString& fileName, const CServerPath& remotePath, const CServer& server) const
 {
 	std::list<t_fileData>::const_iterator iter;
-	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
+	for (iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter) {
 		if (iter->name != fileName)
 			continue;
 
@@ -498,7 +489,7 @@ std::list<CEditHandler::t_fileData>::const_iterator CEditHandler::GetFile(const
 
 void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
 {
-	std::list<t_fileData>::iterator iter = GetFile(fileName);
+	auto iter = GetFile(fileName);
 	if (iter == m_fileDataList[local].end())
 		return;
 
@@ -524,7 +515,7 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName)
 
 void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, const CServerPath& remotePath, const CServer& server)
 {
-	std::list<t_fileData>::iterator iter = GetFile(fileName, remotePath, server);
+	auto iter = GetFile(fileName, remotePath, server);
 	if (iter == m_fileDataList[remote].end())
 		return;
 
@@ -575,7 +566,7 @@ void CEditHandler::FinishTransfer(bool successful, const wxString& fileName, con
 
 bool CEditHandler::StartEditing(const wxString& file)
 {
-	std::list<t_fileData>::iterator iter = GetFile(file);
+	auto iter = GetFile(file);
 	if (iter == m_fileDataList[local].end())
 		return false;
 
@@ -584,7 +575,7 @@ bool CEditHandler::StartEditing(const wxString& file)
 
 bool CEditHandler::StartEditing(const wxString& file, const CServerPath& remotePath, const CServer& server)
 {
-	std::list<t_fileData>::iterator iter = GetFile(file, remotePath, server);
+	auto iter = GetFile(file, remotePath, server);
 	if (iter == m_fileDataList[remote].end())
 		return false;
 
@@ -617,10 +608,8 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 	if (insideCheckForModifications)
 		return;
 
-	if (emitEvent)
-	{
-		wxCommandEvent evt(fzEDIT_CHANGEDFILE);
-		AddPendingEvent(evt);
+	if (emitEvent) {
+		QueueEvent(new wxCommandEvent(fzEDIT_CHANGEDFILE));
 		return;
 	}
 
@@ -629,12 +618,12 @@ void CEditHandler::CheckForModifications(bool emitEvent)
 	for (int i = 0; i < 2; i++)
 	{
 checkmodifications_loopbegin:
-		for (std::list<t_fileData>::iterator iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
+		for (auto iter = m_fileDataList[i].begin(); iter != m_fileDataList[i].end(); ++iter)
 		{
 			if (iter->state != edit)
 				continue;
 
-			wxDateTime mtime;
+			CDateTime mtime;
 			bool is_link;
 			if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file) {
 				m_fileDataList[i].erase(iter);
@@ -646,20 +635,17 @@ checkmodifications_loopbegin:
 			if (!mtime.IsValid())
 				continue;
 
-			if (iter->modificationTime.IsValid() && iter->modificationTime == mtime)
+			if (iter->modificationTime.IsValid() && !iter->modificationTime.Compare(mtime))
 				continue;
 
 			// File has changed, ask user what to do
 
 			m_busyTimer.Stop();
-			wxMouseState mouseState = wxGetMouseState();
-			if (mouseState.LeftDown() || mouseState.MiddleDown() || mouseState.RightDown())
-			{
+			if( !wxDialogEx::CanShowPopupDialog() ) {
 				m_busyTimer.Start(1000, true);
 				insideCheckForModifications = false;
 				return;
 			}
-
 			wxTopLevelWindow* pTopWindow = (wxTopLevelWindow*)wxTheApp->GetTopWindow();
 			if (pTopWindow && pTopWindow->IsIconized())
 			{
@@ -680,7 +666,7 @@ checkmodifications_loopbegin:
 			}
 			else if (remove)
 			{
-				if (i == remote)
+				if (i == static_cast<int>(remote))
 				{
 					if (CLocalFileSystem::GetFileInfo(iter->file, is_link, 0, &mtime, 0) != CLocalFileSystem::file || wxRemoveFile(iter->file))
 					{
@@ -720,7 +706,7 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 	else
 		XRCCTRL(dlg, "ID_DESC_UPLOAD_REMOTE", wxStaticText)->Hide();
 
-	dlg.SetLabel(XRCID("ID_FILENAME"), iter->name);
+	dlg.SetChildLabel(XRCID("ID_FILENAME"), iter->name);
 
 	if (type == local)
 	{
@@ -740,10 +726,10 @@ int CEditHandler::DisplayChangeNotification(CEditHandler::fileType type, std::li
 			XRCCTRL(dlg, "ID_OPENEDAS", wxStaticText)->Hide();
 		}
 		else
-			dlg.SetLabel(XRCID("ID_OPENEDAS"), file);
+			dlg.SetChildLabel(XRCID("ID_OPENEDAS"), file);
 	}
-	dlg.SetLabel(XRCID("ID_SERVER"), iter->server.FormatServer());
-	dlg.SetLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
+	dlg.SetChildLabel(XRCID("ID_SERVER"), iter->server.FormatServer());
+	dlg.SetChildLabel(XRCID("ID_REMOTEPATH"), iter->remotePath.GetPath());
 
 	dlg.GetSizer()->Fit(&dlg);
 
@@ -768,7 +754,7 @@ bool CEditHandler::UploadFile(const wxString& file, bool unedit)
 
 bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterator iter, bool unedit)
 {
-	wxASSERT(type != none);
+	wxCHECK(type != none, false);
 
 	if (iter == m_fileDataList[type].end())
 		return false;
@@ -779,8 +765,8 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 
 	iter->state = unedit ? upload_and_remove : upload;
 
-	wxLongLong size;
-	wxDateTime mtime;
+	int64_t size;
+	CDateTime mtime;
 
 	bool is_link;
 	if (CLocalFileSystem::GetFileInfo(iter->file, is_link, &size, &mtime, 0) != CLocalFileSystem::file)
@@ -790,7 +776,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 	}
 
 	if (!mtime.IsValid())
-		mtime = wxDateTime::Now();
+		mtime = CDateTime::Now();
 
 	iter->modificationTime = mtime;
 
@@ -804,7 +790,7 @@ bool CEditHandler::UploadFile(enum fileType type, std::list<t_fileData>::iterato
 		return false;
 	}
 
-	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, wxLongLong(size.GetHi(), size.GetLo()), type, priority_high);
+	m_pQueue->QueueFile(false, false, file, iter->name, localPath, iter->remotePath, iter->server, size, type, QueuePriority::high);
 	m_pQueue->QueueFile_Finish(true);
 
 	return true;
@@ -865,20 +851,20 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 	if (!COptions::Get()->GetOptionVal(OPTION_EDIT_ALWAYSDEFAULT))
 	{
 		const wxString command = GetCustomOpenCommand(file, program_exists);
-		if (command != _T(""))
+		if (!command.empty())
 			return command;
 
 		if (COptions::Get()->GetOptionVal(OPTION_EDIT_INHERITASSOCIATIONS))
 		{
 			const wxString command = GetSystemOpenCommand(file, program_exists);
-			if (command != _T(""))
+			if (!command.empty())
 				return command;
 		}
 	}
 
 	wxString command = COptions::Get()->GetOption(OPTION_EDIT_DEFAULTEDITOR);
-	if (command[0] == '0')
-		return _T(""); // None set
+	if (command.empty() || command[0] == '0')
+		return wxString(); // None set
 	else if (command[0] == '1')
 	{
 		// Text editor
@@ -894,12 +880,12 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 		command = command.Mid(1);
 
 	if (command.empty())
-		return _T("");
+		return wxString();
 
 	wxString args;
 	wxString editor = command;
 	if (!UnquoteCommand(editor, args))
-		return _T("");
+		return wxString();
 
 	if (!ProgramExists(editor))
 	{
@@ -911,126 +897,12 @@ wxString CEditHandler::GetOpenCommand(const wxString& file, bool& program_exists
 	return command + _T(" \"") + file + _T("\"");
 }
 
-
-static bool PathExpand(wxString& cmd)
-{
-#ifndef __WXMSW__
-	if (cmd[0] == '/')
-		return true;
-#else
-	if (cmd[0] == '\\')
-		// UNC or root of current working dir, whatever that is
-		return true;
-	if (cmd.Len() > 2 && cmd[1] == ':')
-		// Absolute path
-		return true;
-#endif
-
-	// Need to search for program in $PATH
-	wxString path;
-	if (!wxGetEnv(_T("PATH"), &path))
-		return false;
-
-	wxString full_cmd;
-	bool found = wxFindFileInPath(&full_cmd, path, cmd);
-#ifdef __WXMSW__
-	if (!found && cmd.Right(4).Lower() != _T(".exe"))
-	{
-		cmd += _T(".exe");
-		found = wxFindFileInPath(&full_cmd, path, cmd);
-	}
-#endif
-
-	if (!found)
-		return false;
-
-	cmd = full_cmd;
-	return true;
-}
-
-wxString CEditHandler::GetSystemOpenCommand(wxString file, bool &program_exists)
-{
-	wxFileName fn(file);
-
-	const wxString& ext = fn.GetExt();
-	if (ext == _T(""))
-		return _T("");
-
-	for (;;)
-	{
-		wxFileType* pType = wxTheMimeTypesManager->GetFileTypeFromExtension(ext);
-		if (!pType)
-			return _T("");
-
-		wxString cmd;
-		if (!pType->GetOpenCommand(&cmd, wxFileType::MessageParameters(file)))
-		{
-			delete pType;
-			return _T("");
-		}
-		delete pType;
-
-		if (cmd.empty())
-			return wxEmptyString;
-
-		program_exists = false;
-
-		wxString editor;
-		bool is_dde = false;
-#ifdef __WXMSW__
-		if (cmd.Left(7) == _T("WX_DDE#"))
-		{
-			// See wxWidget's wxExecute in src/msw/utilsexc.cpp
-			// WX_DDE#<command>#DDE_SERVER#DDE_TOPIC#DDE_COMMAND
-			editor = cmd.Mid(7);
-			int pos = editor.Find('#');
-			if (pos < 1)
-				return cmd;
-			editor = editor.Left(pos);
-			is_dde = true;
-		}
-		else
-#endif
-		{
-			editor = cmd;
-		}
-
-		wxString args;
-		if (!UnquoteCommand(editor, args, is_dde) || editor.empty())
-			return cmd;
-
-		if (!PathExpand(editor))
-			return cmd;
-
-		if (ProgramExists(editor))
-			program_exists = true;
-
-#ifdef __WXGTK__
-		int pos = args.Find(file);
-		if (pos != -1 && file.Find(' ') != -1 && file[0] != '\'' && file[0] != '"')
-		{
-			// Might need to quote filename, wxWidgets doesn't do it
-			if ((!pos || (args[pos - 1] != '\'' && args[pos - 1] != '"')) &&
-				args[pos + file.Length()] != '\'' && args[pos + file.Length()] != '"')
-			{
-				// Filename in command arguments isn't quoted. Repeat with quoted filename
-				file = _T("\"") + file + _T("\"");
-				continue;
-			}
-		}
-#endif
-		return cmd;
-	}
-
-	return wxEmptyString;
-}
-
 wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_exists)
 {
 	wxFileName fn(file);
 
 	wxString ext = fn.GetExt();
-	if (ext == _T(""))
+	if (ext.empty())
 	{
 		if (fn.GetFullName()[0] == '.')
 			ext = _T(".");
@@ -1046,7 +918,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 		wxString assoc = associations.Left(pos);
 		associations = associations.Mid(pos + 1);
 
-		if (assoc == _T(""))
+		if (assoc.empty())
 			continue;
 
 		wxString command;
@@ -1060,10 +932,10 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 
 		wxString args;
 		if (!UnquoteCommand(prog, args))
-			return _T("");
+			return wxString();
 
-		if (prog == _T(""))
-			return _T("");
+		if (prog.empty())
+			return wxString();
 
 		if (!ProgramExists(prog))
 		{
@@ -1075,7 +947,7 @@ wxString CEditHandler::GetCustomOpenCommand(const wxString& file, bool& program_
 		return command + _T(" \"") + fn.GetFullPath() + _T("\"");
 	}
 
-	return _T("");
+	return wxString();
 }
 
 void CEditHandler::OnChangedFileEvent(wxCommandEvent& event)
@@ -1095,8 +967,8 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 	if (max != -1)
 	{
 		name = TruncateFilename(m_localDir, name, max);
-		if (name == _T(""))
-			return _T("");
+		if (name.empty())
+			return wxString();
 	}
 
 	wxString file = m_localDir + name;
@@ -1115,8 +987,8 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 			cutoff *= 10;
 			max--;
 			name = TruncateFilename(m_localDir, name, max);
-			if (name == _T(""))
-				return _T("");
+			if (name.empty())
+				return wxString();
 		}
 
 		int pos = name.Find('.', true);
@@ -1129,7 +1001,7 @@ wxString CEditHandler::GetTemporaryFile(wxString name)
 			return file;
 	}
 
-	return _T("");
+	return wxString();
 }
 
 wxString CEditHandler::TruncateFilename(const wxString path, const wxString& name, int max)
@@ -1145,7 +1017,7 @@ wxString CEditHandler::TruncateFilename(const wxString path, const wxString& nam
 			if (pathlen + extlen >= max)
 			{
 				// Cannot truncate extension
-				return _T("");
+				return wxString();
 			}
 
 			return name.Left(max - pathlen - extlen) + name.Mid(pos);
@@ -1157,20 +1029,19 @@ wxString CEditHandler::TruncateFilename(const wxString path, const wxString& nam
 
 bool CEditHandler::FilenameExists(const wxString& file)
 {
-	for (std::list<t_fileData>::const_iterator iter = m_fileDataList[remote].begin(); iter != m_fileDataList[remote].end(); ++iter)
-	{
-#ifdef __WXMSW__
-		if (!iter->file.CmpNoCase(file))
-#else
-		if (iter->file == file)
-#endif
+	for (auto const& fileData : m_fileDataList[remote]) {
+		// Always ignore case, we don't know which type of filesystem the user profile
+		// is installed upon.
+		if (!fileData.file.CmpNoCase(file))
 			return true;
 	}
 
-	if (wxFileName::FileExists(file))
-	{
+	if (wxFileName::FileExists(file)) {
 		// Save to remove, it's not marked as edited anymore.
-		wxRemoveFile(file);
+		{
+			wxLogNull log;
+			wxRemoveFile(file);
+		}
 
 		if (wxFileName::FileExists(file))
 			return true;
@@ -1215,7 +1086,7 @@ int CEditHandlerStatusDialog::ShowModal()
 
 	if (!pEditHandler->GetFileCount(CEditHandler::none, CEditHandler::unknown))
 	{
-		wxMessageBox(_("No files are currently being edited."), _("Cannot show dialog"), wxICON_INFORMATION, m_pParent);
+		wxMessageBoxEx(_("No files are currently being edited."), _("Cannot show dialog"), wxICON_INFORMATION, m_pParent);
 		return wxID_CANCEL;
 	}
 
@@ -1549,7 +1420,7 @@ CNewAssociationDialog::CNewAssociationDialog(wxWindow *parent)
 {
 }
 
-bool CNewAssociationDialog::Show(const wxString &file)
+bool CNewAssociationDialog::Run(const wxString &file)
 {
 	if (!Load(m_pParent, _T("ID_EDIT_NOPROGRAM")))
 		return true;
@@ -1563,10 +1434,10 @@ bool CNewAssociationDialog::Show(const wxString &file)
 		m_ext.clear();
 
 	wxStaticText *pDesc = XRCCTRL(*this, "ID_DESC", wxStaticText);
-	pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext.c_str()));
+	pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), m_ext));
 
 	bool program_exists = false;
-	wxString cmd = CEditHandler::Get()->GetSystemOpenCommand(_T("foo.txt"), program_exists);
+	wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 	if (!program_exists)
 		cmd.clear();
 	if (!cmd.empty())
@@ -1589,7 +1460,7 @@ bool CNewAssociationDialog::Show(const wxString &file)
 	else
 	{
 		XRCCTRL(*this, "ID_EDITOR_DESC_NONE", wxStaticText)->Hide();
-		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), cmd.c_str()));
+		pDesc->SetLabel(wxString::Format(pDesc->GetLabel(), cmd));
 	}
 
 	SetCtrlState();
@@ -1627,28 +1498,24 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 	const bool custom = XRCCTRL(*this, "ID_USE_CUSTOM", wxRadioButton)->GetValue();
 	const bool always = XRCCTRL(*this, "ID_ALWAYS", wxCheckBox)->GetValue();
 
-	if (custom)
-	{
+	if (custom) {
 		wxString cmd = XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->GetValue();
 		wxString editor = cmd;
 		wxString args;
-		if (!UnquoteCommand(editor, args) || editor.empty())
-		{
-			wxMessageBox(_("You need to enter a properly quoted command."), _("Cannot set file association"), wxICON_EXCLAMATION);
+		if (!UnquoteCommand(editor, args) || editor.empty()) {
+			wxMessageBoxEx(_("You need to enter a properly quoted command."), _("Cannot set file association"), wxICON_EXCLAMATION);
 			return;
 		}
-		if (!ProgramExists(editor))
-		{
-			wxMessageBox(_("Selected editor does not exist."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
+		if (!ProgramExists(editor)) {
+			wxMessageBoxEx(_("Selected editor does not exist."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
 			return;
 		}
 
 		if (always)
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("2") + cmd);
-		else
-		{
+		else {
 			wxString associations = COptions::Get()->GetOption(OPTION_EDIT_CUSTOMASSOCIATIONS);
-			if (associations.Last() != '\n')
+			if (!associations.empty() && associations.Last() != '\n')
 				associations += '\n';
 			if (m_ext.empty())
 				m_ext = _T("/");
@@ -1656,18 +1523,15 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 			COptions::Get()->SetOption(OPTION_EDIT_CUSTOMASSOCIATIONS, associations);
 		}
 	}
-	else
-	{
+	else {
 		if (always)
 			COptions::Get()->SetOption(OPTION_EDIT_DEFAULTEDITOR, _T("1"));
-		else
-		{
+		else {
 			bool program_exists = false;
-			wxString cmd = CEditHandler::Get()->GetSystemOpenCommand(_T("foo.txt"), program_exists);
+			wxString cmd = GetSystemOpenCommand(_T("foo.txt"), program_exists);
 			if (!program_exists)
 				cmd.clear();
-			if (!cmd.empty())
-			{
+			if (!cmd.empty()) {
 				wxString args;
 				if (!UnquoteCommand(cmd, args))
 					cmd.clear();
@@ -1678,7 +1542,7 @@ void CNewAssociationDialog::OnOK(wxCommandEvent& event)
 #endif
 				)
 			{
-				wxMessageBox(_("The default editor for text files could not be found."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
+				wxMessageBoxEx(_("The default editor for text files could not be found."), _("Cannot set file association"), wxICON_EXCLAMATION, this);
 				return;
 			}
 			if (cmd.Find(' ') != -1)
@@ -1712,13 +1576,13 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
 		return;
 
 	wxString editor = dlg.GetPath();
-	if (editor == _T(""))
+	if (editor.empty())
 		return;
 
 	if (!ProgramExists(editor))
 	{
 		XRCCTRL(*this, "ID_EDITOR", wxWindow)->SetFocus();
-		wxMessageBox(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
+		wxMessageBoxEx(_("Selected editor does not exist."), _("File not found"), wxICON_EXCLAMATION, this);
 		return;
 	}
 
@@ -1727,3 +1591,197 @@ void CNewAssociationDialog::OnBrowseEditor(wxCommandEvent& event)
 
 	XRCCTRL(*this, "ID_CUSTOM", wxTextCtrl)->ChangeValue(editor);
 }
+
+bool CEditHandler::Edit(CEditHandler::fileType type, wxString const fileName, CServerPath const& path, CServer const& server, wxLongLong size, wxWindow * parent)
+{
+	std::vector<FileData> data;
+	FileData d{fileName, size};
+	data.push_back(d);
+
+	return Edit(type, data, path, server, parent);
+}
+
+bool CEditHandler::Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, CServer const& server, wxWindow * parent)
+{
+	if (type == CEditHandler::remote) {
+		wxString const& localDir = GetLocalDirectory();
+		if (localDir.empty()) {
+			wxMessageBoxEx(_("Could not get temporary directory to download file into."), _("Cannot edit file"), wxICON_STOP);
+			return false;
+		}
+	}
+
+	if (data.empty()) {
+		wxBell();
+		return false;
+	}
+
+	if (data.size() > 10) {
+		CConditionalDialog dlg(parent, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return false;
+	}
+
+	bool success = true;
+	int already_editing_action{};
+	for (auto const& file : data) {
+		if (!DoEdit(type, file, path, server, parent, data.size(), already_editing_action)) {
+			success = false;
+		}
+	}
+
+	return success;
+}
+
+bool CEditHandler::DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, CServer const& server, wxWindow * parent, size_t fileCount, int & already_editing_action)
+{
+	bool dangerous = false;
+	bool program_exists = false;
+	wxString cmd = CanOpen(type, file.name, dangerous, program_exists);
+	if (cmd.empty()) {
+		CNewAssociationDialog dlg(parent);
+		if (!dlg.Run(file.name)) {
+			return false;
+		}
+		cmd = CanOpen(type, file.name, dangerous, program_exists);
+		if (cmd.empty()) {
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nNo program has been associated on your system with this file type."), file.name), _("Opening failed"), wxICON_EXCLAMATION);
+			return false;
+		}
+	}
+	if (!program_exists) {
+		wxString msg = wxString::Format(_("The file '%s' cannot be opened:\nThe associated program (%s) could not be found.\nPlease check your filetype associations."), file.name, cmd);
+		wxMessageBoxEx(msg, _("Cannot edit file"), wxICON_EXCLAMATION);
+		return false;
+	}
+	if (dangerous) {
+		int res = wxMessageBoxEx(_("The selected file would be executed directly.\nThis can be dangerous and might damage your system.\nDo you really want to continue?"), _("Dangerous filetype"), wxICON_EXCLAMATION | wxYES_NO);
+		if (res != wxYES) {
+			wxBell();
+			return false;
+		}
+	}
+
+	fileState state;
+	if (type == local)
+		state = GetFileState(file.name);
+	else
+		state = GetFileState(file.name, path, server);
+	switch (state)
+	{
+	case CEditHandler::download:
+	case CEditHandler::upload:
+	case CEditHandler::upload_and_remove:
+	case CEditHandler::upload_and_remove_failed:
+		wxMessageBoxEx(_("A file with that name is already being transferred."), _("Cannot view/edit selected file"), wxICON_EXCLAMATION);
+		return false;
+	case CEditHandler::removing:
+		if (!Remove(file.name, path, server)) {
+			wxMessageBoxEx(_("A file with that name is still being edited. Please close it and try again."), _("Selected file is already opened"), wxICON_EXCLAMATION);
+			return false;
+		}
+		break;
+	case CEditHandler::edit:
+		{
+			int action = already_editing_action;
+			if (!action) {
+				wxDialogEx dlg;
+				if (!dlg.Load(parent, type == CEditHandler::local ? _T("ID_EDITEXISTING_LOCAL") : _T("ID_EDITEXISTING_REMOTE"))) {
+					wxBell();
+					return false;
+				}
+				dlg.SetChildLabel(XRCID("ID_FILENAME"), file.name);
+
+				int choices = COptions::Get()->GetOptionVal(OPTION_PERSISTENT_CHOICES);
+
+				if (fileCount < 2) {
+					xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::Hide);
+				}
+				else {
+					if (choices & edit_choices::edit_existing_always) {
+						xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::SetValue, true);
+					}
+				}
+
+				if (type == CEditHandler::remote && (choices & edit_choices::edit_existing_action)) {
+					xrc_call(dlg, "ID_RETRANSFER", &wxRadioButton::SetValue, true);
+				}
+
+				dlg.GetSizer()->Fit(&dlg);
+				int res = dlg.ShowModal();
+				if (res != wxID_OK && res != wxID_YES) {
+					wxBell();
+					action = -1;
+				}
+				else if (type == CEditHandler::local || xrc_call(dlg, "ID_REOPEN", &wxRadioButton::GetValue)) {
+					action = 1;
+					if (type == CEditHandler::remote) {
+						choices &= ~edit_choices::edit_existing_action;
+					}
+				}
+				else {
+					action = 2;
+					choices |= edit_choices::edit_existing_action;
+				}
+
+				bool always = xrc_call(dlg, "ID_ALWAYS", &wxCheckBox::GetValue);
+				if (always) {
+					already_editing_action = action;
+					choices |= edit_choices::edit_existing_always;
+				}
+				else {
+					choices &= ~edit_choices::edit_existing_always;
+				}
+				COptions::Get()->SetOption(OPTION_PERSISTENT_CHOICES, choices);
+			}
+
+			if (action == -1) {
+				return false;
+			}
+			else if (action == 1) {
+				if (type == CEditHandler::local) {
+					StartEditing(file.name);
+				}
+				else {
+					StartEditing(file.name, path, server);
+				}
+				return true;
+			}
+			else {
+				if (!Remove(file.name, path, server)) {
+					wxMessageBoxEx(_("The selected file is still opened in some other program, please close it."), _("Selected file is still being edited"), wxICON_EXCLAMATION);
+					return false;
+				}
+			}
+		}
+		break;
+	default:
+		break;
+	}
+
+	wxString localFile = file.name;
+	if (!AddFile(type, localFile, path, server)) {
+		if( type == CEditHandler::local) {
+			wxMessageBoxEx(wxString::Format(_("The file '%s' could not be opened:\nThe associated command failed"), file.name), _("Opening failed"), wxICON_EXCLAMATION);
+		}
+		else {
+			wxFAIL;
+			wxBell();
+			return false;
+		}
+	}
+
+	if (type == CEditHandler::remote) {
+		wxString localFileName;
+		CLocalPath localPath(localFile, &localFileName);
+
+		m_pQueue->QueueFile(false, true, file.name, (localFileName != file.name) ? localFileName : wxString(),
+			localPath, path, server, file.size, type, QueuePriority::high);
+		m_pQueue->QueueFile_Finish(true);
+	}
+
+	return true;
+}
