@@ -3,8 +3,18 @@
 
 #include "dialogex.h"
 
+#include <wx/timer.h>
+
 // Handles all aspects about remote file viewing/editing
 
+namespace edit_choices {
+enum type
+{
+	edit_existing_action = 0x1,
+	edit_existing_always = 0x2
+};
+}
+
 class CQueueView;
 class CEditHandler : protected wxEvtHandler
 {
@@ -20,7 +30,7 @@ public:
 		removing
 	};
 
-	enum fileType
+	enum fileType : signed char
 	{
 		none = -1,
 		local,
@@ -43,6 +53,16 @@ public:
 	// pServer may be set only if state isn't unknown
 	int GetFileCount(enum fileType type, enum fileState state, const CServer* pServer = 0) const;
 
+	// Starts editing the given file, queues it if needed. For local files, fileName must include local path.
+	// Can be used to edit files already being added, user is prompted for action.
+	bool Edit(CEditHandler::fileType type, wxString const fileName, CServerPath const& path, CServer const& server, wxLongLong size, wxWindow* parent);
+
+	struct FileData {
+		wxString name;
+		wxLongLong size;
+	};
+	bool Edit(CEditHandler::fileType type, std::vector<FileData> const& data, CServerPath const& path, CServer const& server, wxWindow* parent);
+
 	// Adds the file that doesn't exist yet. (Has to be in unknown state)
 	// The initial state will be download
 	bool AddFile(enum fileType type, wxString& fileName, const CServerPath& remotePath, const CServer& server);
@@ -71,18 +91,18 @@ public:
 	wxString CanOpen(enum fileType type, const wxString& fileName, bool &dangerous, bool& program_exists);
 	bool StartEditing(const wxString& file);
 	bool StartEditing(const wxString& file, const CServerPath& remotePath, const CServer& server);
-	
+
 	struct t_fileData
 	{
 		wxString name; // The name of the file
 		wxString file; // The actual local filename
 		fileState state;
-		wxDateTime modificationTime;
+		CDateTime modificationTime;
 		CServerPath remotePath;
 		CServer server;
 	};
 
-	const std::list<t_fileData>& GetFiles(enum fileType type) const { wxASSERT(type != none); return m_fileDataList[type]; }
+	const std::list<t_fileData>& GetFiles(enum fileType type) const { wxASSERT(type != none); return m_fileDataList[(type == local) ? 0 : 1]; }
 
 	bool UploadFile(const wxString& file, bool unedit);
 	bool UploadFile(const wxString& file, const CServerPath& remotePath, const CServer& server, bool unedit);
@@ -90,11 +110,12 @@ public:
 	// Returns command to open the file. If association is set but
 	// program does not exist, program_exists is set to false.
 	wxString GetOpenCommand(const wxString& file, bool& program_exists);
-	wxString GetSystemOpenCommand(const wxString& file, bool &program_exists);
 
 protected:
+	bool DoEdit(CEditHandler::fileType type, FileData const& file, CServerPath const& path, CServer const& server, wxWindow* parent, size_t fileCount, int & already_editing_action);
+
 	CEditHandler();
-	virtual ~CEditHandler() { }
+	virtual ~CEditHandler() {}
 
 	static CEditHandler* m_pEditHandler;
 
@@ -118,8 +139,10 @@ protected:
 	CQueueView* m_pQueue;
 
 	wxTimer m_timer;
-	
-	void RemoveTemporaryFiles(const wxString& temp);
+	wxTimer m_busyTimer;
+
+	void RemoveTemporaryFiles(wxString const& temp);
+	void RemoveTemporaryFilesInSpecificDir(wxString const& temp);
 
 	wxString GetTemporaryFile(wxString name);
 	wxString TruncateFilename(const wxString path, const wxString& name, int max);
@@ -138,10 +161,12 @@ protected:
 	void OnChangedFileEvent(wxCommandEvent& event);
 };
 
+class CWindowStateManager;
 class CEditHandlerStatusDialog : protected wxDialogEx
 {
 public:
 	CEditHandlerStatusDialog(wxWindow* parent);
+	virtual ~CEditHandlerStatusDialog();
 
 	virtual int ShowModal();
 
@@ -152,6 +177,8 @@ protected:
 
 	wxWindow* m_pParent;
 
+	CWindowStateManager* m_pWindowStateManager;
+
 	DECLARE_EVENT_TABLE()
 	void OnSelectionChanged(wxListEvent& event);
 	void OnUnedit(wxCommandEvent& event);
@@ -164,14 +191,14 @@ class CNewAssociationDialog : protected wxDialogEx
 public:
 	CNewAssociationDialog(wxWindow* parent);
 
-	bool Show(const wxString& file);
+	bool Run(const wxString& file);
 
 protected:
 	void SetCtrlState();
 	wxWindow* m_pParent;
 	wxString m_ext;
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnRadioButton(wxCommandEvent& event);
 	void OnOK(wxCommandEvent& event);
 	void OnBrowseEditor(wxCommandEvent& event);
