@@ -3,6 +3,8 @@
 
 #include <option_change_event_handler.h>
 
+#include <wx/timer.h>
+
 class CFilelistStatusBar : public wxStatusBar, protected COptionChangeEventHandler
 {
 public:
@@ -31,18 +33,18 @@ public:
 	void SetConnected(bool connected);
 protected:
 
-	virtual void OnOptionChanged(int option);
+	virtual void OnOptionsChanged(changed_options_t const& options);
 
 	bool m_connected;
 	int m_count_files;
 	int m_count_dirs;
-	wxLongLong m_total_size;
+	int64_t m_total_size;
 	int m_unknown_size; // Set to true if there are files with unknown size
 	int m_hidden;
 
 	int m_count_selected_files;
 	int m_count_selected_dirs;
-	wxLongLong m_total_selected_size;
+	int64_t m_total_selected_size;
 	int m_unknown_selected_size; // Set to true if there are files with unknown size
 
 	wxTimer m_updateTimer;
