@@ -8,7 +8,7 @@
 #include "conditionaldialog.h"
 #include <algorithm>
 #include "filelist_statusbar.h"
-#ifdef __WXGTK__
+#if defined(__WXGTK__) && !defined(__WXGTK3__)
 #include <gtk/gtk.h>
 #endif
 
@@ -65,23 +65,20 @@ typedef struct fz_tagNMLVODSTATECHANGE
 
 template<class CFileData> WXLRESULT CFileListCtrl<CFileData>::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
 {
-	if (nMsg == WM_APPCOMMAND)
-	{
+	if (nMsg == WM_APPCOMMAND) {
 		DWORD cmd = GET_APPCOMMAND_LPARAM(lParam);
-		if (cmd == APPCOMMAND_BROWSER_FORWARD)
-		{
+		if (cmd == APPCOMMAND_BROWSER_FORWARD) {
 			OnNavigationEvent(true);
 			return true;
 		}
-		else if (cmd == APPCOMMAND_BROWSER_BACKWARD)
-		{
+		else if (cmd == APPCOMMAND_BROWSER_BACKWARD) {
 			OnNavigationEvent(false);
 			return true;
 		}
 
 		return wxListCtrlEx::MSWWindowProc(nMsg, wParam, lParam);
 	}
-	
+
 	return wxListCtrlEx::MSWWindowProc(nMsg, wParam, lParam);
 }
 
@@ -93,8 +90,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 	*result = 0;
 
 	NMHDR* pNmhdr = (NMHDR*)lParam;
-	if (pNmhdr->code == LVN_ODSTATECHANGED)
-	{
+	if (pNmhdr->code == LVN_ODSTATECHANGED) {
 		// A range of items got (de)selected
 
 		if (m_insideSetSelection)
@@ -102,21 +98,20 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 
 		if (!m_pFilelistStatusBar)
 			return true;
-		
+
 		if (wxGetKeyState(WXK_CONTROL) && wxGetKeyState(WXK_SHIFT))
 		{
 			// The behavior of Ctrl+Shift+Click is highly erratic.
 			// Even though it is very slow, we need to manually recount.
 			m_pFilelistStatusBar->UnselectAll();
 			int item = -1;
-			while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-			{
+			while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 				if (m_hasParent && !item)
 					continue;
 
 				const int index = m_indexMapping[item];
 				const CFileData& data = m_fileData[index];
-				if (data.flags == fill)
+				if (data.comparison_flags == fill)
 					continue;
 
 				if (ItemIsDir(index))
@@ -125,19 +120,17 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 					m_pFilelistStatusBar->SelectFile(ItemGetSize(index));
 			}
 		}
-		else
-		{
+		else {
 			fzNMLVODSTATECHANGE* pNmOdStateChange = (fzNMLVODSTATECHANGE*)lParam;
 
 			wxASSERT(pNmOdStateChange->iFrom <= pNmOdStateChange->iTo);
-			for (int i = pNmOdStateChange->iFrom; i <= pNmOdStateChange->iTo; i++)
-			{
+			for (int i = pNmOdStateChange->iFrom; i <= pNmOdStateChange->iTo; ++i) {
 				if (m_hasParent && !i)
 					continue;
 
 				const int index = m_indexMapping[i];
 				const CFileData& data = m_fileData[index];
-				if (data.flags == fill)
+				if (data.comparison_flags == fill)
 					continue;
 
 				if (ItemIsDir(index))
@@ -148,8 +141,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 		}
 		return true;
 	}
-	else if (pNmhdr->code == LVN_ITEMCHANGED)
-	{
+	else if (pNmhdr->code == LVN_ITEMCHANGED) {
 		if (m_insideSetSelection)
 			return true;
 
@@ -161,12 +153,10 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 			m_pFilelistStatusBar->UnselectAll();
 		}
 	}
-	else if (pNmhdr->code == LVN_MARQUEEBEGIN)
-	{
+	else if (pNmhdr->code == LVN_MARQUEEBEGIN) {
 		SetFocus();
 	}
-	else if (pNmhdr->code == LVN_GETDISPINFO)
-	{
+	else if (pNmhdr->code == LVN_GETDISPINFO) {
 		// Handle this manually instead of using wx for it
 		// so that we can set the overlay image
 		LV_DISPINFO *info = (LV_DISPINFO *)lParam;
@@ -176,15 +166,13 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 
 		int column = m_pVisibleColumnMapping[lvi.iSubItem];
 
-		if (lvi.mask & LVIF_TEXT)
-		{
+		if (lvi.mask & LVIF_TEXT) {
 			wxString text = GetItemText(item, column);
 			wxStrncpy(lvi.pszText, text, lvi.cchTextMax - 1);
 			lvi.pszText[lvi.cchTextMax - 1] = 0;
 		}
 
-		if (lvi.mask & LVIF_IMAGE)
-		{
+		if (lvi.mask & LVIF_IMAGE) {
 			if (!lvi.iSubItem)
 				lvi.iImage = OnGetItemImage(item);
 			else
@@ -201,7 +189,7 @@ template<class CFileData> bool CFileListCtrl<CFileData>::MSWOnNotify(int idCtrl,
 }
 #endif
 
-#ifdef __WXGTK__
+#if defined(__WXGTK__) && !defined(__WXGTK3__)
 // Need to call a member function of a C++ template class
 // from a C function.
 // Sadly template functions with C linkage aren't possible,
@@ -210,6 +198,7 @@ class CGtkEventCallbackProxyBase
 {
 public:
 	virtual void OnNavigationEvent(bool forward) = 0;
+	virtual ~CGtkEventCallbackProxyBase() {}
 };
 
 template <class CFileData> class CGtkEventCallbackProxy : public CGtkEventCallbackProxyBase
@@ -241,15 +230,11 @@ static gboolean gtk_button_release_event(GtkWidget*, void *gdk_event, CGtkEventC
 }
 #endif
 
-template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CState* pState, CQueueView* pQueue, bool border /*=false*/)
+template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pParent, CState*, CQueueView* pQueue, bool border /*=false*/)
 : wxListCtrlEx(pParent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxLC_VIRTUAL | wxLC_REPORT | wxLC_EDIT_LABELS | (border ? wxBORDER_SUNKEN : wxNO_BORDER)),
-	CComparableListing(this), CSystemImageList(16)
+	CComparableListing(this)
 {
-#ifdef __WXMSW__
-	m_pHeaderImageList = 0;
-#endif
-	m_header_icon_index.down = m_header_icon_index.up = -1;
-
+	CreateSystemImageList(16);
 	m_pQueue = pQueue;
 
 	m_sortColumn = 0;
@@ -275,78 +260,19 @@ template<class CFileData> CFileListCtrl<CFileData>::CFileListCtrl(wxWindow* pPar
 	m_focusItem = -1;
 #endif
 
-#ifdef __WXGTK__
-	m_gtkEventCallbackProxy = new CGtkEventCallbackProxy<CFileData>(this);
+#if defined __WXGTK__ && !defined(__WXGTK3__)
+	m_gtkEventCallbackProxy.reset(new CGtkEventCallbackProxy<CFileData>(this));
 
 	GtkWidget* widget = GetMainWindow()->GetConnectWidget();
-	g_signal_connect(widget, "button_release_event", G_CALLBACK(gtk_button_release_event), m_gtkEventCallbackProxy.Value());
+	g_signal_connect(widget, "button_release_event", G_CALLBACK(gtk_button_release_event), m_gtkEventCallbackProxy.get());
 #endif
+
+	m_genericTypes[genericTypes::file] = _("File");
+	m_genericTypes[genericTypes::directory] = _("Directory");
 }
 
 template<class CFileData> CFileListCtrl<CFileData>::~CFileListCtrl()
 {
-#ifdef __WXMSW__
-	delete m_pHeaderImageList;
-#endif
-}
-
-template<class CFileData> void CFileListCtrl<CFileData>::InitHeaderImageList()
-{
-#ifdef __WXMSW__
-	// Initialize imagelist for list header
-	m_pHeaderImageList = new wxImageListEx(8, 8, true, 3);
-
-	wxBitmap bmp;
-
-	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("up.png"), wxBITMAP_TYPE_PNG);
-	m_pHeaderImageList->Add(bmp);
-	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("down.png"), wxBITMAP_TYPE_PNG);
-	m_pHeaderImageList->Add(bmp);
-
-	HWND hWnd = (HWND)GetHandle();
-	if (!hWnd)
-	{
-		delete m_pHeaderImageList;
-		m_pHeaderImageList = 0;
-		return;
-	}
-
-	HWND header = (HWND)SendMessage(hWnd, LVM_GETHEADER, 0, 0);
-	if (!header)
-	{
-		delete m_pHeaderImageList;
-		m_pHeaderImageList = 0;
-		return;
-	}
-
-	TCHAR buffer[1000] = {0};
-	HDITEM item;
-	item.mask = HDI_TEXT;
-	item.pszText = buffer;
-	item.cchTextMax = 999;
-	SendMessage(header, HDM_GETITEM, 0, (LPARAM)&item);
-
-	SendMessage(header, HDM_SETIMAGELIST, 0, (LPARAM)m_pHeaderImageList->GetHandle());
-
-	m_header_icon_index.up = 0;
-	m_header_icon_index.down = 1;
-#else
-
-	wxColour colour = wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
-
-	wxString lightness;
-	if (colour.Red() + colour.Green() + colour.Blue() > 3 * 128)
-		lightness = _T("DARK");
-	else
-		lightness = _T("LIGHT");
-
-	wxBitmap bmp;
-
-	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_UP_") + lightness,  wxART_OTHER, wxSize(16, 16));
-	m_header_icon_index.up = m_pImageList->Add(bmp);
-	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_DOWN_") + lightness,  wxART_OTHER, wxSize(16, 16));
-	m_header_icon_index.down = m_pImageList->Add(bmp);
-#endif
 }
 
 template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=-1*/, int direction /*=-1*/, bool updateSelections /*=true*/)
@@ -359,7 +285,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		{
 			const int oldVisibleColumn = GetColumnVisibleIndex(m_sortColumn);
 			if (oldVisibleColumn != -1)
-				SetHeaderIconIndex(oldVisibleColumn, -1);
+				SetHeaderSortIconIndex(oldVisibleColumn, -1);
 		}
 	}
 	else
@@ -375,7 +301,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		column = 0;
 	}
 
-	SetHeaderIconIndex(newVisibleColumn, direction ? m_header_icon_index.down : m_header_icon_index.up);
+	SetHeaderSortIconIndex(newVisibleColumn, direction);
 
 	// Remember which files are selected
 	bool *selected = 0;
@@ -387,7 +313,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 
 #ifndef __WXMSW__
 		// GetNextItem is O(n) if nothing is selected, GetSelectedItemCount() is O(1)
-		if (GetSelectedItemCount())		
+		if (GetSelectedItemCount())
 #endif
 		{
 			int item = -1;
@@ -409,7 +335,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 		m_sortColumn = column;
 		std::vector<unsigned int>::iterator start = m_indexMapping.begin();
 		if (m_hasParent)
-			start++;
+			++start;
 		std::reverse(start, m_indexMapping.end());
 
 		if (updateSelections)
@@ -434,7 +360,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList(int column /*=
 
 	std::vector<unsigned int>::iterator start = m_indexMapping.begin();
 	if (m_hasParent)
-		start++;
+		++start;
 	CSortComparisonObject object = GetSortComparisonObject();
 	std::sort(start, m_indexMapping.end(), object);
 	object.Destroy();
@@ -470,31 +396,53 @@ template<class CFileData> void CFileListCtrl<CFileData>::SortList_UpdateSelectio
 	}
 }
 
-template<class CFileData> CListViewSort::DirSortMode CFileListCtrl<CFileData>::GetDirSortMode()
+template<class CFileData> CFileListCtrlSortBase::DirSortMode CFileListCtrl<CFileData>::GetDirSortMode()
 {
 	const int dirSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_DIRSORT);
 
-	enum CListViewSort::DirSortMode dirSortMode;
+	enum CFileListCtrlSortBase::DirSortMode dirSortMode;
 	switch (dirSortOption)
 	{
 	case 0:
 	default:
-		dirSortMode = CListViewSort::dirsort_ontop;
+		dirSortMode = CFileListCtrlSortBase::dirsort_ontop;
 		break;
 	case 1:
 		if (m_sortDirection)
-			dirSortMode = CListViewSort::dirsort_onbottom;
+			dirSortMode = CFileListCtrlSortBase::dirsort_onbottom;
 		else
-			dirSortMode = CListViewSort::dirsort_ontop;
+			dirSortMode = CFileListCtrlSortBase::dirsort_ontop;
 		break;
 	case 2:
-		dirSortMode = CListViewSort::dirsort_inline;
+		dirSortMode = CFileListCtrlSortBase::dirsort_inline;
 		break;
 	}
 
 	return dirSortMode;
 }
 
+template<class CFileData> CFileListCtrlSortBase::NameSortMode CFileListCtrl<CFileData>::GetNameSortMode()
+{
+	const int nameSortOption = COptions::Get()->GetOptionVal(OPTION_FILELIST_NAMESORT);
+
+	enum CFileListCtrlSortBase::NameSortMode nameSortMode;
+	switch (nameSortOption)
+	{
+	case 0:
+	default:
+		nameSortMode = CFileListCtrlSortBase::namesort_caseinsensitive;
+		break;
+	case 1:
+		nameSortMode = CFileListCtrlSortBase::namesort_casesensitive;
+		break;
+	case 2:
+		nameSortMode = CFileListCtrlSortBase::namesort_natural;
+		break;
+	}
+
+	return nameSortMode;
+}
+
 template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListEvent &event)
 {
 	int col = m_pVisibleColumnMapping[event.GetColumn()];
@@ -526,10 +474,25 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnClicked(wxListE
 	RefreshListOnly(false);
 }
 
+#ifdef __WXMSW__
+namespace {
+wxString GetExt(const wxString& file)
+{
+	wxString ret;
+
+	int pos = file.Find('.', true);
+	if (pos > 0 && file[pos + 1] != 0) // Does neither starts nor end with dot
+		ret = file.Mid(pos + 1);
+
+	return ret;
+}
+}
+#endif
+
 template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString name, bool dir, const wxString& path /*=_T("")*/)
 {
 #ifdef __WXMSW__
-	wxString ext = wxFileName(name).GetExt();
+	wxString ext = GetExt(name);
 	ext.MakeLower();
 	std::map<wxString, wxString>::iterator typeIter = m_fileTypeMap.find(ext);
 	if (typeIter != m_fileTypeMap.end())
@@ -537,7 +500,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 
 	wxString type;
 	int flags = SHGFI_TYPENAME;
-	if (path == _T(""))
+	if (path.empty())
 		flags |= SHGFI_USEFILEATTRIBUTES;
 	else if (path == _T("\\"))
 		name += _T("\\");
@@ -552,45 +515,47 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 		sizeof(shFinfo),
 		flags))
 	{
-		type = shFinfo.szTypeName;
-		if (type == _T(""))
+		if (!*shFinfo.szTypeName)
 		{
-			type = ext;
-			type.MakeUpper();
-			if (!type.IsEmpty())
+			if (!ext.empty())
 			{
+				type = ext;
+				type.MakeUpper();
 				type += _T("-");
 				type += _("file");
 			}
 			else
-				type = _("File");
+				type = m_genericTypes[genericTypes::file];
 		}
 		else
 		{
-			if (!dir && ext != _T(""))
+			type = shFinfo.szTypeName;
+			if (!dir && !ext.empty())
 				m_fileTypeMap[ext.MakeLower()] = type;
 		}
 	}
 	else
 	{
-		type = ext;
-		type.MakeUpper();
-		if (!type.IsEmpty())
+		if (!ext.empty())
 		{
+			type = ext;
+			type.MakeUpper();
 			type += _T("-");
 			type += _("file");
 		}
 		else
-			type = _("File");
+			type = m_genericTypes[genericTypes::file];
 	}
 	return type;
 #else
+	(void)path;
+
 	if (dir)
-		return _("Directory");
+		return m_genericTypes[genericTypes::directory];
 
 	int pos = name.Find('.', true);
 	if (pos < 1 || !name[pos + 1]) // Starts or ends with dot
-		return _("File");
+		return m_genericTypes[genericTypes::file];
 	wxString ext = name.Mid(pos + 1);
 	wxString lower_ext = ext.Lower();
 
@@ -609,7 +574,7 @@ template<class CFileData> wxString CFileListCtrl<CFileData>::GetType(wxString na
 	}
 
 	wxString desc;
-	if (pType->GetDescription(&desc) && desc != _T(""))
+	if (pType->GetDescription(&desc) && !desc.empty())
 	{
 		delete pType;
 		m_fileTypeMap[ext] = desc;
@@ -651,7 +616,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnExitComparisonMode()
 	m_indexMapping.swap(m_originalIndexMapping);
 
 	for (unsigned int i = 0; i < m_fileData.size() - 1; i++)
-		m_fileData[i].flags = normal;
+		m_fileData[i].comparison_flags = normal;
 
 	SetItemCount(m_indexMapping.size());
 
@@ -669,7 +634,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::CompareAddFile(t_fileEn
 	}
 
 	int index = m_originalIndexMapping[m_comparisonIndex];
-	m_fileData[index].flags = flags;
+	m_fileData[index].comparison_flags = flags;
 
 	m_indexMapping.push_back(index);
 }
@@ -686,7 +651,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 	{
 		SetItemState(focus, 0, wxLIST_STATE_FOCUSED);
 		int index = m_indexMapping[focus];
-		if (m_fileData[index].flags == fill)
+		if (m_fileData[index].comparison_flags == fill)
 			focus = -1;
 		else
 			focus = index;
@@ -702,7 +667,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRememberSelec
 		while ((item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
 		{
 			int index = m_indexMapping[item];
-			if (m_fileData[index].flags == fill)
+			if (m_fileData[index].comparison_flags == fill)
 				continue;
 			m_comparisonSelections.push_back(index);
 		}
@@ -753,7 +718,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::ComparisonRestoreSelect
 	}
 }
 
-template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wxListEvent& event)
+template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wxListEvent&)
 {
 	ShowColumnEditor();
 }
@@ -761,7 +726,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnColumnRightClicked(wx
 template<class CFileData> void CFileListCtrl<CFileData>::InitSort(int optionID)
 {
 	wxString sortInfo = COptions::Get()->GetOption(optionID);
-	m_sortDirection = sortInfo[0] - '0';
+	if( !sortInfo.empty() ) {
+		m_sortDirection = sortInfo[0] - '0';
+	}
+	else {
+		m_sortDirection = 0;
+	}
 	if (m_sortDirection < 0 || m_sortDirection > 1)
 		m_sortDirection = 0;
 
@@ -804,7 +774,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnItemSelected(wxListEv
 
 	const int index = m_indexMapping[item];
 	const CFileData& data = m_fileData[index];
-	if (data.flags == fill)
+	if (data.comparison_flags == fill)
 		return;
 
 	if (ItemIsDir(index))
@@ -840,7 +810,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnItemDeselected(wxList
 
 	const int index = m_indexMapping[item];
 	const CFileData& data = m_fileData[index];
-	if (data.flags == fill)
+	if (data.comparison_flags == fill)
 		return;
 
 	if (ItemIsDir(index))
@@ -865,12 +835,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnFocusChanged(wxListEv
 	const int focusItem = event.GetIndex();
 
 	// Need to defer processing, as focus it set before selection by wxWidgets internally
-	wxCommandEvent evt;
-	evt.SetEventType(fz_EVT_FILELIST_FOCUSCHANGE);
-	evt.SetInt(m_focusItem);
-	evt.SetExtraLong((long)focusItem);
+	wxCommandEvent *evt = new wxCommandEvent();
+	evt->SetEventType(fz_EVT_FILELIST_FOCUSCHANGE);
+	evt->SetInt(m_focusItem);
+	evt->SetExtraLong((long)focusItem);
 	m_pending_focus_processing++;
-	AddPendingEvent(evt);
+	QueueEvent(evt);
 
 	m_focusItem = focusItem;
 }
@@ -941,7 +911,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnProcessFocusChange(wx
 
 		const int index = m_indexMapping[i];
 		const CFileData& data = m_fileData[index];
-		if (data.flags == fill)
+		if (data.comparison_flags == fill)
 			continue;
 
 		if (selected)
@@ -966,12 +936,12 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnLeftDown(wxMouseEvent
 	// Left clicks in the whitespace around the items deselect everything
 	// but does not change focus. Defer event.
 	event.Skip();
-	wxCommandEvent evt;
-	evt.SetEventType(fz_EVT_DEFERRED_MOUSEEVENT);
-	AddPendingEvent(evt);
+	wxCommandEvent *evt = new wxCommandEvent();
+	evt->SetEventType(fz_EVT_DEFERRED_MOUSEEVENT);
+	QueueEvent(evt);
 }
 
-template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxCommandEvent& event)
+template<class CFileData> void CFileListCtrl<CFileData>::OnProcessMouseEvent(wxCommandEvent&)
 {
 	if (m_pending_focus_processing)
 		return;
@@ -1028,7 +998,7 @@ template<class CFileData> void CFileListCtrl<CFileData>::OnKeyDown(wxKeyEvent& e
 		for (unsigned int i = m_hasParent ? 1 : 0; i < m_indexMapping.size(); i++)
 		{
 			const CFileData& data = m_fileData[m_indexMapping[i]];
-			if (data.flags != fill)
+			if (data.comparison_flags != fill)
 				SetSelection(i, true);
 			else
 				SetSelection(i, false);
