@@ -118,7 +118,7 @@ void CFilterEditDialog::SaveFilter(CFilter& filter)
 	}
 }
 
-void CFilterEditDialog::OnNew(wxCommandEvent& event)
+void CFilterEditDialog::OnNew(wxCommandEvent&)
 {
 	if (m_currentSelection != -1)
 	{
@@ -131,22 +131,22 @@ void CFilterEditDialog::OnNew(wxCommandEvent& event)
 	wxString name = _("New filter");
 	wxString newName = name;
 	while (m_pFilterListCtrl->FindString(newName) != wxNOT_FOUND)
-		newName = wxString::Format(_T("%s (%d)"), name.c_str(), ++index);
+		newName = wxString::Format(_T("%s (%d)"), name, ++index);
 
 	wxTextEntryDialog dlg(this, _("Please enter a name for the new filter."), _("Enter filter name"), newName);
 	if (dlg.ShowModal() != wxID_OK)
 		return;
 	newName = dlg.GetValue();
 
-	if (newName == _T(""))
+	if (newName.empty())
 	{
-		wxMessageBox(_("No filter name given"), _("Cannot create new filter"), wxICON_INFORMATION);
+		wxMessageBoxEx(_("No filter name given"), _("Cannot create new filter"), wxICON_INFORMATION);
 		return;
 	}
 
 	if (m_pFilterListCtrl->FindString(newName) != wxNOT_FOUND)
 	{
-		wxMessageBox(_("The entered filter name already exists, please choose a different name."), _("Filter name already exists"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("The entered filter name already exists, please choose a different name."), _("Filter name already exists"), wxICON_ERROR, this);
 		return;
 	}
 
@@ -155,7 +155,7 @@ void CFilterEditDialog::OnNew(wxCommandEvent& event)
 
 	m_filters.push_back(filter);
 
-	for (std::vector<CFilterSet>::iterator iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
+	for (auto iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
 	{
 		CFilterSet& set = *iter;
 		set.local.push_back(false);
@@ -179,7 +179,7 @@ void CFilterEditDialog::OnDelete(wxCommandEvent& event)
 	m_filters.erase(m_filters.begin() + item);
 
 	// Remote filter from all filter sets
-	for (std::vector<CFilterSet>::iterator iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
+	for (auto iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
 	{
 		CFilterSet& set = *iter;
 		set.local.erase(set.local.begin() + item);
@@ -203,9 +203,9 @@ void CFilterEditDialog::OnRename(wxCommandEvent& event)
 	const wxString& newName = pDlg->GetValue();
 	delete pDlg;
 
-	if (newName == _T(""))
+	if (newName.empty())
 	{
-		wxMessageBox(_("Empty filter names are not allowed."), _("Empty name"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("Empty filter names are not allowed."), _("Empty name"), wxICON_ERROR, this);
 		return;
 	}
 
@@ -214,7 +214,7 @@ void CFilterEditDialog::OnRename(wxCommandEvent& event)
 
 	if (m_pFilterListCtrl->FindString(newName) != wxNOT_FOUND)
 	{
-		wxMessageBox(_("The entered filter name already exists, please choose a different name."), _("Filter name already exists"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("The entered filter name already exists, please choose a different name."), _("Filter name already exists"), wxICON_ERROR, this);
 		return;
 	}
 
@@ -238,22 +238,22 @@ void CFilterEditDialog::OnCopy(wxCommandEvent& event)
 	const wxString& name = filter.name;
 	wxString newName = name;
 	while (m_pFilterListCtrl->FindString(newName) != wxNOT_FOUND)
-		newName = wxString::Format(_T("%s (%d)"), name.c_str(), ++index);
+		newName = wxString::Format(_T("%s (%d)"), name, ++index);
 
 	wxTextEntryDialog dlg(this, _("Please enter a new name for the copied filter."), _("Enter filter name"), newName);
 	if (dlg.ShowModal() != wxID_OK)
 		return;
 
 	newName = dlg.GetValue();
-	if (newName == _T(""))
+	if (newName.empty())
 	{
-		wxMessageBox(_("Empty filter names are not allowed."), _("Empty name"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("Empty filter names are not allowed."), _("Empty name"), wxICON_ERROR, this);
 		return;
 	}
 
 	if (m_pFilterListCtrl->FindString(newName) != wxNOT_FOUND)
 	{
-		wxMessageBox(_("The entered filter name already exists, please choose a different name."), _("Filter name already exists"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("The entered filter name already exists, please choose a different name."), _("Filter name already exists"), wxICON_ERROR, this);
 		return;
 	}
 
@@ -261,7 +261,7 @@ void CFilterEditDialog::OnCopy(wxCommandEvent& event)
 
 	m_filters.push_back(filter);
 
-	for (std::vector<CFilterSet>::iterator iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
+	for (auto iter = m_filterSets.begin(); iter != m_filterSets.end(); ++iter)
 	{
 		CFilterSet& set = *iter;
 		set.local.push_back(false);
@@ -334,28 +334,25 @@ bool CFilterEditDialog::Validate()
 		return true;
 
 	wxString error;
-	if (!ValidateFilter(error))
-	{
+	if (!ValidateFilter(error)) {
 		m_pFilterListCtrl->SetSelection(m_currentSelection);
-		wxMessageBox(error, _("Filter validation failed"), wxICON_ERROR, this);
+		wxMessageBoxEx(error, _("Filter validation failed"), wxICON_ERROR, this);
 		return false;
 	}
 
 	wxString name = XRCCTRL(*this, "ID_NAME", wxTextCtrl)->GetValue();
-	if (name == _T(""))
-	{
+	if (name.empty()) {
 		m_pFilterListCtrl->SetSelection(m_currentSelection);
 		XRCCTRL(*this, "ID_NAME", wxTextCtrl)->SetFocus();
-		wxMessageBox(_("Need to enter filter name"), _("Filter validation failed"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("Need to enter filter name"), _("Filter validation failed"), wxICON_ERROR, this);
 		return false;
 	}
 
 	int pos = m_pFilterListCtrl->FindString(name);
-	if (pos != wxNOT_FOUND && pos != m_currentSelection)
-	{
+	if (pos != wxNOT_FOUND && pos != m_currentSelection) {
 		m_pFilterListCtrl->SetSelection(m_currentSelection);
 		XRCCTRL(*this, "ID_NAME", wxTextCtrl)->SetFocus();
-		wxMessageBox(_("Filter name already exists"), _("Filter validation failed"), wxICON_ERROR, this);
+		wxMessageBoxEx(_("Filter name already exists"), _("Filter validation failed"), wxICON_ERROR, this);
 		return false;
 	}
 
