@@ -21,7 +21,7 @@
 DECLARE_EVENT_TYPE(fzEVT_POSTSCROLL, -1)
 DEFINE_EVENT_TYPE(fzEVT_POSTSCROLL)
 
-BEGIN_EVENT_TABLE(wxListCtrlEx, wxListCtrl)
+BEGIN_EVENT_TABLE(wxListCtrlEx, wxListCtrlExBase)
 EVT_COMMAND(wxID_ANY, fzEVT_POSTSCROLL, wxListCtrlEx::OnPostScrollEvent)
 EVT_SCROLLWIN(wxListCtrlEx::OnScrollEvent)
 EVT_MOUSEWHEEL(wxListCtrlEx::OnMouseWheel)
@@ -44,15 +44,8 @@ wxListCtrlEx::wxListCtrlEx(wxWindow *parent,
 						   long style,
 						   const wxValidator& validator,
 						   const wxString& name)
-						   : wxListCtrl(parent, id, pos, size, style, validator, name)
 {
-#ifdef __WXMSW__
-	m_pHeaderImageList = 0;
-#endif
-	m_header_icon_index.down = m_header_icon_index.up = -1;
-
-	m_pVisibleColumnMapping = 0;
-	m_prefixSearch_enabled = false;
+	Create(parent, id, pos, size, style, validator, name);
 
 #ifndef __WXMSW__
 	m_editing = false;
@@ -70,22 +63,29 @@ wxListCtrlEx::~wxListCtrlEx()
 	delete [] m_pVisibleColumnMapping;
 }
 
-#ifndef __WXMSW__
-wxScrolledWindow* wxListCtrlEx::GetMainWindow() const
+wxWindow* wxListCtrlEx::GetMainWindow()
 {
-#ifdef __WXMAC__
-	return (wxScrolledWindow*)m_genericImpl->m_mainWin;
+#ifdef __WXMSW__
+	return this;
 #else
-	return (wxScrolledWindow*)m_mainWin;
+	return reinterpret_cast<wxWindow*>(m_mainWin);
 #endif
 }
+
+wxWindow const* wxListCtrlEx::GetMainWindow() const
+{
+#ifdef __WXMSW__
+	return this;
+#else
+	return reinterpret_cast<wxWindow const*>(m_mainWin);
 #endif
+}
 
 void wxListCtrlEx::OnPostScroll()
 {
 }
 
-void wxListCtrlEx::OnPostScrollEvent(wxCommandEvent& event)
+void wxListCtrlEx::OnPostScrollEvent(wxCommandEvent&)
 {
 	OnPostScroll();
 }
@@ -97,8 +97,7 @@ void wxListCtrlEx::OnPreEmitPostScrollEvent()
 
 void wxListCtrlEx::EmitPostScrollEvent()
 {
-	wxCommandEvent evt(fzEVT_POSTSCROLL, wxID_ANY);
-	AddPendingEvent(evt);
+	QueueEvent(new wxCommandEvent(fzEVT_POSTSCROLL, wxID_ANY));
 }
 
 void wxListCtrlEx::OnScrollEvent(wxScrollWinEvent& event)
@@ -121,7 +120,6 @@ void wxListCtrlEx::OnSelectionChanged(wxListEvent& event)
 
 void wxListCtrlEx::ScrollTopItem(int item)
 {
-#ifdef __WXMSW__
 	const int current = GetTopItem();
 
 	int delta = item - current;
@@ -133,10 +131,6 @@ void wxListCtrlEx::ScrollTopItem(int item)
 
 	delta *= rect.GetHeight();
 	ScrollList(0, delta);
-#else
-	GetMainWindow()->Scroll(0, item);
-	EnsureVisible(item);
-#endif
 }
 
 
@@ -174,7 +168,7 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 		if (text.Length() >= m_prefixSearch_prefix.Length() && !m_prefixSearch_prefix.CmpNoCase(text.Left(m_prefixSearch_prefix.Length())))
 			beep = true;
 	}
-	else if (m_prefixSearch_prefix == _T(""))
+	else if (m_prefixSearch_prefix.empty())
 		beep = true;
 
 	int start = item;
@@ -183,8 +177,7 @@ void wxListCtrlEx::HandlePrefixSearch(wxChar character)
 
 	int newPos = FindItemWithPrefix(newPrefix, start);
 
-	if (newPos == -1 && m_prefixSearch_prefix[0] == character && !m_prefixSearch_prefix[1] && item != -1 && beep)
-	{
+	if (newPos == -1 && (m_prefixSearch_prefix.Len() == 1 && m_prefixSearch_prefix[0] == character) && item != -1 && beep) {
 		// Search the next item that starts with the same letter
 		newPrefix = m_prefixSearch_prefix;
 		newPos = FindItemWithPrefix(newPrefix, item + 1);
@@ -295,7 +288,10 @@ void wxListCtrlEx::OnKeyDown(wxKeyEvent& event)
 
 	// Get the actual key
 	BYTE state[256];
-	GetKeyboardState(state);
+	if (!GetKeyboardState(state)) {
+		event.Skip();
+		return;
+	}
 	wxChar buffer[1];
 	int res = ToUnicode(event.GetRawKeyCode(), 0, state, buffer, 1, 0);
 	if (res != 1)
@@ -756,7 +752,7 @@ protected:
 		{
 			wxCheckListBox* pListBox = XRCCTRL(*this, "ID_ACTIVE", wxCheckListBox);
 			pListBox->Check(0);
-			wxMessageBox(_("The filename column can neither be hidden nor moved."), _("Column properties"));
+			wxMessageBoxEx(_("The filename column can neither be hidden nor moved."), _("Column properties"));
 		}
 	}
 };
@@ -772,7 +768,10 @@ void wxListCtrlEx::ShowColumnEditor()
 {
 	CColumnEditDialog dlg;
 
-	dlg.Load(this, _T("ID_COLUMN_SETUP"));
+	if (!dlg.Load(this, _T("ID_COLUMN_SETUP"))) {
+		wxBell();
+		return;
+	}
 
 	wxCheckListBox* pListBox = XRCCTRL(dlg, "ID_ACTIVE", wxCheckListBox);
 
@@ -806,7 +805,7 @@ void wxListCtrlEx::ShowColumnEditor()
 		if (!isChecked && !col)
 		{
 			isChecked = true;
-			wxMessageBox(_("The filename column cannot be hidden."));
+			wxMessageBoxEx(_("The filename column cannot be hidden."));
 		}
 		MoveColumn(col, i);
 		if (m_columnInfo[col].shown != isChecked)
@@ -870,9 +869,9 @@ void wxListCtrlEx::InitHeaderSortImageList()
 
 	wxBitmap bmp;
 
-	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("up.png"), wxBITMAP_TYPE_PNG);
+	bmp.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("up.png"), wxBITMAP_TYPE_PNG);
 	m_pHeaderImageList->Add(bmp);
-	bmp.LoadFile(wxGetApp().GetResourceDir() + _T("down.png"), wxBITMAP_TYPE_PNG);
+	bmp.LoadFile(wxGetApp().GetResourceDir().GetPath() + _T("down.png"), wxBITMAP_TYPE_PNG);
 	m_pHeaderImageList->Add(bmp);
 
 	HWND hWnd = (HWND)GetHandle();
@@ -915,9 +914,9 @@ void wxListCtrlEx::InitHeaderSortImageList()
 	wxBitmap bmp;
 
 	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_UP_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-	m_header_icon_index.up = m_pImageList->Add(bmp);
+	m_header_icon_index.up = GetSystemImageList()->Add(bmp);
 	bmp = wxArtProvider::GetBitmap(_T("ART_SORT_DOWN_") + lightness, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
-	m_header_icon_index.down = m_pImageList->Add(bmp);
+	m_header_icon_index.down = GetSystemImageList()->Add(bmp);
 #endif
 }
 
@@ -977,11 +976,7 @@ void wxListCtrlEx::SetHeaderSortIconIndex(int col, int icon)
 void wxListCtrlEx::RefreshListOnly(bool eraseBackground /*=true*/)
 {
 	// See comment in wxGenericListCtrl::Refresh
-#ifdef __WXMSW__
-	Refresh(eraseBackground);
-#else
 	GetMainWindow()->Refresh(eraseBackground);
-#endif
 }
 
 void wxListCtrlEx::CancelLabelEdit()
@@ -1160,161 +1155,16 @@ bool wxListCtrlEx::HasSelection() const
 }
 
 
-wxRect wxListCtrlEx::GetListRect() const
+wxRect wxListCtrlEx::GetActualClientRect() const
 {
-#ifdef __WXMSW__
-	wxRect windowRect = GetClientRect();
-	{
-		wxRect topRect;
-		if (GetItemRect(0, topRect))
-		{
-			windowRect.height -= topRect.y;
-			windowRect.y += topRect.y;
-		}
-	}
-#else
 	wxRect windowRect = GetMainWindow()->GetClientRect();
+#ifdef __WXMSW__
+	wxRect topRect;
+	if (GetItemRect(0, topRect)) {
+		windowRect.height -= topRect.y;
+		windowRect.y += topRect.y;
+	}
 #endif
 
 	return windowRect;
 }
-
-
-CListCtrlDropTarget::CListCtrlDropTarget(wxListCtrlEx* pListCtrl)
-	: m_pListCtrl(pListCtrl)
-	, m_count()
-{
-	m_timer.SetOwner(this);
-}
-
-
-bool CListCtrlDropTarget::OnDrop(wxCoord x, wxCoord y)
-{
-	m_timer.Stop();
-	return true;
-}
-
-
-wxDragResult CListCtrlDropTarget::OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
-{
-	if (!m_timer.IsRunning() && IsScroll(wxPoint(x, y)))
-	{
-		m_timer.Start(100, true);
-		m_count = 0;
-	}
-	return def;
-}
-
-
-
-void CListCtrlDropTarget::OnLeave()
-{
-	m_timer.Stop();
-}
-
-
-wxDragResult CListCtrlDropTarget::OnEnter(wxCoord x, wxCoord y, wxDragResult def)
-{
-	if (!m_timer.IsRunning() && IsScroll(wxPoint(x, y)))
-	{
-		m_timer.Start(100, true);
-		m_count = 0;
-	}
-	return def;
-}
-
-
-bool CListCtrlDropTarget::IsScroll(wxPoint p) const
-{
-	return IsTopScroll(p) || IsBottomScroll(p);
-}
-
-
-bool CListCtrlDropTarget::IsTopScroll(wxPoint p) const
-{
-	wxRect itemRect;
-	if (!m_pListCtrl->GetItemRect(m_pListCtrl->GetTopItem(), itemRect))
-		return false;
-
-	wxRect windowRect = m_pListCtrl->GetListRect();
-
-	if (itemRect.GetHeight() > windowRect.GetHeight() / 4 ) {
-		itemRect.SetHeight( wxMax( windowRect.GetHeight() / 4, 8 ) );
-	}
-
-	if (p.y < 0 || p.y >= itemRect.GetBottom())
-		return false;
-
-	if (p.x < 0 || p.x > windowRect.GetWidth())
-		return false;
-
-	int top = m_pListCtrl->GetTopItem();
-	if( top <= 0 ) {
-		return false;
-	}
-
-	return true;
-}
-
-
-bool CListCtrlDropTarget::IsBottomScroll(wxPoint p) const
-{
-	wxRect itemRect;
-	if (!m_pListCtrl->GetItemRect(0, itemRect))
-		return false;
-
-	wxRect windowRect = m_pListCtrl->GetListRect();
-	if (itemRect.GetHeight() > windowRect.GetHeight() / 4 ) {
-		itemRect.SetHeight( wxMax( windowRect.GetHeight() / 4, 8 ) );
-	}
-
-	if (p.y > windowRect.GetBottom() || p.y < windowRect.GetBottom() - itemRect.GetHeight())
-		return false;
-
-	if (p.x < 0 || p.x > windowRect.GetWidth())
-		return false;
-
-	int bottom = m_pListCtrl->GetTopItem() + m_pListCtrl->GetCountPerPage();
-	if (bottom >= m_pListCtrl->GetItemCount())
-		return false;
-
-	return true;
-}
-
-
-void CListCtrlDropTarget::OnTimer(wxTimerEvent& /*event*/)
-{
-	wxPoint p = wxGetMousePosition();
-#ifdef __WXMSW__
-	wxWindow* ctrl = m_pListCtrl;
-#else
-	wxWindow* ctrl = m_pListCtrl->GetMainWindow();
-#endif
-	p = ctrl->ScreenToClient(p);
-
-	if (IsTopScroll(p))
-	{
-		int top = m_pListCtrl->GetTopItem();
-		m_pListCtrl->EnsureVisible(top - 1);
-	}
-	else if (IsBottomScroll(p))
-	{
-		int top = m_pListCtrl->GetTopItem();
-		m_pListCtrl->EnsureVisible(top + m_pListCtrl->GetCountPerPage());
-	}
-	else
-	{
-		m_timer.Stop();
-		return;
-	}
-
-	DisplayDropHighlight(p);
-
-	if (m_count < 90)
-		++m_count;
-	m_timer.Start(100 - m_count, true);
-}
-
-BEGIN_EVENT_TABLE(CListCtrlDropTarget, wxEvtHandler)
-EVT_TIMER(wxID_ANY, CListCtrlDropTarget::OnTimer)
-END_EVENT_TABLE()
