@@ -17,17 +17,19 @@ END_EVENT_TABLE()
 
 CMenuBar::CMenuBar()
 	: CStateEventHandler(0)
+	, m_pMainFrame()
 {
 }
 
 CMenuBar::~CMenuBar()
 {
-	for (std::map<wxMenu*, std::map<int, wxMenuItem*> >::iterator menu_iter = m_hidden_items.begin(); menu_iter != m_hidden_items.end(); menu_iter++)
-	{
-		for (std::map<int, wxMenuItem*>::iterator iter = menu_iter->second.begin(); iter != menu_iter->second.end(); iter++)
-			delete iter->second;
+	for (auto hidden_menu : m_hidden_items) {
+		for (auto hidden_item : hidden_menu.second) {
+			delete hidden_item.second;
+		}
 	}
 
+	m_pMainFrame->Disconnect(wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(CMenuBar::OnMenuEvent), 0, this);
 }
 
 CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
@@ -38,7 +40,7 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 
 	menubar->m_pMainFrame = pMainFrame;
 
-	
+
 #if FZ_MANUALUPDATECHECK
 	if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK))
 #endif
@@ -82,6 +84,7 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 
 	menubar->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
 	menubar->Check(XRCID("ID_VIEW_QUICKCONNECT"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT) != 0);
+	menubar->Check(XRCID("ID_VIEW_TOOLBAR"), COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) == 0);
 	menubar->Check(XRCID("ID_VIEW_MESSAGELOG"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
 	menubar->Check(XRCID("ID_VIEW_QUEUE"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE) != 0);
 	menubar->Check(XRCID("ID_VIEW_LOCALTREE"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL) != 0);
@@ -102,6 +105,8 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 		break;
 	}
 
+	menubar->UpdateSpeedLimitMenuItem();
+
 	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2)
 		menubar->HideItem(XRCID("ID_VIEW_MESSAGELOG"));
 
@@ -120,6 +125,7 @@ CMenuBar* CMenuBar::Load(CMainFrame* pMainFrame)
 	menubar->RegisterOption(OPTION_ASCIIBINARY);
 	menubar->RegisterOption(OPTION_PRESERVE_TIMESTAMPS);
 
+	menubar->RegisterOption(OPTION_TOOLBAR_HIDDEN);
 	menubar->RegisterOption(OPTION_SHOW_MESSAGELOG);
 	menubar->RegisterOption(OPTION_SHOW_QUEUE);
 	menubar->RegisterOption(OPTION_SHOW_TREE_LOCAL);
@@ -141,13 +147,13 @@ void CMenuBar::UpdateBookmarkMenu()
 		return;
 
 	// Delete old bookmarks
-	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_global.begin(); iter != m_bookmark_menu_id_map_global.end(); iter++)
+	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_global.begin(); iter != m_bookmark_menu_id_map_global.end(); ++iter)
 	{
 		pMenu->Delete(iter->first);
 	}
 	m_bookmark_menu_id_map_global.clear();
 
-	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.begin(); iter != m_bookmark_menu_id_map_site.end(); iter++)
+	for (std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.begin(); iter != m_bookmark_menu_id_map_site.end(); ++iter)
 	{
 		pMenu->Delete(iter->first);
 	}
@@ -161,7 +167,7 @@ void CMenuBar::UpdateBookmarkMenu()
 			pMenu->Delete(pSeparator);
 	}
 
-	std::list<int>::iterator ids = m_bookmark_menu_ids.begin();
+	auto ids = m_bookmark_menu_ids.begin();
 
 	// Insert global bookmarks
 	std::list<wxString> global_bookmarks;
@@ -169,7 +175,7 @@ void CMenuBar::UpdateBookmarkMenu()
 	{
 		pMenu->AppendSeparator();
 
-		for (std::list<wxString>::const_iterator iter = global_bookmarks.begin(); iter != global_bookmarks.end(); iter++)
+		for (std::list<wxString>::const_iterator iter = global_bookmarks.begin(); iter != global_bookmarks.end(); ++iter)
 		{
 			int id;
 			if (ids == m_bookmark_menu_ids.end())
@@ -180,7 +186,7 @@ void CMenuBar::UpdateBookmarkMenu()
 			else
 			{
 				id = *ids;
-				ids++;
+				++ids;
 			}
 			wxString name(*iter);
 			name.Replace(_T("&"), _T("&&"));
@@ -201,7 +207,7 @@ void CMenuBar::UpdateBookmarkMenu()
 
 	pMenu->AppendSeparator();
 
-	for (std::list<wxString>::const_iterator iter = controls->site_bookmarks->bookmarks.begin(); iter != controls->site_bookmarks->bookmarks.end(); iter++)
+	for (std::list<wxString>::const_iterator iter = controls->site_bookmarks->bookmarks.begin(); iter != controls->site_bookmarks->bookmarks.end(); ++iter)
 	{
 		int id;
 		if (ids == m_bookmark_menu_ids.end())
@@ -212,7 +218,7 @@ void CMenuBar::UpdateBookmarkMenu()
 		else
 		{
 			id = *ids;
-			ids++;
+			++ids;
 		}
 		wxString name(*iter);
 		name.Replace(_T("&"), _T("&&"));
@@ -227,18 +233,19 @@ void CMenuBar::ClearBookmarks()
 	CContextControl* pContextControl = m_pMainFrame ? m_pMainFrame->GetContextControl() : 0;
 	CContextControl::_context_controls* controls = pContextControl ? pContextControl->GetCurrentControls() : 0;
 
-	if (!controls)
-		controls->site_bookmarks = new CContextControl::_context_controls::_site_bookmarks;
+	if (controls && !controls->site_bookmarks)
+		controls->site_bookmarks = std::make_shared<CContextControl::_context_controls::_site_bookmarks>();
 	UpdateBookmarkMenu();
 }
 
 void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 {
 	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (!pState)
+		return;
 
 	std::map<int, wxString>::const_iterator iter = m_bookmark_menu_id_map_site.find(event.GetId());
-	if (iter != m_bookmark_menu_id_map_site.end())
-	{
+	if (iter != m_bookmark_menu_id_map_site.end()) {
 		// We hit a site-specific bookmark
 		CContextControl* pContextControl = m_pMainFrame ? m_pMainFrame->GetContextControl() : 0;
 		CContextControl::_context_controls* controls = pContextControl ? pContextControl->GetCurrentControls() : 0;
@@ -252,38 +259,29 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 		name.Replace(_T("/"), _T("\\/"));
 		name = controls->site_bookmarks->path + _T("/") + name;
 
-		CSiteManagerItemData_Site *pData = CSiteManager::GetSiteByPath(name);
+		std::unique_ptr<CSiteManagerItemData_Site> pData = CSiteManager::GetSiteByPath(name);
 		if (!pData)
 			return;
 
-		if (!pState)
-			return;
-
 		pState->SetSyncBrowse(false);
-		if (!pData->m_remoteDir.IsEmpty() && pState->IsRemoteIdle())
-		{
+		if (!pData->m_remoteDir.empty() && pState->IsRemoteIdle()) {
 			const CServer* pServer = pState->GetServer();
-			if (!pServer || *pServer != pData->m_server)
-			{
-				m_pMainFrame->ConnectToSite(pData);
+			if (!pServer || *pServer != pData->m_server) {
+				m_pMainFrame->ConnectToSite(*pData);
 				pData->m_localDir.clear(); // So not to set again below
 			}
 			else
 				pState->ChangeRemoteDir(pData->m_remoteDir);
 		}
-		if (!pData->m_localDir.empty())
-		{
+		if (!pData->m_localDir.empty()) {
 			bool set = pState->SetLocalDir(pData->m_localDir);
 
-			if (set && pData->m_sync)
-			{
-				wxASSERT(!pData->m_remoteDir.IsEmpty());
+			if (set && pData->m_sync) {
+				wxASSERT(!pData->m_remoteDir.empty());
 				pState->SetSyncBrowse(true, pData->m_remoteDir);
 			}
 		}
 
-		delete pData;
-
 		return;
 	}
 
@@ -298,15 +296,15 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 			return;
 
 		pState->SetSyncBrowse(false);
-		if (!remote_dir.IsEmpty() && pState->IsRemoteIdle())
+		if (!remote_dir.empty() && pState->IsRemoteIdle())
 		{
 			const CServer* pServer = pState->GetServer();
 			if (pServer)
 			{
 				CServerPath current_remote_path = pState->GetRemotePath();
-				if (!current_remote_path.IsEmpty() && current_remote_path.GetType() != remote_dir.GetType())
+				if (!current_remote_path.empty() && current_remote_path.GetType() != remote_dir.GetType())
 				{
-					wxMessageBox(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
+					wxMessageBoxEx(_("Selected global bookmark and current server use a different server type.\nUse site-specific bookmarks for this server."), _("Bookmark"), wxICON_EXCLAMATION, this);
 					return;
 				}
 				pState->ChangeRemoteDir(remote_dir);
@@ -318,7 +316,7 @@ void CMenuBar::OnMenuEvent(wxCommandEvent& event)
 
 			if (set && sync)
 			{
-				wxASSERT(!remote_dir.IsEmpty());
+				wxASSERT(!remote_dir.empty());
 				pState->SetSyncBrowse(true, remote_dir);
 			}
 		}
@@ -364,11 +362,9 @@ void CMenuBar::OnStateChange(CState* pState, enum t_statechange_notifications no
 	}
 }
 
-void CMenuBar::OnOptionChanged(int option)
+void CMenuBar::OnOptionsChanged(changed_options_t const& options)
 {
-	switch (option)
-	{
-	case OPTION_ASCIIBINARY:
+	if (options.test(OPTION_ASCIIBINARY)) {
 		switch (COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY))
 		{
 		default:
@@ -381,61 +377,61 @@ void CMenuBar::OnOptionChanged(int option)
 			Check(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), true);
 			break;
 		}
-		break;
-	case OPTION_PRESERVE_TIMESTAMPS:
+	}
+	if (options.test(OPTION_PRESERVE_TIMESTAMPS)) {
 		Check(XRCID("ID_MENU_TRANSFER_PRESERVETIMES"), COptions::Get()->GetOptionVal(OPTION_PRESERVE_TIMESTAMPS) != 0);
-		break;
-	case OPTION_SHOW_TREE_LOCAL:
+	}
+	if (options.test(OPTION_SHOW_TREE_LOCAL)) {
 		Check(XRCID("ID_VIEW_LOCALTREE"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_LOCAL) != 0);
-		break;
-	case OPTION_SHOW_TREE_REMOTE:
+	}
+	if (options.test(OPTION_SHOW_TREE_REMOTE)) {
 		Check(XRCID("ID_VIEW_REMOTETREE"), COptions::Get()->GetOptionVal(OPTION_SHOW_TREE_REMOTE) != 0);
-		break;
-	case OPTION_SHOW_QUICKCONNECT:
+	}
+	if (options.test(OPTION_SHOW_QUICKCONNECT)) {
 		Check(XRCID("ID_VIEW_QUICKCONNECT"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUICKCONNECT) != 0);
-		break;
-	case OPTION_SHOW_MESSAGELOG:
+	}
+	if (options.test(OPTION_TOOLBAR_HIDDEN)) {
+		Check(XRCID("ID_VIEW_TOOLBAR"), COptions::Get()->GetOptionVal(OPTION_TOOLBAR_HIDDEN) == 0);
+	}
+	if (options.test(OPTION_SHOW_MESSAGELOG)) {
 		Check(XRCID("ID_VIEW_MESSAGELOG"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
-		break;
-	case OPTION_SHOW_QUEUE:
+	}
+	if (options.test(OPTION_SHOW_QUEUE)) {
 		Check(XRCID("ID_VIEW_QUEUE"), COptions::Get()->GetOptionVal(OPTION_SHOW_QUEUE) != 0);
-		break;
-	case OPTION_COMPARE_HIDEIDENTICAL:
+	}
+	if (options.test(OPTION_COMPARE_HIDEIDENTICAL)) {
 		Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->GetOptionVal(OPTION_COMPARE_HIDEIDENTICAL) != 0);
-		break;
-	case OPTION_COMPARISONMODE:
+	}
+	if (options.test(OPTION_COMPARISONMODE)) {
 		if (COptions::Get()->GetOptionVal(OPTION_COMPARISONMODE) != 1)
 			Check(XRCID("ID_COMPARE_SIZE"), true);
 		else
 			Check(XRCID("ID_COMPARE_DATE"), true);
-		break;
-	case OPTION_MESSAGELOG_POSITION:
+	}
+	if (options.test(OPTION_MESSAGELOG_POSITION)) {
 		if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 2)
 			HideItem(XRCID("ID_VIEW_MESSAGELOG"));
-		else
-		{
+		else {
 			ShowItem(XRCID("ID_VIEW_MESSAGELOG"));
 			Check(XRCID("ID_VIEW_MESSAGELOG"), COptions::Get()->GetOptionVal(OPTION_SHOW_MESSAGELOG) != 0);
 		}
-		break;
-	case OPTION_SPEEDLIMIT_ENABLE:
-	case OPTION_SPEEDLIMIT_INBOUND:
-	case OPTION_SPEEDLIMIT_OUTBOUND:
-		{
-			bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) != 0;
-
-			int downloadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
-			int uploadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
-
-			if (!downloadLimit && !uploadLimit)
-				enable = false;
-
-			Check(XRCID("ID_MENU_TRANSFER_SPEEDLIMITS_ENABLE"), enable);
-		}
-		break;
-	default:
-		break;
 	}
+	if (options.test(OPTION_SPEEDLIMIT_ENABLE) || options.test(OPTION_SPEEDLIMIT_INBOUND) || options.test(OPTION_SPEEDLIMIT_OUTBOUND)) {
+		UpdateSpeedLimitMenuItem();
+	}
+}
+
+void CMenuBar::UpdateSpeedLimitMenuItem()
+{
+	bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) != 0;
+
+	int downloadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
+	int uploadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
+
+	if (!downloadLimit && !uploadLimit)
+		enable = false;
+
+	Check(XRCID("ID_MENU_TRANSFER_SPEEDLIMITS_ENABLE"), enable);
 }
 
 void CMenuBar::UpdateMenubarState()
@@ -464,7 +460,7 @@ void CMenuBar::UpdateMenubarState()
 	else
 	{
 		CServer tmp;
-		canReconnect = pState->GetLastServer().GetHost() != _T("");
+		canReconnect = !pState->GetLastServer().GetHost().empty();
 	}
 	Enable(XRCID("ID_MENU_SERVER_RECONNECT"), canReconnect);
 
@@ -479,11 +475,11 @@ void CMenuBar::UpdateMenubarState()
 
 bool CMenuBar::ShowItem(int id)
 {
-	for (std::map<wxMenu*, std::map<int, wxMenuItem*> >::iterator menu_iter = m_hidden_items.begin(); menu_iter != m_hidden_items.end(); menu_iter++)
+	for (auto menu_iter = m_hidden_items.begin(); menu_iter != m_hidden_items.end(); ++menu_iter)
 	{
 		int offset = 0;
 
-		for (std::map<int, wxMenuItem*>::iterator iter = menu_iter->second.begin(); iter != menu_iter->second.end(); iter++)
+		for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter)
 		{
 			if (iter->second->GetId() != id)
 			{
@@ -516,16 +512,16 @@ bool CMenuBar::HideItem(int id)
 
 	pMenu->Remove(pItem);
 
-	std::map<wxMenu*, std::map<int, wxMenuItem*> >::iterator menu_iter = m_hidden_items.insert(std::make_pair(pMenu, std::map<int, wxMenuItem*>())).first;
+	auto menu_iter = m_hidden_items.insert(std::make_pair(pMenu, std::map<int, wxMenuItem*>())).first;
 
-	for (std::map<int, wxMenuItem*>::iterator iter = menu_iter->second.begin(); iter != menu_iter->second.end(); iter++)
+	for (auto iter = menu_iter->second.begin(); iter != menu_iter->second.end(); ++iter)
 	{
 		if (iter->first > (int)pos)
 			break;
-		
+
 		pos++;
 	}
-	
+
 	menu_iter->second[(int)pos] = pItem;
 
 	return true;
