@@ -7,38 +7,29 @@
 #include "timeformatting.h"
 #include "themeprovider.h"
 
-CQueueItem::CQueueItem()
+CQueueItem::CQueueItem(CQueueItem* parent)
+	: m_parent(parent)
 {
-	m_visibleOffspring = 0;
-	m_parent = 0;
-	m_maxCachedIndex = -1;
-
-	m_removed_at_front = 0;
-	m_indent = 0;
 }
 
 CQueueItem::~CQueueItem()
 {
-	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 		delete *iter;
 }
 
-void CQueueItem::SetPriority(enum QueuePriority priority)
+void CQueueItem::SetPriority(QueuePriority priority)
 {
-	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 		(*iter)->SetPriority(priority);
 }
 
 void CQueueItem::AddChild(CQueueItem* item)
 {
-	if (m_removed_at_front)
-	{
+	if (m_removed_at_front) {
 		m_children.erase(m_children.begin(), m_children.begin() + m_removed_at_front);
 		m_removed_at_front = 0;
 	}
-	item->m_indent = m_indent + 1;
 	m_children.push_back(item);
 
 	m_visibleOffspring += 1 + item->m_visibleOffspring;
@@ -93,7 +84,7 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 		child = m_lookupCache[m_maxCachedIndex].child + 1;
 	}
 
-	for (; iter != m_children.end(); iter++, child++)
+	for (; iter != m_children.end(); ++iter, ++child)
 	{
 		if (!item)
 			return *iter;
@@ -130,9 +121,8 @@ unsigned int CQueueItem::GetChildrenCount(bool recursive)
 bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 {
 	int oldVisibleOffspring = m_visibleOffspring;
-	std::vector<CQueueItem*>::iterator iter;
 	bool deleted = false;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 	{
 		if (*iter == pItem)
 		{
@@ -204,7 +194,7 @@ bool CQueueItem::TryRemoveAll()
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
 	m_visibleOffspring = 0;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 	{
 		CQueueItem* pItem = *iter;
 		if (pItem->TryRemoveAll())
@@ -270,7 +260,7 @@ int CQueueItem::GetItemIndex() const
 		return 0;
 
 	int index = 1;
-	for (std::vector<CQueueItem*>::const_iterator iter = pParent->m_children.begin() + pParent->m_removed_at_front; iter != pParent->m_children.end(); iter++)
+	for (std::vector<CQueueItem*>::const_iterator iter = pParent->m_children.begin() + pParent->m_removed_at_front; iter != pParent->m_children.end(); ++iter)
 	{
 		if (*iter == this)
 			break;
@@ -281,62 +271,45 @@ int CQueueItem::GetItemIndex() const
 	return index + pParent->GetItemIndex();
 }
 
-const wxString& CQueueItem::GetIndent() const
-{ 
-	wxASSERT(m_indent < 5);
-	static const wxString indents[5] = 
-	{ 
-		_T(""),
-		_T("  "),
-		_T("    "),
-		_T("      "),
-		_T("        ")
-	};
-	return indents[m_indent];
-}
-
 CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
 					 const wxString& sourceFile, const wxString& targetFile,
 					 const CLocalPath& localPath, const CServerPath& remotePath, wxLongLong size)
-	: m_sourceFile(sourceFile), m_targetFile(targetFile)
-	, m_localPath(localPath), m_remotePath(remotePath)
+	: CQueueItem(parent)
+	, m_sourceFile(sourceFile)
+	, m_targetFile(targetFile.empty() ? CSparseOptional<wxString>() : CSparseOptional<wxString>(targetFile))
+	, m_localPath(localPath)
+	, m_remotePath(remotePath)
 	, m_size(size)
 {
-	m_parent = parent;
-	m_priority = priority_normal;
-
-	flags = 0;
 	if (download)
 		flags |= flag_download;
 	if (queued)
 		flags |= flag_queued;
-	m_errorCount = 0;
-	m_pEngineData = 0;
-	m_defaultFileExistsAction = CFileExistsNotification::unknown;
-	m_edit = CEditHandler::none;
-	m_onetime_action = CFileExistsNotification::unknown;
 }
 
 CFileItem::~CFileItem()
 {
 }
 
-void CFileItem::SetPriority(enum QueuePriority priority)
+void CFileItem::SetPriority(QueuePriority priority)
 {
 	if (priority == m_priority)
 		return;
 
-	CServerItem* parent = (CServerItem*)m_parent;
-	parent->SetChildPriority(this, m_priority, priority);
+	if (m_parent)
+	{
+		CServerItem* parent = static_cast<CServerItem*>(m_parent);
+		parent->SetChildPriority(this, m_priority, priority);
+	}
 	m_priority = priority;
 }
 
-void CFileItem::SetPriorityRaw(enum QueuePriority priority)
+void CFileItem::SetPriorityRaw(QueuePriority priority)
 {
 	m_priority = priority;
 }
 
-enum QueuePriority CFileItem::GetPriority() const
+QueuePriority CFileItem::GetPriority() const
 {
 	return m_priority;
 }
@@ -371,9 +344,9 @@ void CFileItem::SaveItem(TiXmlElement* pElement) const
 		AddTextElement(file, "Size", m_size.ToString());
 	if (m_errorCount)
 		AddTextElement(file, "ErrorCount", m_errorCount);
-	if (m_priority != priority_normal)
-		AddTextElement(file, "Priority", m_priority);
-	AddTextElementRaw(file, "DataType", m_transferSettings.binary ? "1" : "0");
+	if (m_priority != QueuePriority::normal)
+		AddTextElement(file, "Priority", static_cast<int>(m_priority));
+	AddTextElementRaw(file, "DataType", Ascii() ? "0" : "1");
 	if (m_defaultFileExistsAction != CFileExistsNotification::unknown)
 		AddTextElement(file, "OverwriteAction", m_defaultFileExistsAction);
 }
@@ -387,15 +360,41 @@ bool CFileItem::TryRemoveAll()
 	return false;
 }
 
-void CFileItem::SetTargetFile(const wxString &file)
+void CFileItem::SetTargetFile(wxString const& file)
 {
-	wxASSERT(!file.empty());
-	if (file != m_sourceFile)
-		m_targetFile = file;
+	if (!file.empty() && file != m_sourceFile)
+		m_targetFile = CSparseOptional<wxString>(file);
 	else
 		m_targetFile.clear();
 }
 
+void CFileItem::SetStatusMessage(CFileItem::Status status)
+{
+	m_status = status;
+}
+
+wxString const& CFileItem::GetStatusMessage() const
+{
+	static wxString statusTexts[] = {
+		wxString(),
+		_("Incorrect password"),
+		_("Timeout"),
+		_("Disconnecting from previous server"),
+		_("Disconnected from server"),
+		_("Connecting"),
+		_("Connection attempt failed"),
+		_("Interrupted by user"),
+		_("Waiting for browsing connection"),
+		_("Waiting for password"),
+		_("Could not write to local file"),
+		_("Could not start transfer"),
+		_("Transferring"),
+		_("Creating directory")
+	};
+
+	return statusTexts[m_status];
+}
+
 CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath)
 	: CFileItem(parent, queued, true, wxEmptyString, wxEmptyString, localPath, CServerPath(), -1)
 {
@@ -431,9 +430,9 @@ void CFolderItem::SetActive(const bool active)
 }
 
 CServerItem::CServerItem(const CServer& server)
+	: m_activeCount(0)
+	, m_server(server)
 {
-	m_server = server;
-	m_activeCount = 0;
 }
 
 CServerItem::~CServerItem()
@@ -453,8 +452,8 @@ wxString CServerItem::GetName() const
 void CServerItem::AddChild(CQueueItem* pItem)
 {
 	CQueueItem::AddChild(pItem);
-	if (pItem->GetType() == QueueItemType_File ||
-		pItem->GetType() == QueueItemType_Folder)
+	if (pItem->GetType() == QueueItemType::File ||
+		pItem->GetType() == QueueItemType::Folder)
 		AddFileItemToList((CFileItem*)pItem);
 }
 
@@ -463,16 +462,14 @@ void CServerItem::AddFileItemToList(CFileItem* pItem)
 	if (!pItem)
 		return;
 
-	m_fileList[pItem->queued() ? 0 : 1][pItem->GetPriority()].push_back(pItem);
+	m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())].push_back(pItem);
 }
 
 void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
 {
-	std::list<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][pItem->GetPriority()];
-	for (std::list<CFileItem*>::iterator iter = fileList.begin(); iter != fileList.end(); iter++)
-	{
-		if (*iter == pItem)
-		{
+	std::list<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
+	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
+		if (*iter == pItem) {
 			fileList.erase(iter);
 			return;
 		}
@@ -480,45 +477,39 @@ void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
 	wxFAIL_MSG(_T("File item not deleted from m_fileList"));
 }
 
-void CServerItem::SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction)
+void CServerItem::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction)
 {
-	for (std::vector<CQueueItem*>::iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
-	{
+	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		CQueueItem *pItem = *iter;
-		if (pItem->GetType() == QueueItemType_File)
-		{
+		if (pItem->GetType() == QueueItemType::File) {
 			CFileItem* pFileItem = ((CFileItem *)pItem);
-			if (direction == upload && pFileItem->Download())
+			if (direction == TransferDirection::upload && pFileItem->Download())
 				continue;
-			else if (direction == download && !pFileItem->Download())
+			else if (direction == TransferDirection::download && !pFileItem->Download())
 				continue;
 			pFileItem->m_defaultFileExistsAction = action;
 		}
-		else if (pItem->GetType() == QueueItemType_FolderScan)
-		{
-			if (direction == download)
+		else if (pItem->GetType() == QueueItemType::FolderScan) {
+			if (direction == TransferDirection::download)
 				continue;
 			((CFolderScanItem *)pItem)->m_defaultFileExistsAction = action;
 		}
 	}
 }
 
-CFileItem* CServerItem::GetIdleChild(bool immediateOnly, enum TransferDirection direction)
+namespace {
+CFileItem* DoGetIdleChild(std::list<CFileItem*> const* fileList, TransferDirection direction)
 {
 	int i = 0;
-	for (i = (PRIORITY_COUNT - 1); i >= 0; i--)
-	{
-		std::list<CFileItem*>& fileList = m_fileList[1][i];
-		for (std::list<CFileItem*>::iterator iter = fileList.begin(); iter != fileList.end(); iter++)
-		{
-			CFileItem* item = *iter;
+	for (i = static_cast<int>(QueuePriority::count) - 1; i >= 0; --i) {
+		for (auto const& item : fileList[i]) {
 			if (item->IsActive())
 				continue;
 
-			if (direction == both)
+			if (direction == TransferDirection::both)
 				return item;
 
-			if (direction == download)
+			if (direction == TransferDirection::download)
 			{
 				if (item->Download())
 					return item;
@@ -527,42 +518,27 @@ CFileItem* CServerItem::GetIdleChild(bool immediateOnly, enum TransferDirection
 				return item;
 		}
 	}
-	if (immediateOnly)
-		return 0;
-
-	for (i = (PRIORITY_COUNT - 1); i >= 0; i--)
-	{
-		std::list<CFileItem*>& fileList = m_fileList[0][i];
-		for (std::list<CFileItem*>::iterator iter = fileList.begin(); iter != fileList.end(); iter++)
-		{
-			CFileItem* item = *iter;
-			if (item->IsActive())
-				continue;
-
-			if (direction == both)
-				return item;
-
-			if (direction == download)
-			{
-				if (item->Download())
-					return item;
-			}
-			else if (!item->Download())
-				return item;
-		}
-	}
-
 	return 0;
 }
+}
+
+CFileItem* CServerItem::GetIdleChild(bool immediateOnly, TransferDirection direction)
+{
+	CFileItem* item = DoGetIdleChild(m_fileList[1], direction);
+	if( !item && !immediateOnly ) {
+		item = DoGetIdleChild(m_fileList[0], direction);
+	}
+	return item;
+}
 
 bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 {
 	if (!pItem)
 		return false;
 
-	if (pItem->GetType() == QueueItemType_File || pItem->GetType() == QueueItemType_Folder)
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
 	{
-		CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
+		CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
 		RemoveFileItemFromList(pFileItem);
 	}
 
@@ -571,12 +547,10 @@ bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 
 void CServerItem::QueueImmediateFiles()
 {
-	for (int i = 0; i < PRIORITY_COUNT; i++)
-	{
+	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
 		std::list<CFileItem*> activeList;
 		std::list<CFileItem*>& fileList = m_fileList[1][i];
-		for (std::list<CFileItem*>::reverse_iterator iter = fileList.rbegin(); iter != fileList.rend(); iter++)
-		{
+		for (std::list<CFileItem*>::reverse_iterator iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
 			CFileItem* item = *iter;
 			wxASSERT(!item->queued());
 			if (item->IsActive())
@@ -596,15 +570,15 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	if (pItem->queued())
 		return;
 
-	std::list<CFileItem*>& fileList = m_fileList[1][pItem->GetPriority()];
-	for (std::list<CFileItem*>::iterator iter = fileList.begin(); iter != fileList.end(); iter++)
+	std::list<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
+	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter)
 	{
 		if (*iter != pItem)
 			continue;
 
 		pItem->set_queued(true);
 		fileList.erase(iter);
-		m_fileList[0][pItem->GetPriority()].push_front(pItem);
+		m_fileList[0][static_cast<int>(pItem->GetPriority())].push_front(pItem);
 		return;
 	}
 	wxASSERT(false);
@@ -615,7 +589,7 @@ void CServerItem::SaveItem(TiXmlElement* pElement) const
 	TiXmlElement *server = new TiXmlElement("Server");
 	SetServer(server, m_server);
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 		(*iter)->SaveItem(server);
 
 	pElement->LinkEndChild(server);
@@ -624,12 +598,12 @@ void CServerItem::SaveItem(TiXmlElement* pElement) const
 wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
 {
 	wxLongLong totalSize = 0;
-	for (int i = 0; i < PRIORITY_COUNT; i++)
+	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i)
 	{
 		for (int j = 0; j < 2; j++)
 		{
 			const std::list<CFileItem*>& fileList = m_fileList[j][i];
-			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); iter++)
+			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter)
 			{
 				const CFileItem* item = *iter;
 				wxLongLong size = item->GetSize();
@@ -641,12 +615,12 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 		}
 	}
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 	{
-		if ((*iter)->GetType() == QueueItemType_File ||
-			(*iter)->GetType() == QueueItemType_Folder)
+		if ((*iter)->GetType() == QueueItemType::File ||
+			(*iter)->GetType() == QueueItemType::Folder)
 			queuedFiles++;
-		else if ((*iter)->GetType() == QueueItemType_FolderScan)
+		else if ((*iter)->GetType() == QueueItemType::FolderScan)
 			folderScanCount++;
 	}
 
@@ -659,20 +633,16 @@ bool CServerItem::TryRemoveAll()
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
 	m_visibleOffspring = 0;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
-	{
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		CQueueItem* pItem = *iter;
-		if (pItem->TryRemoveAll())
-		{
-			if (pItem->GetType() == QueueItemType_File || pItem->GetType() == QueueItemType_Folder)
-			{
-				CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
+		if (pItem->TryRemoveAll()) {
+			if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
+				CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
 				RemoveFileItemFromList(pFileItem);
 			}
 			delete pItem;
 		}
-		else
-		{
+		else {
 			keepChildren.push_back(pItem);
 			m_visibleOffspring++;
 			m_visibleOffspring += pItem->GetChildrenCount(true);
@@ -682,8 +652,7 @@ bool CServerItem::TryRemoveAll()
 	m_removed_at_front = 0;
 
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
+	while (parent) {
 		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
 		parent = parent->GetParent();
 	}
@@ -701,38 +670,39 @@ void CServerItem::DetachChildren()
 	m_removed_at_front = 0;
 
 	for (int i = 0; i < 2; i++)
-		for (int j = 0; j < PRIORITY_COUNT; j++)
+		for (int j = 0; j < static_cast<int>(QueuePriority::count); j++)
 			m_fileList[i][j].clear();
 }
 
-void CServerItem::SetPriority(enum QueuePriority priority)
+void CServerItem::SetPriority(QueuePriority priority)
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); iter++)
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
 	{
-		if ((*iter)->GetType() == QueueItemType_File)
+		if ((*iter)->GetType() == QueueItemType::File)
 			((CFileItem*)(*iter))->SetPriorityRaw(priority);
 		else
 			(*iter)->SetPriority(priority);
 	}
 
-	for (int i = 0; i < 2; i++)
-		for (int j = 0; j < PRIORITY_COUNT; j++)
-			if (j != priority)
-				m_fileList[i][priority].splice(m_fileList[i][priority].end(), m_fileList[i][j]);
+	for (int i = 0; i < 2; ++i)
+		for (int j = 0; j < static_cast<int>(QueuePriority::count); ++j) {
+			if (j != static_cast<int>(priority))
+				m_fileList[i][static_cast<int>(priority)].splice(m_fileList[i][static_cast<int>(priority)].end(), m_fileList[i][j]);
+		}
 }
 
-void CServerItem::SetChildPriority(CFileItem* pItem, enum QueuePriority oldPriority, enum QueuePriority newPriority)
+void CServerItem::SetChildPriority(CFileItem* pItem, QueuePriority oldPriority, QueuePriority newPriority)
 {
 	int i = pItem->queued() ? 0 : 1;
 
-	for (std::list<CFileItem*>::iterator iter = m_fileList[i][oldPriority].begin(); iter != m_fileList[i][oldPriority].end(); iter++)
+	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter)
 	{
 		if (*iter != pItem)
 			continue;
 
-		m_fileList[i][oldPriority].erase(iter);
-		m_fileList[i][newPriority].push_back(pItem);
+		m_fileList[i][static_cast<int>(oldPriority)].erase(iter);
+		m_fileList[i][static_cast<int>(newPriority)].push_back(pItem);
 		return;
 	}
 
@@ -740,19 +710,12 @@ void CServerItem::SetChildPriority(CFileItem* pItem, enum QueuePriority oldPrior
 }
 
 CFolderScanItem::CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath)
+	: CQueueItem(parent)
+	, m_queued(queued)
+	, m_localPath(localPath)
+	, m_remotePath(remotePath)
+	, m_download(download)
 {
-	m_parent = parent;
-
-	m_download = download;
-	m_localPath = localPath;
-	m_remotePath = remotePath;
-	m_queued = queued;
-	m_remove = false;
-	m_active = false;
-	m_count = 0;
-	m_dir_is_empty = false;
-
-	m_defaultFileExistsAction = CFileExistsNotification::unknown;
 }
 
 bool CFolderScanItem::TryRemoveAll()
@@ -770,7 +733,6 @@ bool CFolderScanItem::TryRemoveAll()
 
 BEGIN_EVENT_TABLE(CQueueViewBase, wxListCtrlEx)
 EVT_ERASE_BACKGROUND(CQueueViewBase::OnEraseBackground)
-EVT_NAVIGATION_KEY(CQueueViewBase::OnNavigationKey)
 EVT_CHAR(CQueueViewBase::OnChar)
 EVT_LIST_COL_END_DRAG(wxID_ANY, CQueueViewBase::OnEndColumnDrag)
 EVT_TIMER(wxID_ANY, CQueueViewBase::OnTimer)
@@ -778,8 +740,9 @@ EVT_KEY_DOWN(CQueueViewBase::OnKeyDown)
 END_EVENT_TABLE()
 
 CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
-	: wxListCtrlEx(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN | wxLC_REPORT | wxLC_VIRTUAL | wxSUNKEN_BORDER | wxTAB_TRAVERSAL),
-	  m_pageIndex(index), m_title(title)
+	: wxListCtrlEx(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN | wxLC_REPORT | wxLC_VIRTUAL | wxSUNKEN_BORDER | wxTAB_TRAVERSAL)
+	, m_pageIndex(index)
+	, m_title(title)
 {
 	m_pQueue = parent;
 	m_insertionStart = -1;
@@ -808,14 +771,14 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 
 CQueueViewBase::~CQueueViewBase()
 {
-	for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
+	for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 		delete *iter;
 }
 
 CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item)
 {
 	std::vector<CServerItem*>::iterator iter;
-	for (iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
+	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 	{
 		if (!item)
 			return *iter;
@@ -837,7 +800,7 @@ int CQueueViewBase::GetItemIndex(const CQueueItem* item)
 	const CQueueItem* pTopLevelItem = item->GetTopLevelItem();
 
 	int index = 0;
-	for (std::vector<CServerItem*>::const_iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
+	for (std::vector<CServerItem*>::const_iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 	{
 		if (pTopLevelItem == *iter)
 			break;
@@ -856,14 +819,14 @@ void CQueueViewBase::OnEraseBackground(wxEraseEvent& event)
 
 wxString CQueueViewBase::OnGetItemText(long item, long column) const
 {
-	if (column < 0 || static_cast<std::size_t>(column) >= m_columns.size())
-		return _T("");
+	if (column < 0 || static_cast<size_t>(column) >= m_columns.size())
+		return wxString();
 
 	CQueueViewBase* pThis = const_cast<CQueueViewBase*>(this);
 
 	CQueueItem* pItem = pThis->GetQueueItem(item);
 	if (!pItem)
-		return _T("");
+		return wxString();
 
 	return OnGetItemText(pItem, m_columns[column]);
 }
@@ -872,20 +835,20 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 {
 	switch (pItem->GetType())
 	{
-	case QueueItemType_Server:
+	case QueueItemType::Server:
 		{
-			CServerItem* pServerItem = reinterpret_cast<CServerItem*>(pItem);
+			CServerItem* pServerItem = static_cast<CServerItem*>(pItem);
 			if (!column)
 				return pServerItem->GetName();
 		}
 		break;
-	case QueueItemType_File:
+	case QueueItemType::File:
 		{
-			CFileItem* pFileItem = reinterpret_cast<CFileItem*>(pItem);
+			CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
 			switch (column)
 			{
 			case colLocalName:
-				return pFileItem->GetIndent() + pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile();
+				return _T("  ") + pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile();
 			case colDirection:
 				if (pFileItem->Download())
 					if (pFileItem->queued())
@@ -904,29 +867,29 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 				{
 					const wxLongLong& size = pFileItem->GetSize();
 					if (size >= 0)
-						return CSizeFormat::Format(size);
+						return CSizeFormat::Format(size.GetValue());
 					else
 						return _T("?");
 				}
 			case colPriority:
 				switch (pFileItem->GetPriority())
 				{
-				case 0:
+				case QueuePriority::lowest:
 					return _("Lowest");
-				case 1:
+				case QueuePriority::low:
 					return _("Low");
 				default:
-				case 2:
+				case QueuePriority::normal:
 					return _("Normal");
-				case 3:
+				case QueuePriority::high:
 					return _("High");
-				case 4:
+				case QueuePriority::highest:
 					return _("Highest");
 				}
 				break;
 			case colTransferStatus:
 			case colErrorReason:
-				return pFileItem->m_statusMessage;
+				return pFileItem->GetStatusMessage();
 			case colTime:
 				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
@@ -934,9 +897,9 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 			}
 		}
 		break;
-	case QueueItemType_FolderScan:
+	case QueueItemType::FolderScan:
 		{
-			CFolderScanItem* pFolderItem = reinterpret_cast<CFolderScanItem*>(pItem);
+			CFolderScanItem* pFolderItem = static_cast<CFolderScanItem*>(pItem);
 			switch (column)
 			{
 			case colLocalName:
@@ -965,14 +928,14 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 			}
 		}
 		break;
-	case QueueItemType_Folder:
+	case QueueItemType::Folder:
 		{
-			CFileItem* pFolderItem = reinterpret_cast<CFolderItem*>(pItem);
+			CFileItem* pFolderItem = static_cast<CFolderItem*>(pItem);
 			switch (column)
 			{
 			case colLocalName:
 				if (pFolderItem->Download())
-					return pFolderItem->GetIndent() + pFolderItem->GetLocalPath().GetPath() + pFolderItem->GetLocalFile();
+					return _T("  ") + pFolderItem->GetLocalPath().GetPath() + pFolderItem->GetLocalFile();
 				break;
 			case colDirection:
 				if (pFolderItem->Download())
@@ -989,7 +952,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 			case colRemoteName:
 				if (!pFolderItem->Download())
 				{
-					if (pFolderItem->GetRemoteFile() == _T(""))
+					if (pFolderItem->GetRemoteFile().empty())
 						return pFolderItem->GetRemotePath().GetPath();
 					else
 						return pFolderItem->GetRemotePath().FormatFilename(pFolderItem->GetRemoteFile());
@@ -998,22 +961,22 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 			case colPriority:
 				switch (pFolderItem->GetPriority())
 				{
-				case 0:
+				case QueuePriority::lowest:
 					return _("Lowest");
-				case 1:
+				case QueuePriority::low:
 					return _("Low");
 				default:
-				case 2:
+				case QueuePriority::normal:
 					return _("Normal");
-				case 3:
+				case QueuePriority::high:
 					return _("High");
-				case 4:
+				case QueuePriority::highest:
 					return _("Highest");
 				}
 				break;
 			case colTransferStatus:
 			case colErrorReason:
-				return pFolderItem->m_statusMessage;
+				return pFolderItem->GetStatusMessage();
 			case colTime:
 				return CTimeFormat::FormatDateTime(pItem->GetTime());
 			default:
@@ -1025,7 +988,7 @@ wxString CQueueViewBase::OnGetItemText(CQueueItem* pItem, ColumnId column) const
 		break;
 	}
 
-	return _T("");
+	return wxString();
 }
 
 int CQueueViewBase::OnGetItemImage(long item) const
@@ -1038,12 +1001,12 @@ int CQueueViewBase::OnGetItemImage(long item) const
 
 	switch (pItem->GetType())
 	{
-	case QueueItemType_Server:
+	case QueueItemType::Server:
 		return 0;
-	case QueueItemType_File:
+	case QueueItemType::File:
 		return 1;
-	case QueueItemType_FolderScan:
-	case QueueItemType_Folder:
+	case QueueItemType::FolderScan:
+	case QueueItemType::Folder:
 			return 3;
 	default:
 		return -1;
@@ -1131,7 +1094,7 @@ void CQueueViewBase::UpdateSelections_ItemRangeAdded(int added, int count)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	for (std::list<int>::const_iterator iter = itemsToSelect.begin(); iter != itemsToSelect.end(); iter++)
+	for (std::list<int>::const_iterator iter = itemsToSelect.begin(); iter != itemsToSelect.end(); ++iter)
 		SetItemState(*iter, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
 }
 
@@ -1212,7 +1175,7 @@ void CQueueViewBase::UpdateSelections_ItemRangeRemoved(int removed, int count)
 
 		item = GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
 	}
-	for (std::list<int>::const_iterator iter = itemsToUnselect.begin(); iter != itemsToUnselect.end(); iter++)
+	for (std::list<int>::const_iterator iter = itemsToUnselect.begin(); iter != itemsToUnselect.end(); ++iter)
 		SetItemState(*iter, 0, wxLIST_STATE_SELECTED);
 }
 
@@ -1242,7 +1205,7 @@ void CQueueViewBase::CreateColumns(std::list<ColumnId> const& extraColumns)
 
 CServerItem* CQueueViewBase::GetServerItem(const CServer& server)
 {
-	for (std::vector<CServerItem*>::iterator iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
+	for (auto iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 	{
 		if ((*iter)->GetServer() == server)
 			return *iter;
@@ -1308,10 +1271,9 @@ void CQueueViewBase::DisplayNumberQueuedFiles()
 	}
 	else
 	{
+		str = m_title;
 		if (m_folderScanCount)
-			str.Printf(m_title + _T(" (0+)"), m_fileCount);
-		else
-			str = m_title;
+			str += _T(" (0+)");
 	}
 	m_pQueue->SetPageText(m_pageIndex, str);
 
@@ -1332,12 +1294,12 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 		m_insertionStart = newIndex;
 	m_insertionCount++;
 
-	if (pItem->GetType() == QueueItemType_File || pItem->GetType() == QueueItemType_Folder)
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
 	{
 		m_fileCount++;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType_FolderScan)
+	else if (pItem->GetType() == QueueItemType::FolderScan)
 	{
 		m_folderScanCount++;
 		m_folderScanCountChanged = true;
@@ -1346,13 +1308,13 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 
 bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount /*=true*/, bool updateSelections /*=true*/)
 {
-	if (pItem->GetType() == QueueItemType_File || pItem->GetType() == QueueItemType_Folder)
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
 	{
 		wxASSERT(m_fileCount > 0);
 		m_fileCount--;
 		m_fileCountChanged = true;
 	}
-	else if (pItem->GetType() == QueueItemType_FolderScan)
+	else if (pItem->GetType() == QueueItemType::FolderScan)
 	{
 		wxASSERT(m_folderScanCount > 0);
 		m_folderScanCount--;
@@ -1375,7 +1337,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 	if (!topLevelItem->GetChild(0))
 	{
 		std::vector<CServerItem*>::iterator iter;
-		for (iter = m_serverList.begin(); iter != m_serverList.end(); iter++)
+		for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
 		{
 			if (*iter == topLevelItem)
 				break;
@@ -1437,17 +1399,10 @@ void CQueueViewBase::RefreshItem(const CQueueItem* pItem)
 #endif
 }
 
-void CQueueViewBase::OnNavigationKey(wxNavigationKeyEvent& event)
-{
-	event.SetEventObject(m_pQueue);
-	m_pQueue->ProcessEvent(event);
-}
-
 void CQueueViewBase::OnChar(wxKeyEvent& event)
 {
 	const int code = event.GetKeyCode();
-	if (code != WXK_LEFT && code != WXK_RIGHT)
-	{
+	if (code != WXK_LEFT && code != WXK_RIGHT) {
 		event.Skip();
 		return;
 	}
