@@ -4,27 +4,27 @@
 #include "aui_notebook_ex.h"
 #include "listctrlex.h"
 #include "edithandler.h"
+#include "optional.h"
 
-#define PRIORITY_COUNT 5
-enum QueuePriority
-{
-	priority_lowest = 0,
-	priority_low = 1,
-	priority_normal = 2,
-	priority_high = 3,
-	priority_highest = 4
+enum class QueuePriority : char {
+	lowest,
+	low,
+	normal,
+	high,
+	highest,
+
+	count
 };
 
-enum QueueItemType
-{
-	QueueItemType_Server,
-	QueueItemType_File,
-	QueueItemType_Folder,
-	QueueItemType_FolderScan,
-	QueueItemType_Status
+enum class QueueItemType {
+	Server,
+	File,
+	Folder,
+	FolderScan,
+	Status
 };
 
-enum TransferDirection
+enum class TransferDirection
 {
 	both,
 	download,
@@ -37,7 +37,7 @@ class CQueueItem
 public:
 	virtual ~CQueueItem();
 
-	virtual void SetPriority(enum QueuePriority priority);
+	virtual void SetPriority(QueuePriority priority);
 
 	virtual void AddChild(CQueueItem* pItem);
 	unsigned int GetChildrenCount(bool recursive);
@@ -51,23 +51,23 @@ public:
 	CQueueItem* GetTopLevelItem();
 	const CQueueItem* GetTopLevelItem() const;
 	int GetItemIndex() const; // Return the visible item index relative to the topmost parent item.
-	virtual void SaveItem(TiXmlElement* pElement) const { }
+	virtual void SaveItem(TiXmlElement*) const {}
 
-	virtual enum QueueItemType GetType() const = 0;
+	virtual QueueItemType GetType() const = 0;
 
 	wxDateTime GetTime() const { return m_time; }
-	void UpdateTime() { m_time = wxDateTime::Now(); }
+	void UpdateTime() { m_time = wxDateTime::UNow(); }
+
+	const std::vector<CQueueItem*>& GetChildren() const { return m_children; }
+	int GetRemovedAtFront() const { return m_removed_at_front; }
 
 protected:
-	CQueueItem();
-	wxString GetIndent();
+	CQueueItem(CQueueItem* parent = 0);
 
 	CQueueItem* m_parent;
 
-	int m_visibleOffspring; // Visible offspring over all sublevels
-	wxString m_indent;
-
-	int m_maxCachedIndex;
+	int m_visibleOffspring{}; // Visible offspring over all sublevels
+	int m_maxCachedIndex{-1};
 
 	struct t_cacheItem
 	{
@@ -86,7 +86,7 @@ private:
 	// Number of items removed at front of list
 	// Increased instead of calling slow m_children.erase(0),
 	// resetted on insert.
-	int m_removed_at_front;
+	int m_removed_at_front{};
 };
 
 class CFileItem;
@@ -95,14 +95,14 @@ class CServerItem : public CQueueItem
 public:
 	CServerItem(const CServer& server);
 	virtual ~CServerItem();
-	virtual enum QueueItemType GetType() const { return QueueItemType_Server; }
+	virtual QueueItemType GetType() const { return QueueItemType::Server; }
 
 	const CServer& GetServer() const;
 	wxString GetName() const;
 
 	virtual void AddChild(CQueueItem* pItem);
 
-	CFileItem* GetIdleChild(bool immadiateOnly, enum TransferDirection direction);
+	CFileItem* GetIdleChild(bool immadiateOnly, TransferDirection direction);
 	virtual bool RemoveChild(CQueueItem* pItem, bool destroy = true); // Removes a child item with is somewhere in the tree of children
 	wxLongLong GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const;
 
@@ -111,17 +111,17 @@ public:
 
 	virtual void SaveItem(TiXmlElement* pElement) const;
 
-	void SetDefaultFileExistsAction(enum CFileExistsNotification::OverwriteAction action, const enum TransferDirection direction);
-
-	int m_activeCount;
+	void SetDefaultFileExistsAction(CFileExistsNotification::OverwriteAction action, const TransferDirection direction);
 
 	virtual bool TryRemoveAll();
 
 	void DetachChildren();
 
-	virtual void SetPriority(enum QueuePriority priority);
+	virtual void SetPriority(QueuePriority priority);
 
-	void SetChildPriority(CFileItem* pItem, enum QueuePriority oldPriority, enum QueuePriority newPriority);
+	void SetChildPriority(CFileItem* pItem, QueuePriority oldPriority, QueuePriority newPriority);
+
+	int m_activeCount;
 
 protected:
 	void AddFileItemToList(CFileItem* pItem);
@@ -132,7 +132,7 @@ protected:
 	// array of item lists, sorted by priority. Used by scheduler to find
 	// next file to transfer
 	// First index specifies whether the item is queued (0) or immediate (1)
-	std::list<CFileItem*> m_fileList[2][PRIORITY_COUNT];
+	std::list<CFileItem*> m_fileList[2][static_cast<int>(QueuePriority::count)];
 };
 
 struct t_EngineData;
@@ -140,22 +140,26 @@ struct t_EngineData;
 class CFileItem : public CQueueItem
 {
 public:
-	CFileItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const wxString& localFile,
-			const wxString& remoteFile, const CServerPath& remotePath, wxLongLong size);
+	CFileItem(CServerItem* parent, bool queued, bool download,
+		const wxString& sourceFile, const wxString& targetFile,
+		const CLocalPath& localPath, const CServerPath& remotePath, wxLongLong size);
+
 	virtual ~CFileItem();
 
-	virtual void SetPriority(enum QueuePriority priority);
-	void SetPriorityRaw(enum QueuePriority priority);
-	enum QueuePriority GetPriority() const;
+	virtual void SetPriority(QueuePriority priority);
+	void SetPriorityRaw(QueuePriority priority);
+	QueuePriority GetPriority() const;
 
-	wxString GetLocalFile() const { return m_localFile; }
-	wxString GetRemoteFile() const { return m_remoteFile; }
+	const wxString& GetLocalFile() const { return !Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
+	const wxString& GetRemoteFile() const { return Download() ? GetSourceFile() : (m_targetFile ? *m_targetFile : m_sourceFile); }
+	const wxString& GetSourceFile() const { return m_sourceFile; }
+	CSparseOptional<wxString> const& GetTargetFile() const { return m_targetFile; }
 	const CLocalPath& GetLocalPath() const { return m_localPath; }
 	const CServerPath& GetRemotePath() const { return m_remotePath; }
-	wxLongLong GetSize() const { return m_size; }
+	const wxLongLong& GetSize() const { return m_size; }
 	void SetSize(wxLongLong size) { m_size = size; }
 	inline bool Download() const { return flags & flag_download; }
-	
+
 	inline bool queued() const { return (flags & flag_queued) != 0; }
 	inline void set_queued(bool q)
 	{
@@ -174,31 +178,60 @@ public:
 			flags &= ~flag_remove;
 	}
 
-	wxString GetIndent() { return m_indent; }
-
-	virtual enum QueueItemType GetType() const { return QueueItemType_File; }
+	virtual QueueItemType GetType() const { return QueueItemType::File; }
 
 	bool IsActive() const { return (flags & flag_active) != 0; }
 	virtual void SetActive(bool active);
-	
+
 	virtual void SaveItem(TiXmlElement* pElement) const;
 
 	virtual bool TryRemoveAll(); // Removes a inactive childrens, queues active children for removal.
 								 // Returns true if item can be removed itself
 
-	void SetLocalFile(const wxString &file);
-	void SetRemoteFile(const wxString &file);
+	void SetTargetFile(wxString const& file);
 
-	int m_errorCount;
-	CEditHandler::fileType m_edit;
+	enum Status : char {
+		none,
+		incorrect_password,
+		timeout,
+		disconnecting,
+		disconnected,
+		connecting,
+		connection_failed,
+		interrupted,
+		wait_browsing,
+		wait_password,
+		local_file_unwriteable,
+		could_not_start,
+		transferring,
+		creating_dir
+	};
 
-	wxString m_statusMessage;
+	wxString const& GetStatusMessage() const;
+	void SetStatusMessage(Status status);
 
-	CFileTransferCommand::t_transferSettings m_transferSettings;
+	unsigned char m_errorCount{};
+	CEditHandler::fileType m_edit{CEditHandler::none};
+	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction{CFileExistsNotification::unknown};
+	CFileExistsNotification::OverwriteAction m_onetime_action{CFileExistsNotification::unknown};
+	QueuePriority m_priority{QueuePriority::normal};
 
-	t_EngineData* m_pEngineData;
+protected:
+	enum : char
+	{
+		flag_download = 0x01,
+		flag_active = 0x02,
+		flag_made_progress = 0x04,
+		flag_queued = 0x08,
+		flag_remove = 0x10,
+		flag_ascii = 0x20
+	};
+	char flags{};
+	Status m_status{};
+
+public:
+	t_EngineData* m_pEngineData{};
 
-	enum CFileExistsNotification::OverwriteAction m_defaultFileExistsAction;
 
 	inline bool made_progress() const { return (flags & flag_made_progress) != 0; }
 	inline void set_made_progress(bool made_progress)
@@ -209,25 +242,23 @@ public:
 			flags &= ~flag_made_progress;
 	}
 
-	enum CFileExistsNotification::OverwriteAction m_onetime_action;
+	bool Ascii() const { return (flags & flag_ascii) != 0; }
+
+	void SetAscii(bool ascii)
+	{
+		if (ascii) {
+			flags |= flag_ascii;
+		}
+		else {
+			flags &= ~flag_ascii;
+		}
+	}
 
 protected:
-	enum
-	{
-		flag_download = 0x01,
-		flag_active = 0x02,
-		flag_made_progress = 0x04,
-		flag_queued = 0x08,
-		flag_remove = 0x10
-	};
-	int flags;
-
-	enum QueuePriority m_priority;
-
-	wxString m_localFile;
-	wxString m_remoteFile;
-	const CLocalPath m_localPath;
-	const CServerPath m_remotePath;
+	wxString const m_sourceFile;
+	CSparseOptional<wxString> m_targetFile;
+	CLocalPath const m_localPath;
+	CServerPath const m_remotePath;
 	wxLongLong m_size;
 };
 
@@ -236,8 +267,8 @@ class CFolderItem : public CFileItem
 public:
 	CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath);
 	CFolderItem(CServerItem* parent, bool queued, const CServerPath& remotePath, const wxString& remoteFile);
-	
-	virtual enum QueueItemType GetType() const { return QueueItemType_Folder; }
+
+	virtual QueueItemType GetType() const { return QueueItemType::Folder; }
 
 	virtual void SaveItem(TiXmlElement* pElement) const;
 
@@ -248,9 +279,9 @@ class CFolderScanItem : public CQueueItem
 {
 public:
 	CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath);
-	virtual ~CFolderScanItem() { }
+	virtual ~CFolderScanItem() {}
 
-	virtual enum QueueItemType GetType() const { return QueueItemType_FolderScan; }
+	virtual QueueItemType GetType() const { return QueueItemType::FolderScan; }
 	CLocalPath GetLocalPath() const { return m_localPath; }
 	CServerPath GetRemotePath() const { return m_remotePath; }
 	bool Download() const { return m_download; }
@@ -262,14 +293,14 @@ public:
 
 	wxString m_statusMessage;
 
-	volatile bool m_remove;
-	bool m_active;
+	volatile bool m_remove{};
+	bool m_active{};
 
-	int m_count;
+	int m_count{};
 
-	enum CFileExistsNotification::OverwriteAction m_defaultFileExistsAction;
+	CFileExistsNotification::OverwriteAction m_defaultFileExistsAction{CFileExistsNotification::unknown};
 
-	bool m_dir_is_empty;
+	bool m_dir_is_empty{};
 	CLocalPath m_current_local_path;
 	CServerPath m_current_remote_path;
 
@@ -286,7 +317,7 @@ public:
 	CStatusItem() {}
 	virtual ~CStatusItem() {}
 
-	virtual enum QueueItemType GetType() const { return QueueItemType_Status; }
+	virtual QueueItemType GetType() const { return QueueItemType::Status; }
 };
 
 class CQueue;
@@ -375,9 +406,8 @@ protected:
 
 	std::vector<ColumnId> m_columns;
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnEraseBackground(wxEraseEvent& event);
-	void OnNavigationKey(wxNavigationKeyEvent& event);
 	void OnChar(wxKeyEvent& event);
 	void OnEndColumnDrag(wxListEvent& event);
 	void OnTimer(wxTimerEvent& event);
