@@ -6,21 +6,11 @@
 #include <sqlite3.h>
 #include <wx/wx.h>
 
-#if HAVE_TR1_UNORDERED_MAP
-#include <tr1/unordered_map>
-#else
 #include <unordered_map>
-#endif
-
-namespace std {
-namespace tr1 {
-}
-using namespace tr1;
-}
 
 #define INVALID_DATA -1
 
-enum _column_type
+enum class Column_type
 {
 	text,
 	integer
@@ -28,14 +18,14 @@ enum _column_type
 
 enum _column_flags
 {
-	not_null,
+	not_null = 1,
 	autoincrement
 };
 
 struct _column
 {
 	const wxChar* const name;
-	_column_type type;
+	Column_type type;
 	unsigned int flags;
 };
 
@@ -63,22 +53,22 @@ namespace server_table_column_names
 }
 
 _column server_table_columns[] = {
-	{ _T("id"), integer, not_null | autoincrement },
-	{ _T("host"), text, not_null },
-	{ _T("port"), integer, 0 },
-	{ _T("user"), text, 0 },
-	{ _T("password"), text, 0 },
-	{ _T("account"), text, 0 },
-	{ _T("protocol"), integer, 0 },
-	{ _T("type"), integer, 0 },
-	{ _T("logontype"), integer, 0 },
-	{ _T("timezone_offset"), integer, 0 },
-	{ _T("transfer_mode"), text, 0 },
-	{ _T("max_connections"), integer, 0 },
-	{ _T("encoding"), text, 0 },
-	{ _T("bypass_proxy"), integer, 0 },
-	{ _T("post_login_commands"), text, 0 },
-	{ _T("name"), text, 0 }
+	{ _T("id"), Column_type::integer, not_null | autoincrement },
+	{ _T("host"), Column_type::text, not_null },
+	{ _T("port"), Column_type::integer, 0 },
+	{ _T("user"), Column_type::text, 0 },
+	{ _T("password"), Column_type::text, 0 },
+	{ _T("account"), Column_type::text, 0 },
+	{ _T("protocol"), Column_type::integer, 0 },
+	{ _T("type"), Column_type::integer, 0 },
+	{ _T("logontype"), Column_type::integer, 0 },
+	{ _T("timezone_offset"), Column_type::integer, 0 },
+	{ _T("transfer_mode"), Column_type::text, 0 },
+	{ _T("max_connections"), Column_type::integer, 0 },
+	{ _T("encoding"), Column_type::text, 0 },
+	{ _T("bypass_proxy"), Column_type::integer, 0 },
+	{ _T("post_login_commands"), Column_type::text, 0 },
+	{ _T("name"), Column_type::text, 0 }
 };
 
 namespace file_table_column_names
@@ -101,18 +91,18 @@ namespace file_table_column_names
 }
 
 _column file_table_columns[] = {
-	{ _T("id"), integer, not_null | autoincrement },
-	{ _T("server"), integer, not_null },
-	{ _T("source_file"), text, 0 },
-	{ _T("target_file"), text, 0 },
-	{ _T("local_path"), integer, 0 },
-	{ _T("remote_path"), integer, 0 },
-	{ _T("download"), integer, not_null },
-	{ _T("size"), integer, 0 },
-	{ _T("error_count"), integer, 0 },
-	{ _T("priority"), integer, 0 },
-	{ _T("ascii_file"), integer, 0 },
-	{ _T("default_exists_action"), integer, 0 }
+	{ _T("id"), Column_type::integer, not_null | autoincrement },
+	{ _T("server"), Column_type::integer, not_null },
+	{ _T("source_file"), Column_type::text, 0 },
+	{ _T("target_file"), Column_type::text, 0 },
+	{ _T("local_path"), Column_type::integer, 0 },
+	{ _T("remote_path"), Column_type::integer, 0 },
+	{ _T("download"), Column_type::integer, not_null },
+	{ _T("size"), Column_type::integer, 0 },
+	{ _T("error_count"), Column_type::integer, 0 },
+	{ _T("priority"), Column_type::integer, 0 },
+	{ _T("ascii_file"), Column_type::integer, 0 },
+	{ _T("default_exists_action"), Column_type::integer, 0 }
 };
 
 namespace path_table_column_names
@@ -125,17 +115,15 @@ namespace path_table_column_names
 }
 
 _column path_table_columns[] = {
-	{ _T("id"), integer, not_null | autoincrement },
-	{ _T("path"), text, not_null }
+	{ _T("id"), Column_type::integer, not_null | autoincrement },
+	{ _T("path"), Column_type::text, not_null }
 };
 
 struct fast_equal
 {
 	bool operator()(wxString const& lhs, wxString const& rhs) const
 	{
-		// wxString is CoW, yet it doesn't even do this fast pointer
-		// comparison in it's less and/or equal operator(s).
-		return lhs.c_str() == rhs.c_str() || lhs == rhs;
+		return lhs == rhs;
 	}
 };
 
@@ -168,27 +156,27 @@ public:
 	bool SaveFile(wxLongLong server, const CFileItem& item);
 	bool SaveDirectory(wxLongLong server, const CFolderItem& item);
 
-	wxLongLong_t SaveLocalPath(const CLocalPath& path);
-	wxLongLong_t SaveRemotePath(const CServerPath& path);
+	int64_t SaveLocalPath(const CLocalPath& path);
+	int64_t SaveRemotePath(const CServerPath& path);
 
 	void ReadLocalPaths();
 	void ReadRemotePaths();
 
-	const CLocalPath& GetLocalPath(wxLongLong_t id) const;
-	const CServerPath& GetRemotePath(wxLongLong_t id) const;
+	const CLocalPath& GetLocalPath(int64_t id) const;
+	const CServerPath& GetRemotePath(int64_t id) const;
 
 	bool Bind(sqlite3_stmt* statement, int index, int value);
-	bool Bind(sqlite3_stmt* statement, int index, wxLongLong_t value);
+	bool Bind(sqlite3_stmt* statement, int index, int64_t value);
 	bool Bind(sqlite3_stmt* statement, int index, const wxString& value);
 	bool Bind(sqlite3_stmt* statement, int index, const char* const value);
 	bool BindNull(sqlite3_stmt* statement, int index);
 
 	wxString GetColumnText(sqlite3_stmt* statement, int index, bool shrink = true);
-	wxLongLong_t GetColumnInt64(sqlite3_stmt* statement, int index, wxLongLong_t def = 0);
+	int64_t GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def = 0);
 	int GetColumnInt(sqlite3_stmt* statement, int index, int def = 0);
 
-	wxLongLong_t ParseServerFromRow(CServer& server);
-	wxLongLong_t ParseFileFromRow(CFileItem** pItem);
+	int64_t ParseServerFromRow(CServer& server);
+	int64_t ParseFileFromRow(CFileItem** pItem);
 
 	bool MigrateSchema();
 
@@ -211,11 +199,11 @@ public:
 	// Caches to speed up saving and loading
 	void ClearCaches();
 
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash, fast_equal> localPaths_;
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash> remotePaths_; // No need for fast_equal as GetSafePath returns unshared string anyhow
+	std::unordered_map<wxString, int64_t, wxStringHash, fast_equal> localPaths_;
+	std::unordered_map<wxString, int64_t, wxStringHash> remotePaths_; // No need for fast_equal as GetSafePath returns unshared string anyhow
 
-	std::map<wxLongLong_t, CLocalPath> reverseLocalPaths_;
-	std::map<wxLongLong_t, CServerPath> reverseRemotePaths_;
+	std::map<int64_t, CLocalPath> reverseLocalPaths_;
+	std::map<int64_t, CServerPath> reverseRemotePaths_;
 };
 
 
@@ -230,7 +218,7 @@ void CQueueStorage::Impl::ReadLocalPaths()
 		res = sqlite3_step(selectLocalPathQuery_);
 		if (res == SQLITE_ROW)
 		{
-			wxLongLong_t id = GetColumnInt64(selectLocalPathQuery_, path_table_column_names::id);
+			int64_t id = GetColumnInt64(selectLocalPathQuery_, path_table_column_names::id);
 			wxString localPathRaw = GetColumnText(selectLocalPathQuery_, path_table_column_names::path);
 			CLocalPath localPath;
 			if (id > 0 && !localPathRaw.empty() && localPath.SetPath(localPathRaw))
@@ -254,7 +242,7 @@ void CQueueStorage::Impl::ReadRemotePaths()
 		res = sqlite3_step(selectRemotePathQuery_);
 		if (res == SQLITE_ROW)
 		{
-			wxLongLong_t id = GetColumnInt64(selectRemotePathQuery_, path_table_column_names::id);
+			int64_t id = GetColumnInt64(selectRemotePathQuery_, path_table_column_names::id);
 			wxString remotePathRaw = GetColumnText(selectRemotePathQuery_, path_table_column_names::path);
 			CServerPath remotePath;
 			if (id > 0 && !remotePathRaw.empty() && remotePath.SetSafePath(remotePathRaw))
@@ -267,9 +255,9 @@ void CQueueStorage::Impl::ReadRemotePaths()
 }
 
 
-const CLocalPath& CQueueStorage::Impl::GetLocalPath(wxLongLong_t id) const
+const CLocalPath& CQueueStorage::Impl::GetLocalPath(int64_t id) const
 {
-	std::map<wxLongLong_t, CLocalPath>::const_iterator it = reverseLocalPaths_.find(id);
+	std::map<int64_t, CLocalPath>::const_iterator it = reverseLocalPaths_.find(id);
 	if (it != reverseLocalPaths_.end())
 		return it->second;
 
@@ -278,9 +266,9 @@ const CLocalPath& CQueueStorage::Impl::GetLocalPath(wxLongLong_t id) const
 }
 
 
-const CServerPath& CQueueStorage::Impl::GetRemotePath(wxLongLong_t id) const
+const CServerPath& CQueueStorage::Impl::GetRemotePath(int64_t id) const
 {
-	std::map<wxLongLong_t, CServerPath>::const_iterator it = reverseRemotePaths_.find(id);
+	std::map<int64_t, CServerPath>::const_iterator it = reverseRemotePaths_.find(id);
 	if (it != reverseRemotePaths_.end())
 		return it->second;
 
@@ -291,7 +279,7 @@ const CServerPath& CQueueStorage::Impl::GetRemotePath(wxLongLong_t id) const
 
 static int int_callback(void* p, int n, char** v, char**)
 {
-	int* i = reinterpret_cast<int*>(p);
+	int* i = static_cast<int*>(p);
 	if (!i || !n || !v || !*v)
 		return -1;
 
@@ -323,9 +311,9 @@ void CQueueStorage::Impl::ClearCaches()
 }
 
 
-wxLongLong_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
+int64_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
 {
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash, fast_equal>::const_iterator it = localPaths_.find(path.GetPath());
+	std::unordered_map<wxString, int64_t, wxStringHash, fast_equal>::const_iterator it = localPaths_.find(path.GetPath());
 	if (it != localPaths_.end())
 		return it->second;
 
@@ -340,7 +328,7 @@ wxLongLong_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
 
 	if (res == SQLITE_DONE)
 	{
-		wxLongLong_t id = sqlite3_last_insert_rowid(db_);
+		int64_t id = sqlite3_last_insert_rowid(db_);
 		localPaths_[path.GetPath()] = id;
 		return id;
 	}
@@ -349,14 +337,14 @@ wxLongLong_t CQueueStorage::Impl::SaveLocalPath(const CLocalPath& path)
 }
 
 
-wxLongLong_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
+int64_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
 {
-	wxString safePath = path.GetSafePath();
-	std::unordered_map<wxString, wxLongLong_t, wxStringHash>::const_iterator it = remotePaths_.find(safePath);
+	wxString const& safePath = path.GetSafePath();
+	std::unordered_map<wxString, int64_t, wxStringHash>::const_iterator it = remotePaths_.find(safePath);
 	if (it != remotePaths_.end())
 		return it->second;
 
-	Bind(insertRemotePathQuery_, path_table_column_names::path, path.GetSafePath());
+	Bind(insertRemotePathQuery_, path_table_column_names::path, safePath);
 
 	int res;
 	do {
@@ -365,9 +353,8 @@ wxLongLong_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
 
 	sqlite3_reset(insertRemotePathQuery_);
 
-	if (res == SQLITE_DONE)
-	{
-		wxLongLong_t id = sqlite3_last_insert_rowid(db_);
+	if (res == SQLITE_DONE) {
+		int64_t id = sqlite3_last_insert_rowid(db_);
 		remotePaths_[safePath] = id;
 		return id;
 	}
@@ -379,12 +366,11 @@ wxLongLong_t CQueueStorage::Impl::SaveRemotePath(const CServerPath& path)
 wxString CQueueStorage::Impl::CreateColumnDefs(_column* columns, size_t count)
 {
 	wxString query = _T("(");
-	for (unsigned int i = 0; i < count; ++i)
-	{
+	for (unsigned int i = 0; i < count; ++i) {
 		if (i)
 			query += _T(", ");
 		query += columns[i].name;
-		if (columns[i].type == integer)
+		if (columns[i].type == Column_type::integer)
 			query += _T(" INTEGER");
 		else
 			query += _T(" TEXT");
@@ -551,34 +537,27 @@ bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, int value)
 }
 
 
-bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, wxLongLong_t value)
+bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, int64_t value)
 {
 	int res = sqlite3_bind_int64(statement, index, value);
 	return res == SQLITE_OK;
 }
 
 
+#ifndef __WXMSW__
 extern "C" {
 static void custom_free(void* v)
 {
-#ifdef __WXMSW__
-	wxStringData* data = reinterpret_cast<wxStringData*>(v) - 1;
-	data->Unlock();
-#else
-	char* s = reinterpret_cast<char*>(v);
+	char* s = static_cast<char*>(v);
 	delete [] s;
+}
+}
 #endif
-}
-}
 
 bool CQueueStorage::Impl::Bind(sqlite3_stmt* statement, int index, const wxString& value)
 {
 #ifdef __WXMSW__
-	// Increase string reference and pass the data to sqlite with a custom deallocator that
-	// reduces the reference once sqlite is done with it.
-	wxStringData* data = reinterpret_cast<wxStringData*>(const_cast<wxChar*>(value.c_str())) - 1;
-	data->Lock();
-	return sqlite3_bind_text16(statement, index, data + 1, data->nDataLength * 2, custom_free) == SQLITE_OK;
+	return sqlite3_bind_text16(statement, index, value.wc_str(), value.length() * 2, SQLITE_TRANSIENT) == SQLITE_OK;
 #else
 	char* out = new char[value.size() * 2];
 	size_t outlen = utf16_.FromWChar(out, value.size() * 2, value.c_str(), value.size());
@@ -678,15 +657,11 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 		break;
 	}
 
-	const enum ServerProtocol protocol = server.GetProtocol();
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
-	{
+	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
 		const std::vector<wxString>& postLoginCommands = server.GetPostLoginCommands();
-		if (!postLoginCommands.empty())
-		{
+		if (!postLoginCommands.empty()) {
 			wxString commands;
-			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter)
-			{
+			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter) {
 				if (!commands.empty())
 					commands += _T("\n");
 				commands += *iter;
@@ -716,16 +691,16 @@ bool CQueueStorage::Impl::SaveServer(const CServerItem& item)
 	if (ret)
 	{
 		sqlite3_int64 serverId = sqlite3_last_insert_rowid(db_);
-		Bind(insertFileQuery_, file_table_column_names::server, serverId);
+		Bind(insertFileQuery_, file_table_column_names::server, static_cast<int64_t>(serverId));
 
 		const std::vector<CQueueItem*>& children = item.GetChildren();
 		for (std::vector<CQueueItem*>::const_iterator it = children.begin() + item.GetRemovedAtFront(); it != children.end(); ++it)
 		{
 			CQueueItem* item = *it;
-			if (item->GetType() == QueueItemType_File)
-				ret &= SaveFile(serverId, *reinterpret_cast<CFileItem*>(item));
-			else if (item->GetType() == QueueItemType_Folder)
-				ret &= SaveDirectory(serverId, *reinterpret_cast<CFolderItem*>(item));
+			if (item->GetType() == QueueItemType::File)
+				ret &= SaveFile(serverId, *static_cast<CFileItem*>(item));
+			else if (item->GetType() == QueueItemType::Folder)
+				ret &= SaveDirectory(serverId, *static_cast<CFolderItem*>(item));
 		}
 	}
 	return ret;
@@ -738,13 +713,14 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 		return true;
 
 	Bind(insertFileQuery_, file_table_column_names::source_file, file.GetSourceFile());
-	if (file.GetTargetFile().empty())
-		BindNull(insertFileQuery_, file_table_column_names::target_file);
+	auto const& targetFile = file.GetTargetFile();
+	if (targetFile)
+		Bind(insertFileQuery_, file_table_column_names::target_file, *targetFile);
 	else
-		Bind(insertFileQuery_, file_table_column_names::target_file, file.GetTargetFile());
+		BindNull(insertFileQuery_, file_table_column_names::target_file);
 
-	wxLongLong_t localPathId = SaveLocalPath(file.GetLocalPath());
-	wxLongLong_t remotePathId = SaveRemotePath(file.GetRemotePath());
+	int64_t localPathId = SaveLocalPath(file.GetLocalPath());
+	int64_t remotePathId = SaveRemotePath(file.GetRemotePath());
 	if (localPathId == -1 || remotePathId == -1)
 		return false;
 
@@ -753,15 +729,15 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 
 	Bind(insertFileQuery_, file_table_column_names::download, file.Download() ? 1 : 0);
 	if (file.GetSize() != -1)
-		Bind(insertFileQuery_, file_table_column_names::size, file.GetSize().GetValue());
+		Bind(insertFileQuery_, file_table_column_names::size, static_cast<int64_t>(file.GetSize().GetValue()));
 	else
 		BindNull(insertFileQuery_, file_table_column_names::size);
 	if (file.m_errorCount)
 		Bind(insertFileQuery_, file_table_column_names::error_count, file.m_errorCount);
 	else
 		BindNull(insertFileQuery_, file_table_column_names::error_count);
-	Bind(insertFileQuery_, file_table_column_names::priority, file.GetPriority());
-	Bind(insertFileQuery_, file_table_column_names::ascii_file, file.m_transferSettings.binary ? 0 : 1);
+	Bind(insertFileQuery_, file_table_column_names::priority, static_cast<int>(file.GetPriority()));
+	Bind(insertFileQuery_, file_table_column_names::ascii_file, file.Ascii() ? 1 : 0);
 
 	if (file.m_defaultFileExistsAction != CFileExistsNotification::unknown)
 		Bind(insertFileQuery_, file_table_column_names::default_exists_action, file.m_defaultFileExistsAction);
@@ -781,14 +757,14 @@ bool CQueueStorage::Impl::SaveFile(wxLongLong server, const CFileItem& file)
 
 bool CQueueStorage::Impl::SaveDirectory(wxLongLong server, const CFolderItem& directory)
 {
-	if (download)
+	if (directory.Download())
 		BindNull(insertFileQuery_, file_table_column_names::source_file);
 	else
 		Bind(insertFileQuery_, file_table_column_names::source_file, directory.GetSourceFile());
 	BindNull(insertFileQuery_, file_table_column_names::target_file);
 
-	wxLongLong_t localPathId = directory.Download() ? SaveLocalPath(directory.GetLocalPath()) : -1;
-	wxLongLong_t remotePathId = directory.Download() ? -1 : SaveRemotePath(directory.GetRemotePath());
+	int64_t localPathId = directory.Download() ? SaveLocalPath(directory.GetLocalPath()) : -1;
+	int64_t remotePathId = directory.Download() ? -1 : SaveRemotePath(directory.GetRemotePath());
 	if (localPathId == -1 && remotePathId == -1)
 		return false;
 
@@ -801,7 +777,7 @@ bool CQueueStorage::Impl::SaveDirectory(wxLongLong server, const CFolderItem& di
 		Bind(insertFileQuery_, file_table_column_names::error_count, directory.m_errorCount);
 	else
 		BindNull(insertFileQuery_, file_table_column_names::error_count);
-	Bind(insertFileQuery_, file_table_column_names::priority, directory.GetPriority());
+	Bind(insertFileQuery_, file_table_column_names::priority, static_cast<int>(directory.GetPriority()));
 	BindNull(insertFileQuery_, file_table_column_names::ascii_file);
 
 	BindNull(insertFileQuery_, file_table_column_names::default_exists_action);
@@ -823,26 +799,28 @@ wxString CQueueStorage::Impl::GetColumnText(sqlite3_stmt* statement, int index,
 
 #ifdef __WXMSW__
 	(void)shrink;
-	const wxChar* text = reinterpret_cast<const wxChar*>(sqlite3_column_text16(statement, index));
+	const wxChar* text = static_cast<const wxChar*>(sqlite3_column_text16(statement, index));
 	if (text)
 		ret = text;
 #else
-	const char* text = reinterpret_cast<const char*>(sqlite3_column_text16(statement, index));
+	const char* text = static_cast<const char*>(sqlite3_column_text16(statement, index));
 	int len = sqlite3_column_bytes16(statement, index);
 	if (text)
 	{
-		wxChar* out = ret.GetWriteBuf( len );
+		wxStringBuffer buffer(ret, len);
+		wxChar* out = buffer;
+
 		int outlen = utf16_.ToWChar( out, len, text, len );
-		ret.UngetWriteBuf( outlen );
-		if (shrink)
-			ret.Shrink();
+		buffer[outlen] = 0;
 	}
+	if (shrink)
+		ret.Shrink();
 #endif
 
 	return ret;
 }
 
-wxLongLong_t CQueueStorage::Impl::GetColumnInt64(sqlite3_stmt* statement, int index, wxLongLong_t def)
+int64_t CQueueStorage::Impl::GetColumnInt64(sqlite3_stmt* statement, int index, int64_t def)
 {
 	if (sqlite3_column_type(statement, index) == SQLITE_NULL)
 		return def;
@@ -858,12 +836,12 @@ int CQueueStorage::Impl::GetColumnInt(sqlite3_stmt* statement, int index, int de
 		return sqlite3_column_int(statement, index);
 }
 
-wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
+int64_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 {
 	server = CServer();
 
 	wxString host = GetColumnText(selectServersQuery_, server_table_column_names::host);
-	if (host == _T(""))
+	if (host.empty())
 		return INVALID_DATA;
 
 	int port = GetColumnInt(selectServersQuery_, server_table_column_names::port);
@@ -873,11 +851,10 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 	if (!server.SetHost(host, port))
 		return INVALID_DATA;
 
-	int protocol = GetColumnInt(selectServersQuery_, server_table_column_names::protocol);
+	int const protocol = GetColumnInt(selectServersQuery_, server_table_column_names::protocol);
 	if (protocol < 0 || protocol > MAX_VALUE)
 		return INVALID_DATA;
-
-	server.SetProtocol((enum ServerProtocol)protocol);
+	server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetColumnInt(selectServersQuery_, server_table_column_names::type);
 	if (type < 0 || type >= SERVERTYPE_MAX)
@@ -905,7 +882,7 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 		if ((long)ACCOUNT == logonType)
 		{
 			wxString account = GetColumnText(selectServersQuery_, server_table_column_names::account);
-			if (account == _T(""))
+			if (account.empty())
 				return INVALID_DATA;
 			if (!server.SetAccount(account))
 				return INVALID_DATA;
@@ -940,8 +917,7 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 			return INVALID_DATA;
 	}
 
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
-	{
+	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
 		std::vector<wxString> postLoginCommands;
 
 		wxString commands = GetColumnText(selectServersQuery_, server_table_column_names::post_login_commands);
@@ -973,13 +949,13 @@ wxLongLong_t CQueueStorage::Impl::ParseServerFromRow(CServer& server)
 }
 
 
-wxLongLong_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
+int64_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 {
 	wxString sourceFile = GetColumnText(selectFilesQuery_, file_table_column_names::source_file);
 	wxString targetFile = GetColumnText(selectFilesQuery_, file_table_column_names::target_file);
 
-	wxLongLong_t localPathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::local_path, false);
-	wxLongLong_t remotePathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::remote_path, false);
+	int64_t localPathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::local_path, false);
+	int64_t remotePathId = GetColumnInt64(selectFilesQuery_, file_table_column_names::remote_path, false);
 
 	const CLocalPath& localPath(GetLocalPath(localPathId));
 	const CServerPath& remotePath(GetRemotePath(remotePathId));
@@ -988,9 +964,9 @@ wxLongLong_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 
 	if (localPathId == -1 || remotePathId == -1)
 	{
-		// QueueItemType_Folder
+		// QueueItemType::Folder
 		if ((download && localPath.empty()) ||
-			(!download && remotePath.IsEmpty()))
+			(!download && remotePath.empty()))
 		{
 			return INVALID_DATA;
 		}
@@ -1003,25 +979,24 @@ wxLongLong_t CQueueStorage::Impl::ParseFileFromRow(CFileItem** pItem)
 	else
 	{
 		wxLongLong size = GetColumnInt64(selectFilesQuery_, file_table_column_names::size);
-		int errorCount = GetColumnInt(selectFilesQuery_, file_table_column_names::error_count);
-		int priority = GetColumnInt(selectFilesQuery_, file_table_column_names::priority, priority_normal);
+		unsigned char errorCount = static_cast<unsigned char>(GetColumnInt(selectFilesQuery_, file_table_column_names::error_count));
+		int priority = GetColumnInt(selectFilesQuery_, file_table_column_names::priority, static_cast<int>(QueuePriority::normal));
 
-		bool binary = GetColumnInt(selectFilesQuery_, file_table_column_names::ascii_file) == 0;
+		bool ascii = GetColumnInt(selectFilesQuery_, file_table_column_names::ascii_file) != 0;
 		int overwrite_action = GetColumnInt(selectFilesQuery_, file_table_column_names::default_exists_action, CFileExistsNotification::unknown);
 
 		if (sourceFile.empty() || localPath.empty() ||
-			remotePath.IsEmpty() ||
+			remotePath.empty() ||
 			size < -1 ||
-			priority < 0 || priority >= PRIORITY_COUNT ||
-			errorCount < 0)
+			priority < 0 || priority >= static_cast<int>(QueuePriority::count))
 		{
 			return INVALID_DATA;
 		}
 
 		CFileItem* fileItem = new CFileItem(0, true, download, sourceFile, targetFile, localPath, remotePath, size);
 		*pItem = fileItem;
-		fileItem->m_transferSettings.binary = binary;
-		fileItem->SetPriorityRaw((enum QueuePriority)priority);
+		fileItem->SetAscii(ascii);
+		fileItem->SetPriorityRaw(QueuePriority(priority));
 		fileItem->m_errorCount = errorCount;
 
 		if (overwrite_action > 0 && overwrite_action < CFileExistsNotification::ACTION_COUNT)
@@ -1065,8 +1040,7 @@ bool CQueueStorage::SaveQueue(std::vector<CServerItem*> const& queue)
 	d_->ClearCaches();
 
 	bool ret = true;
-	if (sqlite3_exec(d_->db_, "BEGIN TRANSACTION", 0, 0, 0) == SQLITE_OK)
-	{
+	if (sqlite3_exec(d_->db_, "BEGIN TRANSACTION", 0, 0, 0) == SQLITE_OK) {
 		for (std::vector<CServerItem*>::const_iterator it = queue.begin(); it != queue.end(); ++it)
 			ret &= d_->SaveServer(**it);
 
@@ -1081,9 +1055,9 @@ bool CQueueStorage::SaveQueue(std::vector<CServerItem*> const& queue)
 	return ret;
 }
 
-wxLongLong_t CQueueStorage::GetServer(CServer& server, bool fromBeginning)
+int64_t CQueueStorage::GetServer(CServer& server, bool fromBeginning)
 {
-	wxLongLong_t ret = -1;
+	int64_t ret = -1;
 
 	if (d_->selectServersQuery_)
 	{
@@ -1131,9 +1105,9 @@ wxLongLong_t CQueueStorage::GetServer(CServer& server, bool fromBeginning)
 }
 
 
-wxLongLong_t CQueueStorage::GetFile(CFileItem** pItem, wxLongLong_t server)
+int64_t CQueueStorage::GetFile(CFileItem** pItem, int64_t server)
 {
-	wxLongLong_t ret = -1;
+	int64_t ret = -1;
 	*pItem = 0;
 
 	if (d_->selectFilesQuery_)
