@@ -6,21 +6,22 @@
 #include "Options.h"
 #include "loginmanager.h"
 #include "Mainfrm.h"
+#include "asksavepassworddialog.h"
 
 BEGIN_EVENT_TABLE(CQuickconnectBar, wxPanel)
 EVT_BUTTON(XRCID("ID_QUICKCONNECT_OK"), CQuickconnectBar::OnQuickconnect)
 EVT_BUTTON(XRCID("ID_QUICKCONNECT_DROPDOWN"), CQuickconnectBar::OnQuickconnectDropdown)
 EVT_MENU(wxID_ANY, CQuickconnectBar::OnMenu)
 EVT_TEXT_ENTER(wxID_ANY, CQuickconnectBar::OnQuickconnect)
-EVT_NAVIGATION_KEY(CQuickconnectBar::OnKeyboardNavigation)
-END_EVENT_TABLE();
+END_EVENT_TABLE()
 
 CQuickconnectBar::CQuickconnectBar()
+	: m_pHost()
+	, m_pUser()
+	, m_pPass()
+	, m_pPort()
+	, m_pMainFrame()
 {
-	m_pHost = 0;
-	m_pUser = 0;
-	m_pPass = 0;
-	m_pPort = 0;
 }
 
 CQuickconnectBar::~CQuickconnectBar()
@@ -30,7 +31,7 @@ CQuickconnectBar::~CQuickconnectBar()
 bool CQuickconnectBar::Create(CMainFrame* pParent)
 {
 	m_pMainFrame = pParent;
-    if (!wxXmlResource::Get()->LoadPanel(this, pParent, _T("ID_QUICKCONNECTBAR")))
+	if (!wxXmlResource::Get()->LoadPanel(this, pParent, _T("ID_QUICKCONNECTBAR")))
 	{
 		wxLogError(_("Cannot load Quickconnect bar from resource file"));
 		return false;
@@ -54,8 +55,6 @@ bool CQuickconnectBar::Create(CMainFrame* pParent)
 		return false;
 	}
 
-	m_pPort->SetMaxLength(5);
-
 	return true;
 }
 
@@ -64,7 +63,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState || !pState->m_pEngine)
 	{
-		wxMessageBox(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -72,7 +71,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	wxString user = m_pUser->GetValue();
 	wxString pass = m_pPass->GetValue();
 	wxString port = m_pPort->GetValue();
-	
+
 	CServer server;
 
 	wxString error;
@@ -83,7 +82,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		wxString msg = _("Could not parse server address:");
 		msg += _T("\n");
 		msg += error;
-		wxMessageBox(msg, _("Syntax error"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(msg, _("Syntax error"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -100,12 +99,12 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	default:
 		{
 			const wxString prefix = server.GetPrefixFromProtocol(protocol);
-			if (prefix != _T(""))
+			if (!prefix.empty())
 				host = prefix + _T("://") + host;
 		}
 		break;
 	}
-	
+
 	m_pHost->SetValue(host);
 	if (server.GetPort() != server.GetDefaultPort(server.GetProtocol()))
 		m_pPort->SetValue(wxString::Format(_T("%d"), server.GetPort()));
@@ -113,18 +112,24 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 		m_pPort->SetValue(_T(""));
 
 	m_pUser->SetValue(server.GetUser());
-	m_pPass->SetValue(server.GetPass());
+	if (server.GetLogonType() != ANONYMOUS)
+		m_pPass->SetValue(server.GetPass());
+	else
+		m_pPass->SetValue(_T(""));
 
 	if (protocol == HTTP || protocol == HTTPS)
 	{
-		wxString error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over SSL (implicit) and\nftpes:// for FTP over SSL (explicit).");
-		wxMessageBox(error, _("Syntax error"), wxICON_EXCLAMATION);
+		wxString error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP with optional encryption,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over TLS (implicit) and\nftpes:// for FTP over TLS (explicit).");
+		wxMessageBoxEx(error, _("Syntax error"), wxICON_EXCLAMATION);
 		return;
 	}
 
 	if (event.GetId() == 1)
 		server.SetBypassProxy(true);
 
+	if (server.GetLogonType() != ANONYMOUS && !CAskSavePasswordDialog::Run(this))
+		return;
+
 	if (!m_pMainFrame->ConnectToServer(server, path))
 		return;
 
@@ -149,7 +154,7 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
 		unsigned int i = 0;
 		for (std::list<CServer>::const_iterator iter = m_recentServers.begin();
 			iter != m_recentServers.end();
-			iter++, i++)
+			++iter, ++i)
 		{
 			wxString name(iter->FormatServer());
 			name.Replace(_T("&"), _T("&&"));
@@ -181,8 +186,8 @@ void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 	if (index >= m_recentServers.size())
 		return;
 
-	std::list<CServer>::const_iterator iter;
-	for (iter = m_recentServers.begin(); index; index--, iter++);
+	std::list<CServer>::const_iterator iter = m_recentServers.begin();
+	std::advance(iter, index);
 
 	CServer server = *iter;
 	if (server.GetLogonType() == ASK)
@@ -194,7 +199,7 @@ void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (!pState || !pState->m_pEngine)
 	{
-		wxMessageBox(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
+		wxMessageBoxEx(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -208,19 +213,3 @@ void CQuickconnectBar::ClearFields()
 	m_pUser->SetValue(_T(""));
 	m_pPass->SetValue(_T(""));
 }
-
-void CQuickconnectBar::OnKeyboardNavigation(wxNavigationKeyEvent& event)
-{
-	if (event.GetDirection() && event.GetEventObject() == XRCCTRL(*this, "ID_QUICKCONNECT_DROPDOWN", wxButton))
-	{
-		event.SetEventObject(this);
-		GetParent()->ProcessEvent(event);
-	}
-	else if (!event.GetDirection() && event.GetEventObject() == m_pHost)
-	{
-		event.SetEventObject(this);
-		GetParent()->ProcessEvent(event);
-	}
-	else
-		event.Skip();
-}
