@@ -12,13 +12,12 @@
 #include "sizeformatting.h"
 #include "timeformatting.h"
 #include "window_state_manager.h"
+#include "xrc_helper.h"
 
-class CSearchFileData : public CGenericFileData
+class CSearchFileData : public CGenericFileData, public CDirentry
 {
 public:
 	CServerPath path;
-
-	CDirentry entry;
 };
 
 class CSearchDialogFileList : public CFileListCtrl<CSearchFileData>
@@ -45,315 +44,17 @@ protected:
 #endif
 
 private:
-	virtual bool CanStartComparison(wxString* pError) { return false; }
+	virtual bool CanStartComparison(wxString*) { return false; }
 	virtual void StartComparison() {}
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, wxDateTime& date, bool &hasTime) { return false; }
-	virtual void CompareAddFile(CComparableListing::t_fileEntryFlags flags) {}
+	virtual bool GetNextFile(wxString&, bool &, wxLongLong &, CDateTime&) { return false; }
+	virtual void CompareAddFile(CComparableListing::t_fileEntryFlags) {}
 	virtual void FinishComparison() {}
-	virtual void ScrollTopItem(int item) {}
+	virtual void ScrollTopItem(int) {}
 	virtual void OnExitComparisonMode() {}
 
 	int m_dirIcon;
 };
 
-// Helper classes for fast sorting using std::sort
-// -----------------------------------------------
-
-class CSearchDialogFileList;
-class CSearchSort : public CListViewSort
-{
-public:
-	CSearchSort(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData> &fileData, enum DirSortMode dirSortMode)
-		: m_fileData(fileData), m_dirSortMode(dirSortMode), m_pListCtrl(pListCtrl)
-	{
-	}
-
-	#define CMP(f, data1, data2) \
-		{\
-			int res = f(data1, data2);\
-			if (res == -1)\
-				return true;\
-			else if (res == 1)\
-				return false;\
-		}
-
-	#define CMP_LESS(f, data1, data2) \
-		{\
-			int res = f(data1, data2);\
-			if (res == -1)\
-				return true;\
-			else\
-				return false;\
-		}
-
-	inline int CmpDir(const CDirentry &data1, const CDirentry &data2) const
-	{
-		switch (m_dirSortMode)
-		{
-		default:
-		case dirsort_ontop:
-			if (data1.is_dir())
-			{
-				if (!data2.is_dir())
-					return -1;
-				else
-					return 0;
-			}
-			else
-			{
-				if (data2.is_dir())
-					return 1;
-				else
-					return 0;
-			}
-		case dirsort_onbottom:
-			if (data1.is_dir())
-			{
-				if (!data2.is_dir())
-					return 1;
-				else
-					return 0;
-			}
-			else
-			{
-				if (data2.is_dir())
-					return -1;
-				else
-					return 0;
-			}
-		case dirsort_inline:
-			return 0;
-		}
-	}
-
-	inline int CmpName(const CDirentry &data1, const CDirentry &data2) const
-	{
-#ifdef __WXMSW__
-		return data1.name.CmpNoCase(data2.name);
-#else
-		return data1.name.Cmp(data2.name);
-#endif
-	}
-
-	inline int CmpSize(const CDirentry &data1, const CDirentry &data2) const
-	{
-		const wxLongLong diff = data1.size - data2.size;
-		if (diff < 0)
-			return -1;
-		else if (diff > 0)
-			return 1;
-		else
-			return 0;
-	}
-
-	inline int CmpStringNoCase(const wxString &data1, const wxString &data2) const
-	{
-		return data1.CmpNoCase(data2);
-	}
-
-	inline int CmpTime(const CDirentry &data1, const CDirentry &data2) const
-	{
-		if (!data1.has_date())
-		{
-			if (data2.has_date())
-				return -1;
-			else
-				return 0;
-		}
-		else
-		{
-			if (!data2.has_date())
-				return 1;
-
-			if (data1.time < data2.time)
-				return -1;
-			else if (data1.time > data2.time)
-				return 1;
-			else
-				return 0;
-		}
-	}
-
-protected:
-	std::vector<CSearchFileData> &m_fileData;
-
-	const enum DirSortMode m_dirSortMode;
-
-	CSearchDialogFileList* m_pListCtrl;
-};
-
-template<class T> class CReverseSort : public T
-{
-public:
-	CReverseSort(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: T(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	inline bool operator()(int a, int b) const
-	{
-		return T::operator()(b, a);
-	}
-};
-
-class CSearchSortName : public CSearchSort
-{
-public:
-	CSearchSortName(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortName> CSearchSortName_Reverse;
-
-class CSearchSortPath : public CSearchSort
-{
-public:
-	CSearchSortPath(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		if (m_fileData[a].path < m_fileData[b].path)
-			return true;
-		if (m_fileData[a].path != m_fileData[b].path)
-			return false;
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortPath> CSearchSortPath_Reverse;
-
-class CSearchSortSize : public CSearchSort
-{
-public:
-	CSearchSortSize(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpSize, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortSize> CSearchSortSize_Reverse;
-
-class CSearchSortType : public CSearchSort
-{
-public:
-	CSearchSortType(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		CSearchFileData &data1 = m_fileData[a];
-		CSearchFileData &data2 = m_fileData[b];
-
-		CMP(CmpDir, data1.entry, data2.entry);
-
-		if (data1.fileType.IsEmpty())
-			data1.fileType = m_pListCtrl->GetType(data1.entry.name, data1.entry.is_dir());
-		if (data2.fileType.IsEmpty())
-			data2.fileType = m_pListCtrl->GetType(data2.entry.name, data2.entry.is_dir());
-
-		CMP(CmpStringNoCase, data1.fileType, data2.fileType);
-
-		CMP_LESS(CmpName, data1.entry, data2.entry);
-	}
-};
-typedef CReverseSort<CSearchSortType> CSearchSortType_Reverse;
-
-class CSearchSortTime : public CSearchSort
-{
-public:
-	CSearchSortTime(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpTime, data1, data2);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortTime> CSearchSortTime_Reverse;
-
-class CSearchSortPermissions : public CSearchSort
-{
-public:
-	CSearchSortPermissions(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpStringNoCase, data1.permissions, data2.permissions);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortPermissions> CSearchSortPermissions_Reverse;
-
-class CSearchSortOwnerGroup : public CSearchSort
-{
-public:
-	CSearchSortOwnerGroup(CSearchDialogFileList* pListCtrl, std::vector<CSearchFileData>& fileData, enum CSearchSort::DirSortMode dirSortMode)
-		: CSearchSort(pListCtrl, fileData, dirSortMode)
-	{
-	}
-
-	bool operator()(int a, int b) const
-	{
-		const CDirentry& data1 = m_fileData[a].entry;
-		const CDirentry& data2 = m_fileData[b].entry;
-
-		CMP(CmpDir, data1, data2);
-
-		CMP(CmpStringNoCase, data1.ownerGroup, data2.ownerGroup);
-
-		CMP_LESS(CmpName, data1, data2);
-	}
-};
-typedef CReverseSort<CSearchSortOwnerGroup> CSearchSortOwnerGroup_Reverse;
-
 // Search dialog file list
 // -----------------------
 
@@ -368,7 +69,7 @@ CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState* pSt
 
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-	m_dirIcon = GetIconIndex(dir);
+	m_dirIcon = GetIconIndex(iconType::dir);
 
 	InitSort(OPTION_SEARCH_SORTORDER);
 
@@ -388,77 +89,73 @@ CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CState* pSt
 
 bool CSearchDialogFileList::ItemIsDir(int index) const
 {
-	return m_fileData[index].entry.is_dir();
+	return m_fileData[index].is_dir();
 }
 
 wxLongLong CSearchDialogFileList::ItemGetSize(int index) const
 {
-	return m_fileData[index].entry.size;
+	return m_fileData[index].size;
 }
 
 CFileListCtrl<CSearchFileData>::CSortComparisonObject CSearchDialogFileList::GetSortComparisonObject()
 {
-	CSearchSort::DirSortMode dirSortMode = GetDirSortMode();
+	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
+	CFileListCtrlSortBase::NameSortMode nameSortMode = GetNameSortMode();
 
-	if (!m_sortDirection)
-	{
+	if (!m_sortDirection) {
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPath(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortPath<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortSize(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortSize<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortType(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortType<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 4)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortTime(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortTime<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 5)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPermissions(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortPermissions<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 6)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortOwnerGroup(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortOwnerGroup<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortName(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CFileListCtrlSortName<std::vector<CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
-	else
-	{
+	else {
 		if (m_sortColumn == 1)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPath_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPath<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 2)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortSize_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortSize<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 3)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortType_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortType<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 4)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortTime_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortTime<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 5)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortPermissions_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortPermissions<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else if (m_sortColumn == 6)
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortOwnerGroup_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 		else
-			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CSearchSortName_Reverse(this, m_fileData, dirSortMode));
+			return CFileListCtrl<CSearchFileData>::CSortComparisonObject(new CReverseSort<CFileListCtrlSortName<std::vector<CSearchFileData>, CSearchFileData>, CSearchFileData>(m_fileData, m_fileData, dirSortMode, nameSortMode, this));
 	}
 }
 
 wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 {
 	if (item < 0 || item >= (int)m_indexMapping.size())
-		return _T("");
+		return wxString();
 	int index = m_indexMapping[item];
 
-	const CDirentry& entry = m_fileData[index].entry;
+	const CDirentry& entry = m_fileData[index];
 	if (!column)
 		return entry.name;
 	else if (column == 1)
 		return m_fileData[index].path.GetPath();
-	else if (column == 2)
-	{
+	else if (column == 2) {
 		if (entry.is_dir() || entry.size < 0)
-			return _T("");
+			return wxString();
 		else
-			return CSizeFormat::Format(entry.size);
+			return CSizeFormat::Format(entry.size.GetValue());
 	}
-	else if (column == 3)
-	{
+	else if (column == 3) {
 		CSearchFileData& data = m_fileData[index];
-		if (data.fileType.IsEmpty())
-		{
+		if (data.fileType.empty()) {
 			if (data.path.GetType() == VMS)
 				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
 			else
@@ -468,20 +165,12 @@ wxString CSearchDialogFileList::GetItemText(int item, unsigned int column)
 		return data.fileType;
 	}
 	else if (column == 4)
-	{
-		if (!entry.has_date())
-			return _T("");
-
-		if (entry.has_time())
-			return CTimeFormat::FormatDateTime(entry.time);
-		else
-			return CTimeFormat::FormatDate(entry.time);
-	}
+		return CTimeFormat::Format(entry.time);
 	else if (column == 5)
-		return entry.permissions;
+		return *entry.permissions;
 	else if (column == 6)
-		return entry.ownerGroup;
-	return _T("");
+		return *entry.ownerGroup;
+	return wxString();
 }
 
 int CSearchDialogFileList::OnGetItemImage(long item) const
@@ -496,7 +185,7 @@ int CSearchDialogFileList::OnGetItemImage(long item) const
 	if (icon != -2)
 		return icon;
 
-	icon = pThis->GetIconIndex(file, pThis->m_fileData[index].entry.name, false);
+	icon = pThis->GetIconIndex(iconType::file, pThis->m_fileData[index].name, false);
 	return icon;
 }
 
@@ -508,23 +197,21 @@ EVT_BUTTON(XRCID("ID_START"), CSearchDialog::OnSearch)
 EVT_BUTTON(XRCID("ID_STOP"), CSearchDialog::OnStop)
 EVT_CONTEXT_MENU(CSearchDialog::OnContextMenu)
 EVT_MENU(XRCID("ID_MENU_SEARCH_DOWNLOAD"), CSearchDialog::OnDownload)
+EVT_MENU(XRCID("ID_MENU_SEARCH_EDIT"), CSearchDialog::OnEdit)
 EVT_MENU(XRCID("ID_MENU_SEARCH_DELETE"), CSearchDialog::OnDelete)
 EVT_CHAR_HOOK(CSearchDialog::OnCharHook)
 END_EVENT_TABLE()
 
 CSearchDialog::CSearchDialog(wxWindow* parent, CState* pState, CQueueView* pQueue)
 	: CStateEventHandler(pState)
+	, m_parent(parent)
+	, m_pQueue(pQueue)
 {
-	m_pQueue = pQueue;
-	m_parent = parent;
-	m_pWindowStateManager = 0;
-	m_searching = false;
 }
 
 CSearchDialog::~CSearchDialog()
 {
-	if (m_pWindowStateManager)
-	{
+	if (m_pWindowStateManager) {
 		m_pWindowStateManager->Remember(OPTION_SEARCH_SIZE);
 		delete m_pWindowStateManager;
 	}
@@ -550,8 +237,8 @@ bool CSearchDialog::Load()
 	m_results->SetFilelistStatusBar(pStatusBar);
 
 	const CServerPath path = m_pState->GetRemotePath();
-	if (!path.IsEmpty())
-		XRCCTRL(*this, "ID_PATH", wxTextCtrl)->ChangeValue(path.GetPath());
+	if (!path.empty())
+		xrc_call(*this, "ID_PATH", &wxTextCtrl::ChangeValue, path.GetPath());
 
 	SetCtrlState();
 
@@ -562,7 +249,10 @@ bool CSearchDialog::Load()
 
 	LoadConditions();
 	EditFilter(m_search_filter);
-	XRCCTRL(*this, "ID_CASE", wxCheckBox)->SetValue(m_search_filter.matchCase);
+
+	xrc_call(*this, "ID_CASE", &wxCheckBox::SetValue, m_search_filter.matchCase);
+	xrc_call(*this, "ID_FIND_FILES", &wxCheckBox::SetValue, m_search_filter.filterFiles);
+	xrc_call(*this, "ID_FIND_DIRS", &wxCheckBox::SetValue, m_search_filter.filterDirs);
 
 	return true;
 }
@@ -586,19 +276,16 @@ void CSearchDialog::Run()
 	m_pState->UnblockHandlers(STATECHANGE_REMOTE_DIR);
 	m_pState->UnblockHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
 
-	if (m_searching)
-	{
-		if (!m_pState->IsRemoteIdle())
-		{
+	if (m_searching) {
+		if (!m_pState->IsRemoteIdle()) {
 			m_pState->m_pCommandQueue->Cancel();
 			m_pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 		}
-		if (!m_original_dir.IsEmpty())
+		if (!m_original_dir.empty())
 			m_pState->ChangeRemoteDir(m_original_dir);
 	}
-	else
-	{
-		if (m_pState->IsRemoteIdle() && !m_original_dir.IsEmpty())
+	else {
+		if (m_pState->IsRemoteIdle() && !m_original_dir.empty())
 			m_pState->ChangeRemoteDir(m_original_dir);
 	}
 }
@@ -607,8 +294,7 @@ void CSearchDialog::OnStateChange(CState* pState, enum t_statechange_notificatio
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
 		ProcessDirectoryListing();
-	else if (notification == STATECHANGE_REMOTE_IDLE)
-	{
+	else if (notification == STATECHANGE_REMOTE_IDLE) {
 		if (pState->IsRemoteIdle())
 			m_searching = false;
 		SetCtrlState();
@@ -617,31 +303,29 @@ void CSearchDialog::OnStateChange(CState* pState, enum t_statechange_notificatio
 
 void CSearchDialog::ProcessDirectoryListing()
 {
-	CSharedPointer<const CDirectoryListing> listing = m_pState->GetRemoteDir();
+	std::shared_ptr<CDirectoryListing> listing = m_pState->GetRemoteDir();
 
-	if (!listing || listing->m_failed)
+	if (!listing || listing->failed())
 		return;
 
 	// Do not process same directory multiple times
 	if (!m_visited.insert(listing->path).second)
 		return;
-	
+
 	int old_count = m_results->m_fileData.size();
 	int added = 0;
 
 	m_results->m_fileData.reserve(m_results->m_fileData.size() + listing->GetCount());
 	m_results->m_indexMapping.reserve(m_results->m_indexMapping.size() + listing->GetCount());
 
-	for (unsigned int i = 0; i < listing->GetCount(); i++)
-	{
+	for (unsigned int i = 0; i < listing->GetCount(); ++i) {
 		const CDirentry& entry = (*listing)[i];
 
-		if (m_search_filter.filters.size() && !CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, listing->path.GetPath(), entry.is_dir(), entry.size, 0, entry.has_date() ? &entry.time : 0))
+		if (!CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, listing->path.GetPath(), entry.is_dir(), entry.size, 0, entry.time))
 			continue;
 
 		CSearchFileData data;
-		data.flags = CComparableListing::normal;
-		data.entry = entry;
+		static_cast<CDirentry&>(data) = entry;
 		data.path = listing->path;
 		data.icon = entry.is_dir() ? m_results->m_dirIcon : -2;
 		m_results->m_fileData.push_back(data);
@@ -653,17 +337,16 @@ void CSearchDialog::ProcessDirectoryListing()
 			m_results->GetFilelistStatusBar()->AddFile(entry.size);
 	}
 
-	m_results->SetItemCount(old_count + added);
-
-	m_results->SortList(-1, -1, true);
-
-	m_results->RefreshListOnly(false);
+	if (added) {
+		m_results->SetItemCount(old_count + added);
+		m_results->SortList(-1, -1, true);
+		m_results->RefreshListOnly(false);
+	}
 }
 
 void CSearchDialog::OnSearch(wxCommandEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		wxBell();
 		return;
 	}
@@ -671,15 +354,13 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 	CServerPath path;
 
 	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
-		wxMessageBox(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
+	if (!pServer) {
+		wxMessageBoxEx(_("Connection to server lost."), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	path.SetType(pServer->GetType());
-	if (!path.SetPath(XRCCTRL(*this, "ID_PATH", wxTextCtrl)->GetValue()) || path.IsEmpty())
-	{
-		wxMessageBox(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
+	if (!path.SetPath(XRCCTRL(*this, "ID_PATH", wxTextCtrl)->GetValue()) || path.empty()) {
+		wxMessageBoxEx(_("Need to enter valid remote path"), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -687,18 +368,18 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 
 	// Prepare filter
 	wxString error;
-	if (!ValidateFilter(error, true))
-	{
-		wxMessageBox(wxString::Format(_("Invalid search conditions: %s"), error.c_str()), _("Remote file search"), wxICON_EXCLAMATION);
+	if (!ValidateFilter(error, true)) {
+		wxMessageBoxEx(wxString::Format(_("Invalid search conditions: %s"), error), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
 	m_search_filter = GetFilter();
-	if (!CFilterManager::CompileRegexes(m_search_filter))
-	{
-		wxMessageBox(_("Invalid regular expression in search conditions."), _("Remote file search"), wxICON_EXCLAMATION);
+	if (!CFilterManager::CompileRegexes(m_search_filter)) {
+		wxMessageBoxEx(_("Invalid regular expression in search conditions."), _("Remote file search"), wxICON_EXCLAMATION);
 		return;
 	}
-	m_search_filter.matchCase = XRCCTRL(*this, "ID_CASE", wxCheckBox)->GetValue();
+	m_search_filter.matchCase = xrc_call(*this, "ID_CASE", &wxCheckBox::GetValue);
+	m_search_filter.filterFiles = xrc_call(*this, "ID_FIND_FILES", &wxCheckBox::GetValue);
+	m_search_filter.filterDirs = xrc_call(*this, "ID_FIND_DIRS", &wxCheckBox::GetValue);
 
 	// Delete old results
 	m_results->ClearSelection();
@@ -719,8 +400,7 @@ void CSearchDialog::OnSearch(wxCommandEvent& event)
 
 void CSearchDialog::OnStop(wxCommandEvent& event)
 {
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		m_pState->m_pCommandQueue->Cancel();
 		m_pState->GetRecursiveOperationHandler()->StopRecursiveOperation();
 	}
@@ -735,8 +415,7 @@ void CSearchDialog::SetCtrlState()
 
 void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 {
-	if (event.GetEventObject() != m_results)
-	{
+	if (event.GetEventObject() != m_results && event.GetEventObject() != m_results->GetMainWindow()) {
 		event.Skip();
 		return;
 	}
@@ -745,8 +424,7 @@ void CSearchDialog::OnContextMenu(wxContextMenuEvent& event)
 	if (!pMenu)
 		return;
 
-	if (!m_pState->IsRemoteIdle())
-	{
+	if (!m_pState->IsRemoteIdle()) {
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_DOWNLOAD"), false);
 		pMenu->Enable(XRCID("ID_MENU_SEARCH_DELETE"), false);
 	}
@@ -770,11 +448,10 @@ public:
 			desc.Printf(wxPLURAL("Selected %d file for transfer.", "Selected %d files for transfer.", count_files), count_files);
 		else if (!count_files)
 			desc.Printf(wxPLURAL("Selected %d directory with its contents for transfer.", "Selected %d directories with their contents for transfer.", count_dirs), count_dirs);
-		else
-		{
+		else {
 			wxString files = wxString::Format(wxPLURAL("%d file", "%d files", count_files), count_files);
 			wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", count_dirs), count_dirs);
-			desc.Printf(_("Selected %s and %s for transfer."), files.c_str(), dirs.c_str());
+			desc.Printf(_("Selected %s and %s for transfer."), files, dirs);
 		}
 		XRCCTRL(*this, "ID_DESC", wxStaticText)->SetLabel(desc);
 
@@ -788,7 +465,7 @@ public:
 
 protected:
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnBrowse(wxCommandEvent& event);
 	void OnOK(wxCommandEvent& event);
 };
@@ -812,15 +489,13 @@ void CSearchDownloadDialog::OnOK(wxCommandEvent& event)
 	wxTextCtrl *pText = XRCCTRL(*this, "ID_LOCALPATH", wxTextCtrl);
 
 	CLocalPath path(pText->GetValue());
-	if (path.empty())
-	{
-		wxMessageBox(_("You have to enter a local directory."), _("Download search results"), wxICON_EXCLAMATION);
+	if (path.empty()) {
+		wxMessageBoxEx(_("You have to enter a local directory."), _("Download search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
-	if (!path.IsWriteable())
-	{
-		wxMessageBox(_("You have to enter a writable local directory."), _("Download search results"), wxICON_EXCLAMATION);
+	if (!path.IsWriteable()) {
+		wxMessageBoxEx(_("You have to enter a writable local directory."), _("Download search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -830,17 +505,15 @@ void CSearchDownloadDialog::OnOK(wxCommandEvent& event)
 void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::list<CServerPath> &selected_dirs)
 {
 	int sel = -1;
-	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1)
-	{
+	while ((sel = m_results->GetNextItem(sel, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != -1) {
 		if (sel > (int)m_results->m_indexMapping.size())
 			continue;
 		int index = m_results->m_indexMapping[sel];
 
-		if (m_results->m_fileData[index].entry.is_dir())
-		{
+		if (m_results->m_fileData[index].is_dir()) {
 			CServerPath path = m_results->m_fileData[index].path;
-			path.ChangePath(m_results->m_fileData[index].entry.name);
-			if (path.IsEmpty())
+			path.ChangePath(m_results->m_fileData[index].name);
+			if (path.empty())
 				continue;
 
 			bool replaced = false;
@@ -850,35 +523,29 @@ void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::list<C
 			// Make sure that selected_dirs does not contain
 			// any directories that are in a parent-child relationship
 			// Resolve by only keeping topmost parents
-			while (iter != selected_dirs.end())
-			{
-				if (*iter == path)
-				{
+			while (iter != selected_dirs.end()) {
+				if (*iter == path) {
 					replaced = true;
 					break;
 				}
 
-				if (iter->IsParentOf(path, false))
-				{
+				if (iter->IsParentOf(path, false)) {
 					replaced = true;
 					break;
 				}
 
-				if (iter->IsSubdirOf(path, false))
-				{
-					if (!replaced)
-					{
+				if (iter->IsSubdirOf(path, false)) {
+					if (!replaced) {
 						*iter = path;
 						replaced = true;
 					}
-					else
-					{
+					else {
 						prev = iter++;
 						selected_dirs.erase(prev);
 						continue;
 					}
 				}
-				iter++;
+				++iter;
 			}
 			if (!replaced)
 				selected_dirs.push_back(path);
@@ -889,26 +556,24 @@ void CSearchDialog::ProcessSelection(std::list<int> &selected_files, std::list<C
 
 	// Now in a second phase filter out all files that are also in a directory
 	std::list<int> selected_files_new;
-	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); iter++)
-	{
-		CServerPath path = m_results->m_fileData[*iter].path;
+	for (auto const& sel : selected_files) {
+		CServerPath path = m_results->m_fileData[sel].path;
 		std::list<CServerPath>::const_iterator path_iter;
-		for (path_iter = selected_dirs.begin(); path_iter != selected_dirs.end(); path_iter++)
-		{
+		for (path_iter = selected_dirs.begin(); path_iter != selected_dirs.end(); ++path_iter) {
 			if (*path_iter == path || path_iter->IsParentOf(path, false))
 				break;
 		}
 		if (path_iter == selected_dirs.end())
-			selected_files_new.push_back(*iter);
+			selected_files_new.push_back(sel);
 	}
 	selected_files.swap(selected_files_new);
-	
+
 	// At this point selected_dirs contains uncomparable
 	// paths and selected_files contains only files not
 	// covered by any of those directories.
 }
 
-void CSearchDialog::OnDownload(wxCommandEvent& event)
+void CSearchDialog::OnDownload(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -921,9 +586,8 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 	if (selected_files.empty() && selected_dirs.empty())
 		return;
 
-	if (selected_dirs.size() > 1)
-	{
-		wxMessageBox(_("Downloading multiple unrelated directories is not yet supported"), _("Downloading search results"), wxICON_EXCLAMATION);
+	if (selected_dirs.size() > 1) {
+		wxMessageBoxEx(_("Downloading multiple unrelated directories is not yet supported"), _("Downloading search results"), wxICON_EXCLAMATION);
 		return;
 	}
 
@@ -934,16 +598,14 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 	wxTextCtrl *pText = XRCCTRL(dlg, "ID_LOCALPATH", wxTextCtrl);
 
 	CLocalPath path(pText->GetValue());
-	if (path.empty() || !path.IsWriteable())
-	{
+	if (path.empty() || !path.IsWriteable()) {
 		wxBell();
 		return;
 	}
 	m_local_target = path;
 
-	const CServer* pServer = m_pState->GetServer();
-	if (!pServer)
-	{
+	CServer const* pServer = m_pState->GetServer();
+	if (!pServer) {
 		wxBell();
 		return;
 	}
@@ -951,28 +613,31 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 	bool start = XRCCTRL(dlg, "ID_QUEUE_START", wxRadioButton)->GetValue();
 	bool flatten = XRCCTRL(dlg, "ID_PATHS_FLATTEN", wxRadioButton)->GetValue();
 
-	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); iter++)
-	{
-		const CDirentry& entry = m_results->m_fileData[*iter].entry;
+	for (auto const& sel : selected_files) {
+		const CDirentry& entry = m_results->m_fileData[sel];
 
 		CLocalPath target_path = path;
-		if (!flatten)
-		{
+		if (!flatten) {
 			// Append relative path to search root to local target path
-			CServerPath remote_path = m_results->m_fileData[*iter].path;
+			CServerPath remote_path = m_results->m_fileData[sel].path;
 			std::list<wxString> segments;
-			while (m_search_root.IsParentOf(remote_path, false) && remote_path.HasParent())
-			{
+			while (m_search_root.IsParentOf(remote_path, false) && remote_path.HasParent()) {
 				segments.push_front(remote_path.GetLastSegment());
 				remote_path = remote_path.GetParent();
 			}
-			for (std::list<wxString>::const_iterator segment_iter = segments.begin(); segment_iter != segments.end(); segment_iter++)
-				target_path.AddSegment(*segment_iter);
+			for (auto const& segment : segments) {
+				target_path.AddSegment(segment);
+			}
 		}
 
-		CServerPath remote_path = m_results->m_fileData[*iter].path;
-		const wxString localName = CQueueView::ReplaceInvalidCharacters(entry.name);
-		m_pQueue->QueueFile(!start, true, target_path, localName, entry.name, remote_path, *pServer, entry.size);
+		CServerPath remote_path = m_results->m_fileData[sel].path;
+		wxString localName = CQueueView::ReplaceInvalidCharacters(entry.name);
+		if (!entry.is_dir() && remote_path.GetType() == VMS && COptions::Get()->GetOptionVal(OPTION_STRIP_VMS_REVISION))
+			localName = StripVMSRevision(localName);
+
+		m_pQueue->QueueFile(!start, true,
+			entry.name, (localName != entry.name) ? localName : wxString(),
+			target_path, remote_path, *pServer, entry.size);
 	}
 	m_pQueue->QueueFile_Finish(start);
 
@@ -982,19 +647,18 @@ void CSearchDialog::OnDownload(wxCommandEvent& event)
 	else
 		mode = start ? CRecursiveOperation::recursive_download : CRecursiveOperation::recursive_addtoqueue;
 
-	for (std::list<CServerPath>::const_iterator iter = selected_dirs.begin(); iter != selected_dirs.end(); iter++)
-	{
+	for (auto const& dir : selected_dirs) {
 		CLocalPath target_path = path;
-		if (!flatten && iter->HasParent())
-			target_path.AddSegment(iter->GetLastSegment());
+		if (!flatten && dir.HasParent())
+			target_path.AddSegment(dir.GetLastSegment());
 
-		m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(*iter, _T(""), target_path, false);
+		m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(dir, _T(""), target_path, false);
 		std::list<CFilter> filters; // Empty, recurse into everything
-		m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(mode, *iter, filters, true, m_original_dir);
+		m_pState->GetRecursiveOperationHandler()->StartRecursiveOperation(mode, dir, filters, true, m_original_dir);
 	}
 }
 
-void CSearchDialog::OnDelete(wxCommandEvent& event)
+void CSearchDialog::OnEdit(wxCommandEvent&)
 {
 	if (!m_pState->IsRemoteIdle())
 		return;
@@ -1007,42 +671,90 @@ void CSearchDialog::OnDelete(wxCommandEvent& event)
 	if (selected_files.empty() && selected_dirs.empty())
 		return;
 
-	if (selected_dirs.size() > 1)
-	{
-		wxMessageBox(_("Deleting multiple unrelated directories is not yet supported"), _("Deleting directories"), wxICON_EXCLAMATION);
+	if (!selected_dirs.empty()) {
+		wxMessageBoxEx(_("Editing directories is not supported"), _("Editing search results"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	CEditHandler* pEditHandler = CEditHandler::Get();
+	if (!pEditHandler) {
+		wxBell();
+		return;
+	}
+
+	const wxString& localDir = pEditHandler->GetLocalDirectory();
+	if (localDir.empty()) {
+		wxMessageBoxEx(_("Could not get temporary directory to download file into."), _("Cannot edit file"), wxICON_STOP);
+		return;
+	}
+
+	const CServer* pServer = m_pState->GetServer();
+	if (!pServer) {
+		wxBell();
+		return;
+	}
+
+	if (selected_files.size() > 10) {
+		CConditionalDialog dlg(this, CConditionalDialog::many_selected_for_edit, CConditionalDialog::yesno);
+		dlg.SetTitle(_("Confirmation needed"));
+		dlg.AddText(_("You have selected more than 10 files for editing, do you really want to continue?"));
+
+		if (!dlg.Run())
+			return;
+	}
+
+	for (auto const& item : selected_files) {
+		const CDirentry& entry = m_results->m_fileData[item];
+		const CServerPath path = m_results->m_fileData[item].path;
+
+		pEditHandler->Edit(CEditHandler::remote, entry.name, path, *pServer, entry.size, this);
+	}
+}
+
+void CSearchDialog::OnDelete(wxCommandEvent&)
+{
+	if (!m_pState->IsRemoteIdle())
+		return;
+
+	// Find all selected files and directories
+	std::list<CServerPath> selected_dirs;
+	std::list<int> selected_files;
+	ProcessSelection(selected_files, selected_dirs);
+
+	if (selected_files.empty() && selected_dirs.empty())
+		return;
+
+	if (selected_dirs.size() > 1) {
+		wxMessageBoxEx(_("Deleting multiple unrelated directories is not yet supported"), _("Deleting directories"), wxICON_EXCLAMATION);
 		return;
 	}
 
 	wxString question;
 	if (selected_dirs.empty())
-		question.Printf(wxPLURAL("Really delete %d file?", "Really delete %d files?", selected_files.size()), selected_files.size());
+		question.Printf(wxPLURAL("Really delete %d file from the server?", "Really delete %d files from the server?", selected_files.size()), selected_files.size());
 	else if (selected_files.empty())
-		question.Printf(wxPLURAL("Really delete %d directory with its contents?", "Really delete %d directories with their contents?", selected_dirs.size()), selected_dirs.size());
-	else
-	{
+		question.Printf(wxPLURAL("Really delete %d directory with its contents from the server?", "Really delete %d directories with their contents from the server?", selected_dirs.size()), selected_dirs.size());
+	else {
 		wxString files = wxString::Format(wxPLURAL("%d file", "%d files", selected_files.size()), selected_files.size());
 		wxString dirs = wxString::Format(wxPLURAL("%d directory with its contents", "%d directories with their contents", selected_dirs.size()), selected_dirs.size());
-		question.Printf(_("Really delete %s and %s?"), files.c_str(), dirs.c_str());
+		question.Printf(_("Really delete %s and %s from the server?"), files, dirs);
 	}
 
-	if (wxMessageBox(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES)
+	if (wxMessageBoxEx(question, _("Confirm deletion"), wxICON_QUESTION | wxYES_NO) != wxYES)
 		return;
 
-	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); iter++)
-	{
-		const CDirentry& entry = m_results->m_fileData[*iter].entry;
+	for (std::list<int>::const_iterator iter = selected_files.begin(); iter != selected_files.end(); ++iter) {
+		const CDirentry& entry = m_results->m_fileData[*iter];
 		std::list<wxString> files_to_delete;
 		files_to_delete.push_back(entry.name);
 		m_pState->m_pCommandQueue->ProcessCommand(new CDeleteCommand(m_results->m_fileData[*iter].path, files_to_delete));
 	}
 
-	for (std::list<CServerPath>::const_iterator iter = selected_dirs.begin(); iter != selected_dirs.end(); iter++)
-	{
+	for (std::list<CServerPath>::const_iterator iter = selected_dirs.begin(); iter != selected_dirs.end(); ++iter) {
 		CServerPath path = *iter;
 		if (!path.HasParent())
 			m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(path, _T(""));
-		else
-		{
+		else {
 			m_pState->GetRecursiveOperationHandler()->AddDirectoryToVisit(path.GetParent(), path.GetLastSegment());
 			path = path.GetParent();
 		}
@@ -1054,8 +766,7 @@ void CSearchDialog::OnDelete(wxCommandEvent& event)
 
 void CSearchDialog::OnCharHook(wxKeyEvent& event)
 {
-	if (IsEscapeKey(event))
-	{
+	if (IsEscapeKey(event)) {
 		EndDialog(wxID_CANCEL);
 		return;
 	}
@@ -1070,7 +781,7 @@ int CSearchDialogFileList::GetOverlayIndex(int item)
 		return -1;
 	int index = m_indexMapping[item];
 
-	if (m_fileData[index].entry.is_link())
+	if (m_fileData[index].is_link())
 		return GetLinkOverlayIndex();
 
 	return 0;
@@ -1081,12 +792,10 @@ void CSearchDialog::LoadConditions()
 {
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
 
-	CXmlFile file;
-	
-	TiXmlElement* pDocument = file.Load(_T("search"));
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
 	}
 
@@ -1102,12 +811,10 @@ void CSearchDialog::SaveConditions()
 {
 	CInterProcessMutex mutex(MUTEX_SEARCHCONDITIONS);
 
-	CXmlFile file;
-	
-	TiXmlElement* pDocument = file.Load(_T("search"));
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("search")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return;
 	}
 
@@ -1118,5 +825,5 @@ void CSearchDialog::SaveConditions()
 
 	CFilterDialog::SaveFilter(pFilter, m_search_filter);
 
-	file.Save();
+	file.Save(true);
 }
