@@ -8,6 +8,7 @@
 class CWindowStateManager;
 class CSearchDialogFileList;
 class CQueueView;
+class CFilelistStatusBar;
 class CSearchDialog : protected CFilterConditionsDialog, public CStateEventHandler
 {
 	friend class CSearchDialogFileList;
@@ -23,17 +24,20 @@ protected:
 
 	void SetCtrlState();
 
+	void SaveConditions();
+	void LoadConditions();
+
 	wxWindow* m_parent;
-	CSearchDialogFileList *m_results;
+	CSearchDialogFileList *m_results{};
 	CQueueView* m_pQueue;
 
-	virtual void OnStateChange(enum t_statechange_notifications notification, const wxString& data);
+	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 
-	CWindowStateManager* m_pWindowStateManager;
+	CWindowStateManager* m_pWindowStateManager{};
 
 	CFilter m_search_filter;
 
-	bool m_searching;
+	bool m_searching{};
 
 	CServerPath m_original_dir;
 	CLocalPath m_local_target;
@@ -44,8 +48,10 @@ protected:
 	void OnSearch(wxCommandEvent& event);
 	void OnStop(wxCommandEvent& event);
 	void OnContextMenu(wxContextMenuEvent& event);
-	void OnDownload(wxCommandEvent& event);
-	void OnDelete(wxCommandEvent& event);
+	void OnDownload(wxCommandEvent&);
+	void OnEdit(wxCommandEvent&);
+	void OnDelete(wxCommandEvent&);
+	void OnCharHook(wxKeyEvent& event);
 
 	std::set<CServerPath> m_visited;
 
