@@ -1,4 +1,4 @@
-#include <filezilla.h>
+	#include <filezilla.h>
 #include "sitemanager.h"
 
 #include "filezillaapp.h"
@@ -6,46 +6,57 @@
 #include "Options.h"
 #include "xmlfunctions.h"
 
-std::map<int, CSiteManagerItemData_Site*> CSiteManager::m_idMap;
+std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> CSiteManager::m_idMap;
 
-bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler)
+bool CSiteManager::Load(CSiteManagerXmlHandler& handler)
+{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+		return false;
+	}
+
+	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
+	if (!pElement) {
+		return false;
+	}
+
+	return Load(pElement, handler);
+}
+
+bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler& handler)
 {
 	wxASSERT(pElement);
-	wxASSERT(pHandler);
 
-	for (TiXmlElement* pChild = pElement->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement())
-	{
-		if (!strcmp(pChild->Value(), "Folder"))
-		{
+	for (TiXmlElement* pChild = pElement->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement()) {
+		if (!strcmp(pChild->Value(), "Folder")) {
 			wxString name = GetTextElement_Trimmed(pChild);
 			if (name.empty())
 				continue;
 
 			const bool expand = GetTextAttribute(pChild, "expanded") != _T("0");
-			if (!pHandler->AddFolder(name, expand))
+			if (!handler.AddFolder(name, expand))
 				return false;
-			Load(pChild, pHandler);
-			if (!pHandler->LevelUp())
+			Load(pChild, handler);
+			if (!handler.LevelUp())
 				return false;
 		}
-		else if (!strcmp(pChild->Value(), "Server"))
-		{
-			CSiteManagerItemData_Site* data = ReadServerElement(pChild);
+		else if (!strcmp(pChild->Value(), "Server")) {
+			std::unique_ptr<CSiteManagerItemData_Site> data = ReadServerElement(pChild);
 
-			if (data)
-			{
-				pHandler->AddSite(data);
+			if (data) {
+				handler.AddSite(std::move(data));
 
 				// Bookmarks
-				for (TiXmlElement* pBookmark = pChild->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark"))
-				{
+				for (TiXmlElement* pBookmark = pChild->FirstChildElement("Bookmark"); pBookmark; pBookmark = pBookmark->NextSiblingElement("Bookmark")) {
 					TiXmlHandle handle(pBookmark);
 
 					wxString name = GetTextElement_Trimmed(pBookmark, "Name");
 					if (name.empty())
 						continue;
 
-					CSiteManagerItemData* data = new CSiteManagerItemData(CSiteManagerItemData::BOOKMARK);
+					auto data = make_unique<CSiteManagerItemData>(CSiteManagerItemData::BOOKMARK);
 
 					TiXmlText* localDir = handle.FirstChildElement("LocalDir").FirstChild().Text();
 					if (localDir)
@@ -55,19 +66,17 @@ bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler
 					if (remoteDir)
 						data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
 
-					if (data->m_localDir.empty() && data->m_remoteDir.IsEmpty())
-					{
-						delete data;
+					if (data->m_localDir.empty() && data->m_remoteDir.empty()) {
 						continue;
 					}
 
-					if (!data->m_localDir.empty() && !data->m_remoteDir.IsEmpty())
+					if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
 						data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
 
-					pHandler->AddBookmark(name, data);
+					handler.AddBookmark(name, std::move(data));
 				}
 
-				if (!pHandler->LevelUp())
+				if (!handler.LevelUp())
 					return false;
 			}
 		}
@@ -76,7 +85,7 @@ bool CSiteManager::Load(TiXmlElement *pElement, CSiteManagerXmlHandler* pHandler
 	return true;
 }
 
-CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElement)
+std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::ReadServerElement(TiXmlElement *pElement)
 {
 	CServer server;
 	if (!::GetServer(pElement, server))
@@ -84,7 +93,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 	if (server.GetName().empty())
 		return 0;
 
-	CSiteManagerItemData_Site* data = new CSiteManagerItemData_Site(server);
+	auto data = make_unique<CSiteManagerItemData_Site>(server);
 
 	TiXmlHandle handle(pElement);
 
@@ -100,7 +109,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 	if (remoteDir)
 		data->m_remoteDir.SetSafePath(ConvLocal(remoteDir->Value()));
 
-	if (!data->m_localDir.empty() && !data->m_remoteDir.IsEmpty())
+	if (!data->m_localDir.empty() && !data->m_remoteDir.empty())
 		data->m_sync = GetTextElementBool(pElement, "SyncBrowsing", false);
 
 	return data;
@@ -109,7 +118,7 @@ CSiteManagerItemData_Site* CSiteManager::ReadServerElement(TiXmlElement *pElemen
 class CSiteManagerXmlHandler_Menu : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, CSiteManagerItemData_Site*> *idMap, bool predefined)
+	CSiteManagerXmlHandler_Menu(wxMenu* pMenu, std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> *idMap, bool predefined)
 		: m_pMenu(pMenu), m_idMap(idMap)
 	{
 		m_added_site = false;
@@ -131,10 +140,10 @@ public:
 
 			// Use same sorting as site tree in site manager
 #ifdef __WXMSW__
-			if (pItem->GetLabel().CmpNoCase(name) > 0)
+			if (pItem->GetItemLabelText().CmpNoCase(name) > 0)
 				break;
 #else
-			if (pItem->GetLabel() > name)
+			if (pItem->GetItemLabelText() > name)
 				break;
 #endif
 		}
@@ -154,7 +163,7 @@ public:
 		return true;
 	}
 
-	virtual bool AddSite(CSiteManagerItemData_Site* data)
+	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site> data)
 	{
 		wxString newName(data->m_server.GetName());
 		int i = GetInsertIndex(m_pMenu, newName);
@@ -163,16 +172,15 @@ public:
 
 		data->m_path = path + _T("/") + CSiteManager::EscapeSegment(data->m_server.GetName());
 
-		(*m_idMap)[pItem->GetId()] = data;
+		(*m_idMap)[pItem->GetId()] = std::move(data);
 
 		m_added_site = true;
 
 		return true;
 	}
 
-	virtual bool AddBookmark(const wxString& name, CSiteManagerItemData* data)
+	virtual bool AddBookmark(const wxString& name, std::unique_ptr<CSiteManagerItemData> data)
 	{
-		delete data;
 		return true;
 	}
 
@@ -213,7 +221,7 @@ public:
 protected:
 	wxMenu* m_pMenu;
 
-	std::map<int, CSiteManagerItemData_Site*> *m_idMap;
+	std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> *m_idMap;
 
 	std::list<wxMenu*> m_parents;
 	std::list<wxString> m_childNames;
@@ -224,7 +232,34 @@ protected:
 	std::list<wxString> m_paths;
 };
 
-wxMenu* CSiteManager::GetSitesMenu()
+
+class CSiteManagerXmlHandler_Stats : public CSiteManagerXmlHandler
+{
+public:
+	virtual bool AddFolder(const wxString& name, bool)
+	{
+		++directories_;
+		return true;
+	}
+
+	virtual bool AddSite(std::unique_ptr<CSiteManagerItemData_Site> data)
+	{
+		++sites_;
+		return true;
+	}
+
+	virtual bool AddBookmark(const wxString& name, std::unique_ptr<CSiteManagerItemData> data)
+	{
+		++bookmarks_;
+		return true;
+	}
+
+	unsigned int sites_{};
+	unsigned int directories_{};
+	unsigned int bookmarks_{};
+};
+
+std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu()
 {
 	ClearIdMap();
 
@@ -232,53 +267,23 @@ wxMenu* CSiteManager::GetSitesMenu()
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	wxMenu* predefinedSites = GetSitesMenu_Predefined(m_idMap);
+	std::unique_ptr<wxMenu> predefinedSites = GetSitesMenu_Predefined(m_idMap);
 
-	CXmlFile file(_T("sitemanager"));
-	TiXmlElement* pDocument = file.Load();
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
+	auto pMenu = make_unique<wxMenu>();
+	CSiteManagerXmlHandler_Menu handler(pMenu.get(), &m_idMap, false);
 
-		if (!predefinedSites)
-			return predefinedSites;
-
-		wxMenu *pMenu = new wxMenu;
-		wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
-		pItem->Enable(false);
-		return pMenu;
+	bool res = Load(handler);
+	if (!res || !pMenu->GetMenuItemCount()) {
+		pMenu.reset();
 	}
 
-	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
-	{
-		if (predefinedSites)
-			return predefinedSites;
-
-		wxMenu *pMenu = new wxMenu;
-		wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
-		pItem->Enable(false);
-		return pMenu;
-	}
-
-	wxMenu* pMenu = new wxMenu;
-	CSiteManagerXmlHandler_Menu handler(pMenu, &m_idMap, false);
-
-	bool res = Load(pElement, &handler);
-	if (!res || !pMenu->GetMenuItemCount())
-	{
-		delete pMenu;
-		pMenu = 0;
-	}
-
-	if (pMenu)
-	{
+	if (pMenu) {
 		if (!predefinedSites)
 			return pMenu;
 
-		wxMenu* pRootMenu = new wxMenu;
-		pRootMenu->AppendSubMenu(predefinedSites, _("Predefined Sites"));
-		pRootMenu->AppendSubMenu(pMenu, _("My Sites"));
+		auto pRootMenu = make_unique<wxMenu>();
+		pRootMenu->AppendSubMenu(predefinedSites.release(), _("Predefined Sites"));
+		pRootMenu->AppendSubMenu(pMenu.release(), _("My Sites"));
 
 		return pRootMenu;
 	}
@@ -286,7 +291,7 @@ wxMenu* CSiteManager::GetSitesMenu()
 	if (predefinedSites)
 		return predefinedSites;
 
-	pMenu = new wxMenu;
+	pMenu = make_unique<wxMenu>();
 	wxMenuItem* pItem = pMenu->Append(wxID_ANY, _("No sites available"));
 	pItem->Enable(false);
 	return pMenu;
@@ -294,60 +299,57 @@ wxMenu* CSiteManager::GetSitesMenu()
 
 void CSiteManager::ClearIdMap()
 {
-	for (std::map<int, CSiteManagerItemData_Site*>::iterator iter = m_idMap.begin(); iter != m_idMap.end(); ++iter)
-		delete iter->second;
-
 	m_idMap.clear();
 }
 
-wxMenu* CSiteManager::GetSitesMenu_Predefined(std::map<int, CSiteManagerItemData_Site*> &idMap)
+bool CSiteManager::LoadPredefined(CSiteManagerXmlHandler& handler)
 {
-	const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-	if (defaultsDir == _T(""))
-		return 0;
+	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+	if (defaultsDir.empty())
+		return false;
 
-	wxFileName name(defaultsDir, _T("fzdefaults.xml"));
+	wxString const name(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	CXmlFile file(name);
 
 	TiXmlElement* pDocument = file.Load();
 	if (!pDocument)
-		return 0;
+		return false;
 
 	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
 	if (!pElement)
-		return 0;
+		return false;
 
-	wxMenu* pMenu = new wxMenu;
-	CSiteManagerXmlHandler_Menu handler(pMenu, &idMap, true);
+	if (!Load(pElement, handler)) {
+		return false;
+	}
 
-	if (!Load(pElement, &handler))
-	{
-		delete pMenu;
+	return true;
+}
+
+std::unique_ptr<wxMenu> CSiteManager::GetSitesMenu_Predefined(std::map<int, std::unique_ptr<CSiteManagerItemData_Site>> &idMap)
+{
+	auto pMenu = make_unique<wxMenu>();
+	CSiteManagerXmlHandler_Menu handler(pMenu.get(), &idMap, true);
+
+	if (!LoadPredefined(handler)) {
 		return 0;
 	}
 
-	if (!pMenu->GetMenuItemCount())
-	{
-		delete pMenu;
+	if (!pMenu->GetMenuItemCount()) {
 		return 0;
 	}
 
 	return pMenu;
 }
 
-CSiteManagerItemData_Site* CSiteManager::GetSiteById(int id)
+std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteById(int id)
 {
-	std::map<int, CSiteManagerItemData_Site*>::iterator iter = m_idMap.find(id);
+	auto iter = m_idMap.find(id);
 
-	CSiteManagerItemData_Site *pData;
-	if (iter != m_idMap.end())
-	{
-		pData = iter->second;
-		iter->second = 0;
+	std::unique_ptr<CSiteManagerItemData_Site> pData;
+	if (iter != m_idMap.end()) {
+		pData = std::move(iter->second);
 	}
-	else
-		pData = 0;
-
 	ClearIdMap();
 
 	return pData;
@@ -358,7 +360,7 @@ bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
 	result.clear();
 
 	wxString name;
-	const wxChar *p = path;
+	wxChar const* p = path.c_str();
 
 	// Undo escapement
 	bool lastBackslash = false;
@@ -384,7 +386,7 @@ bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
 			}
 			else
 			{
-				if (!name.IsEmpty())
+				if (!name.empty())
 					result.push_back(name);
 				name.clear();
 			}
@@ -395,7 +397,7 @@ bool CSiteManager::UnescapeSitePath(wxString path, std::list<wxString>& result)
 	}
 	if (lastBackslash)
 		return false;
-	if (name != _T(""))
+	if (!name.empty())
 		result.push_back(name);
 
 	return !result.empty();
@@ -417,12 +419,11 @@ wxString CSiteManager::BuildPath(wxChar root, std::list<wxString> const& segment
 	return ret;
 }
 
-CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
+std::unique_ptr<CSiteManagerItemData_Site> CSiteManager::GetSiteByPath(wxString sitePath)
 {
-	wxChar c = sitePath[0];
-	if (c != '0' && c != '1')
-	{
-		wxMessageBox(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
+	wxChar c = sitePath.empty() ? 0 : sitePath[0];
+	if (c != '0' && c != '1') {
+		wxMessageBoxEx(_("Site path has to begin with 0 or 1."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -433,47 +434,41 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file;
-	TiXmlElement* pDocument = 0;
-
-	if (c == '0')
-		pDocument = file.Load(_T("sitemanager"));
-	else
-	{
-		const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-		if (defaultsDir == _T(""))
-		{
-			wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+	if (c == '0') {
+		file.SetFileName(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	}
+	else {
+		CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+		if (defaultsDir.empty()) {
+			wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 			return 0;
 		}
-		wxFileName name(defaultsDir, _T("fzdefaults.xml"));
-		pDocument = file.Load(name);
+		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	}
 
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
-
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return 0;
 	}
 
 	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
-	if (!pElement)
-	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+	if (!pElement) {
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
 	std::list<wxString> segments;
 	if (!UnescapeSitePath(sitePath, segments) || segments.empty())
 	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
 	if (!pChild)
 	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -487,16 +482,14 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 	else
 		pBookmark = 0;
 
-	CSiteManagerItemData_Site* data = ReadServerElement(pChild);
+	std::unique_ptr<CSiteManagerItemData_Site> data = ReadServerElement(pChild);
 
-	if (!data)
-	{
-		wxMessageBox(_("Could not read server item."), _("Invalid site path"));
+	if (!data) {
+		wxMessageBoxEx(_("Could not read server item."), _("Invalid site path"));
 		return 0;
 	}
 
-	if (pBookmark)
-	{
+	if (pBookmark) {
 		TiXmlHandle handle(pBookmark);
 
 		wxString localPath;
@@ -507,8 +500,7 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 		TiXmlText* remoteDir = handle.FirstChildElement("RemoteDir").FirstChild().Text();
 		if (remoteDir)
 			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
-		if (!localPath.empty() && !remotePath.IsEmpty())
-		{
+		if (!localPath.empty() && !remotePath.empty()) {
 			data->m_sync = GetTextElementBool(pBookmark, "SyncBrowsing", false);
 		}
 		else
@@ -525,6 +517,8 @@ CSiteManagerItemData_Site* CSiteManager::GetSiteByPath(wxString sitePath)
 
 bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmarks)
 {
+	if (sitePath.empty())
+		return false;
 	wxChar c = sitePath[0];
 	if (c != '0' && c != '1')
 		return false;
@@ -536,22 +530,21 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
 	CXmlFile file;
-	TiXmlElement* pDocument = 0;
-
-	if (c == '0')
-		pDocument = file.Load(_T("sitemanager"));
-	else
-	{
-		const wxString& defaultsDir = wxGetApp().GetDefaultsDir();
-		if (defaultsDir == _T(""))
-			return false;
-		pDocument = file.Load(wxFileName(defaultsDir, _T("fzdefaults.xml")));
+	if (c == '0') {
+		file.SetFileName(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	}
+	else {
+		CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+		if (defaultsDir.empty()) {
+			wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
+			return 0;
+		}
+		file.SetFileName(defaultsDir.GetPath() + _T("fzdefaults.xml"));
 	}
 
-	if (!pDocument)
-	{
-		wxMessageBox(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
-
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
+		wxMessageBoxEx(file.GetError(), _("Error loading xml file"), wxICON_ERROR);
 		return false;
 	}
 
@@ -560,20 +553,15 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		return false;
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
-	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+	if (!UnescapeSitePath(sitePath, segments)) {
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
 
-	TiXmlElement* pBookmark = 0;
 	if (pChild && !strcmp(pChild->Value(), "Bookmark"))
-	{
-		pBookmark = pChild;
 		pChild = pChild->Parent()->ToElement();
-	}
 
 	if (!pChild || strcmp(pChild->Value(), "Server"))
 		return 0;
@@ -596,7 +584,7 @@ bool CSiteManager::GetBookmarks(wxString sitePath, std::list<wxString> &bookmark
 		if (remoteDir)
 			remotePath.SetSafePath(ConvLocal(remoteDir->Value()));
 
-		if (localPath.empty() && remotePath.IsEmpty())
+		if (localPath.empty() && remotePath.empty())
 			continue;
 
 		bookmarks.push_back(name);
@@ -611,20 +599,18 @@ wxString CSiteManager::AddServer(CServer server)
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n") + _("The server could not be added.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
-		return _T("");
+		return wxString();
 	}
 
 	TiXmlElement* pElement = pDocument->FirstChildElement("Servers");
 	if (!pElement)
-		return _T("");
+		return wxString();
 
 	std::list<wxString> names;
 	for (TiXmlElement* pChild = pElement->FirstChildElement("Server"); pChild; pChild = pChild->NextSiblingElement("Server"))
@@ -658,22 +644,18 @@ wxString CSiteManager::AddServer(CServer server)
 	TiXmlElement* pServer = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
 	SetServer(pServer, server);
 
-	char* utf8 = ConvUTF8(name);
-	if (utf8)
-	{
+	wxScopedCharBuffer utf8 = name.utf8_str();
+	if (utf8) {
 		pServer->LinkEndChild(new TiXmlText(utf8));
-		delete [] utf8;
 	}
 
-	wxString error;
-	if (!file.Save(&error))
-	{
+	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
-			return _T("");
+			return wxString();
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
-		return _T("");
+		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
+		return wxString();
 	}
 
 	return _T("0/") + EscapeSegment(name);
@@ -684,7 +666,7 @@ TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxSt
 	for (std::list<wxString>::const_iterator it = segments.begin(); it != segments.end(); ++it)
 	{
 		const wxString & segment = *it;
-		
+
 		TiXmlElement* pChild;
 		for (pChild = pNode->FirstChildElement(); pChild; pChild = pChild->NextSiblingElement())
 		{
@@ -713,10 +695,11 @@ TiXmlElement* CSiteManager::GetElementByPath(TiXmlElement* pNode, std::list<wxSt
 
 bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wxString &local_dir, const CServerPath &remote_dir, bool sync)
 {
-	if (local_dir.empty() && remote_dir.IsEmpty())
+	if (local_dir.empty() && remote_dir.empty())
 		return false;
 
-	if (sitePath[0] != '0')
+	wxChar c = sitePath.empty() ? 0 : sitePath[0];
+	if (c != '0')
 		return false;
 
 	sitePath = sitePath.Mid(1);
@@ -725,13 +708,11 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement* pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n") + _("The bookmark could not be added.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -743,14 +724,14 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	std::list<wxString> segments;
 	if (!UnescapeSitePath(sitePath, segments))
 	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
 	if (!pChild || strcmp(pChild->Value(), "Server"))
 	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
@@ -767,7 +748,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 
 		if (name == old_name)
 		{
-			wxMessageBox(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(_("Name of bookmark already exists."), _("New bookmark"), wxICON_EXCLAMATION);
 			return false;
 		}
 		if (name < old_name && !pInsertBefore)
@@ -781,19 +762,17 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	AddTextElement(pBookmark, "Name", name);
 	if (!local_dir.empty())
 		AddTextElement(pBookmark, "LocalDir", local_dir);
-	if (!remote_dir.IsEmpty())
+	if (!remote_dir.empty())
 		AddTextElement(pBookmark, "RemoteDir", remote_dir.GetSafePath());
 	if (sync)
 		AddTextElementRaw(pBookmark, "SyncBrowsing", "1");
 
-	wxString error;
-	if (!file.Save(&error))
-	{
+	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 
 	return true;
@@ -801,7 +780,8 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 
 bool CSiteManager::ClearBookmarks(wxString sitePath)
 {
-	if (sitePath[0] != '0')
+	wxChar const c = sitePath.empty() ? 0 : sitePath[0];
+	if (c != '0')
 		return false;
 
 	sitePath = sitePath.Mid(1);
@@ -810,13 +790,11 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	// to the same file or one is reading while the other one writes.
 	CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-	CXmlFile file;
-	TiXmlElement* pDocument = file.Load(_T("sitemanager"));
-
-	if (!pDocument)
-	{
+	CXmlFile file(wxGetApp().GetSettingsFile(_T("sitemanager")));
+	TiXmlElement *pDocument = file.Load();
+	if (!pDocument) {
 		wxString msg = file.GetError() + _T("\n") + _("The bookmarks could not be cleared.");
-		wxMessageBox(msg, _("Error loading xml file"), wxICON_ERROR);
+		wxMessageBoxEx(msg, _("Error loading xml file"), wxICON_ERROR);
 
 		return false;
 	}
@@ -826,35 +804,38 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 		return false;
 
 	std::list<wxString> segments;
-	if (!UnescapeSitePath(sitePath, segments))
-	{
-		wxMessageBox(_("Site path is malformed."), _("Invalid site path"));
+	if (!UnescapeSitePath(sitePath, segments)) {
+		wxMessageBoxEx(_("Site path is malformed."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement* pChild = GetElementByPath(pElement, segments);
-	if (!pChild || strcmp(pChild->Value(), "Server"))
-	{
-		wxMessageBox(_("Site does not exist."), _("Invalid site path"));
+	if (!pChild || strcmp(pChild->Value(), "Server")) {
+		wxMessageBoxEx(_("Site does not exist."), _("Invalid site path"));
 		return 0;
 	}
 
 	TiXmlElement *pBookmark = pChild->FirstChildElement("Bookmark");
-	while (pBookmark)
-	{
+	while (pBookmark) {
 		pChild->RemoveChild(pBookmark);
 		pBookmark = pChild->FirstChildElement("Bookmark");
 	}
 
-	wxString error;
-	if (!file.Save(&error))
-	{
+	if (!file.Save(false)) {
 		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
-		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
-		wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
+		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName(), file.GetError());
+		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
 
 	return true;
 }
+
+bool CSiteManager::HasSites()
+{
+	CSiteManagerXmlHandler_Stats handler;
+	Load(handler);
+
+	return handler.sites_ > 0;
+}
