@@ -97,7 +97,7 @@ void CContextManager::RegisterHandler(CStateEventHandler* pHandler, enum t_state
 
 	std::list<t_handler> &handlers = m_handlers[notification];
 	std::list<t_handler>::const_iterator iter;
-	for (iter = handlers.begin(); iter != handlers.end(); iter++)
+	for (iter = handlers.begin(); iter != handlers.end(); ++iter)
 	{
 		if (iter->pHandler == pHandler)
 			return;
@@ -120,7 +120,7 @@ void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_sta
 		for (int i = 0; i < STATECHANGE_MAX; i++)
 		{
 			std::list<t_handler> &handlers = m_handlers[i];
-			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
 			{
 				if (iter->pHandler == pHandler)
 				{
@@ -133,7 +133,7 @@ void CContextManager::UnregisterHandler(CStateEventHandler* pHandler, enum t_sta
 	else
 	{
 		std::list<t_handler> &handlers = m_handlers[notification];
-		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
 		{
 			if (iter->pHandler == pHandler)
 			{
@@ -155,7 +155,7 @@ void CContextManager::NotifyHandlers(CState* pState, t_statechange_notifications
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	const std::list<t_handler> &handlers = m_handlers[notification];
-	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); ++iter)
 	{
 		if (blocked && iter->blockable)
 			continue;
@@ -184,7 +184,7 @@ void CContextManager::NotifyAllHandlers(enum t_statechange_notifications notific
 void CContextManager::NotifyGlobalHandlers(enum t_statechange_notifications notification, const wxString& data /*=_T("")*/, const void* data2 /*=0*/)
 {
 	const std::list<t_handler> &handlers = m_handlers[notification];
-	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); ++iter)
 		iter->pHandler->OnStateChange(0, notification, data, data2);
 }
 
@@ -217,9 +217,6 @@ CState::CState(CMainFrame* pMainFrame)
 CState::~CState()
 {
 	delete m_pServer;
-	
-	delete m_pCertificate;
-	delete m_pSftpEncryptionInfo;
 
 	delete m_pComparisonManager;
 	delete m_pCommandQueue;
@@ -228,7 +225,7 @@ CState::~CState()
 	// Unregister all handlers
 	for (int i = 0; i < STATECHANGE_MAX; i++)
 	{
-		for (std::list<t_handler>::iterator iter = m_handlers[i].begin(); iter != m_handlers[i].end(); iter++)
+		for (auto iter = m_handlers[i].begin(); iter != m_handlers[i].end(); ++iter)
 		{
 			iter->pHandler->m_pState = 0;
 		}
@@ -242,14 +239,9 @@ CLocalPath CState::GetLocalDir() const
 	return m_localDir;
 }
 
-bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
-{
-	if (m_sync_browse.is_changing)
-	{
-		wxMessageBox(_T("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
-		return false;
-	}
 
+bool CState::SetLocalDir(const wxString& dir, wxString *error, bool rememberPreviousSubdir)
+{
 	CLocalPath p(m_localDir);
 #ifdef __WXMSW__
 	if (dir == _T("..") && !p.HasParent() && p.HasLogicalParent())
@@ -263,38 +255,43 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
 	if (!p.ChangePath(dir))
 		return false;
 
-	if (!p.Exists(error))
+	return SetLocalDir(p, error, rememberPreviousSubdir);
+}
+
+bool CState::SetLocalDir(CLocalPath const& dir, wxString *error, bool rememberPreviousSubdir)
+{
+	if (m_sync_browse.is_changing) {
+		wxMessageBoxEx(_T("Cannot change directory, there already is a synchronized browsing operation in progress."), _("Synchronized browsing"));
+		return false;
+	}
+
+	if (!dir.Exists(error))
 		return false;
 
-	if (!m_sync_browse.local_root.empty())
-	{
+	if (!m_sync_browse.local_root.empty()) {
 		wxASSERT(m_pServer);
-		
-		if (p != m_sync_browse.local_root && !p.IsSubdirOf(m_sync_browse.local_root))
-		{
+
+		if (dir != m_sync_browse.local_root && !dir.IsSubdirOf(m_sync_browse.local_root)) {
 			wxString msg = wxString::Format(_("The local directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the local directory?"),
-					p.GetPath().c_str(),
-					m_sync_browse.local_root.GetPath().c_str());
-			if (wxMessageBox(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+					dir.GetPath(),
+					m_sync_browse.local_root.GetPath());
+			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 				return false;
 			SetSyncBrowse(false);
 		}
-		else if (!IsRemoteIdle())
-		{
+		else if (!IsRemoteIdle()) {
 			wxString msg(_("A remote operation is in progress and synchronized browsing is enabled.\nDisable synchronized browsing and continue changing the local directory?"));
-			if (wxMessageBox(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 				return false;
 			SetSyncBrowse(false);
 		}
-		else
-		{
-			CServerPath remote_path = GetSynchronizedDirectory(p);
-			if (remote_path.IsEmpty())
-			{
+		else {
+			CServerPath remote_path = GetSynchronizedDirectory(dir);
+			if (remote_path.empty()) {
 				SetSyncBrowse(false);
 				wxString msg = wxString::Format(_("Could not obtain corresponding remote directory for the local directory '%s'.\nSynchronized browsing has been disabled."),
-					p.GetPath().c_str());
-				wxMessageBox(msg, _("Synchronized browsing"));
+					dir.GetPath());
+				wxMessageBoxEx(msg, _("Synchronized browsing"));
 				return false;
 			}
 
@@ -306,8 +303,22 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
 			return true;
 		}
 	}
-	
-	m_localDir = p;
+
+	if (dir == m_localDir.GetParent() && rememberPreviousSubdir) {
+#ifdef __WXMSW__
+		if (dir.GetPath() == _T("\\")) {
+			m_previouslyVisitedLocalSubdir = m_localDir.GetPath();
+			m_previouslyVisitedLocalSubdir.RemoveLast();
+		}
+		else
+#endif
+			m_previouslyVisitedLocalSubdir = m_localDir.GetLastSegment();
+	}
+	else
+		m_previouslyVisitedLocalSubdir = _T("");
+
+
+	m_localDir = dir;
 
 	COptions::Get()->SetOption(OPTION_LASTLOCALDIR, m_localDir.GetPath());
 
@@ -316,10 +327,9 @@ bool CState::SetLocalDir(const wxString& dir, wxString *error /*=0*/)
 	return true;
 }
 
-bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modified /*=false*/)
+bool CState::SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool modified)
 {
-	if (!pDirectoryListing)
-	{
+	if (!pDirectoryListing) {
 		SetSyncBrowse(false);
 		if (modified)
 			return false;
@@ -329,17 +339,21 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 			m_pDirectoryListing = 0;
 			NotifyHandlers(STATECHANGE_REMOTE_DIR);
 		}
+		m_previouslyVisitedRemoteSubdir = _T("");
 		return true;
 	}
 
 	wxASSERT(pDirectoryListing->m_firstListTime.IsValid());
 
-	if (modified)
-	{
-		if (!m_pDirectoryListing || m_pDirectoryListing->path != pDirectoryListing->path)
-		{
+	if (pDirectoryListing && m_pDirectoryListing &&
+		pDirectoryListing->path == m_pDirectoryListing->path.GetParent())
+		m_previouslyVisitedRemoteSubdir = m_pDirectoryListing->path.GetLastSegment();
+	else
+		m_previouslyVisitedRemoteSubdir = _T("");
+
+	if (modified) {
+		if (!m_pDirectoryListing || m_pDirectoryListing->path != pDirectoryListing->path) {
 			// We aren't interested in these listings
-			delete pDirectoryListing;
 			return true;
 		}
 	}
@@ -347,10 +361,9 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 		m_last_path = pDirectoryListing->path;
 
 	if (m_pDirectoryListing && m_pDirectoryListing->path == pDirectoryListing->path &&
-        pDirectoryListing->m_failed)
+		pDirectoryListing->failed())
 	{
 		// We still got an old listing, no need to display the new one
-		delete pDirectoryListing;
 		return true;
 	}
 
@@ -361,35 +374,31 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 	else
 		NotifyHandlers(STATECHANGE_REMOTE_DIR_MODIFIED);
 
-	if (m_sync_browse.is_changing && !modified)
-	{
+	if (m_sync_browse.is_changing && !modified) {
 		m_sync_browse.is_changing = false;
 		if (m_pDirectoryListing->path != m_sync_browse.remote_root && !m_pDirectoryListing->path.IsSubdirOf(m_sync_browse.remote_root, false))
 		{
 			SetSyncBrowse(false);
 			wxString msg = wxString::Format(_("Current remote directory (%s) is not below the synchronization root (%s).\nSynchronized browsing has been disabled."),
-					m_pDirectoryListing->path.GetPath().c_str(),
-					m_sync_browse.remote_root.GetPath().c_str());
-			wxMessageBox(msg, _("Synchronized browsing"));
+					m_pDirectoryListing->path.GetPath(),
+					m_sync_browse.remote_root.GetPath());
+			wxMessageBoxEx(msg, _("Synchronized browsing"));
 		}
-		else
-		{
+		else {
 			CLocalPath local_path = GetSynchronizedDirectory(m_pDirectoryListing->path);
-			if (local_path.empty())
-			{
+			if (local_path.empty()) {
 				SetSyncBrowse(false);
 				wxString msg = wxString::Format(_("Could not obtain corresponding local directory for the remote directory '%s'.\nSynchronized browsing has been disabled."),
-					m_pDirectoryListing->path.GetPath().c_str());
-				wxMessageBox(msg, _("Synchronized browsing"));
+					m_pDirectoryListing->path.GetPath());
+				wxMessageBoxEx(msg, _("Synchronized browsing"));
 				return true;
 			}
 
 			wxString error;
-			if (!local_path.Exists(&error))
-			{
+			if (!local_path.Exists(&error)) {
 				SetSyncBrowse(false);
 				wxString msg = error + _T("\n") + _("Synchronized browsing has been disabled.");
-				wxMessageBox(msg, _("Synchronized browsing"));
+				wxMessageBoxEx(msg, _("Synchronized browsing"));
 				return true;
 			}
 
@@ -406,7 +415,7 @@ bool CState::SetRemoteDir(const CDirectoryListing *pDirectoryListing, bool modif
 	return true;
 }
 
-CSharedPointer<const CDirectoryListing> CState::GetRemoteDir() const
+std::shared_ptr<CDirectoryListing> CState::GetRemoteDir() const
 {
 	return m_pDirectoryListing;
 }
@@ -467,8 +476,7 @@ void CState::LocalDirCreated(const CLocalPath& path)
 
 void CState::SetServer(const CServer* server)
 {
-	if (m_pServer)
-	{
+	if (m_pServer) {
 		if (server && *server == *m_pServer &&
 			server->GetName() == m_pServer->GetName() &&
 			server->MaximumMultipleConnections() == m_pServer->MaximumMultipleConnections())
@@ -479,27 +487,23 @@ void CState::SetServer(const CServer* server)
 
 		SetRemoteDir(0);
 		delete m_pServer;
-		delete m_pCertificate;
-		m_pCertificate = 0;
-		delete m_pSftpEncryptionInfo;
-		m_pSftpEncryptionInfo = 0;
+		m_pCertificate.reset();
+		m_pSftpEncryptionInfo.reset();
 	}
-	if (server)
-	{
+	if (server) {
 		if (m_last_server != *server)
-			m_last_path.Clear();
+			m_last_path.clear();
 		m_last_server = *server;
 
 		m_pServer = new CServer(*server);
 
 		const wxString& name = server->GetName();
-		if (!name.IsEmpty())
+		if (!name.empty())
 			m_title = name + _T(" - ") + server->FormatServer();
 		else
 			m_title = server->FormatServer();
 	}
-	else
-	{
+	else {
 		m_pServer = 0;
 		m_title = _("Not connected");
 	}
@@ -543,7 +547,7 @@ bool CState::Disconnect()
 
 	if (!IsRemoteConnected())
 		return true;
-	
+
 	if (!IsRemoteIdle())
 		return false;
 
@@ -559,8 +563,8 @@ bool CState::CreateEngine()
 	if (m_pEngine)
 		return true;
 
-	m_pEngine = new CFileZillaEngine();
-	m_pEngine->Init(m_pMainFrame, COptions::Get());
+	m_pEngine = new CFileZillaEngine(m_pMainFrame->GetEngineContext());
+	m_pEngine->Init(m_pMainFrame);
 
 	m_pCommandQueue = new CCommandQueue(m_pEngine, m_pMainFrame, this);
 
@@ -585,7 +589,7 @@ void CState::RegisterHandler(CStateEventHandler* pHandler, enum t_statechange_no
 
 	std::list<t_handler> &handlers = m_handlers[notification];
 	std::list<t_handler>::const_iterator iter;
-	for (iter = handlers.begin(); iter != handlers.end(); iter++)
+	for (iter = handlers.begin(); iter != handlers.end(); ++iter)
 	{
 		if (iter->pHandler == pHandler)
 			return;
@@ -607,7 +611,7 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 		for (int i = 0; i < STATECHANGE_MAX; i++)
 		{
 			std::list<t_handler> &handlers = m_handlers[i];
-			for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+			for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
 			{
 				if (iter->pHandler == pHandler)
 				{
@@ -620,7 +624,7 @@ void CState::UnregisterHandler(CStateEventHandler* pHandler, enum t_statechange_
 	else
 	{
 		std::list<t_handler> &handlers = m_handlers[notification];
-		for (std::list<t_handler>::iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+		for (auto iter = handlers.begin(); iter != handlers.end(); ++iter)
 		{
 			if (iter->pHandler == pHandler)
 			{
@@ -663,7 +667,7 @@ void CState::NotifyHandlers(enum t_statechange_notifications notification, const
 	wxASSERT(notification != STATECHANGE_NONE && notification != STATECHANGE_MAX);
 
 	const std::list<t_handler> &handlers = m_handlers[notification];
-	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); iter++)
+	for (std::list<t_handler>::const_iterator iter = handlers.begin(); iter != handlers.end(); ++iter)
 	{
 		if (m_blocked[notification] && iter->blockable)
 			continue;
@@ -684,7 +688,7 @@ CStateEventHandler::~CStateEventHandler()
 	CContextManager::Get()->UnregisterHandler(this, STATECHANGE_NONE);
 
 	const std::vector<CState*> *states = CContextManager::Get()->GetAllStates();
-	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); iter++)
+	for (std::vector<CState*>::const_iterator iter = states->begin(); iter != states->end(); ++iter)
 		(*iter)->UnregisterHandler(this, STATECHANGE_NONE);
 }
 
@@ -696,7 +700,7 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const w
 	CServerPath path = m_pDirectoryListing->path;
 	if (subdir == _T("..") && path.HasParent())
 		path = path.GetParent();
-	else if (subdir != _T(""))
+	else if (!subdir.empty())
 		path.AddSegment(subdir);
 
 	UploadDroppedFiles(pFileDataObject, path, queueOnly);
@@ -711,14 +715,14 @@ void CState::UploadDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 
 	for (unsigned int i = 0; i < files.Count(); i++)
 	{
-		wxLongLong size;
+		int64_t size;
 		bool is_link;
 		CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(files[i], is_link, &size, 0, 0);
 		if (type == CLocalFileSystem::file)
 		{
 			wxString localFile;
 			const CLocalPath localPath(files[i], &localFile);
-			m_pMainFrame->GetQueue()->QueueFile(queueOnly, false, localPath, localFile, localFile, path, *m_pServer, size);
+			m_pMainFrame->GetQueue()->QueueFile(queueOnly, false, localFile, wxEmptyString, localPath, path, *m_pServer, size);
 			m_pMainFrame->GetQueue()->QueueFile_Finish(!queueOnly);
 		}
 		else if (type == CLocalFileSystem::dir)
@@ -771,11 +775,12 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 	delete [] to;
 	delete [] from;
 #else
+	wxString error;
 	for (unsigned int i = 0; i < files.Count(); i++)
 	{
 		const wxString& file(files[i]);
 
-		wxLongLong size;
+		int64_t size;
 		bool is_link;
 		CLocalFileSystem::local_fileType type = CLocalFileSystem::GetFileInfo(file, is_link, &size, 0, 0);
 		if (type == CLocalFileSystem::file)
@@ -784,24 +789,38 @@ void CState::HandleDroppedFiles(const wxFileDataObject* pFileDataObject, const C
 			CLocalPath sourcePath(file, &name);
 			if (name.empty())
 				continue;
+			wxString target = path.GetPath() + name;
+			if (file == target)
+				continue;
+
 			if (copy)
-				wxCopyFile(file, path.GetPath() + name);
+				wxCopyFile(file, target);
 			else
-				wxRenameFile(file, path.GetPath() + name);
+				wxRenameFile(file, target);
 		}
 		else if (type == CLocalFileSystem::dir)
 		{
+			CLocalPath sourcePath(file);
+			if (sourcePath == path || sourcePath.GetParent() == path)
+				continue;
+			if (sourcePath.IsParentOf(path))
+			{
+				error = _("A directory cannot be dragged into one of its subdirectories.");
+				continue;
+			}
+
 			if (copy)
-				RecursiveCopy(CLocalPath(file), path);
+				RecursiveCopy(sourcePath, path);
 			else
 			{
-				CLocalPath sourcePath(file);
 				if (!sourcePath.HasParent())
 					continue;
 				wxRenameFile(file, path.GetPath() + sourcePath.GetLastSegment());
 			}
 		}
 	}
+	if (!error.empty())
+		wxMessageBoxEx(error, _("Could not complete operation"));
 #endif
 
 	RefreshLocal();
@@ -843,7 +862,7 @@ bool CState::RecursiveCopy(CLocalPath source, const CLocalPath& target)
 		wxString file;
 		while (fs.GetNextFile(file, is_link, is_dir, 0, 0, 0))
 		{
-			if (file == _T(""))
+			if (file.empty())
 			{
 				wxGetApp().DisplayEncodingWarning();
 				continue;
@@ -870,7 +889,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	bool hasDirs = false;
 	bool hasFiles = false;
 	const std::list<CRemoteDataObject::t_fileInfo>& files = pRemoteDataObject->GetFiles();
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 	{
 		if (iter->dir)
 			hasDirs = true;
@@ -890,7 +909,7 @@ bool CState::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, co
 	if (!hasDirs)
 		return true;
 
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
+	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); ++iter)
 	{
 		if (!iter->dir)
 			continue;
@@ -928,64 +947,6 @@ bool CState::IsRemoteIdle() const
 	return m_pCommandQueue->Idle();
 }
 
-wxString CState::GetAsURL(const wxString& dir)
-{
-	// Cheap URL encode
-	wxString encoded;
-	const wxWX2MBbuf utf8 = dir.mb_str();
-
-	const char* p = utf8;
-	while (*p)
-	{
-		// List of characters that don't need to be escaped taken
-		// from the BNF grammar in RFC 1738
-		// Again attention seeking Windows wants special treatment...
-		const unsigned char c = (unsigned char)*p++;
-		if ((c >= 'a' && c <= 'z') ||
-			(c >= 'A' && c <= 'Z') ||
-			(c >= '0' && c <= '9') ||
-			c == '$' ||
-			c == '_' ||
-			c == '-' ||
-			c == '.' ||
-			c == '+' ||
-			c == '!' ||
-			c == '*' ||
-#ifndef __WXMSW__
-			c == '\'' ||
-#endif
-			c == '(' ||
-			c == ')' ||
-			c == ',' ||
-			c == '?' ||
-			c == ':' ||
-			c == '@' ||
-			c == '&' ||
-			c == '=' ||
-			c == '/')
-		{
-			encoded += (wxChar)c;
-		}
-#ifdef __WXMSW__
-		else if (c == '\\')
-			encoded += '/';
-#endif
-		else
-			encoded += wxString::Format(_T("%%%x"), (unsigned int)c);
-	}
-#ifdef __WXMSW__
-	if (encoded.Left(2) == _T("//"))
-	{
-		// UNC path
-		encoded = encoded.Mid(2);
-	}
-	else
-		encoded = _T("/") + encoded;
-#endif
-
-	return _T("file://") + encoded;
-}
-
 void CState::ListingFailed(int error)
 {
 	m_sync_browse.is_changing = false;
@@ -1013,23 +974,23 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 		if (!subdir.empty() && !p.ChangePath(subdir))
 		{
 			wxString msg = wxString::Format(_("Could not get full remote path."));
-			wxMessageBox(msg, _("Synchronized browsing"));
+			wxMessageBoxEx(msg, _("Synchronized browsing"));
 			return false;
 		}
 
 		if (p != m_sync_browse.remote_root && !p.IsSubdirOf(m_sync_browse.remote_root, false))
 		{
 			wxString msg = wxString::Format(_("The remote directory '%s' is not below the synchronization root (%s).\nDisable synchronized browsing and continue changing the remote directory?"),
-					p.GetPath().c_str(),
-					m_sync_browse.remote_root.GetPath().c_str());
-			if (wxMessageBox(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+					p.GetPath(),
+					m_sync_browse.remote_root.GetPath());
+			if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 				return false;
 			SetSyncBrowse(false);
 		}
 		else if (!IsRemoteIdle() && !ignore_busy)
 		{
 			wxString msg(_("Another remote operation is already in progress, cannot change directory now."));
-			wxMessageBox(msg, _("Synchronized browsing"), wxICON_EXCLAMATION);
+			wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_EXCLAMATION);
 			return false;
 		}
 		else
@@ -1039,15 +1000,15 @@ bool CState::ChangeRemoteDir(const CServerPath& path, const wxString& subdir /*=
 			if (local_path.empty())
 			{
 				wxString msg = wxString::Format(_("Could not obtain corresponding local directory for the remote directory '%s'.\nDisable synchronized browsing and continue changing the remote directory?"),
-					p.GetPath().c_str());
-				if (wxMessageBox(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+					p.GetPath());
+				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 					return false;
 				SetSyncBrowse(false);
 			}
 			else if (!local_path.Exists(&error))
 			{
 				wxString msg = error + _T("\n") + _("Disable synchronized browsing and continue changing the remote directory?");
-				if (wxMessageBox(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
+				if (wxMessageBoxEx(msg, _("Synchronized browsing"), wxICON_QUESTION | wxYES_NO) != wxYES)
 					return false;
 				SetSyncBrowse(false);
 			}
@@ -1072,16 +1033,16 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 
 	if (!enable)
 	{
-		wxASSERT(assumed_remote_root.IsEmpty());
+		wxASSERT(assumed_remote_root.empty());
 		m_sync_browse.local_root.clear();
-		m_sync_browse.remote_root.Clear();
+		m_sync_browse.remote_root.clear();
 		m_sync_browse.is_changing = false;
 
 		NotifyHandlers(STATECHANGE_SYNC_BROWSE);
 		return false;
 	}
 
-	if (!m_pDirectoryListing && assumed_remote_root.IsEmpty())
+	if (!m_pDirectoryListing && assumed_remote_root.empty())
 	{
 		NotifyHandlers(STATECHANGE_SYNC_BROWSE);
 		return false;
@@ -1090,7 +1051,7 @@ bool CState::SetSyncBrowse(bool enable, const CServerPath& assumed_remote_root /
 	m_sync_browse.is_changing = false;
 	m_sync_browse.local_root = m_localDir;
 
-	if (assumed_remote_root.IsEmpty())
+	if (assumed_remote_root.empty())
 		m_sync_browse.remote_root = m_pDirectoryListing->path;
 	else
 	{
@@ -1122,7 +1083,7 @@ CLocalPath CState::GetSynchronizedDirectory(CServerPath remote_path)
 		return CLocalPath();
 
 	CLocalPath local_path = m_sync_browse.local_root;
-	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); iter++)
+	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
 		local_path.AddSegment(*iter);
 
 	return local_path;
@@ -1142,7 +1103,7 @@ CServerPath CState::GetSynchronizedDirectory(CLocalPath local_path)
 		return CServerPath();
 
 	CServerPath remote_path = m_sync_browse.remote_root;
-	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); iter++)
+	for (std::list<wxString>::const_iterator iter = segments.begin(); iter != segments.end(); ++iter)
 		remote_path.AddSegment(*iter);
 
 	return remote_path;
@@ -1152,7 +1113,7 @@ bool CState::RefreshRemote()
 {
 	if (!m_pCommandQueue)
 		return false;
-	
+
 	if (!IsRemoteConnected() || !IsRemoteIdle())
 		return false;
 
@@ -1161,30 +1122,26 @@ bool CState::RefreshRemote()
 
 bool CState::GetSecurityInfo(CCertificateNotification *& pInfo)
 {
-	pInfo = m_pCertificate;
+	pInfo = m_pCertificate.get();
 	return m_pCertificate != 0;
 }
 
 bool CState::GetSecurityInfo(CSftpEncryptionNotification *& pInfo)
 {
-	pInfo = m_pSftpEncryptionInfo;
+	pInfo = m_pSftpEncryptionInfo.get();
 	return m_pSftpEncryptionInfo != 0;
 }
 
 void CState::SetSecurityInfo(CCertificateNotification const& info)
 {
-	delete m_pCertificate;
-	m_pCertificate = 0;
-	delete m_pSftpEncryptionInfo;
-	m_pSftpEncryptionInfo = 0;
-	m_pCertificate = new CCertificateNotification(info);
+	m_pSftpEncryptionInfo.reset();
+	m_pCertificate = make_unique<CCertificateNotification>(info);
+	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
 
 void CState::SetSecurityInfo(CSftpEncryptionNotification const& info)
 {
-	delete m_pCertificate;
-	m_pCertificate = 0;
-	delete m_pSftpEncryptionInfo;
-	m_pSftpEncryptionInfo = 0;
-	m_pSftpEncryptionInfo = new CSftpEncryptionNotification(info);
+	m_pCertificate.reset();
+	m_pSftpEncryptionInfo = make_unique<CSftpEncryptionNotification>(info);
+	NotifyHandlers(STATECHANGE_ENCRYPTION);
 }
