@@ -4,10 +4,13 @@
 #include "sizeformatting.h"
 #include "speedlimits_dialog.h"
 #include "statusbar.h"
+#include "themeprovider.h"
 #include "verifycertdialog.h"
 
+#include <wx/dcclient.h>
+
 static const int statbarWidths[3] = {
-	-3, 0, 35
+	-3, 0, 25
 };
 #define FIELD_QUEUESIZE 1
 
@@ -40,7 +43,7 @@ void wxStatusBarEx::SetFieldsCount(int number /*=1*/, const int* widths /*=NULL*
 
 	int oldCount = GetFieldsCount();
 	int* oldWidths = m_columnWidths;
-	
+
 	m_columnWidths = new int[number];
 	if (!widths)
 	{
@@ -88,11 +91,7 @@ void wxStatusBarEx::FixupFieldWidth(int field)
 	if (field != GetFieldsCount() - 1)
 		return;
 
-#ifdef __WXMSW__
-	// Gripper overlaps last field if not maximized
-	if (!m_parentWasMaximized && m_columnWidths[field] > 0)
-		m_columnWidths[field] += 6;
-#elif __WXGTK20__
+#if __WXGTK20__
 	// Gripper overlaps last all the time
 	if (m_columnWidths[field] > 0)
 		m_columnWidths[field] += 15;
@@ -102,29 +101,29 @@ void wxStatusBarEx::FixupFieldWidth(int field)
 void wxStatusBarEx::SetFieldWidth(int field, int width)
 {
 	field = GetFieldIndex(field);
+	if( field < 0 ) {
+		return;
+	}
+
 	m_columnWidths[field] = width;
-
 	FixupFieldWidth(field);
-
 	wxStatusBar::SetStatusWidths(GetFieldsCount(), m_columnWidths);
 }
 
 int wxStatusBarEx::GetFieldIndex(int field)
 {
-	if (field >= 0)
-	{
-		wxASSERT(field <= GetFieldsCount());
+	if (field >= 0) {
+		wxCHECK(field <= GetFieldsCount(), -1);
 	}
-	else
-	{
+	else {
 		field = GetFieldsCount() + field;
-		wxASSERT(field >= 0);
+		wxCHECK(field >= 0, -1);
 	}
 
 	return field;
 }
 
-void wxStatusBarEx::OnSize(wxSizeEvent& event)
+void wxStatusBarEx::OnSize(wxSizeEvent&)
 {
 #ifdef __WXMSW__
 	const int count = GetFieldsCount();
@@ -137,9 +136,9 @@ void wxStatusBarEx::OnSize(wxSizeEvent& event)
 			m_parentWasMaximized = isMaximized;
 
 			if (isMaximized)
-				m_columnWidths[count - 1] -= 6;
+				m_columnWidths[count - 1] -= 16;
 			else
-				m_columnWidths[count - 1] += 6;
+				m_columnWidths[count - 1] += 16;
 
 			wxStatusBar::SetStatusWidths(count, m_columnWidths);
 			Refresh();
@@ -151,16 +150,9 @@ void wxStatusBarEx::OnSize(wxSizeEvent& event)
 #ifdef __WXGTK__
 void wxStatusBarEx::SetStatusText(const wxString& text, int number /*=0*/)
 {
-	// Basically identical to the wx one, but not calling Update
-	wxString oldText = m_statusStrings[number];
-	if (oldText != text)
-	{
-		m_statusStrings[number] = text;
-
-		wxRect rect;
-		GetFieldRect(number, rect);
-
-		Refresh(true, &rect);
+	wxString oldText = GetStatusText(number);
+	if (oldText != text) {
+		wxStatusBar::SetStatusText(text, number);
 	}
 }
 #endif
@@ -204,9 +196,12 @@ void CWidgetsStatusBar::OnSize(wxSizeEvent& event)
 #endif
 }
 
-void CWidgetsStatusBar::AddChild(int field, int idx, wxWindow* pChild)
+bool CWidgetsStatusBar::AddField(int field, int idx, wxWindow* pChild)
 {
 	field = GetFieldIndex(field);
+	if( field < 0 ) {
+		return false;
+	}
 
 	t_statbar_child data;
 	data.field = field;
@@ -215,13 +210,14 @@ void CWidgetsStatusBar::AddChild(int field, int idx, wxWindow* pChild)
 	m_children[idx] = data;
 
 	PositionChildren(field);
+
+	return true;
 }
 
-void CWidgetsStatusBar::RemoveChild(int idx)
+void CWidgetsStatusBar::RemoveField(int idx)
 {
-	std::map<int, struct t_statbar_child>::iterator iter = m_children.find(idx);
-	if (iter != m_children.end())
-	{
+	auto iter = m_children.find(idx);
+	if (iter != m_children.end()) {
 		int field = iter->second.field;
 		m_children.erase(iter);
 		PositionChildren(field);
@@ -232,16 +228,18 @@ void CWidgetsStatusBar::PositionChildren(int field)
 {
 	wxRect rect;
 	GetFieldRect(field, rect);
-	
+
 	int offset = 2;
 
+#ifndef __WXMSW__
 	if (field + 1 == GetFieldsCount())
 	{
-		rect.SetWidth(m_columnWidths[field]);	
+		rect.SetWidth(m_columnWidths[field]);
 		offset += 5 + GetGripperWidth();
 	}
+#endif
 
-	for (std::map<int, struct t_statbar_child>::iterator iter = m_children.begin(); iter != m_children.end(); iter++)
+	for (auto iter = m_children.begin(); iter != m_children.end(); ++iter)
 	{
 		if (iter->second.field != field)
 			continue;
@@ -262,44 +260,11 @@ void CWidgetsStatusBar::SetFieldWidth(int field, int width)
 		PositionChildren(i);
 }
 
-#ifdef __WXMSW__
-class wxStaticBitmapEx : public wxStaticBitmap
-{
-public:
-	wxStaticBitmapEx(wxWindow* parent, int id, const wxBitmap& bmp)
-		: wxStaticBitmap(parent, id, bmp)
-	{
-	};
-
-protected:
-	DECLARE_EVENT_TABLE();
-
-	// Make sure it is truly transparent, i.e. also works with
-	// themed status bars.
-	void OnErase(wxEraseEvent& event)
-	{
-	}
-
-	void OnPaint(wxPaintEvent& event)
-	{
-		wxPaintDC dc(this);
-		dc.DrawBitmap(GetBitmap(), 0, 0, true);
-	}
-};
-
-BEGIN_EVENT_TABLE(wxStaticBitmapEx, wxStaticBitmap)
-EVT_ERASE_BACKGROUND(wxStaticBitmapEx::OnErase)
-EVT_PAINT(wxStaticBitmapEx::OnPaint)
-END_EVENT_TABLE()
-#else
-#define wxStaticBitmapEx wxStaticBitmap
-#endif
-
-class CIndicator : public wxStaticBitmapEx
+class CIndicator : public wxStaticBitmap
 {
 public:
 	CIndicator(CStatusBar* pStatusBar, const wxBitmap& bmp)
-		: wxStaticBitmapEx(pStatusBar, wxID_ANY, bmp)
+		: wxStaticBitmap(pStatusBar, wxID_ANY, bmp)
 	{
 		m_pStatusBar = pStatusBar;
 	}
@@ -308,17 +273,17 @@ protected:
 	CStatusBar* m_pStatusBar;
 
 	DECLARE_EVENT_TABLE()
-	void OnLeftMouseUp(wxMouseEvent& event)
+	void OnLeftMouseUp(wxMouseEvent&)
 	{
 		m_pStatusBar->OnHandleLeftClick(this);
 	}
-	void OnRightMouseUp(wxMouseEvent& event)
+	void OnRightMouseUp(wxMouseEvent&)
 	{
 		m_pStatusBar->OnHandleRightClick(this);
 	}
 };
 
-BEGIN_EVENT_TABLE(CIndicator, wxStaticBitmapEx)
+BEGIN_EVENT_TABLE(CIndicator, wxStaticBitmap)
 EVT_LEFT_UP(CIndicator::OnLeftMouseUp)
 EVT_RIGHT_UP(CIndicator::OnRightMouseUp)
 END_EVENT_TABLE()
@@ -348,6 +313,7 @@ CStatusBar::CStatusBar(wxTopLevelWindow* pParent)
 
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_SERVER, true, false);
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_CHANGEDCONTEXT, false, false);
+	CContextManager::Get()->RegisterHandler(this, STATECHANGE_ENCRYPTION, true, false);
 
 	m_pDataTypeIndicator = 0;
 	m_pEncryptionIndicator = 0;
@@ -377,7 +343,7 @@ CStatusBar::~CStatusBar()
 {
 }
 
-void CStatusBar::DisplayQueueSize(wxLongLong totalSize, bool hasUnknown)
+void CStatusBar::DisplayQueueSize(int64_t totalSize, bool hasUnknown)
 {
 	m_size = totalSize;
 	m_hasUnknownFiles = hasUnknown;
@@ -389,7 +355,7 @@ void CStatusBar::DisplayQueueSize(wxLongLong totalSize, bool hasUnknown)
 	}
 
 	wxString queueSize = wxString::Format(_("Queue: %s%s"), hasUnknown ? _T(">") : _T(""),
-		CSizeFormat::Format(totalSize, true, m_sizeFormat, m_sizeFormatThousandsSep, m_sizeFormatDecimalPlaces).c_str());
+		CSizeFormat::Format(totalSize, true, m_sizeFormat, m_sizeFormatThousandsSep, m_sizeFormatDecimalPlaces));
 
 	SetStatusText(queueSize, FIELD_QUEUESIZE);
 }
@@ -405,7 +371,7 @@ void CStatusBar::DisplayDataType()
 	{
 		if (m_pDataTypeIndicator)
 		{
-			RemoveChild(widget_datatype);
+			RemoveField(widget_datatype);
 			m_pDataTypeIndicator->Destroy();
 			m_pDataTypeIndicator = 0;
 		}
@@ -432,11 +398,11 @@ void CStatusBar::DisplayDataType()
 			desc = _("Current transfer type is set to automatic detection.");
 		}
 
-		wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_OTHER, wxSize(16, 16));
+		wxBitmap bmp = wxArtProvider::GetBitmap(name, wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
 		if (!m_pDataTypeIndicator)
 		{
 			m_pDataTypeIndicator = new CIndicator(this, bmp);
-			AddChild(0, widget_datatype, m_pDataTypeIndicator);
+			AddField(0, widget_datatype, m_pDataTypeIndicator);
 		}
 		else
 			m_pDataTypeIndicator->SetBitmap(bmp);
@@ -450,7 +416,7 @@ void CStatusBar::MeasureQueueSizeWidth()
 	dc.SetFont(GetFont());
 
 	wxSize s = dc.GetTextExtent(_("Queue: empty"));
-	
+
 	wxString tmp = _T(">8888");
 	if (m_sizeFormatDecimalPlaces)
 	{
@@ -458,7 +424,7 @@ void CStatusBar::MeasureQueueSizeWidth()
 		for (int i = 0; i < m_sizeFormatDecimalPlaces; i++)
 			tmp += _T("8");
 	}
-	s.IncTo(dc.GetTextExtent(wxString::Format(_("Queue: %s MiB"), tmp.c_str())));
+	s.IncTo(dc.GetTextExtent(wxString::Format(_("Queue: %s MiB"), tmp)));
 
 	SetFieldWidth(FIELD_QUEUESIZE, s.x + 10);
 }
@@ -466,26 +432,33 @@ void CStatusBar::MeasureQueueSizeWidth()
 void CStatusBar::DisplayEncrypted()
 {
 	const CServer* pServer = 0;
-	const CState* pState = CContextManager::Get()->GetCurrentContext();
+	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (pState)
 		pServer = pState->GetServer();
 
-	if (!pServer || (pServer->GetProtocol() != FTPS && pServer->GetProtocol() != FTPES && pServer->GetProtocol() != SFTP))
-	{
-		if (m_pEncryptionIndicator)
-		{
-			RemoveChild(widget_encryption);
+	bool encrypted = false;
+	if (pServer) {
+		CCertificateNotification* info;
+		if (pServer->GetProtocol() == FTPS || pServer->GetProtocol() == FTPES || pServer->GetProtocol() == SFTP) {
+			encrypted = true;
+		}
+		else if (pServer->GetProtocol() == FTP && pState->GetSecurityInfo(info)) {
+			encrypted = true;
+		}
+	}
+
+	if (!encrypted) {
+		if (m_pEncryptionIndicator) {
+			RemoveField(widget_encryption);
 			m_pEncryptionIndicator->Destroy();
 			m_pEncryptionIndicator = 0;
 		}
 	}
-	else
-	{
-		wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_LOCK"), wxART_OTHER, wxSize(16, 16));
-		if (!m_pEncryptionIndicator)
-		{
+	else {
+		wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_LOCK"), wxART_OTHER,  CThemeProvider::GetIconSize(iconSizeSmall));
+		if (!m_pEncryptionIndicator) {
 			m_pEncryptionIndicator = new CIndicator(this, bmp);
-			AddChild(0, widget_encryption, m_pEncryptionIndicator);
+			AddField(0, widget_encryption, m_pEncryptionIndicator);
 			m_pEncryptionIndicator->SetToolTip(_("The connection is encrypted. Click icon for details."));
 		}
 		else
@@ -495,7 +468,7 @@ void CStatusBar::DisplayEncrypted()
 
 void CStatusBar::UpdateSizeFormat()
 {
-	// 0 equals bytes, however just use IEC binary prefixes instead, 
+	// 0 equals bytes, however just use IEC binary prefixes instead,
 	// exact byte counts for queue make no sense.
 	m_sizeFormat = CSizeFormat::_format(COptions::Get()->GetOptionVal(OPTION_SIZE_FORMAT));
 	if (!m_sizeFormat)
@@ -511,58 +484,36 @@ void CStatusBar::UpdateSizeFormat()
 
 void CStatusBar::OnHandleLeftClick(wxWindow* pWnd)
 {
-	if (pWnd == m_pEncryptionIndicator)
-	{
+	if (pWnd == m_pEncryptionIndicator) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
 		CCertificateNotification *pCertificateNotification = 0;
 		CSftpEncryptionNotification *pSftpEncryptionNotification = 0;
-		if (pState->GetSecurityInfo(pCertificateNotification))
-		{
+		if (pState->GetSecurityInfo(pCertificateNotification)) {
 			CVerifyCertDialog dlg;
-			dlg.ShowVerificationDialog(pCertificateNotification, true);
+			dlg.ShowVerificationDialog(*pCertificateNotification, true);
 		}
-		else if (pState->GetSecurityInfo(pSftpEncryptionNotification))
-		{
+		else if (pState->GetSecurityInfo(pSftpEncryptionNotification)) {
 			CSftpEncryptioInfoDialog dlg;
 			dlg.ShowDialog(pSftpEncryptionNotification);
 		}
 		else
-			wxMessageBox(_("Certificate and session data are not available yet."), _("Security information"));
+			wxMessageBoxEx(_("Certificate and session data are not available yet."), _("Security information"));
 	}
-	else if (pWnd == m_pSpeedLimitsIndicator)
-	{
+	else if (pWnd == m_pSpeedLimitsIndicator) {
 		CSpeedLimitsDialog dlg;
 		dlg.Run(m_pParent);
 	}
+	else if (pWnd == m_pDataTypeIndicator) {
+		ShowDataTypeMenu();
+	}
 }
 
 void CStatusBar::OnHandleRightClick(wxWindow* pWnd)
 {
-	if (pWnd == m_pDataTypeIndicator)
-	{
-		wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TRANSFER_TYPE_CONTEXT"));
-		if (!pMenu)
-			return;
-	
-		const int type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
-		switch (type)
-		{
-		case 1:
-			pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_ASCII"), true);
-			break;
-		case 2:
-			pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), true);
-			break;
-		default:
-			pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_AUTO"), true);
-			break;
-		}
-
-		PopupMenu(pMenu);
-		delete pMenu;
+	if (pWnd == m_pDataTypeIndicator) {
+		ShowDataTypeMenu();
 	}
-	else if (pWnd == m_pSpeedLimitsIndicator)
-	{
+	else if (pWnd == m_pSpeedLimitsIndicator) {
 		wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_SPEEDLIMITCONTEXT"));
 		if (!pMenu)
 			return;
@@ -579,84 +530,101 @@ void CStatusBar::OnHandleRightClick(wxWindow* pWnd)
 	}
 }
 
+void CStatusBar::ShowDataTypeMenu()
+{
+	wxMenu* pMenu = wxXmlResource::Get()->LoadMenu(_T("ID_MENU_TRANSFER_TYPE_CONTEXT"));
+	if (!pMenu)
+		return;
+
+	const int type = COptions::Get()->GetOptionVal(OPTION_ASCIIBINARY);
+	switch (type)
+	{
+	case 1:
+		pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_ASCII"), true);
+		break;
+	case 2:
+		pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_BINARY"), true);
+		break;
+	default:
+		pMenu->Check(XRCID("ID_MENU_TRANSFER_TYPE_AUTO"), true);
+		break;
+	}
+
+	PopupMenu(pMenu);
+	delete pMenu;
+}
+
 void CStatusBar::UpdateSpeedLimitsIcon()
 {
 	bool enable = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_ENABLE) != 0;
 
-	wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, wxSize(16, 16));
+	wxBitmap bmp = wxArtProvider::GetBitmap(_T("ART_SPEEDLIMITS"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall));
+	if (!bmp.Ok()) {
+		return;
+	}
 	wxString tooltip;
 
 	int downloadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
 	int uploadLimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
-	if (!enable || (!downloadLimit && !uploadLimit))
-	{
+	if (!enable || (!downloadLimit && !uploadLimit)) {
 		wxImage img = bmp.ConvertToImage();
 		img = img.ConvertToGreyscale();
 		bmp = wxBitmap(img);
 		tooltip = _("Speed limits are disabled, click to change.");
 	}
-	else
-	{
+	else {
 		tooltip = _("Speed limits are enabled, click to change.");
 		tooltip += _T("\n");
 		if (downloadLimit)
-			tooltip += wxString::Format(_("Download limit: %s/s"), CSizeFormat::FormatUnit(downloadLimit, CSizeFormat::kilo).c_str());
+			tooltip += wxString::Format(_("Download limit: %s/s"), CSizeFormat::FormatUnit(downloadLimit, CSizeFormat::kilo));
 		else
 			tooltip += _("Download limit: none");
 		tooltip += _T("\n");
 		if (uploadLimit)
-			tooltip += wxString::Format(_("Upload limit: %s/s"), CSizeFormat::FormatUnit(uploadLimit, CSizeFormat::kilo).c_str());
+			tooltip += wxString::Format(_("Upload limit: %s/s"), CSizeFormat::FormatUnit(uploadLimit, CSizeFormat::kilo));
 		else
 			tooltip += _("Upload limit: none");
 	}
-	
-	if (!m_pSpeedLimitsIndicator)
-	{
+
+	if (!m_pSpeedLimitsIndicator) {
 		m_pSpeedLimitsIndicator = new CIndicator(this, bmp);
-		AddChild(0, widget_speedlimit, m_pSpeedLimitsIndicator);
+		AddField(0, widget_speedlimit, m_pSpeedLimitsIndicator);
 	}
 	else
 		m_pSpeedLimitsIndicator->SetBitmap(bmp);
 	m_pSpeedLimitsIndicator->SetToolTip(tooltip);
 }
 
-void CStatusBar::OnOptionChanged(int option)
+void CStatusBar::OnOptionsChanged(changed_options_t const& options)
 {
-	switch (option)
-	{
-	case OPTION_SPEEDLIMIT_ENABLE:
-	case OPTION_SPEEDLIMIT_INBOUND:
-	case OPTION_SPEEDLIMIT_OUTBOUND:
+	if (options.test(OPTION_SPEEDLIMIT_ENABLE) || options.test(OPTION_SPEEDLIMIT_INBOUND) || options.test(OPTION_SPEEDLIMIT_OUTBOUND)) {
 		UpdateSpeedLimitsIcon();
-		break;
-	case OPTION_SIZE_FORMAT:
-	case OPTION_SIZE_USETHOUSANDSEP:
-	case OPTION_SIZE_DECIMALPLACES:
+	}
+	if (options.test(OPTION_SIZE_FORMAT) || options.test(OPTION_SIZE_USETHOUSANDSEP) || options.test(OPTION_SIZE_DECIMALPLACES)) {
 		UpdateSizeFormat();
-		break;
-	case OPTION_ASCIIBINARY:
+	}
+	if (options.test(OPTION_ASCIIBINARY)) {
 		DisplayDataType();
-		break;
-	case OPTION_THEME:
+	}
+	if (options.test(OPTION_THEME)) {
 		DisplayDataType();
 		UpdateSpeedLimitsIcon();
 		DisplayEncrypted();
-		break;
-	default:
-		break;
 	}
 }
 
-void CStatusBar::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
+void CStatusBar::OnStateChange(CState*, enum t_statechange_notifications notification, const wxString&, const void*)
 {
-	if (notification == STATECHANGE_SERVER || notification == STATECHANGE_CHANGEDCONTEXT)
-	{
+	if (notification == STATECHANGE_SERVER || notification == STATECHANGE_CHANGEDCONTEXT) {
 		DisplayDataType();
 		DisplayEncrypted();
 	}
+	else if (notification == STATECHANGE_ENCRYPTION) {
+		DisplayEncrypted();
+	}
 }
 
-void CStatusBar::OnSpeedLimitsEnable(wxCommandEvent& event)
+void CStatusBar::OnSpeedLimitsEnable(wxCommandEvent&)
 {
 	int downloadlimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_INBOUND);
 	int uploadlimit = COptions::Get()->GetOptionVal(OPTION_SPEEDLIMIT_OUTBOUND);
@@ -675,7 +643,7 @@ void CStatusBar::OnSpeedLimitsEnable(wxCommandEvent& event)
 		COptions::Get()->SetOption(OPTION_SPEEDLIMIT_ENABLE, 0);
 }
 
-void CStatusBar::OnSpeedLimitsConfigure(wxCommandEvent& event)
+void CStatusBar::OnSpeedLimitsConfigure(wxCommandEvent&)
 {
 	CSpeedLimitsDialog dlg;
 	dlg.Run(m_pParent);
