@@ -1,7 +1,7 @@
 #ifndef __STATUSBAR_H__
 #define __STATUSBAR_H__
 
-#include "option_change_event_handler.h"
+#include <option_change_event_handler.h>
 #include "sizeformatting.h"
 #include "state.h"
 
@@ -24,7 +24,7 @@ public:
 
 	// We override these for two reasons:
 	// - wxWidgets does not provide a function to get the field widths back
-	// - fixup for last field. Under MSW it has a gripper if window is not 
+	// - fixup for last field. Under MSW it has a gripper if window is not
 	//   maximized, under GTK2 it always has a gripper. These grippers overlap
 	//   last field.
 	virtual void SetFieldsCount(int number = 1, const int* widths = NULL);
@@ -51,7 +51,7 @@ protected:
 
 	int* m_columnWidths;
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent& event);
 };
 
@@ -64,9 +64,9 @@ public:
 	// Adds a child window that gets repositioned on window resize
 	// Positioned in the field given in the constructor,
 	// right aligned and in reverse order.
-	void AddChild(int field, int idx, wxWindow* pChild);
-	
-	void RemoveChild(int idx);
+	bool AddField(int field, int idx, wxWindow* pChild);
+
+	void RemoveField(int idx);
 
 	virtual void SetFieldWidth(int field, int width);
 protected:
@@ -81,7 +81,7 @@ protected:
 
 	void PositionChildren(int field);
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnSize(wxSizeEvent& event);
 };
 
@@ -91,7 +91,7 @@ public:
 	CStatusBar(wxTopLevelWindow* parent);
 	virtual ~CStatusBar();
 
-	void DisplayQueueSize(wxLongLong totalSize, bool hasUnknown);
+	void DisplayQueueSize(int64_t totalSize, bool hasUnknown);
 
 	void OnHandleLeftClick(wxWindow* wnd);
 	void OnHandleRightClick(wxWindow* wnd);
@@ -104,20 +104,22 @@ protected:
 
 	void MeasureQueueSizeWidth();
 
-	virtual void OnOptionChanged(int option);
+	void ShowDataTypeMenu();
+
+	virtual void OnOptionsChanged(changed_options_t const& options);
 	virtual void OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2);
 
 	CSizeFormat::_format m_sizeFormat;
 	bool m_sizeFormatThousandsSep;
 	int m_sizeFormatDecimalPlaces;
-	wxLongLong m_size;
+	int64_t m_size;
 	bool m_hasUnknownFiles;
 
 	wxStaticBitmap* m_pDataTypeIndicator;
 	wxStaticBitmap* m_pEncryptionIndicator;
 	wxStaticBitmap* m_pSpeedLimitsIndicator;
 
-	DECLARE_EVENT_TABLE();
+	DECLARE_EVENT_TABLE()
 	void OnSpeedLimitsEnable(wxCommandEvent& event);
 	void OnSpeedLimitsConfigure(wxCommandEvent& event);
 };
