@@ -3,120 +3,99 @@
 #include "Options.h"
 #include <wx/ffile.h>
 #include <wx/log.h>
+#include <wx/base64.h>
 
 #include <local_filesys.h>
 
-CXmlFile::CXmlFile(const wxString& fileName)
+CXmlFile::CXmlFile(wxString const& fileName, wxString const& root)
 {
-	m_pPrinter = 0;
+	if (!root.empty()) {
+		m_rootName = root;
+	}
 	SetFileName(fileName);
-	m_pDocument = 0;
-}
-
-CXmlFile::CXmlFile(const wxFileName& fileName /*=wxFileName()*/)
-{
-	m_pPrinter = 0;
-	SetFileName(fileName);
-	m_pDocument = 0;
 }
 
 void CXmlFile::SetFileName(const wxString& name)
 {
-	m_fileName = wxFileName(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), name + _T(".xml"));
-	m_modificationTime = wxDateTime();
-}
-
-void CXmlFile::SetFileName(const wxFileName& fileName)
-{
-	m_fileName = fileName;
-	m_modificationTime = wxDateTime();
+	wxASSERT(!name.empty());
+	m_fileName = name;
+	m_modificationTime = CDateTime();
 }
 
 CXmlFile::~CXmlFile()
 {
-	delete m_pPrinter;
-	delete m_pDocument;
+	Close();
 }
 
-TiXmlElement* CXmlFile::Load(const wxString& name)
+TiXmlElement* CXmlFile::Load()
 {
-	wxFileName fileName(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), name + _T(".xml"));
-	return Load(fileName);
-}
+	Close();
+	m_error.clear();
 
-TiXmlElement* CXmlFile::Load(const wxFileName& fileName)
-{
-	if (fileName.IsOk())
-		SetFileName(fileName);
+	wxCHECK(!m_fileName.empty(), 0);
 
-	wxCHECK(m_fileName.IsOk(), 0);
+	wxString redirectedName = GetRedirectedName();
 
-	delete m_pDocument;
-	m_pDocument = 0;
-
-	wxString error;
-	TiXmlElement* pElement = GetXmlFile(m_fileName, &error);
-	if (!pElement)
-	{
-		if (!error.empty())
-		{
-			m_error.Printf(_("The file '%s' could not be loaded."), m_fileName.GetFullPath().c_str());
-			if (!error.empty())
-				m_error += _T("\n") + error;
-			else
-				m_error += wxString(_T("\n")) + _("Make sure the file can be accessed and is a well-formed XML document.");
-			m_modificationTime = wxDateTime();
+	GetXmlFile(redirectedName);
+	if (!m_pElement) {
+		wxString err = wxString::Format(_("The file '%s' could not be loaded."), m_fileName);
+		if (m_error.empty()) {
+			err += wxString(_T("\n")) + _("Make sure the file can be accessed and is a well-formed XML document.");
 		}
-		return 0;
+		else {
+			err += _T("\n") + m_error;
+		}
+
+		// Try the backup file
+		GetXmlFile(redirectedName + _T("~"));
+		if (!m_pElement) {
+			// Loading backup failed. If both original and backup file are empty, create new file.
+			if (CLocalFileSystem::GetSize(redirectedName) <= 0 && CLocalFileSystem::GetSize(redirectedName + _T("~")) <= 0) {
+				m_error.clear();
+				CreateEmpty();
+				m_modificationTime = CLocalFileSystem::GetModificationTime(redirectedName);
+				return m_pElement;
+			}
+
+			// File corrupt and no functional backup, give up.
+			m_error = err;
+			m_modificationTime.clear();
+			return 0;
+		}
+
+
+		// Loading the backup file succeeded, restore file
+		bool res;
+		{
+			wxLogNull null;
+			res = wxCopyFile(redirectedName + _T("~"), redirectedName);
+		}
+		if (!res) {
+			// Could not restore backup, give up.
+			Close();
+			m_error = err;
+			m_error += _T("\n") + wxString::Format(_("The valid backup file %s could not be restored"), redirectedName + _T("~"));
+			m_modificationTime.clear();
+			return 0;
+		}
+
+		// We no longer need the backup
+		wxRemoveFile(redirectedName + _T("~"));
+		m_error.clear();
 	}
 
-	{
-		wxLogNull log;
-		m_modificationTime = m_fileName.GetModificationTime();
-	}
-
-	m_pDocument = pElement->GetDocument();
-	return pElement;
-}
-
-TiXmlElement* CXmlFile::GetElement()
-{
-	if (!m_pDocument)
-		return 0;
-
-	TiXmlElement* pElement = m_pDocument->FirstChildElement("FileZilla3");
-	if (!pElement)
-	{
-		delete m_pDocument;
-		m_pDocument = 0;
-		return 0;
-	}
-	else
-		return pElement;
-}
-
-const TiXmlElement* CXmlFile::GetElement() const
-{
-	if (!m_pDocument)
-		return 0;
-
-	const TiXmlElement* pElement = m_pDocument->FirstChildElement("FileZilla3");
-	return pElement;
+	m_modificationTime = CLocalFileSystem::GetModificationTime(redirectedName);
+	return m_pElement;
 }
 
 bool CXmlFile::Modified()
 {
-	wxCHECK(m_fileName.IsOk(), false);
+	wxCHECK(!m_fileName.empty(), false);
 
 	if (!m_modificationTime.IsValid())
-		return false;
-
-	wxLogNull log;
-	wxDateTime modificationTime;
-	if (!m_fileName.FileExists())
 		return true;
 
-	modificationTime = m_fileName.GetModificationTime();
+	CDateTime const modificationTime = CLocalFileSystem::GetModificationTime(m_fileName);
 	if (modificationTime.IsValid() && modificationTime == m_modificationTime)
 		return false;
 
@@ -127,45 +106,40 @@ void CXmlFile::Close()
 {
 	delete m_pDocument;
 	m_pDocument = 0;
+	m_pElement = 0;
 }
 
-bool CXmlFile::Save(wxString* error)
+bool CXmlFile::Save(bool printError)
 {
-	wxCHECK(m_fileName.IsOk(), false);
+	m_error.clear();
 
+	wxCHECK(!m_fileName.empty(), false);
 	wxCHECK(m_pDocument, false);
 
-	bool res = SaveXmlFile(m_fileName, GetElement(), error);
-	wxLogNull log;
-	m_modificationTime = m_fileName.GetModificationTime();
+	bool res = SaveXmlFile();
+	m_modificationTime = CLocalFileSystem::GetModificationTime(m_fileName);
+
+	if (!res && printError) {
+		wxASSERT(!m_error.empty());
+
+		wxString msg = wxString::Format(_("Could not write \"%s\":"), m_fileName);
+		wxMessageBoxEx(msg + _T("\n") + m_error, _("Error writing xml file"), wxICON_ERROR);
+	}
 	return res;
 }
 
 TiXmlElement* CXmlFile::CreateEmpty()
 {
-	delete m_pDocument;
+	Close();
 
 	m_pDocument = new TiXmlDocument();
 	m_pDocument->SetCondenseWhiteSpace(false);
 	m_pDocument->LinkEndChild(new TiXmlDeclaration("1.0", "UTF-8", "yes"));
 
-	return m_pDocument->LinkEndChild(new TiXmlElement("FileZilla3"))->ToElement();
-}
+	m_pElement = new TiXmlElement(m_rootName);
+	m_pDocument->LinkEndChild(m_pElement);
 
-char* ConvUTF8(const wxString& value)
-{
-	// First convert the string into unicode if neccessary.
-	const wxWCharBuffer buffer = wxConvCurrent->cWX2WC(value);
-
-	// Calculate utf-8 string length
-	wxMBConvUTF8 conv;
-	int len = conv.WC2MB(0, buffer, 0);
-
-	// Not convert the string
-	char *utf8 = new char[len + 1];
-	conv.WC2MB(utf8, buffer, len + 1);
-
-	return utf8;
+	return m_pElement;
 }
 
 wxString ConvLocal(const char *value)
@@ -175,26 +149,23 @@ wxString ConvLocal(const char *value)
 
 void RemoveTextElement(TiXmlElement* node)
 {
-	for (TiXmlNode* pChild = node->FirstChild(); pChild; pChild = pChild->NextSibling())
-	{
-		if (!pChild->ToText())
-			continue;
-
-		node->RemoveChild(pChild);
-		break;
+	for (TiXmlNode* pChild = node->FirstChild(); pChild; pChild = pChild->NextSibling()) {
+		if (pChild->ToText()) {
+			node->RemoveChild(pChild);
+			break;
+		}
 	}
 }
 
-void AddTextElement(TiXmlElement* node, const char* name, const wxString& value, bool overwrite)
+TiXmlElement* AddTextElement(TiXmlElement* node, const char* name, const wxString& value, bool overwrite)
 {
 	wxASSERT(node);
 
-	char* utf8 = ConvUTF8(value);
+	wxScopedCharBuffer utf8 = value.utf8_str();
 	if (!utf8)
-		return;
+		return 0;
 
-	AddTextElementRaw(node, name, utf8, overwrite);
-	delete [] utf8;
+	return AddTextElementRaw(node, name, utf8, overwrite);
 }
 
 void AddTextElement(TiXmlElement* node, const char* name, int value, bool overwrite)
@@ -204,27 +175,27 @@ void AddTextElement(TiXmlElement* node, const char* name, int value, bool overwr
 	AddTextElementRaw(node, name, buffer, overwrite);
 }
 
-void AddTextElementRaw(TiXmlElement* node, const char* name, const char* value, bool overwrite)
+TiXmlElement* AddTextElementRaw(TiXmlElement* node, const char* name, const char* value, bool overwrite)
 {
 	wxASSERT(node);
 	wxASSERT(value);
 
 	TiXmlElement *element = 0;
-	if (overwrite)
-	{
+	if (overwrite) {
 		element = node->FirstChildElement(name);
 		if (element)
 			RemoveTextElement(element);
 	}
 
-	if (!element)
-	{
+	if (!element) {
 		element = new TiXmlElement(name);
 		node->LinkEndChild(element);
 	}
 
 	if (*value)
 		element->LinkEndChild(new TiXmlText(value));
+
+	return element;
 }
 
 void AddTextElement(TiXmlElement* node, const wxString& value)
@@ -232,12 +203,11 @@ void AddTextElement(TiXmlElement* node, const wxString& value)
 	wxASSERT(node);
 	wxASSERT(value);
 
-	char* utf8 = ConvUTF8(value);
+	wxScopedCharBuffer utf8 = value.utf8_str();
 	if (!utf8)
 		return;
 
 	AddTextElementRaw(node, utf8);
-	delete [] utf8;
 }
 
 void AddTextElement(TiXmlElement* node, int value)
@@ -273,11 +243,11 @@ wxString GetTextElement(TiXmlElement* node, const char* name)
 
 	TiXmlElement* element = node->FirstChildElement(name);
 	if (!element)
-		return _T("");
+		return wxString();
 
 	TiXmlNode* textNode = element->FirstChild();
 	if (!textNode || !textNode->ToText())
-		return _T("");
+		return wxString();
 
 	return ConvLocal(textNode->Value());
 }
@@ -295,15 +265,13 @@ wxString GetTextElement(TiXmlElement* node)
 {
 	wxASSERT(node);
 
-	for (TiXmlNode* pChild = node->FirstChild(); pChild; pChild = pChild->NextSibling())
-	{
-		if (!pChild->ToText())
-			continue;
-
-		return ConvLocal(pChild->Value());
+	for (TiXmlNode* pChild = node->FirstChild(); pChild; pChild = pChild->NextSibling()) {
+		if (pChild->ToText()) {
+			return ConvLocal(pChild->Value());
+		}
 	}
 
-	return _T("");
+	return wxString();
 }
 
 int GetTextElementInt(TiXmlElement* node, const char* name, int defValue /*=0*/)
@@ -323,13 +291,11 @@ int GetTextElementInt(TiXmlElement* node, const char* name, int defValue /*=0*/)
 
 	int value = 0;
 	bool negative = false;
-	if (*p == '-')
-	{
+	if (*p == '-') {
 		negative = true;
 		p++;
 	}
-	while (*p)
-	{
+	while (*p) {
 		if (*p < '0' || *p > '9')
 			return defValue;
 
@@ -359,13 +325,11 @@ wxLongLong GetTextElementLongLong(TiXmlElement* node, const char* name, int defV
 
 	wxLongLong value = 0;
 	bool negative = false;
-	if (*p == '-')
-	{
+	if (*p == '-') {
 		negative = true;
 		p++;
 	}
-	while (*p)
-	{
+	while (*p) {
 		if (*p < '0' || *p > '9')
 			return defValue;
 
@@ -405,145 +369,123 @@ bool GetTextElementBool(TiXmlElement* node, const char* name, bool defValue /*=f
 	}
 }
 
-bool LoadXmlDocument(TiXmlDocument* pXmlDocument, const wxString& file, wxString* error /*=0*/)
+bool CXmlFile::LoadXmlDocument(wxString const& file)
 {
 	wxFFile f(file, _T("rb"));
-	if (!f.IsOpened())
-	{
-		if (error)
-		{
-			const wxChar* s = wxSysErrorMsg();
-			if (s && *s)
-				*error = s;
-			else
-				*error = _("Unknown error opening the file. Make sure the file can be accessed and is a well-formed XML document.");
-		}
+	if (!f.IsOpened()) {
+		const wxChar* s = wxSysErrorMsg();
+		if (s && *s)
+			m_error = s;
+		else
+			m_error = _("Unknown error opening the file. Make sure the file can be accessed and is a well-formed XML document.");
 		return false;
 	}
 
-	if (!pXmlDocument->LoadFile(f.fp()))
-	{
-		if (pXmlDocument->ErrorId() != TiXmlBase::TIXML_ERROR_DOCUMENT_EMPTY)
-		{
-			if (error)
-			{
-				const char* s = pXmlDocument->ErrorDesc();
-				error->Printf(_("The XML document is not well-formed: %s"), wxString(s, wxConvLibc).c_str());
-			}
-			return false;
+	m_pDocument = new TiXmlDocument;
+	m_pDocument->SetCondenseWhiteSpace(false);
+	if (!m_pDocument->LoadFile(f.fp()) && m_pDocument->ErrorId() != TiXmlBase::TIXML_ERROR_DOCUMENT_EMPTY) {
+		const char* s = m_pDocument->ErrorDesc();
+		if (s && *s) {
+			m_error.Printf(_("The XML document is not well-formed: %s"), wxString(s, wxConvLibc));
 		}
+		else
+			m_error = _("Unknown error opening the file. Make sure the file can be accessed and is a well-formed XML document.");
+		delete m_pDocument;
+		m_pDocument = 0;
+		return false;
 	}
 	return true;
 }
 
 // Opens the specified XML file if it exists or creates a new one otherwise.
 // Returns 0 on error.
-TiXmlElement* GetXmlFile(wxFileName file, wxString* error /*=0*/)
+bool CXmlFile::GetXmlFile(wxString const& file)
 {
-	if (wxFileExists(file.GetFullPath()) && file.GetSize() > 0)
-	{
-		// File does exist, open it
+	Close();
 
-		TiXmlDocument* pXmlDocument = new TiXmlDocument;
-		pXmlDocument->SetCondenseWhiteSpace(false);
-		if (!LoadXmlDocument(pXmlDocument, file.GetFullPath(), error))
-		{
-			delete pXmlDocument;
-			return 0;
-		}
-
-		TiXmlElement* pElement = pXmlDocument->FirstChildElement("FileZilla3");
-		if (!pElement)
-		{
-			if (pXmlDocument->FirstChildElement())
-			{
-				// Not created by FileZilla3
-				delete pXmlDocument;
-
-				if (error)
-					*error = _("Unknown root element, the file does not appear to be generated by FileZilla.");
-				return 0;
-			}
-			pElement = pXmlDocument->LinkEndChild(new TiXmlElement("FileZilla3"))->ToElement();
-		}
-
-		return pElement;
+	if (CLocalFileSystem::GetSize(file) <= 0) {
+		return false;
 	}
-	else
-	{
-		// File does not exist, return empty XML document.
-		TiXmlDocument* pXmlDocument = new TiXmlDocument();
-		pXmlDocument->SetCondenseWhiteSpace(false);
-		pXmlDocument->LinkEndChild(new TiXmlDeclaration("1.0", "UTF-8", "yes"));
 
-		pXmlDocument->LinkEndChild(new TiXmlElement("FileZilla3"));
-
-		return pXmlDocument->FirstChildElement("FileZilla3");
+	// File exists, open it
+	if (!LoadXmlDocument(file)) {
+		return false;
 	}
+
+	m_pElement = m_pDocument->FirstChildElement(m_rootName);
+	if (!m_pElement) {
+		if (m_pDocument->FirstChildElement()) {
+			// Not created by FileZilla3
+			Close();
+			m_error = _("Unknown root element, the file does not appear to be generated by FileZilla.");
+			return false;
+		}
+		m_pElement = m_pDocument->LinkEndChild(new TiXmlElement(m_rootName))->ToElement();
+	}
+
+	return true;
 }
 
-bool SaveXmlFile(const wxFileName& file, TiXmlNode* node, wxString* error /*=0*/, bool move /*=false*/)
+wxString CXmlFile::GetRedirectedName() const
 {
-	if (!node)
-		return true;
-
-	const wxString& fullPath = file.GetFullPath();
-
-	TiXmlDocument* pDocument = node->GetDocument();
+	wxString redirectedName = m_fileName;
+	bool isLink = false;
+	if (CLocalFileSystem::GetFileInfo(m_fileName, isLink, 0, 0, 0) == CLocalFileSystem::file) {
+		if (isLink) {
+			CLocalPath target(CLocalFileSystem::GetSymbolicLinkTarget(m_fileName));
+			if (!target.empty()) {
+				redirectedName = target.GetPath();
+				redirectedName.RemoveLast();
+			}
+		}
+	}
+	return redirectedName;
+}
 
+bool CXmlFile::SaveXmlFile()
+{
 	bool exists = false;
 
 	bool isLink = false;
 	int flags = 0;
-	if (CLocalFileSystem::GetFileInfo( fullPath, isLink, 0, 0, &flags ) == CLocalFileSystem::file)
-	{
+
+	wxString redirectedName = GetRedirectedName();
+	if (CLocalFileSystem::GetFileInfo(redirectedName, isLink, 0, 0, &flags) == CLocalFileSystem::file) {
 #ifdef __WXMSW__
 		if (flags & FILE_ATTRIBUTE_HIDDEN)
-			SetFileAttributes(fullPath, flags & ~FILE_ATTRIBUTE_HIDDEN);
+			SetFileAttributes(redirectedName, flags & ~FILE_ATTRIBUTE_HIDDEN);
 #endif
 
 		exists = true;
 		bool res;
-		if (!move)
 		{
 			wxLogNull null;
-			res = wxCopyFile(fullPath, fullPath + _T("~"));
+			res = wxCopyFile(redirectedName, redirectedName + _T("~"));
 		}
-		else
-		{
-			wxLogNull null;
-			res = wxRenameFile(fullPath, fullPath + _T("~"));
-		}
-		if (!res)
-		{
-			const wxString msg = _("Failed to create backup copy of xml file");
-			if (error)
-				*error = msg;
-			else
-				wxMessageBox(msg);
+		if (!res) {
+			m_error = _("Failed to create backup copy of xml file");
 			return false;
 		}
 	}
 
-	wxFFile f(fullPath, _T("w"));
-	if (!f.IsOpened() || !pDocument->SaveFile(f.fp()))
+	bool success = false;
 	{
-		wxRemoveFile(fullPath);
-		if (exists)
-		{
+		wxFFile f(redirectedName, _T("w"));
+		success = f.IsOpened() && m_pDocument->SaveFile(f.fp());
+	}
+
+	if (!success) {
+		wxRemoveFile(redirectedName);
+		if (exists) {
 			wxLogNull null;
-			wxRenameFile(fullPath + _T("~"), fullPath);
+			wxRenameFile(redirectedName + _T("~"), redirectedName);
 		}
-		const wxString msg = _("Failed to write xml file");
-		if (error)
-			*error = msg;
-		else
-			wxMessageBox(msg);
+		m_error = _("Failed to write xml file");
 		return false;
 	}
 
 	if (exists)
-		wxRemoveFile(fullPath + _T("~"));
+		wxRemoveFile(redirectedName + _T("~"));
 
 	return true;
 }
@@ -553,7 +495,7 @@ bool GetServer(TiXmlElement *node, CServer& server)
 	wxASSERT(node);
 
 	wxString host = GetTextElement(node, "Host");
-	if (host == _T(""))
+	if (host.empty())
 		return false;
 
 	int port = GetTextElementInt(node, "Port");
@@ -563,11 +505,11 @@ bool GetServer(TiXmlElement *node, CServer& server)
 	if (!server.SetHost(host, port))
 		return false;
 
-	int protocol = GetTextElementInt(node, "Protocol");
-	if (protocol < 0)
+	int const protocol = GetTextElementInt(node, "Protocol");
+	if (protocol < 0 || protocol > ServerProtocol::MAX_VALUE) {
 		return false;
-
-	server.SetProtocol((enum ServerProtocol)protocol);
+	}
+	server.SetProtocol(static_cast<ServerProtocol>(protocol));
 
 	int type = GetTextElementInt(node, "Type");
 	if (type < 0 || type >= SERVERTYPE_MAX)
@@ -581,21 +523,39 @@ bool GetServer(TiXmlElement *node, CServer& server)
 
 	server.SetLogonType((enum LogonType)logonType);
 
-	if (server.GetLogonType() != ANONYMOUS)
-	{
+	if (server.GetLogonType() != ANONYMOUS) {
 		wxString user = GetTextElement(node, "User");
 
 		wxString pass;
-		if ((long)NORMAL == logonType || (long)ACCOUNT == logonType)
-			pass = GetTextElement(node, "Pass");
+		if ((long)NORMAL == logonType || (long)ACCOUNT == logonType) {
+			TiXmlElement* passElement = node->FirstChildElement("Pass");
+			if (passElement) {
+				pass = GetTextElement(passElement);
+
+				wxString encoding = GetTextAttribute(passElement, "encoding");
+
+				if (encoding == _T("base64")) {
+					wxMemoryBuffer buf = wxBase64Decode(pass);
+					if (!buf.IsEmpty()) {
+						pass = wxString::FromUTF8(static_cast<const char*>(buf.GetData()), buf.GetDataLen());
+					}
+					else {
+						pass.clear();
+					}
+				}
+				else if (!encoding.empty()) {
+					pass.clear();
+					server.SetLogonType(ASK);
+				}
+			}
+		}
 
 		if (!server.SetUser(user, pass))
 			return false;
 
-		if ((long)ACCOUNT == logonType)
-		{
+		if ((long)ACCOUNT == logonType) {
 			wxString account = GetTextElement(node, "Account");
-			if (account == _T(""))
+			if (account.empty())
 				return false;
 			if (!server.SetAccount(account))
 				return false;
@@ -622,10 +582,9 @@ bool GetServer(TiXmlElement *node, CServer& server)
 		server.SetEncodingType(ENCODING_AUTO);
 	else if (encodingType == _T("UTF-8"))
 		server.SetEncodingType(ENCODING_UTF8);
-	else if (encodingType == _T("Custom"))
-	{
+	else if (encodingType == _T("Custom")) {
 		wxString customEncoding = GetTextElement(node, "CustomEncoding");
-		if (customEncoding == _T(""))
+		if (customEncoding.empty())
 			return false;
 		if (!server.SetEncodingType(ENCODING_CUSTOM, customEncoding))
 			return false;
@@ -633,20 +592,16 @@ bool GetServer(TiXmlElement *node, CServer& server)
 	else
 		server.SetEncodingType(ENCODING_AUTO);
 
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
-	{
+	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
 		std::vector<wxString> postLoginCommands;
 		TiXmlElement* pElement = node->FirstChildElement("PostLoginCommands");
-		if (pElement)
-		{
+		if (pElement) {
 			TiXmlElement* pCommandElement = pElement->FirstChildElement("Command");
-			while (pCommandElement)
-			{
+			while (pCommandElement) {
 				TiXmlNode* textNode = pCommandElement->FirstChild();
-				if (textNode && textNode->ToText())
-				{
+				if (textNode && textNode->ToText()) {
 					wxString command = ConvLocal(textNode->Value());
-					if (command != _T(""))
+					if (!command.empty())
 						postLoginCommands.push_back(command);
 				}
 
@@ -682,17 +637,22 @@ void SetServer(TiXmlElement *node, const CServer& server)
 
 	enum LogonType logonType = server.GetLogonType();
 
-	if (server.GetLogonType() != ANONYMOUS)
-	{
+	if (server.GetLogonType() != ANONYMOUS) {
 		AddTextElement(node, "User", server.GetUser());
 
-		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT)
-		{
+		if (server.GetLogonType() == NORMAL || server.GetLogonType() == ACCOUNT) {
 			if (kiosk_mode)
 				logonType = ASK;
-			else
-			{
-				AddTextElement(node, "Pass", server.GetPass());
+			else {
+				wxString pass = server.GetPass();
+				auto const& buf = pass.utf8_str(); // wxWidgets has such an ugly string API....
+				std::string utf8(buf.data(), buf.length());
+
+				wxString base64 = wxBase64Encode(utf8.c_str(), utf8.size());
+				TiXmlElement* passElement = AddTextElement(node, "Pass", base64);
+				if (passElement) {
+					SetTextAttribute(passElement, "encoding", _T("base64"));
+				}
 
 				if (server.GetLogonType() == ACCOUNT)
 					AddTextElement(node, "Account", server.GetAccount());
@@ -730,12 +690,9 @@ void SetServer(TiXmlElement *node, const CServer& server)
 		break;
 	}
 
-	const enum ServerProtocol protocol = server.GetProtocol();
-	if (protocol == FTP || protocol == FTPS || protocol == FTPES)
-	{
-		const std::vector<wxString>& postLoginCommands = server.GetPostLoginCommands();
-		if (!postLoginCommands.empty())
-		{
+	if (CServer::SupportsPostLoginCommands(server.GetProtocol())) {
+		std::vector<wxString> const& postLoginCommands = server.GetPostLoginCommands();
+		if (!postLoginCommands.empty()) {
 			TiXmlElement* pElement = node->LinkEndChild(new TiXmlElement("PostLoginCommands"))->ToElement();
 			for (std::vector<wxString>::const_iterator iter = postLoginCommands.begin(); iter != postLoginCommands.end(); ++iter)
 				AddTextElement(pElement, "Command", *iter);
@@ -744,7 +701,7 @@ void SetServer(TiXmlElement *node, const CServer& server)
 
 	AddTextElementRaw(node, "BypassProxy", server.GetBypassProxy() ? "1" : "0");
 	const wxString& name = server.GetName();
-	if (name != _T(""))
+	if (!name.empty())
 		AddTextElement(node, "Name", name);
 }
 
@@ -752,12 +709,11 @@ void SetTextAttribute(TiXmlElement* node, const char* name, const wxString& valu
 {
 	wxASSERT(node);
 
-	char* utf8 = ConvUTF8(value);
+	wxScopedCharBuffer utf8 = value.utf8_str();
 	if (!utf8)
 		return;
 
 	node->SetAttribute(name, utf8);
-	delete [] utf8;
 }
 
 wxString GetTextAttribute(TiXmlElement* node, const char* name)
@@ -766,7 +722,7 @@ wxString GetTextAttribute(TiXmlElement* node, const char* name)
 
 	const char* value = node->Attribute(name);
 	if (!value)
-		return _T("");
+		return wxString();
 
 	return ConvLocal(value);
 }
@@ -858,15 +814,14 @@ void CXmlFile::GetRawDataHere(char* p) // p has to big enough to hold at least G
 
 bool CXmlFile::ParseData(char* data)
 {
-	delete m_pDocument;
+	Close();
 	m_pDocument = new TiXmlDocument;
 	m_pDocument->SetCondenseWhiteSpace(false);
 	m_pDocument->Parse(data);
 
-	if (!m_pDocument->FirstChildElement("FileZilla3"))
-	{
-		delete m_pDocument;
-		m_pDocument = 0;
+	m_pElement = m_pDocument->FirstChildElement(m_rootName);
+	if (!m_pElement) {
+		Close();
 		return false;
 	}
 
